if { $DEBUG } {
    # setup gui for debugging io to fitscript pipe.
    # use the intxt text widget to record sent commands
    # and the outtxt widget to record replies.
    # uses globals readfinished readlines from pbcor
    # RWE March 2018 start this iconified along with the console
    # and add cmnds to the File menu to deiconify .debug and console show

    set db [toplevel .debug]
    wm title $db "pbcor script command console"
    wm iconname $db "pbcorcr"
    wm deiconify $db

    proc Scrolled_Text { f args } {
		frame $f
		eval {text $f.text \
		  -xscrollcommand [list $f.xscroll set] \
		  -yscrollcommand [list $f.yscroll set]} $args
		scrollbar $f.xscroll -orient horizontal \
	    	-command [list $f.text xview]
		scrollbar $f.yscroll -orient vertical \
	    	-command [list $f.text yview]
		grid $f.text $f.yscroll -sticky news
		grid $f.xscroll -sticky news
		grid rowconfigure $f 0 -weight 1
		grid columnconfigure $f 0 -weight 1
		return $f.text
    }
    
    proc sendCur {} {
		global intxt outtxt
		global readfinished readlines
	
		if { [catch {set line \
			 [$intxt get "insert linestart" "insert lineend"]}] } {
	    	return
		}
		set line [string trim $line]
		if { [string length $line] < 1 } { return }
		pipeWrite $line
		#$outtxt insert end "pipeWrite: $line\n"
		#$outtxt insert end "pipeRead:\n"
		#vwait readfinished
		# for debugging pipeRead dumps to outtxt
    }

    proc sendClr {} {
		global outtxt
		$outtxt delete 1.0 end
    }
    
    set sendmb [frame $db.mb]
    set sendin [frame $db.in]
    set sendout [frame $db.out]
    
    # try packing the frames for the text widgets as in .txtvw
    pack $sendmb -in $db -side top -expand 0 -fill both
	pack $sendin -in $db -side top -expand 1 -fill both
    pack $sendout -in $db -side top -expand 1 -fill both

    #grid $sendmb -sticky news -pady 5
    #grid $sendin -sticky news
    #grid $sendout -sticky news
    #grid rowconfigure . 1 -weight 1
    #grid rowconfigure . 2 -weight 1
    #grid columnconfigure . 0 -weight 1


    set sendCur [button $sendmb.ex -text "EXEC current line" \
		     -command sendCur]
    
    set sendClr [button $sendmb.clr -text "Clear Output" \
		     -command sendClr]


    grid $sendCur $sendClr -sticky news

	# for now leave out the labels so only text widget is in its frame
    #set inlbl [label $sendin.lbl -text "EXEC line text editor" -justify center]

    set intxt [scrolledText $sendin -1 -1 -height 6 -width 60]
    #set intxt [Scrolled_Text $sendin.ftxt]
    #$intxt configure -height 6 -width 60

	# RWE Mar 2018 try adding <Return> binding to the input text widget to call sendCur
	bind $intxt <Return> {sendCur}

    #set outlbl [label $sendout.lbl -text "RESULT" -justify center]
    #set outtxt [Scrolled_Text $sendout.ftxt]
    set outtxt [scrolledText $sendout -1 -1 -height 20 -width 60 -wrap none]
    #$outtxt configure -height 20 -width 60 -wrap none
    
   	#pack $intxt -in $db.in -side top -expand 1 -fill both
    #pack $outtxt -in $db.out -side top -expand 1 -fill both 

	#grid $inlbl -sticky news
    #grid $sendin.ftxt -sticky news
    
    #grid $outlbl -sticky news
    #grid $sendout.ftxt -sticky news
    
    update idletasks

    # will save all input to fitscript pipe in a file
    # for later debugging by running pbscript through ddd
    # The debugFPT is now opened n checkSessionRecovery where the directory should be correct
    #set debugFPT [open debugCMDS w]

    # Having the local console obviates the need for the debugServer client window
	# RWE Oct 2019 remove debugServer stuff

    if { $mswindows } {
    	console show
    	console title "pbcor tcl console"
    }
}


# package loading
if [catch {package require Tclpbs} pbcorver] {
    Dialog_Warn  "FATAL ERROR: FAILED to load required Tclpbs library. pbcorDir= $pbcorDir. $pbcorver"
    exit
}
# init the Tclpbs package
pbsinit
# check that we dont segfault on the first command. This seems to prevent SEGFAULT later
set cmd "c S"
pbs $cmd

if [catch {package require Bltxs} bltver] {
    Dialog_Warn "FATAL ERROR: FAILED to load Bltxs which is required for plotting. $bltver"
    exit
}
set BLT 1
set PLT 1
#set IMAGE 1
namespace import blt::*
if [catch {package require Graphtcl} graphver] {
	puts "$graphver failed to source graph.tcl for Bltxs"
}


set IMG 1
if [catch {package require Img} imgver] {
	set IMG 0
}


buildPrevSoln

buildTXTwarn

# pwd should get correct datadirectory as the prog start directory except for windows. See i df
# now currentDir as pwd is done in pbcorGlobals so we dont repeat here
# set currentDir [pwd]
# We want to know the working directory before building main gui
# but we want to check for saved directory using the library and pipeWrite
# for Windows
if { $mswindows } {
    # try i df
    # NB debugCMDS will not be open for the i df cmnd
    set saveDEBUG $DEBUG
    set DEBUG 0
    pipeWrite "i df"
    if { $nerr > 0 } {
        selectDataDirectory 0
    } else {
        set currentDir [lindex $readlines 0]
    }
    set DEBUG $saveDEBUG
}
set saveWindowDir $currentDir

# now we should be in currentDir so we can open the debugCMDS if needed
if {$DEBUG} {
	set debugFPT [open debugCMDS w]
}

set dircellfiles [dirCellFiles]

###################################################################
# start building the main GUI window

wm title . "PolarizedBeam Corection for 3-axis    $currentDir"
wm iconname . "PB cor"
wm geometry . +10+10

# main frame with a left(files/options) and right(group/read) subframe
set a [frame .a -height 500 -width 500]
grid $a
set left [frame $a.left -bd 2 -relief raised]
set right [frame $a.right -bd 2 -relief raised]

# use canvas in the top part of sright to scroll the group selector
# see the buildGroupSelector proc for contents of right

pack $left $right -side left -fill both -expand true
#grid $left $right -padx 5

set top [frame $left.top -bd 2 -relief flat -bg tan]
set mid [frame $left.mid -bd 2 -relief flat]
set bot [frame $left.bot -bd 2 -relief flat -bg tan]
grid $top -sticky new
grid $mid -sticky ew
grid $bot -sticky ews
set heightlines 2
array set bframes [list 1 $top 2 $mid 3 $bot]

# make 3 frames for left: top mid bot.
# these are used for combinations of
# fileselector, prep-solve and auto-prep (f/g/a)
# depending on the users height request for the application
# through heightlines variable.
# heightlines = 1 10L only only one of top/mid/bot is viewed and selectable
# heigthlines = 2 26L use  two of three frames
# heightlines = 3 36L use  all 3 left frames for all 3 (f/g/a)


clearIVC

# try moving makeTolArray and direct coding
# makeTolArray keeps crashing segFault. Looks OK if the Tclpbs cmnds run
# so something with tcltk
# Try running the code in global level instead, which works from wish
# NB this crash does not happen when ActiveTcl wish8.5 is used

# This pipeWrite causing SEGFAULT
# I think this is first call to my tclpbs library. Try the bare call
set cmd "c S"
#pbs $cmd
pipeWrite $cmd

set nl [llength $readlines]

for {set i 1} {$i < $nl} {incr i} {
	set ln [lindex $readlines $i]
	if { [llength $ln] < 3 } { continue }
	set stndName [lindex $ln 1]
	set ok 1
	foreach na $notallowed {
		if { [string match $na $stndName] } {
			set ok 0
			break
		}
	}
	if { $ok < 1 } { continue }
	set stndTol [lindex $ln 2]
	set stndTolArr($stndName) $stndTol
	lappend stndTolNames $stndName
}


# if the cell editor is not standalone then the toplevel for it is not
# . but $celledtPre
if { ! $standalone } {
    foreach {nam val} [array get helptext] {
		if { [string match .cf* $nam] || [string match .tf* $nam] || [string match .ef* $nam] } {
	    	unset helptext($nam)
	    	set nam $celledtPre$nam
	    	set helptext($nam) $val
		}
    }
}    

#####################################################
#  main application startup

trace add variable iwin {write} selectPlotWindow
trace add variable calc {write} recordCalc
trace add variable grpTitles {write} setPlotTitle


# build group-select first so that bld buttons exist
buildShowText -width 60 -height 30 -wrap none -font {courier 10}
buildGroupSelector
buildFileSelector
buildGroupPrep
buildAutoGroup
# RWE add BG jobs 2019
buildBGgui
#puts "about to getDirectoryCellFiles"
getDirectoryCellFiles

wm withdraw $tv

# start with heightlines=1 so remove the mid and bot frames
#grid remove $bframes(2)
#grid remove $bframes(3)
# decided to start with heightlines=2
set heightlines 3
set mainwidth 3
update idletasks
#buildButtonConfig 0

set gnum 1
setGroup
# update so we are sure main window is completely mapped
update idletasks
if { $PLT } { buildPlotSelector }
update idletasks
set fileFilter *

#puts "before session recovery: plotconfig=\n[array get plotconfig]"
set sortgroups 0
checkSessionRecovery
set sortgroups 1
#sortGroupsIncreasing
#puts "after session recovery: plotconfig=\n[array get plotconfig]"

