proc selectDataDirectory {recover} {
	# NB tcltk needs to have the correct directory which can be set using cd
	# and Tclpbs separately needs to have the correct directory for its file operations
	# set by  i d dirPath
	# NB dirPath must be quoted if it has spaces
    global currentDir mswindows
    global readlines nread nerr
    set selDir [tk_chooseDirectory \
			-title "choose your DATA directory for pbcor"]
    if { [string length [string trim $selDir]] < 1 } { return }
    set currentDir $selDir
    #puts "tk_chooseDirectory returned $currentDir"
    cd "$currentDir"
    set checkcd [pwd]
    if {$checkcd != $currentDir} {
    	Dialog_Warn "ERROR: FAILED to cd to $currentDir  ended up in $checkcd"
    }
    #puts "After tk-chooseDirectory: currentDir= $currentDir"
    if { $mswindows } {
    	#puts "calling pipeWrite i d $currentDir"
    	# The windows system call will return path with backslashes which wont print correctly
        pipeWrite "i d \"$currentDir\""
        if {$nerr > 0} {
        	Dialog_Warn [lindex $readlines 0]
        	#set currentDir [lrange [lindex $readlines 1] 3 end]
        } else {
        	set rl [lindex $readlines 0]
        	#puts "newcurrentDir from readlines 0= $rl"
        	# replace backslash with forward slash
        	# oops list commands can process backslash
        	# so get newcurrentDir from the return string using string operations
        	set icolon [string first : $rl]
        	incr icolon
        	set newcurrentDir [string trim [string range $rl $icolon end]]
        	#set newcurrentDir [lrange $rl 3 end]
        	regsub -all {\\} $newcurrentDir "/" snewcurrentDir
        	regsub -all {\\} $currentDir "/" scurrentDir
        	if {$snewcurrentDir != $scurrentDir} {
        		Dialog_Warn "ERROR: readline0= $rl\nbefore regsub newcurrent=$newcurrentDir\ncurrentDir=$currentDir\nincorrect directory $newcurrentDir was set with $currentDir requested"
        	}
        }
    }
    if [winfo exists .] {
	wm title . "PolarizedBeam Correction    $currentDir"
    }
    if { ! $recover } {
        return
    }
    checkSessionRecovery
}

############ dialog procedures

proc Scroll_Set {sb geo offset size} {
    if { $offset != 0.0 || $size != 1.0 } {
	    eval $geo
	    $sb set $offset $size
    } else {
	    set manager [lindex $geo 0]
	    $manager forget $sb
    }
}

proc scrolledText {cf sbx sby args} {
    # now cf is the container frame for scrollbars and
    # a subframe that contains the text widget
    # so that scrollbars dont attach directly to the text widget
    # Now cf is created and mananged outside this proc
    # sbh and sbv select the scrollbar placement
    # sbx -1=top  0=none 1=bottom
    # sby -1=left 0=none 1=right
    # This does work for a text-file-viewer
    # but so far I have not got it working when the text widget has embedded frames
    if { ! [winfo exists $cf] } {
	    puts "scrolledText called with undefined parent frame"
	    return
    }
    # create the subframe to hold the text widget
    # but the scrollbar widgets will be children of cf
    set txtrow 0
    set txtcol 0
    if { $sbx > 0 } {
	set sbxrow 1
	set sbyrow 0
	set txtrow 0
    } elseif { $sbx < 0 } {
	set sbxrow 0
	set sbyrow 1
	set txtrow 1
    }
    if { $sby > 0 } {
	set sbycol 1
	set sbxcol 0
	set txtcol 0
    } elseif { $sby < 0 } {
	set sbycol 0
	set sbxcol 1
	set txtcol 1
    }

    set f [frame $cf.f]
    set txt [text $f.text]
    if { $sbx != 0 } {
	    $txt configure -xscrollcommand [list Scroll_Set $cf.xscroll \
			[list grid $cf.xscroll -row $sbxrow -column $sbxcol \
				  -sticky we]]
    }
    if { $sby != 0 } {
	    $txt configure -yscrollcommand [list Scroll_Set $cf.yscroll \
			[list grid $cf.yscroll -row $sbyrow -column $sbycol \
				  -sticky ns]]
    }
    eval {$txt configure} $args
    if { $sbx != 0 } {
	    scrollbar $cf.xscroll -orient horizontal \
	        -command [list $txt xview]
	    grid $cf.xscroll -row $sbxrow -column $sbxcol -sticky news
    }
    if { $sby != 0 } {
	    scrollbar $cf.yscroll -orient vertical \
	        -command [list $txt yview]
	    grid $cf.yscroll -row $sbyrow -column $sbycol -sticky news
    }
    grid $f -row $txtrow -column $txtcol -sticky news
    # allow the txt container frame to grow, not any sb
    grid rowconfigure $cf $txtrow -weight 1
    grid columnconfigure $cf $txtcol -weight 1
    # manage the text widget inside its container frame
    grid $txt -sticky news
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1

    return $txt
}

proc scrolledTextPbcor {f args} {
    frame $f
    text $f.text \
	-xscrollcommand [list Scroll_Set $f.xscroll \
			     [list grid $f.xscroll -row 1 -column 0 \
				  -sticky we]] \
	-yscrollcommand [list Scroll_Set $f.yscroll \
			     [list grid $f.yscroll -row 0 -column 1 \
				  -sticky ns]]
    eval {$f.text configure} $args
    scrollbar $f.xscroll -orient horizontal \
	-command [list $f.text xview]
    scrollbar $f.yscroll -orient vertical \
	-command [list $f.text yview]
    grid $f.text $f.yscroll -sticky news
    grid $f.xscroll -sticky news
    grid rowconfigure $f 0 -weight 1
    grid columnconfigure $f 0 -weight 1
    return $f.text
}

proc buildPrevSoln {} {
	global plotprev
	toplevel .plotprev
	wm title .plotprev "Pbcor plot previous solution"
	frame .plotprev.top
	frame .plotprev.bot
	set plotprev [listbox .plotprev.bot.lb -width 40 -height 24]
	set plotprevsel [button .plotprev.top.sel -command "plotprevSel" -text "Plot selected solutions"]
	set plotsoln [button .plotprev.top.sol -command "getPrevSolutions" -text "Get previous solutions"]
	grid .plotprev.top
	grid .plotprev.bot
	grid $plotprevsel $plotsoln
	grid $plotprev
}

proc getPrevSolutions {} {
	global plotprev
	set solns [glob -nocomplain *.p]
	$plotprev delete 0 end
	set sols {}
	foreach sol $solns {
		if [regexp {_g([0-9]+)} $sol wm gnum] {
			set sln [list $gnum $sol]
			lappend sols $sln
		}
	}
	set solns [lsort -integer -index 0 -increasing $sols]
	foreach s $solns {
		$plotprev insert end [lindex $s 1]
	}
}

proc plotprevSel {} {
	global plotprev
	set wown [selection own]
	if { $wown != $plotprev } {return}
	set fli [$plotprev curselection]
	foreach fi $fli {
		set f [$plotprev get $fi]
		recoverPrevPlot $f
	}
}

proc buildTXTwarn {} {
	global txtwarn
	toplevel .tw
	wm title .tw "pbcor log"
	frame .tw.top
	#frame .tw.bot
	set txtwarn [scrolledTextPbcor .tw.bot]
	set txtwarnclr [button .tw.top.clr -command "$txtwarn delete 1.0 end" -text "Clear"]
	grid .tw.top
	grid .tw.bot
	grid $txtwarnclr
	grid $txtwarn
}

proc Text_Warn {msg} {
	global txtwarn
	set fpt [open pbcor.log a]
	$txtwarn insert end "\n"
	puts $fpt "\n"
	set dt [clock format [clock seconds] -format "%T %D"]
	$txtwarn insert end "$dt\n"
	puts $fpt "$dt\n"
	$txtwarn insert end $msg
	puts $fpt $msg
	close $fpt
	wm deiconify .tw
}


proc Dialog_Warn {msg} {
    set nl [regsub -all {\n} $msg " " result]
    if { $nl < 31 } {
	tk_messageBox -type ok -icon warning -message $msg
    } else {
	Dialog_WarnC $msg
    }
}


proc Dialog_WarnC {string} {
    # custom Dialog using message aspect = 120
    # which means width = 1.2*height
    global warn warnStack msgfont

    if { [llength [after info]] > 0 } {
	after idle [list Dialog_Warn $string]
	return
    }

    set f .warn
    set c $f.f
    set b $c.buttons
    set m $c.msg
    if [Dialog_Create $f "Warning"] {
	frame $c
	pack $c -side top -expand true -fill both
	frame $b
	frame $m
	pack $m -side top -expand true -fill both
	pack $b -side top

	button $b.ok -text OK -command {set warn(ok) 1}
	pack $b.ok -side top
	message $m.msg -text $string -fg red -padx 10 -aspect 120
	pack $m.msg -side top -expand true -fill both


	#set msgfont [$m.msg cget -font]
    } else {
	if { [string match .warn [grab current]] } {
	    lappend warnStack $string
	    return
	}
    }

    #set lines [split $string \n]

    #set pixwid 0
    #foreach line $lines
	#set ll [font measure $msgfont $line]
	#if { $ll > $pixwid } { set pixwid $ll }
    #
    #set pixwid [expr 10 + $pixwid]

    # sometimes dialog comes up with text and or buttons off widget??
    # try not setting the width
    # still getting truncated window size in Windows??
    $m.msg configure -text $string
    #-width $pixwid
    set warn(ok) 0
    Dialog_Wait $f warn ""
    Dialog_Dismiss $f
}


proc Dialog_Create {top title args} {
    global dialog
    if [winfo exists $top] {
	switch -- [wm state $top] {
	    normal { raise $top }
	    withdrawn -
	    iconified {
		wm deiconify $top
		catch {wm geometry $top $dialog(geo,$top)}
	    }
	}
	return 0
    } else {
	eval {toplevel $top} $args
	wm title $top $title
	return 1
    }
}
proc Dialog_Wait {top varName {focus {}} } {
    global pausing $varName
    if { ![winfo exists $top] } { return }
    set varname $varName\(ok\)
    eval set var $$varname
    bind $top <Destroy> [list set $varname $var]
    raise $top
    update idletasks
    if {[string length $focus] == 0} { set focus $top }
    set old [focus -displayof $top]
    focus $focus
    # getting hung up on tkwait visibility
    tkwait visibility $top
    grab $top

    # continually raise the dialog window (some wm's may allow hiding it)

    set ntry 0
    set pausing 0
    set var 0
    set i 0
    while { 1 } {
	set id [after 20 "setPausing $varname 1"]
	tkwait variable $varname
	# after 20 if varname not set by user then after event happens
	# which sets pausing 1 and varname 1
	eval set var $$varname
	if { ! $pausing } {
	    after cancel $id
	    break
	}
	incr i
	if { $i > 100 } { raise $top; set i 0 }
	set pausing 0
	#incr ntry
    }
    catch {grab release $top}
    focus $old
}
proc Dialog_Dismiss {top} {
    global dialog warnStack
    if { [string match .warn $top] && [llength $warnStack] > 0 } {
	set string [lindex $warnStack 0]
	set warnStack [lreplace $warnStack 0 0]
	Dialog_Warn $string
	return
    }
    catch {
	set dialog(geo,$top) [winfo geometry $top]
	wm withdraw $top
    }
}
proc setPausing {varname value} {
    upvar $varname vname
    global pausing
    set pausing 1
    set vname $value
}




############## startup utility procs
# NB array set aname list, only appends elements in list to current array
# need a proc to clear all elements or to set the array = values from list

proc arrayClear {aname} {
    upvar #0 $aname v
    foreach nam [array names v] {
	unset v($nam)
    }
}
proc arraySet {aname lst} {
    arrayClear $aname
    upvar #0 $aname v
    array set v $lst
}

# recursive file finder. Remember to cd to orig working dir when done.
proc FindFile { startDir namePat } {
    set pwd [pwd]
    # BUG don't quote cd $startDir as it will break startDir into words
    # and no longer handle pathes with blanks
    # FindFile is choking on non-existent directories when searching the
    # PATH so add a trap for existence and directory
    if { [string length $startDir] < 1 } { return "" }
    if { ! [file exists $startDir] || ! [file isdirectory $startDir] } {
	return ""
    }
    if [catch {cd $startDir} err] { 
	    #Dialog_Warn "Can't cd to $startDir.\n"
	    return ""
    }
    #puts "FindFile: looking in $startDir for $namePat"
    set match [glob -nocomplain -- $namePat]
    if { [llength $match] > 0 } {
	return [file join $startDir [lindex $match 0]]
    }
    foreach fil [glob -nocomplain *] {
	if [file isdirectory $fil] {
	    set match [FindFile [file join $startDir $fil] $namePat]
	    if { [llength $match] > 0 } {
		return [file join $startDir $fil [lindex $match 0]]
	    }
	}
    }
    cd $pwd
    return ""
}

proc newfilestack {filename} {
    global fptstack curFPT
    if [catch "open $filename r" fileId] {
		set rl "ERROR: newfilestack failed to open $filename"
		pipeReadLine $rl
		return 0
    }
	set curFPT $fileId
	set fptstack [linsert $fptstack 0 $fileId]
    return 1
}

proc filestack {filename} {
    global readline curFPT fptstack
    # called from pipeWrite in library mode to implement cmndfile reading
    # if open filename succeeds increment the filestack
    # and start reading lines from this file
    # and passing them to pipeWrite
    # when EOF is reached close the file and remove it from the stack
    # if an @file is found in the current file it will be trapped
    # by pipeWrite to recall this filestack proc
    # and increment the stack
    # the stack is just implemented as a list
    # with new fileptrs inserted at the beginning of the list
    # so that when the file is closed just remove the list first element
    # and proceed with the new first element if it exists

    # newfilestack will try to open the file and set curFPT
    # if the fileopen fails newfilestack returns 0

    if { ! [newfilestack $filename] } { return }

    # as long as there are lines from curFPT
    # or there are more files on the stack the following loop will run

    while { 1 } {
	set nb [gets $curFPT cmd]
	if { $nb < 0 } {
	    # eof  close curFPT and remove it from the stack
	    close $curFPT
	    set fptstack [lreplace $fptstack 0 0]
	    # if stack is empty we are done with cmnd files
	    if { [llength $fptstack] < 1 } { return }
	    set curFPT [lindex $fptstack 0]
	} elseif { $nb > 0 } {
	    if { [string match @* $cmd] } {
		set cmd [string trimleft $cmd @]
		newfilestack $cmd
		# if newfilestack succeeds curFPT will update
		# if it failes an error message get printed and no update
	    } else {
		pipeWrite $cmd
	    }
	}
	# this will skip blank lines in cmnd files
    }
}

proc pipeWrite {cmd} {
    global readfinished
    global nread nerr nwarn
    global readlines errlines warnlines
    global DEBUG debugFPT
    global readline
    global outtxt debugFPT
    global ncmnds
    global recordScript recordStarted recordScriptName recordFPT

    # clear the readlines list and the readfinished flag
    set readlines {}
    set errlines ""
    set warnlines ""
    set nread 0
    set nerr 0
    set nwarn 0
    set readfinished 0

	#puts ""
	#puts "pipeWrite starting DEBUG= $DEBUG debugFPT exists= [info exists debugFPT]"
    if { $DEBUG && [info exists debugFPT] } {
    	incr ncmnds
    	#if {$ncmnds < 15} {puts "writing to debugFPT= $debugFPT $cmd"}
	    $outtxt insert end "$cmd\n"
	    puts $debugFPT $cmd
	    flush $debugFPT
	    # since debug can run into crashes we save the file by closing it after every write
	    close $debugFPT
	    # then reopen
	    set debugFPT [open debugCMDS a+]
    }
    
    # If recordScript flag set for background execution, debug still OK
    # Just dump the cmd to the recordScriptName file and return
    if { $recordScript } {
    	if { ! $recordStarted } {
    		set recordFPT [open $recordScriptName w]
    		set recordStarted 1
    	}
    	puts $recordFPT $cmd
    	return
    }

    # now handle Tclpbs scripting extension library
    # NB cmndfile reading via atsign filename needs to be handled here
    # as libpbscript is using a dyn string for the result
	# getting segfault if I comment out this puts so maybe something weird with tcl timing
	# and or that pbs iw writing to stdout
	# RWE Feb 2019 try copy fits version which does not quote the @* or @
	puts -nonewline ""
    if { [string match @* $cmd] } {
	    set cmd [string trimleft $cmd @]
	    filestack $cmd
	    return
    }
    # if not @cmndfile execute cmd and get result in lines
    set lines [pbs $cmd]
    # separate the lines into global readline for each one
    # and call pipeRead each line
    foreach line [split $lines \n] {
	    pipeReadLine $line
    }
}

proc pipeReadLine {line} {
    # called directly when Tclpbs ext is used
    global readfinished
    global readlines errlines warnlines
    global nread nerr nwarn
    global DEBUG sessionrecover saverecoverErr
    global skipblanklines
    global readline

	# Tclpbs
	# the split cmnd has discarded the nl
	#set line $readline
	set nb [string length $line]

    # puts "got $nb bytes back from pipe"
    if { $nb < 0 } {
		return
		# this should have been caught as eof
    }
	if { $DEBUG } {
		global outtxt
		$outtxt insert end "$line\n"
	}
	if { $skipblanklines } {
		set line [string trim $line]
		if { [string length $line] < 1 } { return }
	}
	# end of output=END is specific to pbscript
	# collect readlines until END
	if [string match $line "END"] {
		set readfinished 1
	} else {
		if { [string match ERROR* $line] } {
			append errlines "$line\n"
			incr nerr
	    } elseif { [string match WARN* $line] } {
			append warnlines "$line\n"
			incr nwarn
	    } else {
			lappend readlines $line
			incr nread
	    }
    }
}

# gui procs

proc makeSubToplevel {name title iconname} {
    toplevel $name
    wm title $name "$title"
    wm iconname $name $iconname
    wm protocol $name WM_DELETE_WINDOW "setWindowState $name"
    wm withdraw $name
    return $name
}

proc buildGroupPrep {} {
    # this goes in mid

    global cellFiles nsl1Files cellmen nsl1men
    global monoOptions monitorOptions
    global BLT PLT

    global group gnum
    global fss
    
    global nsf sf
    global conenw conlbw conmmw

    # these are the setup variables/arrays
    global cellFile nsl1File monofilter monitor
    global nsflbls sflbls
    global nsfcons sfcons
    #global nsfv sfv
    global acszero acswght
    global cm cen

    # store the setup for each group in group(i)
    global bframes

    set f $bframes(2)
    clearGrid $f

    set title $f.title
    set exe $f.exec
    set optac $f.ac
    set con $f.cm

    set solvact $exe.solvact
    set solvsel $exe.solvsel
    set solvuns $exe.solvuns
    set textsln $exe.textsln
    set plotsln $exe.plotsln

    set cellmen $exe.cells
    set nsl1men $exe.nsl1
    set monocmen $exe.monoc
    set monitmen $exe.monit
    set nsfmen $exe.nsf
    set sfmen $exe.sf

    set cellmenlbl $exe.cellslbl
    set nsl1menlbl $exe.nsl1lbl
    set monocmenlbl $exe.monoclbl
    set monitmenlbl $exe.monitlbl
    set nsfmenlbl $exe.nsflbl
    set sfmenlbl $exe.sflbl

    set acszerow $optac.acszero
    set acswghtw $optac.acswght
    set acswghtT $optac.acswghtT

    set conen $con.en
    set conlb $con.lb
    set conmm $con.mm
    set i 0
    set conenw($i) $conen$i
    for {set i 1} {$i <= 4} {incr i} {
	set conenw($i) $conen$i
	set conlbw($i) $conlb$i
	for {set j 1} {$j <= 4} {incr j} {
	    set conmmw($i$j) $conmm$i$j
	}
    }

    if { ! [winfo exists $exe] } {

	frame $exe
	frame $optac
	frame $con

	#button $solvact -text "SOLVEcur" -width 9
	#   -command "solveGroup 0"
	# -background green -activebackground yellow
	#button $solvsel -text "SOLVEsel" -width 9
	# -command "solveGroup 1"
	# -background green -activebackground yellow
	# button $solvuns -text "SOLVEunsel" -width 9
	# -command "solveGroup 2"
	# -background green -activebackground yellow
	# button $textsln -text "TEXTsoln" -width 9
	# -command "showText"
	#-background green -activebackground yellow
	#button $plotsln -text "PLOTsoln" -width 9
	# -command "showPlot"
	# -background green -activebackground yellow
	#if { $PLT < 1 } { $plotsln configure -state disabled }

	label $title -text "group options" \
	    -font {helvetica 24 bold} -foreground #f00

	label $cellmenlbl -text cellFile
	label $nsl1menlbl -text nsl1Files
	label $monocmenlbl -text mono/filter
	label $monitmenlbl -text monitor
	if { [llength $cellFiles] < 1 } { lappend cellFiles none }
	eval tk_optionMenu $cellmen cellFile $cellFiles
	if { [llength $nsl1Files] < 1 } { lappend nsl1Files none }
	menubutton $nsl1men -text nsl1Files -menu $nsl1men.menu -relief raised
	menu $nsl1men.menu
	#puts "at menubutton winfo exists nsl1men= [winfo exists $nsl1men]"
	#eval tk_optionMenu $nsl1men nsl1File $nsl1Files
	eval tk_optionMenu $monocmen monofilter $monoOptions
	eval tk_optionMenu $monitmen monitor $monitorOptions

	$cellmen configure -width 20
	$nsl1men configure -width 25
	$monocmen configure -width 20
	$monitmen configure -width 20

	label $nsfmenlbl -text nsfCons
	label $sfmenlbl -text sfCons
	buildHelpPop
	bind $nsfmenlbl <Enter> "showHelp $nsfmenlbl ."
	bind $nsfmenlbl <Leave> withdrawHelp
	bind $sfmenlbl <Enter> "showHelp $sfmenlbl ."
	bind $sfmenlbl <Leave> withdrawHelp
	eval tk_optionMenu $nsfmen nsfcons $nsflbls
	eval tk_optionMenu $sfmen sfcons $sflbls

	$nsfmen configure -width 20
	$sfmen configure -width 20

	trace variable nsfcons w setNSF
	trace variable sfcons w setSF

	checkbutton $acszerow -variable acszero \
	    -text "CS=0 if no data"
	checkbutton $acswghtw -variable acswght
	#-text "peak weights"
    button $acswghtT -text "Edit Weights" -bg green -activebackground yellow -command groupPeaks


	set i 0
	#label $conenw($i) -text enable
	for {set i 1} {$i <= 4} {incr i} {
	    checkbutton $conenw($i) -variable cen($i) \
			     -text $fss($i)=
	    bind $conenw($i) <Enter> "showHelp $conenw($i) ."
	    bind $conenw($i) <Leave> withdrawHelp
	    label $conlbw($i) -text $fss($i)
	    for {set j 1} {$j <= 4} {incr j} {
		entry $conmmw($i$j) -textvariable cm($i$j) \
			 -width 5 -relief sunken
		set cm($i$j) 0
		if { $i == $j } { $conmmw($i$j) configure -state disabled }
	    }
	}	
    }

    grid $title -sticky ew
    grid $exe -sticky w
    grid $optac -sticky w
    grid $con -sticky w

    grid $acszerow $acswghtT $acswghtw -sticky w

    #$cellmen configure -width 24
    #$monocmen configure -width 24
    #$monitmen configure -width 24
    #$nsfmen configure -width 24
    #$sfmen -configure -width 24

    grid $cellmenlbl $cellmen -sticky w
    grid $nsl1menlbl $nsl1men -sticky w
    grid $monocmenlbl $monocmen -sticky w
    grid $monitmenlbl $monitmen -sticky w
    grid $nsfmenlbl $nsfmen -sticky w
    grid $sfmenlbl $sfmen -sticky w


    #grid $solvact -sticky e -padx 10 -row 0 -column 2
    #grid $solvsel -sticky e -padx 10 -row 1 -column 2
    #grid $solvuns -sticky e -padx 10 -row 2 -column 2
    #grid $textsln -sticky e -padx 10 -row 3 -column 2
    #grid $plotsln -sticky e -padx 10 -row 4 -column 2

    #grid $conenw(0) -row 0 -column 0
    for {set i 1} {$i <= 4} {incr i} {
	grid $conenw($i) -row $i -column 0 
	grid $conlbw($i) -row 0 -column $i
	for {set j 1} {$j <= 4} {incr j} {
	    grid $conmmw($i$j) -row $i -column $j
	}
    }
}

# procs for prep-solve

proc setCSrow {row args} {
    global conenw conmmw
    if {[llength $args] < 4} { return }
    for {set i 1; set j 0} {$j < 4} {incr i; incr j} {
	$conmmw($row$i) delete 0 end
	$conmmw($row$i) insert 0 [lindex $args $j]
    }
    $conenw($row) select
}
proc setNSF {varName indx op} {
    #upvar $varName var
    #global nsf nsfv
    global nsfcons
    global conenw
    global nsflbls

    if { [set i [lsearch -exact $nsflbls $nsfcons]] < 0 } { return }

    if { $i == 0 } {
	$conenw(1) deselect
	$conenw(4) deselect
    } elseif { $i == 1 } {
	setCSrow 1 0 0 0 0
	setCSrow 4 0 0 0 0
    } elseif { $i == 2 } {
	setCSrow 4 1 0 0 0
	$conenw(1) deselect
    } elseif { $i == 3 } {
	setCSrow 1 0 0 0 1
	$conenw(4) deselect
    }
}

proc setSF {varName indx op} {
    global conenw
    global sflbls
    global sfcons
    if { [set i [lsearch -exact $sflbls $sfcons]] < 0 } { return }

    if { $i == 0 } {
	$conenw(2) deselect
	$conenw(3) deselect
    } elseif { $i == 1 } {
	setCSrow 2 0 0 0 0
	setCSrow 3 0 0 0 0
    } elseif { $i == 2 } {
	setCSrow 3 0 1 0 0
	$conenw(2) deselect
    } elseif { $i == 3 } {
	setCSrow 2 0 0 1 0
	$conenw(3) deselect
    }
}

proc makeTolArray {} {
    # called during startup before checkSessionRecovery
    #grab all the standard column tolerances into array
    global stndTolArr stndTolNames notallowed 
    global readfinished readlines
	set cmd "c S"
    pipeWrite $cmd
    
    for {set i 1} {$i < [llength $readlines]} {incr i} {
		set ln [lindex $readlines $i]
		if { [llength $ln] < 3 } { continue }
		set stndName [lindex $ln 1]
		set ok 1
		foreach na $notallowed {
	    	if { [string match $na $stndName] } { set ok 0 ; break }
		}
		if { $ok < 1 } { continue }
		set stndTol [lindex $ln 2]
		set stndTolArr($stndName) $stndTol
		lappend stndTolNames $stndName
    }
}

proc setStndTolerances {} {
    global stndTolNames stndTolArr
    global readfinished
    # set the stndTol in libDF from the users stndTolArr
    foreach snam $stndTolNames {
    	if { ! [info exists stndTolArr($snam)] } { continue }
	    pipeWrite "c St $snam $stndTolArr($snam)"
    }
}

proc setStndTol {name} {
    # binding to choosing a tol col type from menu
    global stndTolArr tolerance
    if { [info exists stndTolArr($name)] } {
	    set tolerance $stndTolArr($name)
	    set toltype $name
    }
}
proc getStndTol {} {
    # binding to Leave tolerance entry
    # if new entry value is different than stndTolArr, mark as edited
    # thru instrType=-1
    # also clear relaxTol for any edits
    global stndTolArr tolerance toltype instrType relaxToler
    if { ! [info exists stndTolArr($toltype)] } { return }
    if { ! ($tolerance == $stndTolArr($toltype)) } {
    	#set instrType -1
    	set relaxToler 0
    }
    set stndTolArr($toltype) $tolerance
    updateTolMenuLabel $toltype
}

proc saveStndTolerances {} {
	global saveStndTol stndTolArr
	array set saveStndTol [array get stndTolArr]
}
proc swapSavedTolerances {} {
	global saveStndTol stndTolArr
	set stndList [array get saveStndTol]
	array set saveStndTol [array get stndTolArr]
	array set stndTolArr $stndList
}
proc setAllTolerances {val} {
	global stndTolArr
	foreach sc [array names stndTolArr] {
		set stndTolArr($sc) $val
	}
}


proc nextEmptyGroup {ig} {
    global group
    if { $ig < 0 } { set ig 0 }
    while {1} {
	incr ig
	if { ! [info exists group($ig)] } { return $ig }
	set fls [getFilesInGroup $ig]
	if { [llength $fls] < 1 } { return $ig }
    }
}
proc getEntriesForGroup {ig} {
    # return list of entries for group ig
    global lbxs lbxi
    set igs [$lbxs(1) get 0 end]
    set lbxi {}
    for {set i 0} {$i < [llength $igs]} {incr i} {
	set igi [lindex $igs $i]
	if { $igi == $ig } { lappend lbxi $i }
    }
    return $lbxi
}

proc getFilesInGroup {ig} {
    global lbxs lbxi
    global group

    if { $ig > 0 && ! [info exists group($ig)] } { return {} }
    # if ig < 1 return all files
    # else just files with ig group number
    set fls [$lbxs(0) get 0 end]
    if { $ig < 1 } { return $fls }

    set lbxi [getEntriesForGroup $ig]
    set ret {}
    for {set i 0} {$i < [llength $lbxi]} {incr i} {
	set igi [lindex $lbxi $i]
	lappend ret [lindex $fls $igi]
    }
    return $ret
}


proc appendLbxData {data} {
    # should have all full lbxs updates go thru this proc
    # as it supplies formatting and listbox width adjustment
    # and returns lbxs data to normal state
    #
    # data is a list of rowlists to append to lbxs

    global lbxs nlbxs lbxlabels lbxfmts
    set nd [llength $data]
    if { $nd < 1 } { return }
    # make sure all lbxs have the normal data state when appending data
    normalLbxValues
    for {set j 0} {$j < $nd} {incr j} {
	set rowdata [lindex $data $j]
	set rowl [llength $rowdata]
	if { $rowl == 1 } { set rowdata [join $rowdata] }
	if { [set rowl [llength $rowdata]] != $nlbxs } {
	    Dialog_Warn \
		"appendLbxData: ll=$rowl req $nlbxs data in list:\n$rowdata"
	}
	for {set i 0} {$i < $nlbxs} {incr i} {
	    # ith lbx value=idata can be a list of np values
	    set idata [lindex $rowdata $i]
	    set np [llength $idata]
	    # format the np values in idata if fmt defined
	    if { [string length $lbxfmts($i)] > 0 } {
		for {set k 0} {$k < $np} {incr k} {
		    set kdata [lindex $idata $k]
		    if [catch "format $lbxfmts($i) $kdata"] { break }
		    set kdata [format $lbxfmts($i) $kdata]
		    set idata [lreplace $idata $k $k $kdata]
		}
	    }
	    $lbxs($i) insert end [join $idata]

	}
    }
    for {set i 0} {$i < $nlbxs} {incr i} {
	setLbxWidth $lbxs($i) [lindex $lbxlabels $i]
    }
}

proc getLbxFiles {} {
    # get short filenames from lbxs(0)
    global lbxs lbxfiles
    set lbxfiles [$lbxs(0) get 0 end]
}

proc getLbxData {fil} {
    # data should be grp ind except for grp num
    global lbxs nlbxs lbxfiles
    #set fils [$lbxs(0) get 0 end]
    set data {}
    if { [set i [lsearch -exact $lbxfiles $fil]] < 0 } { return $data }
    lappend data [lindex $lbxfiles $i]
    for {set j 1} {$j < $nlbxs} {incr j} {
	lappend data [$lbxs($j) get $i]
    }
    return $data
}

proc nslMenuMarkAssign {assignedFileList} {
	global nsl1men
	set m $nsl1men.menu
	if {![winfo exists $m]} {return}
	set n [$m index end]
	# when p ho nslFile is run it collects failed assignments
	# pass these to this proc and configure the nsl menu to show green on success
	# this menu is supposed to have the none checkbutton always
	# so if the end index is zero there are no file entries
	if {$n < 2} {return}
	set n1 [expr $n - 1]
	# get each file entry. If it isnt in the fail list mark its background green
	# i think entry 0 is for tearoff so skip that index
	for {set i 1} {$i < $n1} {incr i} {
		set e [$m entrycget $i -label]
		if {[lsearch $assignedFileList $e] >= 0} {
			$m entryconfigure $i -background green
		} else {
			$m entryconfigure $i -background white
		}
	}
}

proc writeGroupOpts {} {
    # write the current group options to pbscript
    # to setup for solving
    global gnum
    global cm cen acszero monitor monofilter 
    global cellFile nsl1Files nsl1FilesEnable
    global monoOptions monitorOptions
    global acswght
    global fss fst fsg
    global pksdata
    global errlines nerr nwarn warnlines readfinished

    pipeWrite "p h $cellFile"
    
    # all cellFile errors are fatal
    if { $nwarn > 0 || $nerr > 0 } {
	    Dialog_Warn "PROBLEMS for cellFile for group $gnum:\n$warnlines\n$errlines"
        if { $nerr > 0 } { return 0 }
    }

    # apply any a4nsl1 files for this group
    #puts "writingGroupOpts group=$gnum nsl1FilesEnable= $nsl1FilesEnable nsl1Files= $nsl1Files"
    if {$nsl1FilesEnable == 1} {
    	set errmsg ""
    	set nsl1Assigned {}
    	foreach nsl1File $nsl1Files {
        	pipeWrite "p ho $nsl1File"
        	if {$nerr > 0} {set errmsg "$errmsg$errlines\n"}
        	if {$nwarn < 1} {lappend nsl1Assigned $nsl1File}
    	}
    	if {[string length $errmsg] > 0} {
    		Dialog_Warn "FAILED to assign nsl files\n$errmsg"
    	}
    	#puts "nsl1Assigned= $nsl1Assigned"
    	nslMenuMarkAssign $nsl1Assigned
    } else {
    	pipeWrite "p ho none"
    	nslMenuMarkAssign {}
    }
    
    # clear any previous coordinate peaks
    pipeWrite "g lcd"
    
    # write any group coordinate peak correlations, start by making sure current is saved
    savePkGroup
    
    if { $acswght && [info exists pksdata($gnum)] && [llength $pksdata($gnum)] > 0 } {
        set npk [llength $pksdata($gnum)]
        foreach pklst $pksdata($gnum) {
            set uid [lindex $pklst 0]
            set fff [lindex $pklst 1]
            set cor [lindex $pklst 2]
            set cmn [lindex $pklst 3]
            set cmx [lindex $pklst 4]
            set pos [lindex $pklst 5]
            set fwhm [lindex $pklst 6]
            # sanity checking done in savePkGroup
            pipeWrite "g lc $uid $cor $cmn $cmx $pos $fwhm"
            if [string match $fff all] {
                set fssw "uu,dd,du,ud"
            } elseif { [string match $fff none] || [string length $fff] < 1 } {
                set fssw "X"
            } else {
                set fssw [join $fff ","]
            }
            pipeWrite "g lcf $uid $fssw"
        }
    }

    # check independent S (non-zero coef in equ) are not used in cnstrnt eq
    for {set i 1} {$i <= 4} {incr i} {
	if { ! $cen($i) } { continue }
	for {set j 1} {$j <= 4} {incr j} {
	    if { $i == $j } { continue }
	    if { [scan $cm($i$j) {%g} cmval] < 1 } {
		Dialog_Warn "In group $gnum\nConstraint equ for $fss($i) has non-numeric entry for $fss($j).\nPlease fix this before solving group."
	    }
	    if { [expr abs($cm($i$j))] != 0 } {
		if { $cen($j) } {
		    Dialog_Warn "In group $gnum\nConstraint equ for $fss($i) uses ind var $fss($j)\nwhich is then constrained.\nPlease fix this before solving group."
		    return 0
		}
	    }
	}
    }

    #   gui1-4 libPB1-4 fs index translation
    # offoff 1 -> 1
    # ONoff  2 -> 3
    # offON  3 -> 4
    # ONON   4 -> 2

    #   libPB1-4  gui1-4
    # offoff 1 -> 1
    # ONON   2 -> 4
    # ONoff  3 -> 2
    # offON  4 -> 3

    # p f c e i offoff ONON ONoff offON
    # p f c e i i1     i4   i2    i3
    #set fst(1) 1
    #set fst(2) 3
    #set fst(3) 4
    #set fst(4) 2

    for {set i 1} {$i <= 4} {incr i} {
	set fi $fst($i)
	set cmvs ""
	for {set j 1} {$j <= 4} {incr j} {
	    set fj $fsg($j)
	    append cmvs " $cm($i$fj)"
	}
	if { $cen($i) } {
	    pipeWrite "p f c e $fi $cmvs"
	    
	} else {
	    pipeWrite "p f c e -$fi"
	    
	}
    }
	# the menu strings are saved in pbsession so these should always be the same
	# in the menus although the indices could change and the number of values could
	# change as they did when we added velocity selector or unfiltered MACS
	# so we just need to make sure in Tclpbs that the passed indices to p f m c|t are handled
	# as intended
	# Mar 2017 the monoOptions list is
	# monochormator+filter options
	# set monoOptions [list "Be filter" "PGbt7 no filter" "PGbt7 PGfilter" "PGbt7 velsel" "PGbt7 velsel + PGfilter" "MACS no filter"]
	# beam monitor options
	# set monitorOptions [list "before He3 pol" "after He3 pol"]
    set imon [lsearch -exact $monitorOptions $monitor]
    set iflt [lsearch -exact $monoOptions $monofilter]
    if { $imon < 0 } { set imon 1 }
    if { $iflt < 0 } { set iflt 0 }

    pipeWrite "p f m c $imon"
    pipeWrite "p f m t $iflt"
    pipeWrite "p f c a $acszero"
    
    return 1
}

proc findScannedVariableForSets {flst} {
# find common scanned variable in the sets in setlst. filelst RWE 2019
# to make background compatibile
# from first E then Qxyz then ttm  om  Temp Bfld
# ie in the order that is in stndColNames
# return the list [index bestTotalFrac stndcolumnName] in stndScanNames or -1 if fail
# En x1 x2 x3 tts om Temp Bfld
# NB RWE June 2019 this is changed to always use deflttol, so stndTolArr will be deflt when this is called
    global readfinished nread readlines
    global stndScanNames stndTolArr
    if { [llength $flst] < 1 } { return -1 }
    
	array set totalfrac {}
	set bestTotalfrac 0.
	set bestVar ""
	set bestI -1
	set secondTotalfrac 0.
	set secondVar ""
	set secondI -1
    for {set i 0} {$i < [llength $stndScanNames]} {incr i} {
		set cnam [lindex $stndScanNames $i]
		set tnam $cnam
		if [string match x* $cnam] {set tnam "x"}
		if {! [info exists stndTolArr($tnam)] } { continue }
		set tol $stndTolArr($tnam)
		set ok 1
		set npts 0
		set nfrac 0.
		foreach fl $flst {
	    	pipeWrite "d>$fl"
	    	# get npts
	    	if { $nread < 2 } { continue }
	    	set dlin [lindex $readlines 1]
	    	set dlin [string trimleft $dlin "* "]
			set npt [lindex $dlin 1]
			if {$npt < 1} {continue}

	    	pipeWrite "c al $cnam"
	    
	    	if { $nread < 2 } { continue }
	    	if {[scan [lindex $readlines 1] {%s %d %d %s} \
			     ct id co lb] < 4} { continue }
	    	# using tolerance = 0 to check for any identical list members. sc comes back as 1 if all unique values
	    	# RWE Feb 2019 change to DistinctFraction at current tolerance setting
	    	#pipeWrite "c ss $co"
	    	pipeWrite "c ssf $co $tol"
	    	if { $nread < 2 } { continue }
	    	set dlin [lindex $readlines 1]
	    	set frac [lindex $dlin 2]
	    	set nfrac [expr $nfrac + ($npt * $frac)]
	    	set npts [expr $npts + $npt]
	    
	    	#if { $nread < 2 } { continue }
	    	#if {[scan [lindex $readlines 1] {%d %d %d} co sc mo] < 3} {
			#	continue
	    	#}
	    	# if fails to be a scan variable for some set
	    	# break and try next scan variable
	    	#if { ! $sc } { set ok 0 ; break }
		}
		if {$npts < 2} {continue}
		set totalfrac($cnam) [expr $nfrac/$npts]
		if { $totalfrac($cnam) > $bestTotalfrac } {
			set secondTotalfrac $bestTotalfrac
			set secondVar $bestVar
			set secondI $bestI
			set bestTotalfrac $totalfrac($cnam)
			set bestVar $cnam
			set bestI $i
		}
	}
	#if { $ok > 0 } { return $i }

    return [list $bestI $bestTotalfrac $bestVar $secondI $secondTotalfrac $secondVar]
}

proc getScanVar {} {
	global gnum Xscan useXscan

	set Xscan 0
	set useXscan 0
	set fls [getFilesInGroup $gnum]
	if { [llength $fls] < 1 } {
	    return -1
	}
	#fileListToSetList $fls
	# fileListToSetList also puts set numbers into globalsetlist
	# return index in stndScanNames
	# RWE changed to use filelist
	
	#set iscan [findScannedVariableForSets $globalsetlist]
	# RWE June 2019 This was a bug I found after implementing bin files. find ScannedVariable should be based on defltTol always instead of current user
	saveStndTolerances
	setdefltTol
	set bestScan [findScannedVariableForSets $fls]
	swapSavedTolerances
	# bestscan is list indx frac varname
	set iscan [lindex $bestScan 0]
	set frac [lindex $bestScan 1]
	set stndscanvar [lindex $bestScan 2]
	if { $iscan < 0 } {
		# allow retry with zero tolerances important to find something to plot against
		saveStndTolerances
		setAllTolerances 0.
		set bestScan [findScannedVariableForSets $fls]
		# return to user tolerances
		swapSavedTolerances
		# bestscan is list indx frac varname
		set iscan [lindex $bestScan 0]
		set frac [lindex $bestScan 1]
		set stndscanvar [lindex $bestScan 2]
		if { $iscan < 0 } {
	    #set Xscan 0. Following should probably never happen unless group has only few pts
	    	Dialog_Warn "ERROR: No common scan variable in group $gnum files using zero tolerances."
	    	#set relaxToler 0
	    	return 0
	    } else {
	    	Dialog_Warn "WARNING: Common scan variable $stndscanvar in group $gnum obtained after setting zero tolerances has distinct fraction= $frac."
	    }
	} else {
		# if frac is less than unity warn
		if { $frac < 1. } {
			Dialog_Warn "WARNING: Common scan variable $stndscanvar in group $gnum obtained with default tolerances has distinct fraction= $frac."
		}
	}
	set useXscan 1
	set Xscan [expr $iscan + 1]
	return $Xscan
}

proc setRelaxToler {} {
	# only called by relaxTol check button
	# for current group
	# uses current tolerance settings
	global gnum
	global relaxToler
	global globalsetlist
	global Xscan
	global stndTolNames stndTolArr toltype tolerance
	global stndScanNames
	# update scan tolerances to relaxed for current group gnum. return 0 on failure
	if {$relaxToler == 0} {
		#setdefltTol
		return 1
	}
	# must find a scan variable to set relaxed tolerance
	if {[getScanVar] <= 0} { return 0 }
	
	set relaxToler 1

	# iX is the plot variable selection
	# this is only called if relaxToler is set true
	# using deflt relaxed tolerances except for scanned variable
	# check that some stndCol is varied in all these files
	# relax all the tolerances but the common scan variable
	# save the current tolerances in case user want to fall back to them
	
	saveStndTolerances
	set iscan [expr $Xscan - 1]
	set csvar [lindex $stndScanNames $iscan]
    if { [string match x* $csvar] } {
		set savetol $stndTolArr(x)
    } else {
		set savetol $stndTolArr($csvar)
    }
    # relax all then return the scan variable to its input tolerance		
	foreach snam $stndScanNames {
		set tnam $snam
		if [string match x* $snam] {set tnam x}
		set stndTolArr($tnam) 1000.
	}
    if { [string match x* $csvar] } {
		set stndTolArr(x) $savetol
    } else {
		set stndTolArr($csvar) $savetol
    }
	# now fix up the tolerance gui
	foreach styp $stndTolNames {
	    updateTolMenuLabel $styp
	}
	# update tolerance GUI
    if { [info exists stndTolArr($toltype)] } {
        set tolerance $stndTolArr($toltype)
    }

	return 1
}


proc solveGroup {is args} {
	# solve a group only called by the GUI SOLVE button or SOLVEbg button when args > 0
    # is==0 solve the current group
    # is>0  solve group number
    # now only does one group
    # This works for background execution as well when recordScript flag is on=1
    # Note that in recordScript mode no commands are executed.
    # The move to setlist now uses filenames so a correct setlist will be created in
    # the background process.
    # All gui warnings errors will get bypassed but the C-libraray errors will appear
    # in the output from the background process which gets redirected to filaname.BGout
    
    # one missing thing is what is X for plotting in the background process
    
    # can launch to backgroun using SOLVEbg button next to SOLVE button
    # NB when recordScript is on currently NO cmnds are executed
    #  We expect to get any error messages from the C-code directed to file.BGout

    global readfinished readlines
    global nread nerr errlines nwarn warnlines
    global gnum calcgroup
    global cellFile
    global npts nmsr calc isfop fopgroup
    global textf
    global BLT PLT
    global grpScale grpScales
    global dsmode
    global pltDet ny
    global globalsetlist
    global relaxToler
    global stndColNames stndTolNames stndTolArr stndScanNames iX Xscan useXscan XscanU useXscanU xaxlbl Xlabel
    global multisoln
    
    global recordScript watchBG bgWatchOK

    if { $is == 0 } {
    	# use current group
	    set grp $gnum
    } elseif { $is > 0 } {
	    # use passed group index
	    set grp $is
    }
    
    # for background jobs somehtings requiring the gui need to get done before we turn on recording
    # we need the group filelist
    set fls [getFilesInGroup $grp]
    set nfls [llength $fls]
    puts "solveGroup nfls in group $grp= $nfls  recordScript= $recordScript\n"
	if { $nfls < 1 } {
		#puts "NO files in group=$grp"
	    append emptygroups " $grp"
	    return 0
	}
	# now handle relaxToler flag and finding Xscan for plot
	if { ! $recordScript } {
		if { $relaxToler > 0 } {
			if { [setRelaxToler] <= 0 } { return 0 }
		} else {
			# using current tolerances but still check for something to plot against
			# getScanVar can fail in which case Xscan will be zero
			# sets both useXscan and Xscan
			getScanVar
		}
	}
	puts "solveGroup finished getScanVar with recordScript= $recordScript\n"
    
	puts "solveGroup starting grp= $grp"
	if { [llength $args] > 0 && ! $recordScript } {
		# launch as backgroud. Will reenter solveGroup with recordScript set true
		puts "solveGroup starting background grp= $grp"
		setPbcorJobRun 1 [list $grp]
		return
	}    

    # make sure we aren't in dsmode
    set dsmode 0
    updatePlotSelector

    # save current group
    saveGroup
    set emptygroups ""


	# make grp the current pbgroup in C-library
	pipeWrite "g$grp"
	
	# avoid setting group number if possible to allow lockCells cellEdit
	if { $gnum != $grp } { setGnum $grp }
	set textf(0) ""
	set textf(1) ""
	set textf(2) ""
	set calc 0
	set calcgroup($grp) 0
	set isf 0
	if [info exists fopgroup($grp)] {set isf $fopgroup($grp)}
	#set fopgroup($grp) $isfop
	# skip file operation groups
	if { $isf } { return 0 }
	hiliteCalcGroup

	# apply all constraints and options to this group
	# set the cellfile and then solve
	
	# p f m imonitor imono
	# imonitor 0 none 1 before-pol 2 after-pol
	# imono    0 BT7PG-no-filter 1 w/2cmPGfilter  2 BT4PGnofilter
	# p f c a autoconstrain
	# p f c e index 4coefs
	# p f c e -index disable

	# send group options to pbscript
	# this reads the selected group cellfile and then does any nsl table assigns
	if { ![writeGroupOpts] } { return 0 }

	# send the stndTolArr for this group
	# RWE Jan 2011
	# having to set tolerances before solving
	# or adjust them after points/msrs are disabled
	# is painful.
	# I would like a default option to be
	# just setting a tolerance for the one
	# chosen independent variable, first from E Q
	# then try tts om.
	# Should be able to pre-screen the files in the group
	# to chose this independent variable,
	# then if the default flag is set make all other tolerances
	# large enough to be of no consequence

	# get the files for this group listed in lbxs(0)
	# debug list all datasets so we can check that sets going to this group make sense
	#ipeWrite "d"

	if {$recordScript} {
		# for background need to reread filee into datasets
		foreach fil $fls {
	    	readFile $fil 1
	    	# continue on fatal read error
	    	#puts "fil=$fil readFile returned $ret"
	    }
	}
	
	#puts "starting fileListToSetList"
	fileListToSetList $fls
	#puts "returned from fileListToSetList"
	# fileListToSetList also puts set numbers into globalsetlist
	# return index in stndScanNames
	#puts "starting findScannedVariableForSets"
	
	
	
	puts "solveGroup useXscan= $useXscan Xscan= $Xscan  useXscanU= $useXscanU XscanU= $XscanU\n"
	# allow saveColumns user interface to select X vie useXscanU=1 XscanU > 0
	# groupXcolSet sets the stnd global x index iX>0 and stnd xlabel in Xlabel
	if { $useXscanU && $XscanU > 0 } {
		groupXcolSet $XscanU
	} else {
		# if getScanVar was succesful
		if { $useXscan && $Xscan > 0 } {
			groupXcolSet $Xscan
		} elseif { $iX > 0 } {
		# punt to last set value if valid
			groupXcolSet $iX
		} else {
		# if all fails En should always be assigned for polarized beam so fall back to it
			groupXcolSet 1
		}
	}
	puts "solveGroup set iX= $iX Xlabel= $Xlabel\n"
	# set group xLabel using global is non blank else fall back to the stnd label
	# Xlabel and setting group xlabel now handled in groupXcolSet



	# iX is the plot variable selection from SelectColumn

	# send tolerances to library code
	setStndTolerances

	# move this setlist to pbgroup $gnum
	# would like to keep correspondence between pbgroups and gnums

	# move setlist 1 to this group. This clears any previous list
	#puts "move set list 1 to current group= $grp"
	pipeWrite "g sl 1"
	
	# once we have files in group force set the group outfile name but first send the multisoln flag 
	# when multisoln is true solutions are versioned
	# default is multisoln
	if { ![info exists multisoln] } { set multisoln 1 }
	pipeWrite "g lfm $multisoln"
	# now get back the root group outfile name
	pipeWrite "g ofs 1"
	# read it back
	if {! $recordScript } {
		if {$nerr > 0} {return 0}
		set ofil [lindex [lindex $readlines 0] 3]
		if {$ofil == "NONE"} {return 0}
	}

	# set the bgjob flag true if recordScript
	set bgjob 0
	if { $recordScript } {
		set bgjob 1
		# disable using the watchServer
		pipeWrite "g pm 0"
		if {$watchBG && $bgWatchOK} {
			# if watchServer requested and runs set the flag to use it in C-code
			if [checkWatch 1] {
				# the server is running, tell the C-code to use the watchSerer
				pipeWrite "g pm 1"
			}
		}
	}
	pipeWrite "g lfb $bgjob"	

	# set the group scalefactor if it is non-zero
	if { [info exists grpScales($grp)] && $grpScales($grp) > 0. } {
	    set grpScale $grpScales($grp)
	}
	if { $grpScale > 0 } {
	    #puts "solveGroup: setting scl=$grpScale"
	    pipeWrite "g ps $grpScale"
	    
	}

	# set the detector index for soln, 0=all
	# detector index is in range 1:ny where ny is number of Y cols for
	# all the datasets in the group
	# NB when group solved g->ny is set to number of detectors solved
	# and g->y = 0 for all detectors else is detector index 1:ny
	# which is what is in the Yi data column of the pset
	if { ! [info exists pltDet($gnum)] } { set pltDet($gnum) 0 }
	pipeWrite "g y $pltDet($gnum)"
	
	
	# prep the group
	# most of the error checking is done when solution requested (p cmd)
	# NB we do NOT want to reread the cell file 
	puts "solveGroup starting prep\n"
	pipeWrite "g p"
	puts "solveGroup returned from prep nwarn= $nwarn\n"
	
	if { $nwarn > 0 || $nerr > 0 } {
	    set msg ""
	    if { $nwarn > 0 } {
			append msg "group $grp prep WARNINGS:\n$warnlines"
	    }
	    if { $nerr > 0 } {
			append msg "group $grp prep ERRORS:\n$errlines"
	    }
	    puts "solveGroup calling Text_Warn nwarn= $nwarn nerr= $nerr.\n"
	    Text_Warn $msg
	    if { $nerr > 0 } { return 0 }
	}
	# after prep, check group basics to see that nmsr and pts>0
	# NB g l prints all groups.
	pipeWrite "g$grp"
	
	if { ! $recordScript } {
		set nscan \
	    [scan [lindex $readlines 1] {%[*]%d %d %d %d %d} \
		 star igrp calc nset npts nmsr]
		if { $nscan < 6 || $npts < 1 || $nmsr < 1 } {
	    	Dialog_Warn "ERROR: group $grp solution failed to get npts>0 nmsr>0"
	    	return 0
		}
	}

	# also get back the group ny=number of detectors solved
	# which is set during prep (must be same for all sets)
	pipeWrite "g ly"
	
	if {! $recordScript} {
		if {[scan [lindex $readlines 0] {%s %d %s %d} star igrp dum ny] < 4 && ! $recordScript} {
	    	Dialog_Warn "group $grp solution failed to get ny"
		}
	}

	# set the group output control flags for extended output level 1
	# this is for the .m .p datasets and files
	# the other defaults are:
	# f=1 flags to header
	# h=0 no data headers output
	# c=# comment char
	# x=1 x-col sel output
	# m=0 no extended output, which we are changing here
	# e.g. this makes srcfile last column of .m output
	pipeWrite "g oc m 1"
	

	# solve
	# NB dont reread the cellfile
	pipeWrite "p"
	puts "solveGroup p nwarn= $nwarn nerr= $nerr\n"
	
	if { $nerr > 0 || $nwarn > 0 } {
	    set msg ""
	    if { $nwarn > 0 } {
			append msg "group $grp calc WARNINGS:\n$warnlines"
	    }
	    if { $nerr > 0 } {
			append msg "group $grp calc ERRORS:\n$errlines"
	    }
	    puts "calling Text_Warn\n"
	    Text_Warn $msg
	    if { $nerr > 0 } { return 0 }
	}

	# readlines only line should have
	# group # correctOK ic datasetsOK id
	if {! $recordScript} {
		if { $nread < 1 } {
	    	Dialog_Warn "group $grp calc didnt report success"
	    	return 0
		}
		set nscan \
	    [scan [lindex $readlines 0] {%s %d %s %d %s %d} \
		 glbl igrp clbl cok slbl sok]
		if { $nscan < 4 || ! $cok } {
	    	Dialog_Warn "group $grp calc failed"
	    	return 0
		}
		if { $nscan < 6 || ! $sok } {
	    	Dialog_Warn "group $grp calc OK but didnt produce result datasets"
	    	return 0
		}
	}
	
	pipeWrite "g$grp"
	
	if {! $recordScript} {
		set nscan \
	    [scan [lindex $readlines 1] {%[*]%d %d %d %d %d} \
		 star igrp calc nset npts nmsr]
		if { $nscan < 6 || $npts < 1 || $nmsr < 1 } {
	    	append errs "group $grp ERROR: could not get nmsr/npts after calc\n"
	    	return 0
		}
	}

	# write the group results
	pipeWrite "g w"
	

	if { $nerr > 0 } {
	    Dialog_Warn "group $grp write ERRORS:\n$errlines"
	    return 0
	}
	if { $nwarn > 0 } {
	    Dialog_Warn "group $grp write WARNINGS:\n$warnlines"
	}

	# after this point is post solution processing which wont be done in background
	if { ! $recordScript } {
		recoverFG $grp $ofil
	}
	return 1
}

proc recoverBG {grp fnm} {
	# used after backgroun completes or user wants to load a previous solution
	# Uses the .p .m files to reconstruct
	# This must be rebust across pbcor restarts where group outfile is not yet saved for BG job.
	# But bgRunning is to be saved and has the outfile and group for background job
}

proc recoverFG {grp fnm} {
	global gnum PLT
	global readlines
	# called from solveGroup non-background so iX Xlabel will have been set
	# since they are need by background job 
	
	# solution via gui or BG should have made group datasets .p and .m
	# check the group root-file-name to find out what files will be written

	if {$gnum != $grp} {gotoGroup $grp}
	pipeWrite "g$grp"
	
	#pipeWrite "g f"

	#set nscan \
	#    [scan [lindex $readlines 0] {%s %d %s %s} \
	#	 glbl igrp rlbl root]
	#if { $nscan < 4 || [string match NA $root] } {
	#    append errs "group $grp ERROR: couldn't get root filename\n"
	#    return 0
	#}

	# check that root.p and root.m exist
	set textf(0) $fnm
	set textf(1) $fnm
	append textf(1) .p
	set textf(2) $fnm
	append textf(2) .m

	foreach ix {1 2} {
	    set fil $textf($ix)
	    if { ![istextfile $fil] } {
		Dialog_Warn "group $grp ERROR: failed to write $fil\n"
		set textf($ix) ""
	    }
	}

	# get the scale factor used for the solution
	pipeWrite "g ps"
	
	# returns:  group # scalefactor=  number
	set nscan [scan [lindex $readlines 0] {%s %d %s %g} \
		       glbl igrp rlbl scl]
	if { $nscan < 4 || $scl <= 0. } {
	    Dialog_Warn "group $grp ERROR: failed to get soln scale-factor.\n"
	} else {
	    set grpScale $scl
	    set grpScales($grp) $scl
	}

	# save the textfile names
	set calc 1
	set calcgroup($grp) 1
	set isfop 0
	set fopgroup($grp) 0
	# save the src and .m and .p file names
	# also makes solnsets list
	groupSrcFiles

	# save this group
	saveGroup

	# hilite calc group number
	hiliteCalcGroup
	# nolonger need trace on gnum for groupSrcFiles

	if { $PLT } {
	    groupPlotSetup
	    savePlot
	    showPlot
	}
}

proc checkWatch {star} {
	global mswindows bgWatchOK bgWatch
	# check if pbcorWatchServer is running. If not start it
	# On windows can use tasklist
	# On Unices should have ps flags | grep convWatch
	set running 0
	if {$mswindows} {
		set xlns [exec tasklist.exe /FI "IMAGENAME eq pbcorWatchServer.exe" /FO "LIST"]
	} else {
		set xlns [exec ps -aux | grep "pbcorWatchServer"]
	}
	puts "checkWatch $star xlns= $xlns"
	foreach ln $xlns {
		if [string match *pbcorWatch* $ln] {
			set running 1
			break
		}
	}
	if {$star && ! $running && $bgWatchOK } {
		exec $bgWatch &
		set running 1
	}
	return $running
}

proc updatePbcorJobs {} {
	# whenever readFiles to current group call this
	# checks if group is NOT fop and has files and is NOT already listed
	# add new group.
	# NB sessionRecovery only recovers forground solutions. This can recover any completed bg jobs

	global gnum pbcorlbxs fopgroup multisoln bgRecovered
	#puts "starting updatePbcorJobs"
	if {![winfo exists .pbcorBG]} {buildBGgui}
	# op= a(add) d(delete) u(updateData)
	set grpsnow [$pbcorlbxs(0) get 0 end]
	set ig [lsearch $grpsnow $gnum]
	#puts "calling getFileInGroup"
	set flst [getFilesInGroup $gnum]
	#puts "returned from getFilesInGroup"
	if {[llength $flst] < 1} {return}
	if {[info exists fopgroup($gnum)] && $fopgroup($gnum) > 0} {return}

	set firstfile [lindex $flst 0]
	# here we need to determine the outfilename for this group from firstfile
	# If not multisoln then outfile = firstfile_g$gnum
	# If multisoln
	# need an algorithm to look for existing solutions *.p and see if they are background
	# and look for running jobs that havent written *.p yet but have a *.START without *.EXIT
	# so this is robust across pbcor program starts
	
	# are there any completed solutions?
	# We need both foreground and background to see which is most recent
	# bgjobs for this group with a *_g$gnum*.START
	set laststat "NA"
	if { $ig >= 0 } { set laststat [$pbcorlbxs(1) get $ig] }
	set pfils [glob -nocomplain *_g$gnum*.p]
	set ok 0
	if { [llength $pfils] > 0 } {
		set pfil [latestFile $pfils]
		# is this latest soln a background solution that we want to add as Finished
		set root [regsub {\.p} $pfil {}]
		if { [file exists $root.START] } {
			set bgfile $root
			# NB is .p exists then the background should have completed but we check anyway
			if { [file exists $root.EXIT] } {
				set bgstatus "Finished"
			} else {
				set bgstatus "Running"
			}
		} else {
			# latest was foreground soln
			set bgfile $root
			set bgstatus "NA"
		}
		set ok 1
	}
	if { ! $ok } {
		set bfils [glob -nocomplain *_g$gnum*.START]
		if {[llength $bfils] > 0} {
			# no completed solutions, but still may be a background running job
			set bfil [latestFile $bfils]
			set root [regsub {\.START} $bfil {}]
			set bgfile $root
			set bgstatus "Running"
		} else {
			set bgfile $firstfile
			set bgstatus "NA"
		}
	}
	# now update the gui
	
	if {$ig >= 0} {
		# update the filename and status
		$pbcorlbxs(1) delete $ig
		$pbcorlbxs(1) insert $ig $bgstatus
		$pbcorlbxs(2) delete $ig
		$pbcorlbxs(2) insert $ig $bgfile
	} else {
		# group not listed yet so add it
		$pbcorlbxs(0) insert end $gnum
		$pbcorlbxs(1) insert end $bgstatus
		$pbcorlbxs(2) insert end $bgfile
	}
	# recover any newly finished background jobs.
	if { $bgstatus == "Finished" && $laststat != "Finished" } {
		if [recoverPlotBG $gnum $bgfile] {
			set bgRecovered($bgfile) 1
		} else {
			set bgRecovered($bgfile) 0
		}
	}
	if { $bgstatus == "Running" } {
		set bgRecovered($bgfile) 0
	}
}


proc pbcorBGstatus {glst fnamelst} {
	global bgRecovered gnum
	global pollBG pollBGsec
	global groups
	
	# check BG job status from the gui
	# called from updateBGstatus
	# which is called from the convBGjobs toplevel or from convBGrun in pbcor.tcl via updateBGstatus
	# how to tell when a BG job is finished
	# fname.BGout must exist and can be opened for reading
	# the C-code writes the file fname.EXIT  as fname timestamp
	# started jobs have fname.START
	# whenever a BG job starts and fname.START is created, call updateBGstatus to get the status set to Running

	if { [llength $glst] < 1 } { return }
	set status {}
	set runningjobs 0
	set savegnum $gnum
	foreach g $glst fname $fnamelst {
		# find the most recent background start for this job
		set istart 0
		if [file exists $fname.START] {set istart 1}

		if {$istart == 0} {
			lappend status "NA"
			continue
		}
		# found a start. find the most recent exit
		set iexit 0
		if [file exists $fname.EXIT] {
			set iexit 1
			if {[info exists bgRecovered($fname)] && $bgRecovered($fname) == 0 && [info exists bgjobs($fname)]} {
				# OK to recover this bgJob
				# must go to its group
				# RWE Mar 2019 decided not to use vaolatile bgjobs, just use gui as valid source of info
				gotoGroup $g

				# after a background job we need to get the resulting .p and .m file
				if { ![recoverPlotBG $g $fname] } {
					Dialog_Warn "FAILED to recover pbcor data from BG job $fname."
				}
				# get the fit params from the BG .fit file
				# NB recoverFit has made sure there is a fit instance and set the fit set via l sof

    			#plotFit plotFit already done in recoverFit
				
				set bgRecovered($fname) 1
			}
		}

		if {$iexit == 0} {
			lappend status "Running"
			set bgRecovered($fname) 0
			incr runningjobs
			continue
		}
		lappend status "Finished"
	}
	gotoGroup $savegnum
	# if polling BG jobs schedule another polled check if any jobs are running

	return $status
}

proc updateBGstatus {} {
	global pbcorlbxs
	global pollBG bgPolling pollBGsec
	# this is initially called from gui button
	# but also called when polling is set in the gui.
	# Will check when a job completes and plot it
	# decided to update bg status all files in the convol job list
	#set so [selection own]
	#if $so != $convlbxs(0)
	#	Dialog_Warn "Selection must be in the convBGjobs files listbox."
	#	return
	#
	#set sel [selection get]
	if {![winfo exists .pbcorBG]} {
		buildBGgui
		return
	}
	set fnames [$pbcorlbxs(2) get 0 end]
	set grps [$pbcorlbxs(0) get 0 end]
	# check all possible jobs listed in the gui
	# pbcorBGstatus can recover background jobs. will plot any finished jobs that havent already been recovered
	set status [pbcorBGstatus $grps $fnames]
	# convBGstatus proc is in fits.tcl
	set runningjobs 0
	for {set i 0} {$i < [llength $fnames]} {incr i} {
		$pbcorlbxs(1) delete $i
		set istat [lindex $status $i]
		$pbcorlbxs(1) insert $i $istat
		if { $istat == "Running" } { incr runningjobs }
	}
	# get any status updates requested. Assumes the afterID is gone once the last poll completes
	set afterjobs [after info]
	if {$runningjobs > 0 && $pollBG} {
		# get list of current after jobs. If pbPolling is in this list don't start another
		if { [lsearch $afterjobs $bgPolling] < 0 } {
			set pollsecs 15
			if {[scan "$pollBGsec" %d] > 0} {set pollsecs $pollBGsec}
			set pollmsecs [expr 1000*$pollsecs]
			set bgPolling [after $pollmsecs updateBGstatus]
		}
	}
	if {$runningjobs < 1} {
		foreach id $afterjobs {after cancel $id}
		set bgPolling -1
	}
}

proc buildBGgui {} {
	global watchBG pollBG pollBGsec pbcorlbxs
	# background GUI
	makeSubToplevel .pbcorBG "Background Pbcor Jobs" "PbcorBG" 
	set pbcorbg [frame .pbcorBG.f]
	set pbcorbgB [frame .pbcorBG.b]
	grid $pbcorbg
	grid $pbcorbgB

# puts "passed makeSubToplevel .convBG  OK"

# convBGjobs
# has congruent listboxes like groupselector with columns
# filename   group   fit   status
# ...
# Then
# For Current Selection:
# fitting ON
# fitting OFF
# start BG job
# kill BG job

# NB the cmndline version of Tkfits= TclfitsCL can be run
# from the tcl exec cmnd with POSIX io redirection
# So prepare an input script as filename.BGin
# and run in background: exec TclfitsCL < BGin > BGout &

# can check the BGout to see if status is done or running


	set pbcorlbxlabels \
	{group status filename}
	set pbcorwidths \
	{6 16 48}
	set pbcorsorts \
	{ -integer -ascii -ascii}

	array set pbcorlbxs \
	[scrolledlistboxes $pbcorbg $pbcorlbxlabels $pbcorwidths $pbcorsorts 0 {}]
	# bind $convlbxs(0) <ButtonRelease-1>  {+ showSelfile}

	# add the control buttons in pbcorbgB
	set pbcorClbl [label $pbcorbgB.l -text "For Selected Groups:"]
	# set convCfit [button $convbgB.f -text "Set Fit" -command "setConvJobFit 1" -bg green]
	# $set convCnfit [button $convbgB.nf -text "Set NO Fit" -command "setConvJobFit 0" -bg green]
	set pbcorCstart [button $pbcorbgB.s -text "Start BG" -command "setPbcorJobRun 1" -bg green]
	set pbcorCstop [button $pbcorbgB.x -text "Kill BG" -command "setPbcorJobRun 0" -bg green]

	set pbcorClbla [label $pbcorbgB.la -text "For All Groups:"]
	#set pbcorCwcbl [label $pbcorbgB.wcl -text "Watch BG"]
	set pbcorCwcb [checkbutton $pbcorbgB.wc -variable watchBG -text "Watch BG" -justify right]
	set pbcorCup [button $pbcorbgB.u -text "Update Status" -command "updateBGstatus" -bg green]
	#set pbcorCpcbl [label $pbcorbgB.pcl -text "Poll BG"]
	set pbcorCpcb [checkbutton $pbcorbgB.pc -variable pollBG -text "Poll BG" -justify right]
	set pbcorCpcel [label $pbcorbgB.pcel -text "Poll secs"]
	set pbcorCpce [entry $pbcorbgB.pce -textvariable pollBGsec -width 4 -justify left]

	grid $pbcorClbl $pbcorCstart $pbcorCstop 
	grid $pbcorClbla $pbcorCwcb $pbcorCup 
	grid $pbcorCpcb $pbcorCpcel $pbcorCpce
}


proc setPbcorJobRun {t args} {
	global gnum group 
	global multisoln watchBG bgWatchOK
	global bgExec bgExecOK
	global bgRecovered
	global pbcorlbxs
	global bgjobs
	global nread nerr readlines
	
	# start point from gui to run or kill a background job
	# can be called from outside the GUI as well for current group using non-empty args list

	
	if {! $bgExecOK } {
		Dialog_Warn "Pbcor background executable, $bgExec, not found or not executable."
		# this is fatal so stop any background requests
		return -1
	}
	


	if { [llength $args] > 0 } {
		# called from the SOLVEbg button. add group to gui if missing and set the group column 
		# in the gui to select this group.
		set sel [lindex $args 0]
		if { ![info exists group($sel)] } {
			Dialog_Warn "Pbcor background attempt failed for non-existent group= $sel."
			return -1
		}
		# sel should be a group number listed in the gui
		set gns [$pbcorlbxs(0) get 1 end]
		if { [set isel [lsearch $gns $sel]] < 0 } {
			$pbcorlbxs(0) insert end $sel
			$pbcorlbxs(1) insert end NA
			$pbcorlbxs(2) insert end NA
			$pbcorlbxs(0) selection set end
		} else {
			$pbcorlbxs(0) selection set $isel
		}
	}
	set so [selection own]
	if {$so != $pbcorlbxs(0)} {
		Dialog_Warn "Selection must be in the pbcorBGjobs groups listbox."
		return
	}
	set sel [$pbcorlbxs(0) curselection]
	set start 0
	if {$t == 1} { set start 1 }
	# whenever a BG job starts make sure the group filename is up to date
	
	# raise the GUI to remind user that background is starting
	wm deiconify .pbcorBG
	
	# pbcorBG gui allows launching multiple groups at once
	foreach s $sel {
		set g [$pbcorlbxs(0) get $s]
		if { ! $start } {
			pbcorBGkill [$pbcorlbxs(2) get $s]
		} else {
			# get the unique group outfile in the C-code and put it in the gui
			# this requirs setting the group filelist from the groups files
			# which is done also in solveGroup. Here we are not yet recording script
			# but we need to do this to get the correct output file name for the gui
			# first make sure we are the requested group in the C-library
			pipeWrite "g$g"
			
			set fls [getFilesInGroup $g]
			if { [llength $fls] < 1 } {
				#puts "NO files in group=$g"
	    		Dialog_Warn "ERROR: No files in group $g for background job."
	    		continue
	    	}
			#puts "starting fileListToSetList"
			# puts file list in C-code setlist 1
			fileListToSetList $fls
			#puts "move set list 1 to current group= $grp"
			pipeWrite "g sl 1"
			
			# send the multisoln flag for this group. deflt is ON
			if { ![info exists multisoln] } { set multisoln 1 }
			pipeWrite "g lfm $multisoln"
			# finally get the unique group outfile name used to identify all the backgroun job files
			pipeWrite "g ofs 1"
			if {$nread < 1 || $nerr > 0} {
				Dialog_Warn "ERROR: Failed to get group $g outfile name."
				continue
			}
			set fn [lindex [lindex $readlines 0] 3]
			$pbcorlbxs(2) delete $s
			$pbcorlbxs(2) insert $s $fn
			$pbcorlbxs(1) delete $s
			$pbcorlbxs(1) insert $s "Running"
			set bgRecovered($fn) 0

			# startup the background run using the unique filename
			pbcorBGrun $g $fn
		}
	}
}

proc recordBGscript {gn fname} {
	global recordScript recordFile recordScriptName
	global recordStarted recordFPT
	
	# now we can start making the background script
	# when recordScript is on, pipeWrite just records the cmd in recordScriptName file	
	set recordScript 1
	set recordFile $fname
	set recordScriptName $fname.BGin

	
	# just call solveGroup $gn
	set setupOK [solveGroup $gn]
	# this should complete the commands that need recording

	set recordScript 0
	close $recordFPT
	set recordStarted 0
	# Cant dispatch background job if setup fails
	if { ! $setupOK } { return 0 }
	return 1
}



proc pbcorBGrun {gn fname} {
	global gnum recordScriptName
	global bgExec bgExecOK bgWatchOK bgRunning bgRecovered pollBG pollBGsec bgPolling
	global par
	global pbcorWatch
	global nread readlines
		
	# start by turning on recording to recordFile fname.BGin
	# this script will get passed to TkfitsCL < fname.BGin > fname.BGout
	# make sure we are in requested group
	# so that we can get to fname
	# also resol is per group
	
	if {! $bgExecOK } {
		Dialog_Warn "Pbcor background executable, $bgExec, not found or not executable."
		# this is fatal so stop any background requests
		return 0
	}
	

	# start the watch BG server if requested
	if {$pbcorWatch && $bgWatchOK} {
		checkWatch 1
	}

	# before we start recording get to the group 
	if { $gn != $gnum } {
		gotoGroup $gn

	}
	# record the background script which is just solveGroup cmnds
	if {! [recordBGscript $gn $fname]} {
		Dialog_Warn "Pbcor background setup failed for group $gn output file $fname"
		return 0
	}
	# add it to the list of running jobs. I am currently not using bgRunning
	if {[lsearch $bgRunning $fname] < 0} {
		lappend bgRunning $fname
	}

	# for background jobs to be robust if the fits program stops
	# a $fname.START file is created in fitLsq for convolutions
	# NB fname has .cvi appended for convolutions
	#set fptX [open $fname.START "w"]
	#catch { close $fptX }
	# make sure fname doesnt have a STOP or EXIT file
	# also mark this file as NOT recovered
	if {[file exists $fname.STOP]} { file delete $fname.STOP }
	if {[file exists $fname.EXIT]} { file delete $fname.EXIT }
	set bgRecovered($fname) 0
	# dispatch the background job
	exec $bgExec < $recordScriptName >& $fname.BGout &
	# user can manually do an updateBGstatus to indicate that this job is running and raise the .convBG window
	# or turn on polling with the Poll BG check box in the Convolution Background Jobs .convBG window and resol(pollBGsec)
	if {$pollBG} {
		# if bgPolling >= 0 try to cancel it
		if { $bgPolling >= 0 } { after cancel $bgPolling }
		set pollsecs 15
		if {[scan $pollBGsec %d] > 0} {set pollsecs $pollBGsec}
		set pollmsecs [expr 1000*$pollsecs]
		set bgPolling [after $pollmsecs updateBGstatus]
	}
	
	if [winfo exists .pbcorBG] {raise .pbcorBG}
	# since it may take some time to create the .START file user should do the updateBGstatus manually
	#updateBGstatus
	return 1
}

proc pbcorBGkill {fname} {
	# put fname in the convStop.log file so that convolution can check for stop
	# first see if this is running by calling convBGstatus which returns NONE Running Finished
	# 
	global bgRunning
	set bgstatus [pbcorBGstatus [list $fname]]
	set status [lindex $bgstatus 0]
	if [$status != "Running"] {
		Dialog_Warn "$fname is NOT running in background."
		return 0
	}
	# create fname.STOP
	set fptK [open $fname.STOP "w"]
    catch {close $fptK}
	return 1
}

# neutron fast background stuff
proc subtractBG {} {
    global bg
    if { ![winfo exists $bg] } { buildFBG }
    if { ![winfo ismapped $bg] } { wm deiconify $bg }
    raise $bg
}
proc scaleSETS {} {
    global scl
    if { ![winfo exists $scl] } { buildSCL }
    if { ![winfo ismapped $scl] } { wm deiconify $scl }
    raise $scl
}
proc buildFBG {} {
    global bg
    global FBGcps FBGcpsErr

    set bg [toplevel .bg]
    wm title $bg "fast background subtraction"
    wm iconname $bg "PB bg"

    # entry FBGcps  FBGcpsErr
    set bge $bg.e
    set bgcpsL $bge.cpsL
    set bgcps $bge.cps
    set bgerrL $bge.errL
    set bgerr $bge.err

    set bgb $bg.b
    set bgcurg $bgb.curg
    set bgselg $bgb.selg
    set bgself $bgb.self

    frame $bge
    frame $bgb
    grid $bge
    grid $bgb

    label $bgcpsL -text "fast bg (cps)"
    entry $bgcps -textvariable FBGcps -relief sunken -bd 2
    label $bgerrL -text "+-"
    entry $bgerr -textvariable FBGcpsErr -relief sunken -bd 2

    button $bgself -text "sub bg sel files" -command "subtractFBG 0" \
	    -background green -activebackground yellow
    button $bgcurg -text "sub bg cur group" -command "subtractFBG 1" \
	    -background green -activebackground yellow
    button $bgselg -text "sub bg sel groups" -command "subtractFBG 2" \
	    -background green -activebackground yellow

    grid $bgcpsL $bgcps $bgerrL $bgerr
    grid $bgself $bgcurg $bgselg -sticky ew
}

proc buildSCL {} {
    global scl
    global extraSclFactor

    set scl [toplevel .scl]
    wm title $scl "scale datasets"
    wm iconname $scl "PB scl"

    # entry extraScaleFactor
    set sclw $scl.w
    set sclwl $sclw.l

    set scle $scl.e
    set sclel $scle.l
    set sclee $scle.e

    set sclc $scl.c
    set sclself $sclc.self
    set sclcurg $sclc.curg
    set sclselg $sclc.selg

    frame $sclw
    frame $scle
    frame $sclc
    grid $sclw
    grid $scle
    grid $sclc

    label $sclwl \
	-text "There is something systematically wrong with your data." \
	-font {helvetica 18 bold} -fg #f00
    label $sclel -text "extra scale factor"
    entry $sclee -textvariable extraSclFactor -relief sunken -bd 2

    button $sclself -text "scl sel files" -command "scaleSets 0" \
	    -background green -activebackground yellow
    button $sclcurg -text "scl cur group" -command "scaleSets 1" \
	    -background green -activebackground yellow
    button $sclselg -text "scl sel groups" -command "scaleSets 2" \
	    -background green -activebackground yellow

    grid $sclwl
    grid $sclel $sclee
    grid $sclself $sclcurg $sclselg -sticky ew
}


proc cloneSet {sset fnam} {
    # clone sset, set its filename and return the new setnumber
    global readlines readfinished
    pipeWrite "d$sset c"
    
    # get the cloned setnumber
    if { [scan [lindex $readlines 0] {%s %d} lbl bset] < 2 } {
	Dialog_Warn "failed to get cloned src file setnumber.\n"
	return 0
    }
    # name the clone src.fbg
    pipeWrite "d$bset nm $fnam"
    
    return $bset
}

proc subtractFBG {in} {
    global lbxs lbxlabels lbxi fbglbx gnum
    global dsFile srcfiles
    global readlines readfinished nerr errlines nread
    global fbgFile FBGcps FBGcpsErr

    # subtract fast background from
    # selected files in=0  lbxs(0) or lbxs(1)
    # current group  in=1
    # selected groupsin=2  lbxs(0) or lbxs(1)
    # FBG is here just proportional to Time
    # although eventually may add proportional to angle ttm tts tta ?
    # the FBG toplevel lets user enter FBGcps +- cps and
    # then command buttons to subtract selfiles or groups

    # RWE Oct 2010 changes procedure to subtract fbg
    # now just create the fbg and fbgErr columns in same set
    # Then foreach y# e# create new columns fbgSub# fbgSubErr#
    # and use c ds A,Aerr,B,Berr,C,Cerr
    # reset the y# e# columns to the new C,Cerr columns
    # finally rename the dataset by appending .fbg.s
    # and update the name in lbxs(0)

    # old procedure was
    # use libDF clone each source file, returns the dsnumber
    # d nm to set its new name= src.fbg.x
    # put src and bg file into setlist via fileListToSetlist
    # make sure we have the fop for this setlist (1) with columns -1 -2 -3 2
    # Qx Qy Qz En via: s1 o c 2 -1 -2 -3
    # which should distinguish the datapoints for subtraction
    # purposes
    # prepare the data for the background set using
    # c df commands: c dfn 1 to init to one ind var
    # c dfx Time to make function value proportional to Time
    # c dfc1 FBGcps   c dfe1 FBGcpsErr
    # c df DetectorCol,DetectorErrCol
    # set the subtractor to the FBGsetnumber for this setlist
    # s ss FBGsetnumber
    # do the subtraction
    # s s
    # output should be src.s
    # can also find the result set from s1 l
    # which reports in second line op lnr opc lns
    # lnr is setlist link to result
    # that setlist should have the result file

    # get the short filenames, according to source
    if { $in == 1 } {
	set fls [getFilesInGroup $gnum]
    } elseif { $in == 0 } {
	set fls [filesSelected 1]
    } elseif { $in == 2 } {
	set fls [filesInGroupsSelected 1]
    }
    set nfl [llength $fls]
    if { $nfl < 1 } {
	Dialog_Warn "No files to subtract FBG from.\n"
	return
    }

    set alreadySub {}
    for {set i 0} {$i < $nfl} {incr i} {
	set fil [lindex $fls $i]
	if { [string match *.s $fil] || [string match *.fbg* $fil] } {
	    lappend alreadySub $fil
	}
    }

    if { [llength $alreadySub] > 0 } {
	if { ![Dialog_Query "Files [join $alreadySub \n] have already been used in subtraction. Continue anyway?"] } { return }
    }

    for {set i 0} {$i < [llength $fls]} {incr i} {
	# clone the source file
	set fil [lindex $fls $i]
	# puts "subtractFBG for file $fil"

	set sset $dsFile($fil)
	pipeWrite "d$sset"
	
	# get the dataset ndy
	set nscan [scan [string trimleft [lindex $readlines 1] "* "] \
		       {%d %d %d %d %d %d %s} \
		       is npts ncol ndy nass df srcf]
	if { $nscan < 7 || $ndy < 1 } {
	    Dialog_Warn "Failed to get set $sset ny.\n"
	    return
	}
	# also get nx
	pipeWrite "d nx"
	
	set nscan [scan [lindex $readlines 0] {%s %d %s %d} dum is dum2 nx]
	if { $nscan < 4 || $nx < 1 } {
	    Dialog_Warn "Failed to get set $sset nx.\n"
	    return
	}

	# clone to new set and rename. bset becomes the current dataset
	# RWE Oct 2010 no longer cloning to new set
	#if { ! [set bset [cloneSet $sset "$fil\.fbg.x"]] } { return }

	# instead make sure fbg and fbgErr columns exist
	pipeWrite "c n fbg fbgErr"
	

	# setup to calc fbg vs Time
	pipeWrite "c dfn 1"
	
	pipeWrite "c dfx Time"
	
	# get the $fil FBG, ie any previous subtraction result
	if { ! [info exists fbgFile($fil)] } { set fbgFile($fil) [list 0 0] }
	if { [llength $fbgFile($fil)] < 2 } { lappend fbgFile($fil) 0 }
	# if there was a previous subtraction with error, this additional
	# subtraction will calculate an additional propagated error
	set prevFBG [lindex $fbgFile($fil) 0]
	set prevERR [lindex $fbgFile($fil) 1]

	pipeWrite "c dfc1 $FBGcps"
	
	pipeWrite "c dfe1 $FBGcpsErr"
	
	#puts 
	#"subtractFBG for file $fil set=$sset npts=$npts ny=$ny\nstart det loop"
	# put the fbg calc in the fbg and fbgErr columns
	pipeWrite "c ls fbg"
	
	if { [scan [lindex $readlines 0] {%s %s %d} typ lbl fbgcol] < 3 } {
	    Dialog_Warn "failed to get fbg output column."
	    return
	}
	pipeWrite "c ls fbgErr"
	
	if { [scan [lindex $readlines 0] {%s %s %d} typ lbl fbgErrcol] < 3 } {
	    Dialog_Warn "failed to get fbgErr output column."
	    return
	}
	# use these columns for fbg and fbgErr calc
	pipeWrite "c df $fbgcol,$fbgErrcol"
	

	# now loop thru assigned detectors
	for {set j 1} {$j <= $ndy} {incr j} {
	    # find the y# and e# column numbers
	    pipeWrite "c al y$j"
	    
	    set rl [lindex $readlines 1]
	    if { [scan $rl {%s %d %d %s} typ inx ycol ylbl] < 4 } {
		Dialog_Warn "failed to get src file y$j col"
		return
	    }
	    pipeWrite "c al e$j"
	    
	    set rl [lindex $readlines 1]
	    if { [scan $rl {%s %d %d %s} typ inx ecol elbl] < 4 } {
		Dialog_Warn "failed to get src file e$j col"
		return
	    }
	    # make sure we have the output columns for, fbgSub# fbgSubErr#
	    # using labels ylbl-fbg elbl-fbg
	    set newylbl "$ylbl\-fbg"
	    set newelbl "$elbl\-fbg"
	    pipeWrite "c n $newylbl $newelbl"
	    
	    # get their column numbers
	    pipeWrite "c ls $newylbl"
	    
	    if {[scan [lindex $readlines 0] {%s %s %d} typ lbl fbgsubcol] < 3} {
		Dialog_Warn "failed to get $newylbl output column."
		return
	    }
	    pipeWrite "c ls $newelbl"
	    
	    if {[scan [lindex $readlines 0] {%s %s %d} \
		     typ lbl fbgsubErrcol] < 3 } {
		Dialog_Warn "failed to get $newelbl output column."
		return
	    }

	    # do the simple column subtraction
	    # y$j - fbg = fbgSub$j  fbgSubErr$j ^2 = fbgErr ^2 + e$j ^2 
	    pipeWrite \
		"c ds $ycol,$ecol,$fbgcol,$fbgErrcol,$fbgsubcol,$fbgsubErrcol"
	    
	    # there shouldnt be any errors from this
	    # reassign y# e# to the fbgsub columns
	    pipeWrite "c ad y$j $fbgsubcol"
	    
	    pipeWrite "c ad e$j $fbgsubErrcol"
	    
	    # if the column for y$j is in the ICE header as Signal
	    # update that as well
	    pipeWrite "d hr \#Signal"
	    
	    if { $nread > 1 } {
		if { [scan [lindex $readlines 1] {%s %d %s} \
			  dum col slbl] == 3 } {
		    set newSig "\#Signal $fbgsubcol $newylbl"
		    if { [string match $slbl $ylbl] } {
			pipeWrite "d hr \#Signal \"$newSig\""
			
		    }
		}
	    }
	}
	# ready to put these two sets into setlist 1 for the subtraction
	#pipeWrite "s1 lc"
	#
	#pipeWrite "s l+ $sset $bset"
	#
	# define the subtractor
	#pipeWrite "s ss $bset"
	#
	# make sure the fop has x columns to compare, add En if room
	#set xcs ""
	#set bws ""
	#for {set j 1} {$j <= $nx} {incr j} {
	#    append xcs -$j " "
	#    append bws 0.001 0. " "
	#}
	#if { $nx < 4 } { append xcs 2; append bws 0.001 0. }
	#pipeWrite "s o c $xcs"
	#
	#pipeWrite "s o b $bws"
	#
	# do the subtraction
	#pipeWrite "s s"
	#
	# check for errors

	#if { $nerr > 0 } {
	#    Dialog_Warn "FBG subtraction ERRORS for $fil:\n$errlines"
	#    continue
	#}
	
	# list the setlist to find out which setlist it wrote the result to
	#pipeWrite "s1 l"
	#
	# the second line of output should have: op lnr opc lns, lnr is link
	# to result
	#if { [scan [lindex $readlines 1] {%s %s %s %s %d %d %d %d} \
	#	  lop llnr lopc llns op lnr opc lns] < 8 } {
	#    Dialog_Warn "failed to get subtraction result setlist.\n"
	#    return
	#}
	# list the result setlist to find the result dataset number
	#pipeWrite "s$lnr l"
	#
	# the 4th line should have: ilst match iset filename
	#if { [scan [lindex $readlines 3] {%d %d %d %s} \
	#	  ilst mtch rset rfil] < 4 } {
	#    Dialog_Warn "failed to get subtraction result dataset.\n"
	#    return
	#}
	# certainly want to write the result to disk and then
	# replace the src in lbxs(0) with the bgsub file
	# should we also write the background file, I guess
	#pipeWrite "d$bset w"
	#
	#pipeWrite "d$rset w"
	#

	# can get the lbx0 index for the processed file from lbxi
	# which was set when the fls was constructed.
	# rename the outputfile for sset
	# need to unset dsFile($fil) which was = sset since the filename changes

	set indx [lindex $lbxi $i]
	$lbxs(0) delete $indx
	unset dsFile($fil)
	set newfile "$fil\.fbg.s"
	pipeWrite "d nm $newfile"
	
	pipeWrite "d w"
	
	set dsFile($newfile) $sset

	set fshort [file tail $newfile]
	$lbxs(0) insert $indx $fshort

	set srcfiles($sset) $fshort
	if { ! [info exists fbgFile($fshort)] } {
	    set fbgFile($fshort) "0 0"
	}
	#puts "before update fbgFile($fil)= $fbgFile($fil)"
	set FBGres [expr $prevFBG + $FBGcps]
	set FBGerr [expr $prevERR*$prevERR + $FBGcpsErr*$FBGcpsErr]
	if { $FBGerr > 0. } { set FBGerr [expr sqrt($FBGerr)] }
	set fbgFile($fshort) "$FBGres $FBGerr"
	#puts "after update fbgFile($fshort)= $fbgFile($fshort)"
	$lbxs($fbglbx) delete $indx
	$lbxs($fbglbx) insert $indx $fbgFile($fshort)
    }
    #puts "lbxs(0) exists= [info exists lbxs(0)]"
    if { [info exists lbxs(0)] } {
	setLbxWidth $lbxs(0) [lindex $lbxlabels 0]
	setLbxWidth $lbxs($fbglbx) [lindex $lbxlabels $fbglbx]
	hiliteActiveGroup
    }
}

proc scaleSets {in} {
    global lbxs lbxlabels lbxi gnum
    global dsFile srcfiles
    global readlines readfinished nerr errlines nread
    global extraSclFactor

    # scale sets from
    # selected files in=0  lbxs(0) or lbxs(1)
    # current group  in=1
    # selected groupsin=2  lbxs(0) or lbxs(1)
    # This scales all defined detector y and yerr
    # There is no associated error and the scale factor
    # is untouched.
    # This is only useful for trying to fix systematic intensity pbs.

    # create the Scl column in same set
    # Then foreach y# e# create new columns y#-old e#-old
    # and use c dy Scl,Ycol   c dy Scl,YerrCol
    # dont need to reset the y# e# columns
    # just write the old data to new columns
    # finally rename the dataset by appending .y
    # and update the name in lbxs(0)

    # get the short filenames, according to source
    if { $in == 1 } {
	set fls [getFilesInGroup $gnum]
    } elseif { $in == 0 } {
	set fls [filesSelected 1]
    } elseif { $in == 2 } {
	set fls [filesInGroupsSelected 1]
    }
    set nfl [llength $fls]
    if { $nfl < 1 } {
	Dialog_Warn "No files scale.\n"
	return
    }

    set alreadyScl {}
    for {set i 0} {$i < $nfl} {incr i} {
	set fil [lindex $fls $i]
	if { [string match *.y* $fil] } {
	    lappend alreadyScl $fil
	}
    }

    if { [llength $alreadyScl] > 0 } {
	if { ![Dialog_Query "Files [join $alreadyScl \n] have already been scaled. Continue anyway?"] } { return }
    }

    for {set i 0} {$i < [llength $fls]} {incr i} {
	set fil [lindex $fls $i]
	set sset $dsFile($fil)
	pipeWrite "d$sset"
	
	# get the dataset ndy
	set nscan [scan [string trimleft [lindex $readlines 1] "* "] \
		       {%d %d %d %d %d %d %s} \
		       is npts ncol ndy nass df srcf]
	if { $nscan < 7 || $ndy < 1 } {
	    Dialog_Warn "Failed to get set $sset ny.\n"
	    return
	}
	# also get nx
	pipeWrite "d nx"
	
	set nscan [scan [lindex $readlines 0] {%s %d %s %d} dum is dum2 nx]
	if { $nscan < 4 || $nx < 1 } {
	    Dialog_Warn "Failed to get set $sset nx.\n"
	    return
	}

	# create the extraScl column with constant sequence
	pipeWrite "c dm extraScl $extraSclFactor 0."

	# read back the created column number
	pipeWrite "c ls extraScl"
	
	if { [scan [lindex $readlines 0] {%s %s %d} typ lbl sclcol] < 3 } {
	    Dialog_Warn "failed to get scale output column."
	    return
	}

	# now loop thru assigned detectors
	for {set j 1} {$j <= $ndy} {incr j} {
	    # find the y# and e# column numbers
	    pipeWrite "c al y$j"
	    
	    set rl [lindex $readlines 1]
	    if { [scan $rl {%s %d %d %s} typ inx ycol ylbl] < 4 } {
		Dialog_Warn "failed to get src file y$j col"
		return
	    }
	    pipeWrite "c al e$j"
	    
	    set rl [lindex $readlines 1]
	    if { [scan $rl {%s %d %d %s} typ inx ecol elbl] < 4 } {
		Dialog_Warn "failed to get src file e$j col"
		return
	    }
	    # make sure we have the output columns for, scly# sclErr#
	    # using labels ylbl-old elbl-old
	    set newylbl "$ylbl\-old"
	    set newelbl "$elbl\-old"
	    pipeWrite "c n $newylbl $newelbl"
	    
	    # get their column numbers
	    pipeWrite "c ls $newylbl"
	    
	    if {[scan [lindex $readlines 0] {%s %s %d} typ lbl oldYcol] < 3} {
		Dialog_Warn "failed to get $newylbl output column."
		return
	    }
	    pipeWrite "c ls $newelbl"
	    
	    if {[scan [lindex $readlines 0] {%s %s %d} typ lbl oldEcol] < 3} {
		Dialog_Warn "failed to get $newelbl output column."
		return
	    }

	    # copy the current y and err values to y-old err-old
	    pipeWrite "c dy $ycol,$oldYcol"
	    pipeWrite "c dy $ecol,$oldEcol"

	    # do the scaling in place on y yerr
	    # y$j = scl * y$j   e$j = scl * e$j
	    pipeWrite "c dy $ycol,$ycol $extraSclFactor"
	    pipeWrite "c dy $ecol,$ecol $extraSclFactor"
	    
	    # there shouldnt be any errors from this
	    # now we dont need to reassign y# e# as they are
	    # now result scale columns
	    # this avoids detector naming inconsistencies
	    # Also dont need any updating of ICE Signal column spec
	}

	set indx [lindex $lbxi $i]
	$lbxs(0) delete $indx
	unset dsFile($fil)
	set newfile "$fil\.y"
	pipeWrite "d nm $newfile"
	
	pipeWrite "d w"
	
	set dsFile($newfile) $sset

	set fshort [file tail $newfile]
	$lbxs(0) insert $indx $fshort

	set srcfiles($sset) $fshort
    }

    if { [info exists lbxs(0)] } {
	setLbxWidth $lbxs(0) [lindex $lbxlabels 0]
	hiliteActiveGroup
    }
}

########### text file viewer procs ###################

proc istextfile {fil} {
    return [expr [file exists $fil] && [file isfile $fil] && \
	    [file readable $fil]]
}

proc dsFromSn {n} {
    # n setnmuber 1 - ngrpsets
    # returns pbscript setnumber
    global dsns ngrpsets
    if { $n < 1 || $n > $ngrpsets } { return 0 }
    return [lindex $dsns [expr $n - 1]]
}
proc snFromDs {ds} {
    # ds pbscript datasetnumber >= 1
    # returns setnumber in group list 1 - ngrpsets
    # deprecated
    global dsns ngrpsets
    for {set i 0; set j 1} {$i < $ngrpsets} {incr i; incr j} {
	set dsni [lindex $dsns $i]
	if { $dsni == $n } { return $j }
    }
    return 0
}
proc fileFromSn {sn} {
    # deprecated
    global srcfiles ngrpsets
    if {$sn < 1 } {set sn 1}
    if {$sn > $ngrpsets} {set sn $ngrpsets}
    set dsni [dsFromSn $sn]
    return $srcfiles($dsni)
}
proc snFromFile {fnam} {
    global solnfiles
    if { [set i [lsearch $solnfiles $fnam]] >= 0 } { return [expr $i + 1] }
    #for {set i 0; set j 1} {$i < $ngrpsets} {incr i; incr j}
    #	set dsni [lindex $dsns $i]
    #	if { [string match $fnam $srcfiles($dsni)] } { return $j }
    return 0
}

proc fileViewer {args} {
    global tv
    if { ! [winfo exists $tv] } {
	buildShowText -width 60 -height 20 -wrap none
    }
    wm deiconify $tv
    # no longer raise the window on every call
    # only raise if fileViewer button pressed
    raise $tv
    showText
}

# put file text in toplevel text-viewer tv
proc showText {args} {
	global gnum
    global tv txtw tvtitle viewfile
    global textf mp snum natn natext
    global srcfiles solnfiles dsns ngrpsets
    global sellectlist lbxs currentDir
    # deiconify the scrolledText window
    # and load the .p file
    # srcfiles dsns ngrpsets should already be available from grpSrcFiles
    # mp selects  1 .d   2 .m   3 src   4 select   5 native root.A B C D no longer used
    if { ! [winfo exists $tv] } {
		buildShowText -width 60 -height 20 -wrap none
    }
    # wm deiconify $tv
    # no longer raise the window on every call
    # only raise if fileViewer button pressed
    # raise $tv
    $txtw delete 1.0 end
    set viewfile ""
    if { $mp < 3 } {
    	set pfils [glob -nocomplain *_g$gnum*.p]
		if { [llength $pfils] > 0 } {
			set pfil [latestFile $pfils]
			set mfil [regsub {\.p} $pfil ".m"]
			set bfil [regsub {\.p} $pfil ".bin"]
			if {$mp == 0 } {
				set viewfile $bfil
			} elseif {$mp == 1} {
				set viewfile $pfil
			} else {
				set viewfile $mfil
			}
		}
    } elseif { $mp == 3 || $mp == 5 } {
		# print the group sets, will incl p m
		# groupSrcFiles will set array srcfiles(setnumber) = filename
		if { $ngrpsets < 1 } {
	    Dialog_Warn "NO src files listed for current group"
	    return
		}
		if { $snum < 1 } { set snum 1 }	
		if { $snum > $ngrpsets } { set snum $ngrpsets }
		set viewfile [file tail [lindex $solnfiles [expr $snum - 1]]]
		if {$mp == 5} {
			# strip down to .x and add the .pbcor extension
			set root [regsub {\.x.*} $viewfile {.x}]
			set viewfile $root.pbcor
		}
    } elseif { $mp == 4 } {
		# sel can be in sellectlist or lbxs(0)
		set sel [selectionOwned 1 1 1]
		if { [llength $sel] < 1 } { return }
		set viewfile [lindex $sel 0]

		#set fil [file join $currentDir $fil]
    } elseif { $mp == 5 } {
		# native $textf(0).A/B/C/D = natn 1-4
		if { $natn < 1 || $natn > 4 } { set natn 1 }
		set viewfile $textf(0)$natext($natn)
    }

    if { ! [istextfile $viewfile] } {
		#Dialog_Warn "$viewfile is not a readable textfile"
		return
    }
    set in [open $viewfile]
    
    $txtw insert end [read $in]
    close $in
    wm title $tv "$tvtitle   $viewfile"
    # always start the text window at top left
    wm geometry $tv +10+10
}

proc groupSrcFiles {args} {
    # only called from solveGroup foreground which puts solution into datasets for
    # the foreground Tclpbs
    # this is not session robust as it was
    # designed for call after solveGroup,
    # so have session recovery resolve any groups with calc=1.

    # what is needed is the list of solnfiles
    # solnfiles should be identical to the groupL grpF

    # so retask srcfiles as simply the inverse of dsFile
    # not to be session saved,
    # instead save solnfiles foreach group
    # along with ngrpsets which counts solnfiles
    # also get mset pset from textf
    # after textf is session recovered,
    # regenerate mset pset at the end.
    # isrcplot can be saved per group.

    # call this whenever group is solved
    # then save on gnum change via setGroup/saveGroup
    # then srcfiles, pset, mset are available
    # isrcplot($sn) flags srcdata viewed
    # init to all on

    global gnum
    global srcfiles solnfiles solnsets solnresultsets solnresultfiles 
    global ngrpsets dsns isrcplot
    global pset mset
    global nread readlines
    # applies to current group

    set narg [llength $args]
    if { $narg > 0 } {
		set g [lindex $args 0]
    } else {
		set g $gnum
    }
    set solnfiles {}
    set solnsets($g) {}
    set solnresultfiles {}
    set solnresultsets($g) {}
    set pset($g) 0
    set mset($g) 0

    pipeWrite "g$g"
    

    pipeWrite "g ls"
    
    # DATASETS for group #
    # gset iset npts ncol srcfile
    # NB this can recognize the m and p sets (not integer sn)
    # which are handled separately in showText

    for {set i 2} {$i < $nread} {incr i} {
		set rl [lindex $readlines $i]
		set nscan [scan $rl {%d %d %d %d %s} igs ids npt nc srcf]
		if { $nscan < 5 } {
	    	if { [scan $rl {%s %d} igs ids] == 2 } {
				if { [string match p $igs] } {
		    		set pset($gnum) $ids
		    		lappend solnresultsets($g) $ids
		    		lappend solnresultfiles [file tail $srcf]
				} elseif { [string match m $igs] } {
		    		set mset($gnum) $ids
		    		lappend solnresultsets($g) $ids
		    		lappend solnresultfiles [file tail $srcf]
				}
	    	}
	    	continue
		}
		lappend solnfiles [file tail $srcf]
		lappend solnsets($g) $ids
    }
    #set dsns [array names srcfiles]
    set ngrpsets [llength $solnfiles]
    for {set i 1} {$i <= $ngrpsets} {incr i} { set isrcplot($i) 1 }

    # also get native setnumbers
    pipeWrite "g lsn"
    
    # native DATASETS for group #
    # gset iset npts ncol srcfile
    # order is A D B C = offoff ONON ONoff offON

    for {set i 2} {$i < $nread} {incr i} {
		set rl [lindex $readlines $i]
		set nscan [scan $rl {%s %d %d %d %s} igs ids npt nc srcf]
		lappend solnresultfiles [file tail $srcf]
		lappend solnresultsets($g) $ids
    }

    if { $narg > 0 } { return }
    setSn 1
    updateSrcFileMenus
}

proc setSn {sn} {
    # still called from setGroup
    global snum mp isrcon isrcplot srcfile srcfiles
    set snum $sn
    if { $mp == 3 } { showText }
    if { [info exists isrcplot($snum)] } {
	set isrcon $isrcplot($snum)
    }
    set ids [dsFromSn $snum]
    if { [info exists srcfiles($ids)] } {
	set srcfile [file tail $srcfiles($ids)]
    }
}
proc incrSn {} {
    # deprecated
    global snum ngrpsets
    if { $snum >= $ngrpsets } { return }
    incr snum
    setSn $snum
}
proc decrSn {} {
    # deprecated
    global snum
    if { $snum <= 1 } { return }
    incr snum -1
    setSn $snum
}

proc columnsFromFileView {mode} {
    # edit menu buttons call this to 
    # mode-0 save columns from a fileview to a new file
    # mode-1 or to update the Yselect columns
    # mode-2 or to Xselect the plot x-coord

    # plotx for groups is iX and for DS is spcfg(xcol) spcfg(xlbl)
    # in the setplot(file) array
    # spsn is current DSplot setnumber in dsmode

    # the yselmode must be set before call
    # as it determines the functionality in the toplevel .vcol
    # that is opened here

    global yselmode xselmode
    global viewfile viewcollabels yviewcollabels viewncol xviewcollabels
    global nread readlines readfinished
    global saveset tv gnum dsFile

    # if the fileViewer is not open or viewfile is not defined  use the first dataset in the current grp
    if { ([string length $viewfile] < 2) || ![file exists $viewfile] || ! [info exists tv] || ! [winfo exists $tv] || ! [winfo ismapped $tv]} {
		set grpfils [getFilesInGroup $gnum]
		if { [llength $grpfils] < 1 } {
	    	Dialog_Warn "current group is empty."
	    	return
		}
		set viewfile [lindex $grpfils 0]
		pipeWrite "d>$viewfile"
		if { $nread < 2 } {
	    	Dialog_Warn "couldnt get set for $viewfile"
	    	return
		}
		scan [string trimleft [lindex $readlines 1] "* "] {%d} iset
    } else {
		# check that viewfile belongs to a dataset
		# d no is not working so just check in dsFile array
		#pipeWrite "d no $viewfile"
		#
		# returns: dataset # filename= filename  #=0 if not found
		#set rl [lindex $readlines 0]
		#if { [scan $rl {%s %d} dum iset] < 1 || $iset < 1 }
		pipeWrite "d>$viewfile"
		if { $nread < 2 } {
			set ret [readFile $viewfile 1]
	    	if { [llength $ret] < 1 } {
				Dialog_Warn \
				"Current file in viewer $viewfile is not in a dataset and read failed."
				return
			}
			set iset [lindex $ret 0]
		} else {
			scan [string trimleft [lindex $readlines 1] "* "] {%d} iset
		}
    }

    # open a toplevel that lets user select columns from this dataset
    # make it the current dataset and get the list of columnlabels
    # for the user to select from
    puts "columnsFromFileView viewfile= $viewfile"
    pipeWrite "d>$viewfile"
    
    set rl [lindex $readlines 1]
    #puts "columnsFromFileView: d$iset returned nread=$nread\n[lindex $readlines 0]\n$rl"
    set nscan [scan [string trimleft $rl "* "] {%d %d %d %d %d %d %s} \
		   iset npts ncol ndy nas df src]
    if { $nscan < 7 } {
	  Dialog_Warn "failed to get dataset info for set $viewfile.\n"
	  return
    }
    # use column statistics to get column labels
    # icol lbl min max nchng vary ave
    pipeWrite "c s"
    
    if { $nread <= $ncol } {
		Dialog_Warn "failed to return all column labels for dataset $iset.\n"
		return
    }
    # icol columnlabel followed by data lines
    arrayClear viewcollabels
    set yviewcollabels {}
    set xviewcollabels {}
    for {set i 1} {$i <= $ncol} {incr i} {
		set rl [lindex $readlines $i]
		set nscan \
	    [scan $rl {%d %s %s %s %d %d %s} icol labl mn mx nch var ave]
		if { $nscan < 7 } {
		# HKL syntax is messing the string read skip  this check
	    # Dialog_Warn "failed to read column label from\n$rl"
	    # return
		}
		set viewcollabels($icol) $labl
		lappend yviewcollabels [list $labl $var]
		if { $var == 2 } { set var 0 }
		lappend xviewcollabels [list $labl $var]
    }
    # sort on decreasing vary number so probable ycolumns which
    # vary randomly have vary=2 will be at top
    set yviewcollabels [lsort -integer -decreasing -index 1 $yviewcollabels]
    set xviewcollabels [lsort -integer -decreasing -index 1 $xviewcollabels]
    set viewncol $ncol

    # for 

    # make sure this toplevel is built, mapped and raised
    # and has menu of current columns list
    set saveset $iset
    set yselmode 0
    set xselmode 0
    if { $mode == 1 } {
	set yselmode 1
    } elseif { $mode == 2 } {
	set xselmode 1
    }
    openSaveColumns
    # openSaveColumns will create SaveColumns=.vcol gui then modify the .vcol gui depending on yselmode xselmode
    # then call updateSaveColumns
}

proc openSaveColumns {} {
    # called from columnsFromFileView mode
    # use 3 modes, save or ycolumns or xcolumn
    # forget saveAs SAVE inclHead in ycolumn and xcolumn mode
    # sort with stat vary columns at top of list for ycolumn mode
    # init the outlbx with current Yselect
    # savecolbut from .vcol gui does the action
    global svcws yselmode lastyselmode xselmode lastxselmode savecolbut

    if { ! [winfo exists .vcol] } { buildSaveColumns }
    if { $yselmode && ! $lastyselmode } {
		foreach w $svcws { grid remove $w }
		wm title .vcol "select Y columns"
		$savecolbut configure -text "APPLY"
		set lastyselmode 1
		set lastxselmode 0
    } elseif { $xselmode && ! $lastxselmode } {
		foreach w $svcws { grid remove $w }
		wm title .vcol "select X column"
		$savecolbut configure -text "APPLY"
		set lastyselmode 0
		set lastxselmode 1
    } elseif { (! $yselmode && $lastyselmode) || \
		   (! $xselmode && $lastxselmode) } {
		foreach w $svcws { grid $w }
		wm title .vcol "save viewfile columns to file"
		$savecolbut configure -text "SAVE"
		set lastyselmode 0
		set lastxselmode 0
    }
    updateSaveColumns
    wm deiconify .vcol
    raise .vcol
}


proc updateYColumn {} {
    # deprecated by combining with updateSaveColumns
    # put the viewfile column labels into YColumn toplevel
    global lbY viewfile viewsave
    global viewncol viewcollabels
    set lbh $viewncol
    if { $lbh > 16 } { set lbh 16 }
    $lbY configure -height $lbh
    $lbY delete 0 end
    for {set i 1} {$i <= $viewncol} {incr i} {
	$lbY insert end $viewcollabels($i)
    }
    set viewfile [file tail $viewfile]
    set viewsave $viewfile
}

proc getXcolumns {iset} {
    global nread readlines readfinished
    global xcolumns xlst xvary

    arrayClear xcolumns
    arrayClear xvary
    foreach x $xlst {
		pipeWrite "c$iset al $x"
	
		if { $nread < 2 } {
	    	set xcolumns($x) {}
	    	set xvary($x) 0
	    	continue
		}
		if { [scan [lindex $readlines 1] {%s %d %d %s} std inx col lbl] < 4 } {
	    	set xcolumns($x) {}
	    	set xvary($x) 0
	    	continue
		}
		pipeWrite "c s $col"
	
		if {[scan [lindex $readlines 1] {%d %s %g %g %d %d} \
			 icol lbla mn mx nxch var] < 6} {
	    	set xcolumns($x) {}
	    	set xvary($x) 0
	    	continue
		}	
		set xcolumns($x) [list $col $lbl $var]
		set xvary($x) $var
    }
}

proc updateSaveColumns {} {
	# sets Xselect or Yselect to a column label(s)
    # called by openSaveColumns, which opens the saveColumns gui
    # put the viewfile column labels into SaveColumns toplevel
    # also handle yselmode by moving Yselect columns to lbout
    # and sorting column labels with varying at top
    # also handle xselmode by putting En x1 x2 x3 labels into lbsrc
    # and move current Xselect to lbout
    global yselmode Yselect xselmode Xselect
    global lbsrc lbout viewfile viewsave
    global viewncol viewcollabels yviewcollabels xviewcollabels
    set lbh $viewncol
    if { $lbh > 16 } { set lbh 16 }
    $lbsrc configure -height $lbh
    $lbout configure -height $lbh
    $lbsrc delete 0 end
    $lbout delete 0 end
    for {set i 1; set j 0} {$i <= $viewncol} {incr i; incr j} {
		if { $yselmode } {
	    	set vi [lindex [lindex $yviewcollabels $j] 0]
		} elseif { $xselmode } {
	    	set vi [lindex [lindex $xviewcollabels $j] 0]
		} else {
	    	set vi $viewcollabels($i)
		}
		if { $yselmode && [lsearch $Yselect $vi] >= 0 } {
	    	$lbout insert end $vi
		} elseif { $xselmode && [string length $Xselect] > 0 && \
		       [string match $Xselect $vi] > 0 } {
	    	$lbout insert end $vi
		} else {
	    	$lbsrc insert end $vi
		}
    }
    if { $yselmode } {
		set newYselect [$lbout get 0 end]
		foreach old $Yselect {
	    	if { [lsearch $newYselect $old] < 0 } {
				if [Dialog_Query "Do you want to change Y-columns from:\n$Yselect\nto:\n$newYselect?\n"] { set Yselect $newYselect }
				break
	    	}
		}
		#set Yselect [$lbout get 0 end]
    }
    if { $xselmode } {
		set newXselect [$lbout get 0 end]
		foreach old $Xselect {
	    	if { [lsearch $newXselect $old] < 0 } {
				if [Dialog_Query "Do you want to change X-column from:\n$Xselect\nto:\n$newXselect?\n"] { set Xselect $newXselect }
				break
	    	}
		}
    }

    set viewfile [file tail $viewfile]
    set viewsave $viewfile
}

proc globToOut {} {
    global lbsrc lbout globtoout
    if { [string length $globtoout] < 1 } { return }
    set lst [$lbsrc get 0 end]
    set sel {}
    for {set i 0} {$i < [llength $lst]} {incr i} {
	if { [string match $globtoout [lindex $lst $i]] } {
	    lappend sel $i
	}
    }
    if { [llength $sel] < 1 } { return }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbsrc get $ix]
	$lbout insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbsrc delete $ix
    }
}
proc globToSrc {} {
    global lbsrc lbout globtosrc
    if { [string length $globtosrc] < 1 } { return }
    set lst [$lbout get 0 end]
    set sel {}
    for {set i 0} {$i < [llength $lst]} {incr i} {
	if { [string match $globtosrc [lindex $lst $i]] } {
	    lappend sel $i
	}
    }
    if { [llength $sel] < 1 } { return }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbout get $ix]
	$lbsrc insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbout delete $ix
    }
}
proc toOut {} {
    global lbsrc lbout
    set sel [$lbsrc curselection]
    if { [llength $sel] < 1 } { return }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbsrc get $ix]
	$lbout insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbsrc delete $ix
    }
}
proc toSrc {} {
    global lbsrc lbout
    set sel [$lbout curselection]
    if { [llength $sel] < 1 } { return }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbout get $ix]
	$lbsrc insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbout delete $ix
    }
}

# utility for saving user selected columns to a new file for plotting progs
# also handles yselmode which just stores selected Y columns in Yselect
# Yselect columns are assigned to current grp and on any subsequent readFile


proc saveColumns {} {
	# main action executed from .vcol gui APPLY or SAVE button
    # activated by SAVE or APPLY button in the openSaveColumns toplevel
    global yselmode Yselect xselmode Xselect
    global viewsave inclhead saveset currentDir
    global lbout viewcollabels viewncol
    global readlines readfinished
    global dsmode spcfg spsn iX xcolumns ixlst
    global useXscanU XscanU
    # the useXscanU flag is set to 1 if a valid X columnd is selected
    # saveset is the dataset determined in columnsFromFileView as fileviewed or current group first file
    # make a list of the output columnindices. If no output is selected, punt, set useXscan 0 in xselmode
    set outlbls [$lbout get 0 end]
    puts "saveColumns starting with outlbls= $outlbls\n"
    if { [set nout [llength $outlbls]] < 1 } {
    	if {$xselmode} {
    		# this turns off user select X
    		set useXscanU 0
    		return
    	}
    	if {$yselmode} {return}
		Dialog_Warn "No columns selected for save"
		return
    }

    if { $yselmode } {
		# apply the Y column selection to current group
		set Yselect $outlbls
		applyYselectGroup 0
		#puts "saveColumns yselmode set Yselect=$Yselect"
		# applyYselectGroup calls updateSrcFileMenus
		# puts "saveColumns after update Yselect=$Yselect"
		return
    }

    foreach {ix lbl} [array get viewcollabels] { set inverse($lbl) $ix }
	puts "saveColumns inverse= [array get inverse]\n"
    if { $xselmode } {
		# if dsmode have to update spcfg ? setplot
		# else group plot just update iX can only be En x1 x2 x3 tts om Temp 1-7
		# use the last dataset from fileview!!!
		getXcolumns $saveset
		set Xselect [lindex $outlbls 0]
		set useXscanU 0
		
		if { ! $dsmode } {
		    set cn $inverse($Xselect)
		    # check that this column is stnd En x1-3 tts or om
		    set ok 0
		    foreach xl $ixlst {
				if { $cn == [lindex $xcolumns($xl) 0] } {
					puts "saveColumns found cn= $cn == from xcolumns xl=$xl = [lindex $xcolumns($xl) 0]\n"
			    	set ok 1
			    	# xi becomes the item number >= 1 of the stnd col Xlabel from ixlst
			    	set xi [expr [lsearch $ixlst $xl] + 1]
			    	break
				}
		    }
		    if { ! $ok } {
				Dialog_Warn "group mode Xselect is not one of $ixlst\n"
		    } else {
		    	# this sets iXU and puts it into the group xStndName
		    	# but no this choice is group agnostic, applies to all groups while set
				#groupXcolSetU $xi
				set useXscanU 1
				set XscanU $xi
			# this selection will be valid only thru the next
			# groupPlotSetup
		    }
		    puts "saveColumns xselmode set=$saveset Xselect= $Xselect useXscanU= $useXscanU XscanU= $XscanU"
		} else {
		    # dsmode just find the column number from inverse
		    set spcfg(xcol) $inverse($Xselect)
		    set spcfg(xlbl) $Xselect
		    showDSplot
		}
		return
    }


    # save columns to file mode
    set outix {}
    for {set i 0} {$i < $nout} {incr i} {
		lappend outix $inverse([lindex $outlbls $i])
    }
    set outi [join $outix ,]
    # now ready to copy the source dataset using just the spec columns
    pipeWrite "d$saveset c $outi"
    
    # this will return the output setnumber
    if { [scan [lindex $readlines 0] {%s %d} dum outset] < 2 } {
		Dialog_Warn "Set column extraction failed."
		return
    }
    # delete the header if inclhead == 0
    if { $inclhead == 0 } {
		pipeWrite "d$outset hd"
	
    }
    # set the filename for outset
    pipeWrite "d$outset nm [file join $currentDir $viewsave]"
    
    # now write the outset to disk in raw mode (no DFheader stuff)
    pipeWrite "d$outset wr"
    
    pipeWrite "d$outset"
    
    # finally delete the outset
    pipeWrite "d-$outset"
    
}

proc buildSaveColumns {} {
    global viewfile viewsave inclhead
    global lbout lbsrc
    global svcws lastyselmode lastxselmode
    global globtoout globtosrc savecolbut

    set vc [toplevel .vcol]
    wm title $vc "save viewfile columns to file"
    wm iconname $vc SaveCol
    # mb has viewfileName  saveAsName  saveButton  inclHeaderCheckBox
    set mb [frame $vc.mb]
    set vfNameLbl [label $mb.vfl -text viewfile]
    set vfName [label $mb.vf -textvariable viewfile -width 32 \
		    -relief sunken -bd 1 -bg white]
    set saNameLbl [label $mb.sal -text saveAs]
    set saName [entry $mb.sa -textvariable viewsave -width 32]
    set inch [checkbutton $mb.inch -variable inclhead -text inclHead]
    # savecolbut is the main action for the gui. It calls saveColumns
    set savecolbut [button $mb.save -text "SAVE" \
		  -command saveColumns \
		  -background green -activebackground yellow]


    set f [frame $vc.f]
    set lbsrc [listbox $f.src -relief raised -bd 2 \
		  -yscrollcommand "$f.srcscroll set" -selectmode extended]
    set srcscroll [scrollbar $f.srcscroll -command "$f.src yview"]
    set move [frame $f.move]
    set lbout [listbox $f.out -relief raised -bd 2 \
		  -yscrollcommand "$f.outscroll set" -selectmode extended]
    set outscroll [scrollbar $f.outscroll -command "$f.out yview"]

    set toOut $move.out
    set toSrc $move.src
    button $toOut -command toOut -image rightarrow \
	-background green -activebackground yellow
    button $toSrc -command toSrc -image leftarrow \
	-background green -activebackground yellow

    set gsrcb [button $f.srcb -text "glob" \
		   -command "globToOut" \
		  -background green -activebackground yellow]
    set goutb [button $f.outb -text "glob" \
		   -command "globToSrc" \
		  -background green -activebackground yellow]
    set gsrce [entry $f.srce -textvariable globtoout]
    set goute [entry $f.oute -textvariable globtosrc]


    grid $mb
    grid $f

    grid $vfNameLbl $vfName $savecolbut
    grid $saNameLbl $saName $inch

    grid $gsrcb -row 0 -column 1
    grid $gsrce -row 0 -column 2
    grid $goutb -row 0 -column 4
    grid $goute -row 0 -column 5

    grid $srcscroll -row 1 -column 0 -sticky news
    grid $lbsrc -row 1 -column 1 -columnspan 2 -sticky news
    grid $move -row 1 -column 3
    grid $lbout -row 1 -column 4 -columnspan 2 -sticky news
    grid $outscroll -row 1 -column 6 -sticky news

    grid $toOut
    grid $toSrc
    set svcws [list $saNameLbl $saName $inch]
    set lastyselmode 0
    set lastxselmode 0
}

proc buildYColumn {} {
    # deprecated by combination with buildSaveColumns
    # since multiple Y columns are allowed, mirror the savecolumns toplevel
    global viewfile viewsave yselect
    global lbY
    set yc [toplevel .ycol]
    wm title $yc "Set Y-column from viewfile"
    wm iconname $yc Ycol
    # mb has viewfileName  setYButton  YselectLabel
    set mb [frame $yc.mb]
    set vfNameLbl [label $mb.vfl -text viewfile]
    set vfName [label $mb.vf -textvariable viewfile -width 32 \
		    -relief sunken -bd 1 -bg white]

    #set setY [button $mb.save -text "Set Y column"
    #		  -command setYColumn
    #      -background green -activebackground yellow]

    set selYlbl [label $mb.ysell -text "selected Y"]
    set selY [label $mb.ysel -textvariable yselect -width 32 \
		  -relief sunken -bd 1 -bg white]

    set f [frame $yc.f]
    set lbY [listbox $f.src -relief raised -bd 2 \
		 -yscrollcommand "$f.srcscroll set" -selectmode extended]
    set srcscroll [scrollbar $f.srcscroll -command "$f.src yview"]

    grid $mb
    grid $f

    grid $vfNameLbl $vfName
    grid $selYlbl   $selY
    #grid $setY -row 0 -column 2 -rowspan 2
    grid $srcscroll $lbY -sticky news
}
proc openYColumn {} {
    if { ! [winfo exists .ycol] } { buildYColumn }
    updateYColumn
    wm deiconify .ycol
    raise .ycol
}

####################  plot a dataset or FOP #########################

proc getFOPpset {} {
    global fopgroup gnum
    if { ! [info exists fopgroup($gnum)] || ! $fopgroup($gnum) } {
	Dialog_Warn \
	    "current group not file Op result for plotting."
	return {}
    }
    set fopfiles [getFilesInGroup $gnum]
    set ok 0
    foreach fil $fopfiles {
	if [regexp {.*_g[0-9]+\.p} $fil] {
	    return $fil
	}
    }
    return ""
}

proc checkDSfile {spfile} {
    global dsFile
    if { ! [info exists dsFile($spfile)] || $dsFile($spfile) < 1 } {
	# try to read the file
	set errs ""
	set ret [readFile $spfile 1]
	if { [llength $ret] < 1 } { return 0 }
	set dsFile($spfile) [lindex $ret 0]
    }
    return $dsFile($spfile)
}

proc plotDS {args} {
    # binding to Button1 in sellectlist or lbxs(0)
    # if dsmode, plot a dataset. NOT PB-plot although can handle multiple y
    # so can plot FOP result(.p file) from PB
    # if selected fil is already in a dataset, use that
    # else readFile

    # this proc finds the dataset, and determines dataset column-labels
    # and column-data indices for plotting, then calls showDSplot

    # store the column labels/stats in splabelsarr and splabelslst
    # try to figure out what X Y Err are for the plot
    # e.g. should recognize Suu Sdd .. pb-datapts .p file data
    # or use mostDistinct to find an X if x assigned (and E stnd)
    # if no y assigned chose a random vary column

    global readlines nread readfinished
    global spfile spsn
    global setplot spcfg grpTitles gnum
    global dsmode dseditm dseditY fsSymb gfsSymb
    global dsFile errs fopgroup gnum

    set plotfop 0
    if { [llength $args] > 0 } {
	set plotfop 1
    } elseif { ! $dsmode } { return {} }

    # save any previous DS file plot configuration
    if { [info exists spfile] && [string length $spfile] > 0 } {
	set setplot($spfile) [array get spcfg]
    }

    if { ! $plotfop } {
	set sel [selectionOwned 1 1 1]
	if { [llength $sel] < 1 } {
	    Dialog_Warn \
		"Selection is not in one of the file selector listboxes.\n"
	    return {}
	}
	set fil [lindex $sel 0]
	set spsn 0
	set spfile [file tail $fil]
    } else {
	# current group should be fop
	# get the _g$gnum.p file from lbxs(0) this group groupfiles
	set spfile [getFOPpset]
	if { [string length [string trim $spfile]] < 1 } {
	    Dialog_Warn "Couldn't find fop file .p for plotting."
	    return {}
	}
    }

    # look for the dataset by input src filename in case
    # user is using sellectlist for the filename

    set spsn [checkDSfile $spfile]
    if { $spsn < 1 } {
	Dialog_Warn "file $spfile is not in a dataset and read failed.\n$errs\n"
	return
    }
    pipeWrite "d$spsn"
    
    set rl [lindex $readlines 1]
    if { [scan [string trimleft $rl "* "] {%d %d} iset npt] < 2 || $npt < 1 } {
	Dialog_Warn "file $spfile has no data.\n"
	return
    }


    # store the column stats
    pipeWrite "c s"
    
    arrayClear splabelsarr
    set splbls {}
    for {set i 1} {$i <= $nread} {incr i} {
	set rl [lindex $readlines $i]
	if { [scan $rl {%d %s %g %g %d %d} icol lbl mn mx nch var] < 6 } {
	    continue
	}
	set splabelsarr($lbl) [list $var $icol]
	lappend splbls [list $lbl $var $icol]
    }
    set splabelslst [array names splabelsarr]

    # if file previously read, should be able to recover columns from setplot

    # for plotfop can set title from grpTitles($gnum)

    if { $plotfop && [info exists grpTitles($gnum)] } {
	set spcfg(title) $grpTitles($gnum)
    } else {
	set spcfg(title) ""
    }

    set colsOK 0
    if { [info exists setplot($spfile)] } {
	array set spcfg $setplot($spfile)
	# try to determine plot columns from spcfg if file prev read
	set colsOK [getSPcfgCols]
	if { [info exists spcfg(y1-lbl)] && \
		 [string match $spcfg(y1-lbl) Suu] } {
	    configSxy
	    set setplot($spfile) [array get spcfg]
	}
	if { $colsOK } {
	    showDSplot
	    return
	}
    }

    # try assigned cols
    pipeWrite "c al"
    
    if [info exists stndcol] { unset stndcol }
    for {set i 1} {$i <= $nread} {incr i} {
	set rl [lindex $readlines $i]
	set stdlbl [lindex $rl 0]
	set indx [lindex $rl 1]
	set icol [lindex $rl 2]
	set lbl [lindex $rl 3]
	if { $indx > 0 } { append stdlbl $indx }
	set stndcol($stdlbl) [list $lbl $icol]
    }
    set ya [array names stndcol y*]
    set ea [array names stndcol e*]
    set xa [concat En [array names stndcol x*] ttm om]
    # need at least 1 y and 1 x or En or other col-var=1 
    # if En is assigned and varies==1 monotonic  use that for x
    # or use a monotonic x variable x1 ...
    set xok 0
    foreach x $xa {
	if { ! [info exists stndcol($x)] } { continue }
	set lbl [lindex $stndcol($x) 0]
	# check for vary = 1 -> monotonic
	if { [lindex $splabelsarr($lbl) 0] == 1 } {
	    set spcfg(xlbl) $lbl
	    set spcfg(xcol) [lindex $stndcol($x) 1]
	    set xok 1
	    break
	}
    }

    array set ndistinct {}
    if { ! $xok } {
	# look for npt distinct values using c sd colLbl OR colnum
	foreach x $xa {
	    if { ! [info exists stndcol($x)] } { continue }
	    set icol [lindex $stndcol($x) 1]
	    pipeWrite "c sd $icol"
	    
	    if { $nread < 2 } { continue }
	    set nd [llength [lindex $readlines 1]] 
	    set ndistinct($x) $nd
	    if { $nd == $npt } {
		set lbl [lindex $stndcol($x) 0]
		set spcfg(xlbl) $lbl
		set spcfg(xcol) $icol
		set xok 1
		break
	    }
	}
    }
    if { ! $xok } {
	set nd 0
	foreach x $xa {
	    if { ! [info exists stndcol($x)] } { continue }
	    if { ! [info exists ndistinct($x)] } { continue }
	    if { $ndistinct($x) > $nd } {
		set nd $ndistinct($x)
		set xd $x
	    }
	}
	if { $nd > 0 } {
	    set icol [lindex $stndcol($xd) 1]
	    set lbl [lindex $stndcol($xd) 0]
	    set spcfg(xlbl) $lbl
	    set spcfg(xcol) $icol
	    set xok 1
	}
    }


    set ndy [llength $ya]
    set isSxy 0
    foreach y $ya {
	set spcfg(${y}-lbl) [lindex $stndcol($y) 0]
	set spcfg(${y}-col) [lindex $stndcol($y) 1]
	if { [lsearch {Suu Sdd Sdu Sud} $spcfg(${y}-lbl)] >= 0 } { set isSxy 1}
    }
    set ne [llength $ea]
    foreach e $ea {
	set spcfg(${e}-lbl) [lindex $stndcol($e) 0]
	set spcfg(${e}-col) [lindex $stndcol($e) 1]
    }
    if { $ne != $ndy || $ndy == 0 } { set ne 0 }
    # if at this point ndy == 0 
    # first check for Suu Sdd Sdu Sud labels and Exx for errors
    # if there are no Sxx labels find a vary==2,1,0 column

    if { $ndy == 0 } {
	foreach sxx {Suu Sdd Sdu Sud} exx {Euu Edd Edu Eud} {
	    if { [info exists splabelsarr($sxx)] } {
		incr ndy
		set spcfg(y${ndy}-lbl) $sxx
		set spcfg(y${ndy}-col) [lindex $splabelsarr($sxx) 1]
	    }
	    if { [info exists splabelsarr($exx)] } {
		incr ne
		set spcfg(e${ndy}-lbl) $exx
		set spcfg(e${ndy}-col) [lindex $splabelsarr($exx) 1]
	    }
	}
	if { $ndy > 0 } { set isSxy 1 }
    }
    set splbls [lsort -decreasing -integer -index 1 $splbls]
    set nsplbls [llength $splbls]
    if { $ndy == 0 } {
	set ne 0
	set splb [lindex $splbls 0]
	set spcfg(y1-lbl) [lindex $splb 0]
	set spcfg(y1-col) [lindex $splb 2]
	set ndy 1
    }
    if { ! $xok } {
	for {set i [expr $nsplbls - 1]} {$i > 0} {incr i -1} {
	    set splbl [lindex $splbls $i]
	    if { [lindex $lbl 1] > 0 } {
		set spcfg(xlbl) [lindex $splb 0]
		set spcfg(xcol) [lindex $splb 2]
		set xok 1
		break
	    }
	}
    }
    if { ! $xok || $ndy < 1 } {
	Dialog_Warn "couldn't find x-y columns in $spfile.\n"
	return
    }
    set spcfg(ny) $ndy
    set spcfg(ylbl) ""
    $dseditm delete 0 end
    $dseditm add radio -variable dseditY -label all -value all \
	-command "viewElem 1"
    for {set i 1} {$i <= $ndy} {incr i} {
	append spcfg(ylbl) "$spcfg(y${i}-lbl) "
	# put the ylabels as radio into dseditm
	$dseditm add radio -variable dseditY \
	    -label $spcfg(y${i}-lbl) -value $spcfg(y${i}-lbl) \
	    -command "viewElem 1"
	# make sure y#-gsym pixels color exist
	if { ![info exists spcfg(y${i}-color)] } { set spcfg(y${i}-color) blue }
	if { ![info exists spcfg(y${i}-pixels)] } { set spcfg(y${i}-pixels) 6 }
	if { ![info exists spcfg(y${i}-fill)] } {set spcfg(y${i}-fill) defcolor}
	if { ![info exists spcfg(y${i}-sym)] } { set spcfg(y${i}-sym) plus }
	if { ![info exists spcfg(y${i}-gsym)] } { set spcfg(y${i}-gsym) 1 }
    }
    set spcfg(sym) circle-fill
    if { $isSxy } { configSxy }
    set spcfg(color) blue
    set spcfg(pixels) 6
    set spcfg(fill) defcolor
    set spcfg(logscale) 0
    set dseditY all
    set spcfg(errs) 1
    set spcfg(syms) 1
    if { $ne != $ndy } { set spcfg(errs) 0 }

    # save spcfg
    set setplot($spfile) [array get spcfg]
    # ready to showDSplot
    showDSplot
}

proc configSxy {} {
    global spcfg fsSymb gfsSymb
    for {set i 1} {$i <= 4} {incr i} {
	set spcfg(y${i}-sym) $fsSymb($i)
	set spcfg(y${i}-gsym) $gfsSymb($i)
    }
}

proc getSPcfgCols {} {
    global spcfg
    if { ! [info exists spcfg(ny)] } { return 0 }
    for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	if { ! [info exists spcfg(y${i}-col)] } { return 0 }
	if { ! [info exists spcfg(y${i}-lbl)] } { return 0 }
    }
    return 1
}

proc showDSplot {} {
    # called to actually show the DS plot after columns determined
    # or for reconfig plot
	# NO longer support GNUplot

    global spfile spsn spcfg
    global BLT
    global gvds gwds gvtitle
    global gdspipe gnuprog

    if { ! $BLT } { return }
	if { ! [info exists gvds] || ! [winfo exists $gvds] } {
	    # create a new top level and put a graph widget in it
	    set gvds [toplevel .gds]
	    set gwds [graph $gvds.gds]
	    pack $gwds -side top -fill both -expand true
	    wm iconname $gvds
	}
	wm title $gvds "$gvtitle  file: $spfile"
	bltDSGraphSetup
	raise $gvds
	# apply the per group graph options. Wont need any updating for BLT
	# Do NOT call applyGraphOptions 
	#applyGraphOptions 0
	return
}



proc buildPlotColumns {} {
    # build toplevel .pc plotcolumns gui to select plot columns
    # to plot any dataset
    # store by short-filename
    # when this is called a file(spfile) has been selected and is
    # already in a dataset(spsn)
    # the master set labels list is in splabelsarr(lbl) which are
    #  [list vary icol]
    # the previous pcxye gets saved as pcxyelast

    # the plotcolumns(labels) symbols symbol-sizes colors (multiple y)
    # axis-scaling axis-labels plot-title
    # array set spconfig $setplot(short-fil)
    # set setplot(short-fil) [array get spconfig]
    # spconfig array elements for:
    #   ny  xlbl
    #   y1-lbl y1-elbl y1-symbolname y1-symbolsize y1-colora y1-colorb
    #   y1-showsym   y1-showerr ...
    #   xaxlbl yaxlbl xaxautomin xaxautomax xaxmin xaxmax
    #                 yax same-but-incl log
    # load all into spconfig

    # this toplevel will set spconfig(xlbl) spconfig(y1-lbl) spconfig(y1-elbl)
    # etc for multiple y

    # for x-selection use only columns that vary (vary>0) with non-rand pref(1)
    # for y-selection put columns that vary rand (vary==2) at top
    # to do this put collabel vary as list foreach collabel
    # then first sort -dictionary on -index 0 then -integer


    global spconfig spfile spsn
    # the move label procs will use the listbox widgets
    global lbspx lbspy lbspe lbspsrc

    set pc [toplevel .pc]
    wm title $pc "plot columns"
    wm iconname $pc PlotCol
    # mb has: Apply(c1)  X-radio(c2)  Y-radio(c4) E-radio(c5)
    # but I think this will build OK all on one grid (single frame)
    set apply [button $pc.apply -text Apply \
		   -command applySetColumns \
		   -background green -activebackground yellow]

    # xye radio select buttons
    set xr [radiobutton $pc.x -variable pcxye -value 1 -text X -command pcradio]
    set yr [radiobutton $pc.y -variable pcxye -value 2 -text Y -command pcradio]
    set er [radiobutton $pc.e -variable pcxye -value 3 -text Err \
	       -command pcradio]

    # 4 lbxs
    set lbspsrc [listbox $pc.lbs -relief raised -bd 2 \
		  -yscrollcommand "$pc.sscroll set" -selectmode extended]
    set sscroll [scrollbar $pc.sscroll -orient vertical \
		     -command "$pc.lbs yview"]

    set lbspx [listbox $pc.lbx -relief raised -bd 2 \
		    -selectmode single]
    set lbspy [listbox $pc.lby -relief raised -bd 2 \
		  -yscrollcommand "$pc.yscroll set" -selectmode extended]
    set yscroll [scrollbar $pc.yscroll -orient vertical \
		     -command "$pc.lby yview"]
    set lbspe [listbox $pc.lbe -relief raised -bd 2 \
		  -yscrollcommand "$pc.escroll set" -selectmode extended]
    set escroll [scrollbar $pc.escroll -orient vertical \
		     -command "$pc.lbe yview"]

    
    # 4 lbx labels
    set xlbl [label $pc.xlbl -text X-column]
    set ylxl [label $pc.ylbl -text Y-columns]
    set elbl [label $pc.elbl -text Err-columns]
    set slbl [label $pb.slbl -text Src-columns]

    # move col-labels from src to selected lbox and vice versa
    set toOut [button $pc.out -command pctoOut -image rightarrow \
		   -background green -activebackground yellow]
    set toSrc [button $pc.src -command pctoSrc -image leftarrow \
		   -background green -activebackground yellow]


    grid $apply -row 1 -column 1
    grid $xr -row 1 -column 2
    grid $yr -row 1 -column 4
    grid $er -row 1 -column 5

    grid $sscroll -row 4 -rowspan 17 -column 0 -sticky ns
    grid $lbspsrc -row 4 -rowspan 17 -column 1 -columnspan 2 -sticky news
    grid $slbl    -row 3 -column 1 -columnspan 2 -sticky ew

    grid $toOut   -row 8 -column 3 -padx 10
    grid $toSrc   -row 11 -column 3 -padx 10

    grid $lbspx   -row 4 -column 4 -columnspan 2 -sticky news
    grid $xlbl    -row 3 -column 4 -columnspan 2 -sticky ew

    grid $lbspy   -row 7 -rowspan 6 -column 4 -columnspan 2 -sticky news
    grid $ylbl    -row 6 -column 4 -columnspan 2 -sticky ew
    grid $yscroll -row 7 -rowspan 6 -column 6 -sticky ns

    grid $lbspe   -row 15 -rowspan 6 -column 4 -columnspan 2 -sticky news
    grid $elbl    -row 14 -column 4 -columnspan 2 -sticky ew
    grid $escroll -row 15 -rowspan 6 -column 6 -sticky ns
}

proc pctoOut {} {
    global lbspsrc lbspx lbspy lbspe
    global pcxye
    set sel [$lbspsrc curselection]
    if { [llength $sel] < 1 } { return }
    switch -exact -- $pcxye {
	1 {set lbout $lbspx}
	2 {set lbout $lbspy}
	3 {set lbout $lbspe}
    }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbsrc get $ix]
	$lbout insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbsrc delete $ix
    }
}
proc pctoSrc {} {
    global lbspsrc lbspx lbspy lpspe
    global pcxye
    switch -exact -- $pcxye {
	1 {set lbout $lbspx}
	2 {set lbout $lbspy}
	3 {set lbout $lbspe}
    }
    set sel [$lbout curselection]
    if { [llength $sel] < 1 } { return }
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	set lbl [$lbout get $ix]
	$lbspsrc insert end $lbl
    }
    set sel [lsort -decreasing -integer $sel]
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set ix [lindex $sel $i]
	$lbout delete $ix
    }
}

proc pcradio {} {
    # choose and sort from master list according to pcxye
    # the master list splabelsarr is array
    # splabelsarr(collabel) = [list vary icol]
    # so we can find the vary or icol for any label quickly
    # and the master list of all labels is just [array names splabelsin]
    # which is in splabelslst
    # the invariant columns are in splabelsinv

    # the current list in lbspsrc is the truncated list splabels
    # because some labels may have been selected (moved to x y OR e lbx)
    # in order to select/sort the current splabels
    # make a list of lists on elem foreach label appending the vary value.
    # then do -index sorts and truncations
    # the result goes into the lbspsrc listbox

    # for x-selection use only columns that vary (vary>0) with non-rand pref(1)
    # for y-selection put columns that vary rand (vary==2) at top
    # to do this put collabel vary as list foreach collabel
    # then first sort -dictionary on -index 0 then -integer

    global splabelsarr splabelslst
    global lbspsrc pcxye pcxyelast
    # this gets called on radiobutton pcxye change
    # so if pcxyelast > 1 & pcxye > 1 just return
    if { $pcxyelast > 1 && $pcxye > 1 } { return }

    # take the master and remove every label in x y e lbxs
    # if pcxye == 1 also remove every label that has vary = 0
    # do sort on icol then on vary (incr for x) (decr for ye)

    $lbspsrc delete 0 end
    set splabels $splabelslst
    foreach lbl [$lbspx get 0 end] {
	if { [set i [lsearch $splabels $lbl]] < 0 } { continue }
	set splabels [lreplace $splabels $i $i]
    }
    foreach lbl [$lbspy get 0 end] {
	if { [set i [lsearch $splabels $lbl]] < 0 } { continue }
	set splabels [lreplace $splabels $i $i]
    }
    foreach lbl [$lbspe get 0 end] {
	if { [set i [lsearch $splabels $lbl]] < 0 } { continue }
	set splabels [lreplace $splabels $i $i]
    }
    if { $pcxye == 1 } {
	foreach lbl $splabels {
	    if { [lindex $splabelsarr($lbl) 0] == 0 } {
		if { [set i [lsearch $splabels $lbl]] < 0 } { continue }
		set splabels [lreplace $splabels $i $i]
	    }
	}
    }
    if { [llength $splabels] < 1 } { return }
    set nlst {}
    foreach lbl $splabels {
	lappend nlst [list $lbl [lindex $splabelsarr($lbl) 0] \
			  [lindex $splabelsarr($lbl) 1]]
    }
    set nlst [lsort -integer -increasing -index 2 $nlst]
    if { $pcxye == 1 } {
	set sd -increasing
    } else {
	set sd -decreasing
    }
    set nlst [lsort -integer $sd -index 1 $nlst]
    foreach n $nlst { $lbspsrc insert end $n }
}

# showText
proc buildShowText {args} {
    # top frame for group and file type selection
    # bottom is scrolledText
    global tv txtw
    global gnum
    global mp snum sfvm natn natnlabels
    global tvtitle
    global mbbg mbab

    set tv [toplevel .txtvw]
    wm title $tv $tvtitle
    wm iconname $tv "PB txt"
    wm iconify $tv

    set mb [frame $tv.mb]

    set inc $mb.inc
    set dec $mb.dec
    set cur $mb.cur
    set curlbl $mb.curlbl

    set dptf $mb.dptf
    set msrf $mb.msrf
    set binf $mb.binf
    set natf $mb.natf
    set srcf $mb.srcf
    set self $mb.self

    #set sninc $mb.sninc
    #set sndec $mb.sndec
    #set sncur $mb.sncur
    #set sncurlbl $mb.sncurlbl
    set sfv $mb.sfv
    set nfv $mb.nfv

    set fl $mb.file
    menubutton $fl -text File -menu $fl.menu -bg $mbbg -relief raised \
	    -activebackground $mbab
    set fm [menu $fl.menu -tearoff 0]
    $fm add command -label "save columns from current file to new file" \
	-command "columnsFromFileView 0"

    button $inc -command incrGroup -image uparrow \
	-background green -activebackground yellow
    button $dec -command decrGroup -image downarrow \
	-background green -activebackground yellow
    label  $cur -textvariable gnum -width 3 -relief sunken -bd 1 -bg white
    label  $curlbl -text group

    radiobutton $dptf -variable mp -text soln-datapts -value 1
    radiobutton $msrf -variable mp -text soln-msrmnts -value 2
    radiobutton $binf -variable mp -text presoln-bins -value 0
    radiobutton $natf -variable mp -text soln-native -value 5
    radiobutton $srcf -variable mp -text srcfiles -value 3
    radiobutton $self -variable mp -text select -value 4
    trace variable mp w showText

    # replace these with just a menu of the src files with radio
    #button $sninc -command incrSn -image uparrow \
    #	-background green -activebackground yellow
    #button $sndec -command decrSn -image downarrow \
    #	-background green -activebackground yellow
    #label  $sncur -textvariable snum -width 3 -relief sunken -bd 1 -bg white
    #label  $sncurlbl -text dataset

    menubutton $sfv -text srcFile -menu $sfv.menu -bg $mbbg -relief raised \
	    -activebackground $mbab
    set sfvm [menu $sfv.menu -tearoff 0]
    #menubutton $nfv -text native -menu $nfv.menu -bg $mbbg -relief raised \
	#    -activebackground $mbab
    #set nfvm [menu $nfv.menu -tearoff 0]
    #for {set i 1} {$i <= 4} {incr i} {
	#$nfvm add radio -variable natn \
	#    -label $natnlabels($i) -value $i \
	#    -command showText
    #}

	# remove nfv menu

    grid $fl $curlbl $cur $inc $dec \
	$dptf $msrf $binf $natf $srcf $sfv $self
    #$sninc $sndec $sncur $sncurlbl $self


    set tf $tv.tf
    set txtw [eval scrolledTextPbcor $tf $args]
    pack $mb -side top -fill x
    pack $tf -side top -fill both -expand true
}

################# group plot procs ###########################

proc openColorSelector {msr} {
    global colorsel csl colorselmsr
    global mcolorFracs mcolors
    global scolorFracs scolors
    if { ! [winfo exists .cs] } { buildColorSelector }
    if { ! [winfo ismapped .cs] } { wm deiconify .cs }
    raise .cs
    $csl delete 0 end
    set colorselmsr $msr
    if { $msr } {
		wm title .cs "measurement dataset colors"
		set nf [llength $mcolorFracs]
		set nc [llength $mcolors]
		if { $nc > $nf } { set nc $nf }
		if { $nf > $nc } { set nf $nc }
		for {set i 0} {$i < $nc} {incr i} {
	    	set c [lindex $mcolors $i]
	    	$csl insert end "[lindex $mcolorFracs $i] $c"
	    	$csl itemconfigure $i -fg \#$c
		}
    } else {
		wm title .cs "detector colors"
		set nf [llength $scolorFracs]
		set nc [llength $scolors]
		if { $nc > $nf } { set nc $nf }
		if { $nf > $nc } { set nf $nc }
		for {set i 0} {$i < $nc} {incr i} {
	    	set c [lindex $scolors $i]
	    	$csl insert end "[lindex $scolorFracs $i] $c"
	    	$csl itemconfigure $i -fg \#$c
		}
    }
}

proc updateColors {} {
    global csl colorselmsr
    global scolorFracs scolors mcolorFracs mcolors
    set clst [$csl get 0 end]
    if { $colorselmsr } {
		set mcolorFracs {}
		set mcolors {}
    } else {
		set scolorFracs {}
		set scolors {}
    }
    for {set i 0} {$i < [llength $clst]} {incr i} {
		set cl [lindex $clst $i]
		if { [llength $cl] < 2 } { continue }
		set f [lindex $cl 0]
		set c [string trim [lindex $cl 1] \#]
		if { $colorselmsr } {
		    lappend mcolorFracs $f
		    lappend mcolors $c
		} else {
		    lappend scolorFracs $f
		    lappend scolors $c
		}
    }
}

proc setColorFrac {} {
    global csl colorfrac
    set csel [$csl curselection]
    if { [llength $csel] < 1 } {
		Dialog_Warn "No color selected in listbox"
		return
    }
    if { [string length $colorfrac] < 1 || [scan $colorfrac {%g} x] < 1 } {
		Dialog_Warn "No numeric fraction entered"
		return
    }
    set ic [lindex $csel 0]
    set val [$csl get $ic]
    set val [lreplace $val 0 0 $colorfrac]
    set col [$csl itemcget $ic -fg]
    $csl delete $ic
    $csl insert $ic $val
    $csl itemconfigure $ic -fg $col
    updateColors
}
proc editColor {} {
    global csl
    set csel [$csl curselection]
    if { [llength $csel] < 1 } {
		Dialog_Warn "No color selected in listbox"
		return
    }
    set ic [lindex $csel 0]
    set newc [tk_chooseColor]
    if { [string length $newc] < 1 } { return }
    set val [$csl get $ic]
    set val [lreplace $val 1 1 [string trim $newc \#]]
    $csl delete $ic
    $csl insert $ic $val
    $csl itemconfigure $ic -fg $newc
    updateColors
}
proc delCurColor {} {
    global csl
    set csel [$csl curselection]
    if { [llength $csel] < 1 } {
		Dialog_Warn "No color selected in listbox"
		return
    }
    set ic [lindex $csel 0]
    $csl delete $ic
    updateColors
}
proc insCurColor {} {
    global csl
    set csel [$csl curselection]
    if { [llength $csel] < 1 } {
		Dialog_Warn "No color selected in listbox"
		return
    }
    set ic [lindex $csel 0]
    $csl insert $ic [$csl get $ic]
    updateColors
}
proc appCurColor {} {
    global csl
    set csel [$csl curselection]
    if { [llength $csel] < 1 } {
		Dialog_Warn "No color selected in listbox"
		return
    }
    set ic [lindex $csel 0]
    $csl insert end [$csl get $ic]
    updateColors
}

proc buildColorSelector {} {
    global colorsel csl colorfrac
    set colorsel [toplevel .cs]
    set csf [frame .cs.f]
    set csl [listbox .cs.lb -selectmode single]
    grid $csf
    grid $csl
    set edfl [label $csf.edfl -text "edit-frac"]
    set edfe [entry $csf.edfe -textvariable colorfrac -width 6]
    set edc [button $csf.edc -command editColor -text "edit-color" \
		 -background green -activebackground yellow]
    set del [button $csf.del -command delCurColor -text "delete" \
		 -background green -activebackground yellow]
    set ins [button $csf.ins -command insCurColor -text "insert" \
		 -background green -activebackground yellow]
    set app [button $csf.app -command appCurColor -text "append" \
		 -background green -activebackground yellow]
    grid $edfl $edfe $edc
    grid $del  $ins  $app
    bind $edfe <Tab> setColorFrac
    bind $edfe <Return> setColorFrac
}

proc vectorExists {vecName} {
    return [expr [llength [vector names ::$vecName]] \
	    || [llength [vector names $vecName]]]
}

proc Ndistinct {lst} {
    # return number of distinct real values in list
    set ll [llength $lst]
    if { $ll < 1 } { return 0 }
    set slst [lsort -increasing -real $lst]
    set ndist 1 ;
    set a [lindex $slst 0]
    for {set j 1} {$j < $ll} {incr j} {
	set b [lindex $slst $j]
	if { $b > $a } { incr ndist }
	set a $b
    }
    return $ndist
}

proc mostDistinct {args} {
    # args is list of lists of data
    # sort increasing to easily calc number of distinct values
    # return index of list with most distinct values
    # on tie, earliest list wins

    # NB this is a more robust way than using vary=1,2 from
    # column statistics, as there is no guarantee that
    # the input data will be sorted incr or decr
    # Dosnt need any tolerances

    set nl [llength $args]
    if { $nl < 1 } { return 0 }
    if { $nl == 1 } { return 1 }
    set ibest 1
    set mostdist 0
    for {set i 0} {$i < $nl} {incr i} {
		set ndist [Ndistinct [lindex $args $i]]
		if { $ndist > $mostdist } {
	    	set mostdist $ndist
	    	set ibest [expr $i + 1]
		}
    }
    return $ibest
}

proc checkPens {} {
    global g maxgrpsets ngrpsets ny pen spen gnum fopgroup nDetectors
    # create any needed pens for BLT
    # uses group global ny and ngrpsets
    # ngrpsets is assigned in groupSrcFiles used in solveGroup and execCurFOP
    # ny is the total number of detectors via Yselect etc
    # user can solve all detectors or 1
    # we create the pens for all detectors

    set penlst [$g pen names]
    #for {set i $is} {$i <= $ngrpsets} {incr i}

    if { ! [info exists fopgroup($gnum)] || ! $fopgroup($gnum) } {
    	puts "checkPens ngrpsets= $ngrpsets"
		for {set i 1} {$i <= $ngrpsets} {incr i} {
		    for {set j 1} {$j <= 4} {incr j} {
				set pen($j$i) p$j$i
				if { [lsearch $penlst p$j$i] < 0 } { $g pen create p$j$i }
				if { [lsearch $penlst r$j$i] < 0 } { $g pen create r$j$i }
		    }
		}
    }

    if { $ngrpsets > $maxgrpsets } { set maxgrpsets $ngrpsets }  
    # RWE April 2019 no longer need to index pens by detector number for different colors
    # now we are just plot or no plot (by pixels=0) detectors chosen by user then thru sWts  
    # so change ny to 2, where 1 in ON 2 is OFF
    for {set i 1} {$i <= $nDetectors} {incr i} {
		for {set j 1} {$j <= 4} {incr j} {
	    	set spen($j$i) sp$j$i
	    	if { [lsearch $penlst sp$j$i] < 0 } { $g pen create sp$j$i }
		}
    }
}
 proc constList {npt valu} {
    # return a list with npt elements all valu
    set rl {}
    for {set i 0} {$i < $npt} {incr i} { lappend rl $valu }
    return $rl
}

proc groupXcolSet {ix} {
    global nread readlines readfinished
    global gnum pset mset stndX iX Xscan gws xaxlbl Xlabel
    # get the group .p .m x datalists for
    # ix=1-6 stnd names  En x1 x2 x3 tts om
    # and reset the graph standard x-vectors
    # ::xpts$gnum ::xmsr$gnum
    # this doesn't seem to reset the graph so just set iX and resolve
    # to get the new plot
    
    # iX is item number>0 in xaxlabl xaxlbl(iX) or for stnd col nmae in stndX
	puts "groupXcolSet ix= $ix\n"
    set iX $ix
    if { $iX < 1 || $iX > 8 } { return }
    set Xlabel $xaxlbl($iX)
    # set this for currentGroup
    pipeWrite "g lx $stndX($iX)"
    pipeWrite "g lxl $xaxlbl($iX)" 
}

proc groupXcolSetU {ix} {
    global nread readlines readfinished
    global gnum pset mset stndX iXU XscanU gws xaxlbl
    # This is not used as user X column applies to all groups
    # This is the one called fro saveColumns user selected
    # get the group .p .m x datalists for
    # ix=1-6 stnd names  En x1 x2 x3 tts om
    # and reset the graph standard x-vectors
    # ::xpts$gnum ::xmsr$gnum
    # this doesn't seem to reset the graph so just set iX and resolve
    # to get the new plot
    
    # iX is item number>0 in xaxlabl xaxlbl(iX) or for stnd col nmae in stndX
	puts "groupXcolSet ix= $ix\n"
    set iXU $ix
    # set this for currentGroup
    pipeWrite "g lx $stndX($iXU)"
}

proc getGrpSolInds {gn} {
	# return list of all solution indices for group gn
	set pflst [glob -nocomplain *_g$gn*.p]
	set ilst {}
	foreach pf $pflst {
		if { [regexp {.*_g[0-9]+\.([0-9]+)} $pf wm vers] } {
			lappend ilst $vers
		} else {
			lappend ilst -1
		}
	}
	return [lsort -integer -increasing $ilst]
}

proc latestFile {fl} {
	# return the latest filename from the filename list fl
	#puts "latestFile filelist= $fl\n"
	set latesttime 0
	set latestfile ""
	foreach f $fl {
		set mtime [file mtime $f]
		#puts "latestFile $f  mtime= $mtime\n"
		if { $mtime > $latesttime } {
			set latesttime $mtime
			set latestfile $f
			#puts "latestFile set latestfile= $latestfile"
		}
	}
	return $latestfile
}

proc getGrpSolByInd {gn i} {
	# return filename of solution_g$gn.i.p file for i > 0
	# else if i == 0 return the most recent in time solution
	# else if i < 0 return the non-indexed soln solution_g$gn.p
		
	set pflst [glob -nocomplain *_g$gn*.p]
	if { $i == 0 } {
		# return most recent solution file indexed or not
		return [latestFile $pflst]
	}
	if { $i < 0 } {
		# return the non-indexed solution. There can only be one such file
		set pflst [glob -nocomplain *_g$gn.p]
		if { [llength $pflst] < 1 } { return "" }
		return [lindex $pflst 0]
	}
	# finally handle request for version i > 0
	foreach pf $pflst {
		if { [regexp {.*_g[0-9]+\.([0-9]+)} $pf wm vers] } {
			if { $i == $vers } {
				return $pf
			}
		}
	}
	# may fail
	return ""
}

proc  recoverPlotBG {gn fl} {
	global solnindx
	# recover a background job solution plot
	# for group gn and soln file root fl
	# NB the fl may not be the most recent solution if user does a foreground solution
	# that somehow finishes earlier.
	# In any case this will plot the background solution in gn plot window like recoverPlot
	# Should get wm to put the solution filename in the window border along with the group number
	gotoGroup $gn
	# extract the solution index from fl
	if { ![regexp {_g[0-9]+\.([0-9]+)} $fl wm vers] } {
		set solnindx($gn) -1
		# must be unversioned solution file
	} else {
		set solnindx($gn) $vers
	}
	return [recoverPlot]
}

# solveGroup auto plots result and I used this to recover session.
# Since MACS started using very large datasets that take minutes to solve
# I want to just plot on recover instead of redoing the time intensive solution.
# make a new recoverPlot proc and a global flag to choose either solveGroup or 
# alternatively read in the gnum.p and gnum.m sets followed by groupPlotSetup savePlot showPlot

proc recoverPlot {} {
	global gnum nerr nread readlines
	global mset pset iX Xlabel xaxlbl ixlst
	global solnindx solnfile
	global recoverPlotBySolve
	global bgRecovered
	
	# this gets the plot for current group as used by sessionRecover
	# As of Mar 2019 stop using solveGroup to recover the plot
	# NB solveGroup does the setup for iX and Xlabel
	# so here try to get Xstnd and iX rom the pset
	
	if {$recoverPlotBySolve} {
		solveGroup 0
		return 1
	}
	# with long solve times it is a good option to use the existing solution files to
	# recover a plot instead of the time consuming solve.
	# find the most recent .p and .m files for this group and read them to datasets
	# and set pset(gnum) mset(gnum)
	# recovery of old solutions is allowed using non-zero solnindx($gnum) but this is
	# going to be handled in another proc since it must use a different set of vectors and plot windows
	# here we still plots to the group plot window.
	# this can handle background job plotting to the group window by setting the solnindx of the background job
	
	# NB this may end up putting the same solution file into multiple datasets.
	# This should not be an issue as it is for actual data files which must be unique
	# in the dataset list since we typically recover the dataset by filename in that case.

	if { ![info exists solnindx($gnum)] } { set solnindx($gnum) 0 }
	set pfl [getGrpSolByInd $gnum $solnindx($gnum)]
	#puts "recoverPlot gnum=$gnum pfl= $pfl\n"
	if {[string length $pfl] < 2 || ! [file exists $pfl]} {return 0}
	set solnfile [regsub {\.p$} $pfl ""]
	set mfl [regsub {\.p$} $pfl .m]
	if {[string length $mfl] < 2 || ! [file exists $mfl]} {return 0}
	# put p file in a dataset
	pipeWrite "f+ $pfl"
	if {$nerr > 0 || $nread < 2} {return 0}
	set rl [lindex $readlines 1]
	if { [scan [string trimleft $rl "* "] {%d %d} ipset npts] < 2 || $npts < 1 } {
		#Dialog_Warn "file $pfl has no data.\n"
		return 0
	}

	# also try to get Xstnd
	puts "recoverPlot trying to find Xstnd from pset= $ipset pfl= $pfl\n"
	set iX 0
	set Xlabel ""
	pipeWrite "d>$pfl"
    pipeWrite "d hl Xstnd"
    if { $nread > 1 } {
	    set xstnd [lindex [lindex $readlines 1] 1]
    	set iXnum [lsearch $ixlst $xstnd]
    	if { $iXnum >= 0 } {
    		set iX [expr $iXnum + 1]
    		set Xlabel $xaxlbl($iX)
    	}
    }
    puts "recoverPlot found iX= $iX\n"
	
	# put m file in set
	pipeWrite "f+ $mfl"
	if {$nerr > 0 || $nread < 2} {return 0}
	set rl [lindex $readlines 1]
	if { [scan [string trimleft $rl "* "] {%d %d} imset nmsr] < 2 || $nmsr < 1 } {
		#Dialog_Warn "file $mfl has no data.\n"
		return 0
	}
	
	set pset($gnum) $ipset
	set mset($gnum) $imset
	#puts "recoverPlot about to call groupPlotSetup\n"
	groupPlotSetup
	savePlot
	showPlot
	# in case this was run as background (no way to know if it finsished while pbcor not running)
	# mark it as recovered
	set bgRecovered($solnfile) 1
	return 1
}

proc recoverPrevPlot {pfl} {
	global nerr nread readlines
	global msetprev psetprev
	global npts nmsr
	if {![file exists $pfl]} {return 0}
	set mfl [regsub {\.p$} $pfl .m]
	if {![file exists $mfl]} {return} 0
	pipeWrite "f+ $pfl"
	if {$nerr > 0 || $nread < 2} {return 0}
	set rl [lindex $readlines 1]
	if { [scan [string trimleft $rl "* "] {%d %d} ipset npts] < 2 || $npts < 1 } {
		#Dialog_Warn "file $pfl has no data.\n"
		return 0
	}

	# put m file in set
	pipeWrite "f+ $mfl"
	if {$nerr > 0 || $nread < 2} {return 0}
	set rl [lindex $readlines 1]
	if { [scan [string trimleft $rl "* "] {%d %d} imset nmsr] < 2 || $nmsr < 1 } {
		#Dialog_Warn "file $mfl has no data.\n"
		return 0
	}

	set psetprev $ipset
	set msetprev $imset
	recoverPrevPlotSetup $pfl
	recoverPlotShow $pfl
	return 1
}


proc groupPlotSetup {} {
    # call this after a group is calc and PLT enabled vie recoverFG or from recoverPlot via sessionRecovery
    # this will init plot data and vectors
    # showPlot to display the results for cur grp is a separate call
    # RWE Jan 2011 changed to handle fop groups (no msr or R) as well


    global g gnum
    global mset pset
    global nmsr npts
    global ngrpsets maxgrpsets ny nDetectors fopgroup
    # following are lists of data list names
    global pDatNames mDatNames
    global pStdNames mStdNames
    global Xpts Xmsr
    global Yi
    global Src Rf iSrc stylst sstylst rstylst iX Xlabel
    global iWts suuWts sddWts sduWts sudWts
    global isrcplot
    global readlines readfinished nread
    global pen pencolor spen spencolor
    global mcolorFracs mcolors
    global scolorFracs scolors
    global fsSymb gfsSymb
    global titlefont tickfont
    global BLT GNU
    global pcolumns nsetfs
    global textf
    global gnufil gnustyl gnucol
    global pltDet
    global ynames
    global useXscan Xscan xaxlbl Xlabel

    # the corresponding vectors have same name l.c.
    # mset and pset set in groupSrcFiles when write gnum
    # should try to get iX set before calling this

    # call after solveGroup
    # get all the vector data lists
    # decide what to vary in the plot
    # already have ndatasets and srcfiles listings, pset mset
    # from groupSrcFiles

    # would like to set it up as for groupOptions
    # where there is a list of commands that gets run
    # when group changes
    # need vector names:

    # get vector data using
    # c al stndCol(x# y# En -> ctyp indx dscol label
    # c dl label -> lbl data ...
    # from .d need y1-4 = Suu Sdd Sdu Sud e1-4 x1-3 = Qxyz En
    # from .m need y1 = Cc e1 = Ccerr x1-3 = Qxyz En fs srcfile
    # symbols uu=+ du=arrow ud=triangle dd=circle
    # colors S-blue sets red->green ff0000->00ff00
    # use element -hide
    # ? can I use .m as a single element
    # could turn off a pen by setting symbol blank
    # so could have a pen for each set but then
    # how do change symbols for spin state
    # would need 4 pens per set
    # p1-4 weigths 1-4 p5-8 weights 5-8 etc
    # so weight for each msr = 4(setn-1)+fs
    # pens p{fs}{sn}
    # the hack to turn off the view for a set is to
    # set the color for its pens to the plotbackground color

    # also use pens to color code the Sxy elements by detector index yi
    # need ny pens for each of the 4 Sxy = 4*ny pens
    # but only ny different wt values per element

    if { $ngrpsets < 1 } {
		Dialog_Warn "No group sets"
		return
    }
    set isfop 0
    if { [info exists fopgroup($gnum)] } {set isfop $fopgroup($gnum)}
    if { (! $isfop && $mset($gnum) < 1) || $pset($gnum) < 1 } {
		Dialog_Warn "NO msr/pts datasets for plotting current group"
		return
    }

    # nothing in this proc should depend on BLT graph existing yet
    # if { $BLT } { checkPens }

    arrayClear pcolumns

    # get the pset(.p)  data lists
    pipeWrite "d$pset($gnum)"
    
    #  NO iX is alrady handled in solveGroup which ends up calling this
    # so do the pset Xstnd in the recoverPlot function while at the pset get any saved Xstnd
    #set iXs 0
    #pipeWrite "d hl Xstnd"
    #if { $nread > 1 } {
	#    set xstnd [lindex [lindex $readlines 1] 1]
    #	set iXnum [lsearch $ixlst $xstnd]
    #	if { $iXnum >= 0 } { set iXs [expr $iXnum + 1] }
    #}

    
    foreach pdat $pDatNames pstd $pStdNames {
		global $pdat
		pipeWrite "c al $pstd"
	
	# ctyp indx dscol label
		if { $nread < 2 || [scan [lindex $readlines 1] {%s %d %d} \
				ct inx dsc] < 3 } {
	    	# set data to zeros and undefined column number=0
	    	set $pdat [constList $npts 0]
	    	set pcolumns($pdat) 0
	    	continue
	    	Dialog_Warn "failed to get $pdat data"
	    	#return
		}
		pipeWrite "c dl $dsc"
	
		# get rid of the leading label
		set $pdat [lreplace [lindex $readlines 0] 0 0]
		set npr [eval llength $$pdat]
		if { $npr < $npts } {
	    	Dialog_Warn "not enough pts read=$npr for grp $gnum npts=$npts  at p $pdat data"
	    	return
		}
		# save the column numbers for GNU
		set pcolumns($pdat) $dsc
    }
    # also get the detector index from the .p set yi column 1:ny=NdetSolved
    # or single detector value in range 1:ny
    # RWE Apr 2019 this changes yi to di detector indx column which is PTAI in MACS 3-axis files
    #pipeWrite "c dl di"
    # get Yi directly from global pDatNames DI
    
    set Yi $DI
    if { [llength $Yi] < $npts } {
		Dialog_Warn "not enough pts read for p data Yi"
		return
    }

    if { ! $isfop } {
	# get the mset(.m)  data lists
		pipeWrite "d$mset($gnum)"
	
		foreach mdat $mDatNames mstd $mStdNames {
	    	global $mdat
	    	pipeWrite "c al $mstd"
	    
	    	if { $nread < 2 || [scan [lindex $readlines 1] {%s %d %d} \
				    ct inx dsc] < 3 } {
				# set data to zeros
				set $mdat [constList $nmsr 0]
				continue
			#Dialog_Warn "failed to get p data"
			#return
	    	}
	    	pipeWrite "c dl $dsc"
	    
	    	# get rid of the leading label
	    	set $mdat [lreplace [lindex $readlines 0] 0 0]
	    	if { [eval llength $$mdat] < $nmsr } {
				Dialog_Warn "not enough msr read for m data"
				return
	    	}
		}
		# also get the src column from mset
		pipeWrite "c dl src"
	
		set Src [lreplace [lindex $readlines 0] 0 0]
		if { [llength $Src] < $nmsr } {
	    	Dialog_Warn "not enough msrs read for m data"
	    	return
		}
		# also get the R flipping ratio column from mset
		pipeWrite "c dl R"
	
		set Rf [lreplace [lindex $readlines 0] 0 0]
		if { [llength $Rf] < $nmsr } {
	    	Dialog_Warn "not enough msrs read for m data"
	    	return
		}
	
		# get the snum indices for each srcfile in .m, and set the weights
		set iSrc {}
		set iWts {}

		arrayClear nsetfs
		for {set i 1} {$i <= $ngrpsets } {incr i} {
	    	for {set j 1} {$j <= 4} {incr j} {
				set nsetfs($i,$j) 0
	    	}
		}
		for {set i 0} {$i < $nmsr} {incr i} {
	    	set fil [file tail [lindex $Src $i]]
	    	set is [snFromFile $fil]
	    	if { $is < 1 || $is > $ngrpsets } {
				Dialog_Warn "failed to index src files"
				return
	    	}
	    	lappend iSrc $is
	    	# get msr pt fs
	    	set ifs [lindex $Fs $i]
	    	lappend iWts [expr 4*($is - 1) + $ifs]
	    	incr nsetfs($is,$ifs)
		}
    }

    # need number of detectors solved g->ny
    # ny for group should have been set in solveGroup
    # by reading it back with g ly. Repeat here for FOP group
    # remember there are only two choices for ny
    # 1 or all detectors

    pipeWrite "g ly"
    
    if { [scan [lindex $readlines 0] {%s %d %s %d} dum idum nyl ny] < 4 } {
		set ny 1
    }
 
    if { $nDetectors < 1 } { set nDetectors 1 }

    set suuWts {}
    set sddWts {}
    set sduWts {}
    set sudWts {}
	
    for {set i 0} {$i < $npts} {incr i} {
		set is [lindex $Yi $i]
		if { $is < 1 || ($ny > 1 && $is > $ny) } {
	    	Dialog_Warn "detector index $is out of range"
	    	return
		}
		if { $nDetectors == 1 } { set is 1 }
		lappend suuWts [expr 4*($is - 1) + 1]
		lappend sddWts [expr 4*($is - 1) + 2]
		lappend sduWts [expr 4*($is - 1) + 3]
		lappend sudWts [expr 4*($is - 1) + 4]
    }

    # software configure the pens and make the styles list
    # NB initially all sets are viewed (pens have color)
    # so should init the iSrc list

    if { ! $isfop } {
		set dcolor 1
		set stylst {}
		set rstylst {}
		if { $ngrpsets > 1 } { set dcolor [expr 1./($ngrpsets-1)] }
		for {set i 1} {$i <= $ngrpsets} {incr i} {
	   		set pcolor [hexColorList $mcolorFracs $mcolors [expr ($i-1)*$dcolor]]
	    	for {set j 1} {$j <= 4} {incr j} {
				set pen($j$i) p$j$i
				set pencolor($j$i) $pcolor
				set pw [expr 4*($i-1)+$j]
				lappend stylst [list p$j$i $pw $pw]
				lappend rstylst [list r$j$i $pw $pw]
	    	}
	    	# record the view state in isrcplot array already done
	    	set isrcplot($i) 1
		}
    }

    # determine the bestX from datapt lists  En Qx Qy Qz TTS OM TEMP
    # first check if current iX is all distinct
    # we need the group ny from above to do this check

    # RWE Dec 2010 allow color coding of Sxy soln pts for multiple detector

    set sdcolor 1
    set sstylst {}
    set sdoff 0.2
    if { $ny > 1 } {
		set sdcolor [expr 1./($ny-1)]
		set sdoff 0.
    }

    for {set i 1} {$i <= $nDetectors} {incr i} {
		set spcolor \
	    [hexColorList $scolorFracs $scolors [expr $sdoff + ($i-1)*$sdcolor]]
		for {set j 1} {$j <= 4} {incr j} {
	    	set spen($j$i) sp$j$i
	    	set spencolor($j$i) $spcolor
	    	set pw [expr 4*($i-1)+$j]
	    	lappend sstylst [list sp$j$i $pw $pw]
		}
    }

    # Now what to use for X on the plot
    # solution should have set Xscan (if not Xscan = 0)
    # or user may have used Xselect to unset useXscan

    # if useXscan = 0 and iX > 0 use iX
    # if useXscan = 0 and iX = 0 use mostDistinct
    # if useXscan > 0 set iX = Xscan
    
    # This is now the final choice for X plot variable, and only happens after solution
    # This fixes the problem that choice if scan variable failed in setup by looking at the
    # solution data in .p .m files
    # Really need to make this choice before solution so background has access
    # so we need some type of mostDistinct search in solveGroup
    
    # useXscan is true unless user picks a valid X in selectColumn

    #if { $useXscan } { set iX $Xscan }
    # iX should have been determined already. If it is invalid use mostDistinct
    # which is not supposed to fail
    if { $iX < 1 } {
    	set iX [mostDistinct $En $Qx $Qy $Qz $TTS $OM $TEMP $BFLD]
    	set Xlabel $xaxlbl($iX)
    }

    if { $iX == 1 } {
		set pcolumns(X) $pcolumns(En)
		set Xpts $En
		if { ! $isfop } { set Xmsr $Enm }
    } elseif { $iX == 2 } {
		set pcolumns(X) $pcolumns(Qx)
		set Xpts $Qx
		if { ! $isfop } { set Xmsr $Qxm }
    } elseif { $iX == 3 } {
		set pcolumns(X) $pcolumns(Qy)
		set Xpts $Qy
		if { ! $isfop } { set Xmsr $Qym }
    } elseif { $iX == 4 } {
		set pcolumns(X) $pcolumns(Qz)
		set Xpts $Qz
		if { ! $isfop } { set Xmsr $Qzm }
    } elseif { $iX == 5 } {
		set pcolumns(X) $pcolumns(TTS)
		set Xpts $TTS
		if { ! $isfop } { set Xmsr $TTSm }
    } elseif { $iX == 6 } {
		set pcolumns(X) $pcolumns(OM)
		set Xpts $OM
		if { ! $isfop } { set Xmsr $OMm }
    } elseif { $iX == 7 } {
		set pcolumns(X) $pcolumns(TEMP)
		set Xpts $TEMP
		if { ! $isfop } { set Xmsr $TEMPm }
    } elseif { $iX == 8 } {
		set pcolumns(X) $pcolumns(BFLD)
		set Xpts $BFLD
		if { ! $isfop } { set Xmsr $BFLDm }
    }
}

proc recoverPrevPlotSetup {pfl} {
	# this is recoverPlot for previous solutions which means versioning is in effect (otherwise previous overwritten)
    # call this after a group is calc and PLT enabled
    # this will init plot data and vectors
    # recoverPlotShow to display the results for this


    global g gnum
    global msetprev psetprev
    global nmsr npts
    global ngrpsets maxgrpsets ny nDetectors
    # use global nDetectors instead of ny
    global solnfiles
    # following are lists of data list names
    global pDatNames mDatNames
    global pStdNames mStdNames
    global Xpts Xmsr
    global Yi
    global Src Rf iSrc stylst sstylst rstylst iX
    global iWts suuWts sddWts sduWts sudWts
    global isrcplot
    global readlines readfinished nread
    global pen pencolor spen spencolor
    global mcolorFracs mcolors
    global scolorFracs scolors
    global fsSymb gfsSymb
    global titlefont tickfont
    global BLT GNU
    global pcolumns nsetfs
    global textf
    global pltDet
    global ynames
    global useXscan Xscan useXscanU XscanU ixlst xaxlbl Xlabel

    # the corresponding vectors have same name l.c.
    # mset and pset set in groupSrcFiles when write gnum

    # call after solveGroup
    # get all the vector data lists
    # decide what to vary in the plot
    # already have ndatasets and srcfiles listings, pset mset
    # from groupSrcFiles

    # would like to set it up as for groupOptions
    # where there is a list of commands that gets run
    # when group changes
    # need vector names:

    # get vector data using
    # c al stndCol(x# y# En -> ctyp indx dscol label
    # c dl label -> lbl data ...
    # from .d need y1-4 = Suu Sdd Sdu Sud e1-4 x1-3 = Qxyz En
    # from .m need y1 = Cc e1 = Ccerr x1-3 = Qxyz En fs srcfile
    # symbols uu=+ du=arrow ud=triangle dd=circle
    # colors S-blue sets red->green ff0000->00ff00
    # use element -hide
    # ? can I use .m as a single element
    # could turn off a pen by setting symbol blank
    # so could have a pen for each set but then
    # how do change symbols for spin state
    # would need 4 pens per set
    # p1-4 weigths 1-4 p5-8 weights 5-8 etc
    # so weight for each msr = 4(setn-1)+fs
    # pens p{fs}{sn}
    # the hack to turn off the view for a set is to
    # set the color for its pens to the plotbackground color

    # also use pens to color code the Sxy elements by detector index yi
    # need ny pens for each of the 4 Sxy = 4*ny pens
    # but only ny different wt values per element



    # nothing in this proc should depend on BLT graph existing yet
    # if { $BLT } { checkPens }
    
    # which group
    
    puts "recoverPrevPlotSetup pfl= $pfl"
    
    if { ![regexp {_g([0-9]+)} $pfl wm gn] } {
    	Dialog_Warn "ERROR: recoverPrevPlotSetup failed to get group from filename $pfl"
    	return
    }
    set solnfiles [getFilesInGroup $gn]
    set ngrpsets [llength $solnfiles]
    puts "recoverPrevPlotSetup call gotoGroup $gn will replot for that group."
    gotoGroup $gn
    set isfop 0
	# getScanVar may fail then Xscan < 1 useXscan = 0   
    getScanVar

    arrayClear pcolumns

    # get the pset(.p)  data lists
    pipeWrite "d$psetprev"
    set npts [lindex [lindex $readlines 1] 2]
    # as usual if we can find valid x in file it can be used unless useXscan has been set from the Xcolumn interface
    # get back the Xstnd line from header to read the stnd x column. d hl Xstnd finds Xstnd. Returns Xstnd value
    set iXs 0
    pipeWrite "d hl Xstnd"
    if { $nread > 1 } {
	    set xstnd [lindex [lindex $readlines 1] 1]
    	set iXnum [lsearch $ixlst $xstnd]
    	if { $iXnum >= 0 } { set iXs [expr $iXnum + 1] }
    }
    
    foreach pdat $pDatNames pstd $pStdNames {
		global $pdat
		pipeWrite "c al $pstd"
	
	# ctyp indx dscol label
		if { $nread < 2 || [scan [lindex $readlines 1] {%s %d %d} \
				ct inx dsc] < 3 } {
	    	# set data to zeros and undefined column number=0
	    	set $pdat [constList $npts 0]
	    	set pcolumns($pdat) 0
	    	continue
	    	#Dialog_Warn "failed to get p data"
	    	#return
		}
		pipeWrite "c dl $dsc"
	
		# get rid of the leading label
		set $pdat [lreplace [lindex $readlines 0] 0 0]
		set npr [eval llength $$pdat]
		if { $npr < $npts } {
	    	Dialog_Warn "not enough pts read=$npr for grp $gnum npts=$npts  at p $pdat data"
	    	return
		}
		# save the column numbers
		set pcolumns($pdat) $dsc
    }
    # also get the detector index from the .p set yi column 1:ny=NdetSolved
    # or single detector value in range 1:ny
    # NB even for MACS the yi column gets listed as 1 for number of detectors
    # get the detector index from di = PTAI for MACS
    #pipeWrite "c dl di"
    
    set Yi $DI
    if { [llength $Yi] < $npts } {
		Dialog_Warn "not enough pts read for p data Yi"
		return
    }


	# get the mset(.m)  data lists
	pipeWrite "d$msetprev"
	set nmsr [lindex [lindex $readlines 1] 2]
	
	foreach mdat $mDatNames mstd $mStdNames {
	    	global $mdat
	    	pipeWrite "c al $mstd"
	    
	    	if { $nread < 2 || [scan [lindex $readlines 1] {%s %d %d} \
				    ct inx dsc] < 3 } {
				# set data to zeros
				set $mdat [constList $nmsr 0]
				continue
			#Dialog_Warn "failed to get p data"
			#return
	    	}
	    	pipeWrite "c dl $dsc"
	    
	    	# get rid of the leading label
	    	set $mdat [lreplace [lindex $readlines 0] 0 0]
	    	if { [eval llength $$mdat] < $nmsr } {
				Dialog_Warn "not enough msr read for m data"
				return
	    	}
	}
	# also get the src column from mset
	pipeWrite "c dl src"
	
	set Src [lreplace [lindex $readlines 0] 0 0]
	if { [llength $Src] < $nmsr } {
	    Dialog_Warn "not enough msrs read for m data"
	    return
	}
	# also get the R flipping ratio column from mset
	pipeWrite "c dl R"
	
	set Rf [lreplace [lindex $readlines 0] 0 0]
	if { [llength $Rf] < $nmsr } {
	    Dialog_Warn "not enough msrs read for m data"
	    return
	}
	
	# get the snum indices for each srcfile in .m, and set the weights
	set iSrc {}
	set iWts {}

	arrayClear nsetfs
	for {set i 1} {$i <= $ngrpsets } {incr i} {
	    	for {set j 1} {$j <= 4} {incr j} {
				set nsetfs($i,$j) 0
	    	}
	}
	
		
	for {set i 0} {$i < $nmsr} {incr i} {
	    set fil [file tail [lindex $Src $i]]
	    set is [snFromFile $fil]
	    if { $is < 1 || $is > $ngrpsets } {
			Dialog_Warn "failed to index src files"
			return
	    }
	    lappend iSrc $is
	    # get msr pt fs
	    set ifs [lindex $Fs $i]
	    lappend iWts [expr 4*($is - 1) + $ifs]
	    incr nsetfs($is,$ifs)
	}


    # need number of detectors solved g->ny
    # ny for group should have been set in solveGroup
    # by reading it back with g ly. Repeat here for FOP group
    # remember there are only two choices for ny
    # 1 or all detectors


 
	set ny 1

    set suuWts {}
    set sddWts {}
    set sduWts {}
    set sudWts {}
	
    for {set i 0} {$i < $npts} {incr i} {
		set is [lindex $Yi $i]
		if { $is < 1 || ($nDetectors > 1 && $is > $nDetectors) } {
	    	Dialog_Warn "detector index $is out of range"
	    	return
		}
		if { $nDetectors == 1 } { set is 1 }
		lappend suuWts [expr 4*($is - 1) + 1]
		lappend sddWts [expr 4*($is - 1) + 2]
		lappend sduWts [expr 4*($is - 1) + 3]
		lappend sudWts [expr 4*($is - 1) + 4]
    }

    # software configure the pens and make the styles list
    # NB initially all sets are viewed (pens have color)
    # so should init the iSrc list

    
	set dcolor 1
	set stylst {}
	set rstylst {}
	if { $ngrpsets > 1 } { set dcolor [expr 1./($ngrpsets-1)] }
	for {set i 1} {$i <= $ngrpsets} {incr i} {
	   		set pcolor [hexColorList $mcolorFracs $mcolors [expr ($i-1)*$dcolor]]
	    	for {set j 1} {$j <= 4} {incr j} {
				set pen($j$i) p$j$i
				set pencolor($j$i) $pcolor
				set pw [expr 4*($i-1)+$j]
				lappend stylst [list p$j$i $pw $pw]
				lappend rstylst [list r$j$i $pw $pw]
	    	}
	    	# record the view state in isrcplot array already done
	    	set isrcplot($i) 1
	}

    # determine the bestX from datapt lists  En Qx Qy Qz TTS OM TEMP
    # first check if current iX is all distinct
    # we need the group ny from above to do this check

    # RWE Dec 2010 allow color coding of Sxy soln pts for multiple detector

    set sdcolor 1
    set sstylst {}
    set sdoff 0.2
    if { $ny > 1 } {
		set sdcolor [expr 1./($ny-1)]
		set sdoff 0.
    }

    for {set i 1} {$i <= $nDetectors} {incr i} {
		set spcolor \
	    [hexColorList $scolorFracs $scolors [expr $sdoff + ($i-1)*$sdcolor]]
		for {set j 1} {$j <= 4} {incr j} {
	    	set spen($j$i) sp$j$i
	    	set spencolor($j$i) $spcolor
	    	set pw [expr 4*($i-1)+$j]
	    	lappend sstylst [list sp$j$i $pw $pw]
		}
    }

    # Now what to use for X on the plot
    # solution should have set Xscan (if not Xscan = 0)
    # or user may have used Xselect to unset useXscan

    # if useXscan = 0 and iX > 0 use iX
    # if useXscan = 0 and iX = 0 use mostDistinct
    # if useXscan > 0 set iX = Xscan
    
    # This is now the final choice for X plot variable, and only happens after solution
    # This fixes the problem that choice if scan variable failed in setup by looking at the
    # solution data in .p .m files
    # Really need to make this choice before solution so background has access
    # so we need some type of mostDistinct search in solveGroup
    
    # for recovering from a solution file the same algorithm as in solveGroup is used
    # if user has selected X from saveColumns then that choice overrides anything set in the
    # soln file
    set iX 0
    if { $useXscanU && $XscanU > 0 } {
    	set iX $XscanU
    } elseif {$iXs > 0} {
    	# try the one from solnfile
    	set iX $iXs
    } elseif { $useXscan && $Xscan > 0 } {
    	# use getScanVar value
    	set iX $Xscan
    }
    if { $iX < 1 } {
    	# if none of these work use mostDistinct of the standard X datalists. This is supposed to NOT fail
    	set iX [mostDistinct $En $Qx $Qy $Qz $TTS $OM $TEMP $BFLD]
    }
    # set the xaxis label as stnd label
    set Xlabel $xaxlbl($iX)

	# set the Xpts and Xmsr data lists
    if { $iX == 1 } {
		set pcolumns(X) $pcolumns(En)
		set Xpts $En
		if { ! $isfop } { set Xmsr $Enm }
    } elseif { $iX == 2 } {
		set pcolumns(X) $pcolumns(Qx)
		set Xpts $Qx
		if { ! $isfop } { set Xmsr $Qxm }
    } elseif { $iX == 3 } {
		set pcolumns(X) $pcolumns(Qy)
		set Xpts $Qy
		if { ! $isfop } { set Xmsr $Qym }
    } elseif { $iX == 4 } {
		set pcolumns(X) $pcolumns(Qz)
		set Xpts $Qz
		if { ! $isfop } { set Xmsr $Qzm }
    } elseif { $iX == 5 } {
		set pcolumns(X) $pcolumns(TTS)
		set Xpts $TTS
		if { ! $isfop } { set Xmsr $TTSm }
    } elseif { $iX == 6 } {
		set pcolumns(X) $pcolumns(OM)
		set Xpts $OM
		if { ! $isfop } { set Xmsr $OMm }
    } elseif { $iX == 7 } {
		set pcolumns(X) $pcolumns(TEMP)
		set Xpts $TEMP
		if { ! $isfop } { set Xmsr $TEMPm }
    } elseif { $iX == 8 } {
		set pcolumns(X) $pcolumns(BFLD)
		set Xpts $BFLD
		if { ! $isfop } { set Xmsr $BFLDm }
    }
}


proc setPlotGeo {} {
    # put gv under the main window
    global gv tv
    if { ![info exists gv] || ![winfo exists $gv] || ![winfo ismapped $gv] } {
	 return
    }
    set gg [wm geometry $gv]
    set mg [wm geometry .]
    #set tg ""
    #if { [winfo exists $tv] && [winfo ismapped $tv] } {
    #set tg [wm geometry $tv]
    #}
    if {[scan $mg {%dx%d%d%d} xl yl x y] < 4} {return}
    # try to place gv below main
    if {[scan $gg {%dx%d%d%d} xlg ylg xg yg] < 4} {return}
    set yg [expr $y + $yl + 5]
    set xg $x
    set gg [format %dx%d%+d%+d $xlg $ylg $xg $yg]
    wm geometry $gv $gg
}

proc showPlot {} {
	# called by recoverFG after solveGroup, by recoverPlot in sessionRecover, by enterGroupShow
    # showPlot plots the current group via all local variables
    # into the current iwin
    # PLT flag must be set and group must be calc==1

    # call this when gnum changes and PLT, or plot request
    # groupSrcFiles
    # groupPlotSetup gets called before this
    # to set up all the data lists

    global PLT BLT GNU gnuplotFPT screen toscreen gnuftyp gnuPSopt gnuprog
    global groupiwin grpTitles
    global saveGnuCmnds
    global g gv gnum calc snum srcfiles solnfile gvtitle gpipe gpipes gws gvs iwin
    global ngrpsets fopgroup
    global pDatNames pVecNames
    global mDatNames mVecNames
    global Xpts Xmsr iX Rf xaxlbl titlefont tickfont
    global stylst rstylst sstylst iWts suuWts sddWts sduWts sudWts
    global plotbg pencolor spencolor fsSymb
    global isrcplot eview viewS textf pcolumns nsetfs
    global eview viewS plotsymbolsize plotconfig
    global gnufil gnustyl gnusstyl gnucol
    global yctslbl y2ctslbl y2Rlbl

    if { ! $PLT } { return }
    if { [info exists fopgroup($gnum)] && $fopgroup($gnum) } {
    } else {
		if { ! $calc } { return }
    }

    # how about getting an automatic plot of fop group results .p file

    # only build one plot control window = gv
    # Apr 2010 changed to put plot-selector below group-selector in main window
    if { ![winfo exists $gv.title] } { buildPlotSelector }
    #if { ![winfo ismapped $gv] } { wm deiconify $gv }
    #wm title $gv "$gvtitle   group $gnum  plotwindow $iwin"
    #setPlotGeo
    #raise $gv

    set groupiwin($iwin) $gnum

    # may need to open a new gnuplot channel, or blt toplevel for graph
    if { $BLT } {
		if { ! [info exists gws($iwin)] || ! [winfo exists $gws($iwin)] } {
	    	# create a new top level and put a graph widget in it
	    	set gvs($iwin) [toplevel .g$iwin]
	    	raise $gvs($iwin)
	    	set gws($iwin) [graph $gvs($iwin).g$iwin]
	    	pack $gws($iwin) -side top -fill both -expand true
		}
		# make this graph current g
		set g $gws($iwin)
		wm title $gvs($iwin) "$gvtitle  group $gnum $solnfile"
		bltGraphSetup
		# apply the per group graph options. Wont need any updating for BLT
		# make sure new graphs get mapped before applyGraphOptions
		# in case plotconfigall which looks for mapped windows
		update idletasks
		applyGraphOptions 0
		raise $gvs($iwin)
		return
    }
}

proc recoverPlotShow {pfl} {
	# show plot for previous solutions
	# graph toplevels are g$gnum_$vers
    # showPlot plots the current group via all local variables
    # into the current iwin
    # PLT flag must be set and group must be calc==1

    # call this when gnum changes and PLT, or plot request
    # groupSrcFiles
    # groupPlotSetup gets called before this
    # to set up all the data lists

    global PLT BLT GNU gnuplotFPT screen toscreen gnuftyp gnuPSopt gnuprog
    global groupiwin grpTitles
    global saveGnuCmnds
    global g gv gnum calc snum srcfiles gvtitle gpipe gpipes gws gvs iwin
    global ngrpsets fopgroup
    global pDatNames pVecNames
    global mDatNames mVecNames
    global Xpts Xmsr iX Rf xaxlbl titlefont tickfont
    global stylst rstylst sstylst iWts suuWts sddWts sduWts sudWts
    global plotbg pencolor spencolor fsSymb
    global isrcplot eview viewS textf pcolumns nsetfs
    global eview viewS plotsymbolsize plotconfig
    global gnufil gnustyl gnusstyl gnucol
    global yctslbl y2ctslbl y2Rlbl

    if { ! $PLT } { return }


    # how about getting an automatic plot of fop group results .p file


    #if { ![winfo ismapped $gv] } { wm deiconify $gv }
    #wm title $gv "$gvtitle   group $gnum  plotwindow $iwin"
    #setPlotGeo
    #raise $gv

	# get the olution root filename from pfl
	regexp {_g([0-9]+)\.([0-9]+)\.p} $pfl wm gn vers
	set root [regsub {\.p} $pfl {}]
	
	set gtop ".g$gn\_$vers"
	set prevplotwin($root) $gtop
    #set groupiwin($iwin) $gnum

    # may need to open a new  blt toplevel for graph

	if { ! [winfo exists $gtop] } {
	    	# create a new top level and put a graph widget in it
	    set gp [toplevel $gtop]
	    raise $gp
	    set gpg [graph $gtop.g]
	    pack $gpg -side top -fill both -expand true
	} else {
		set gpg $gtop.g
	}

	wm title $gtop "$gvtitle  group $gn soln $root"
	# need a different that bltGraphSetup since vectors names must be different
	bltRecoverGraphSetup $gpg $gn.$vers $gn
		# apply the per group graph options. Wont need any updating for BLT
		# make sure new graphs get mapped before applyGraphOptions
		# in case plotconfigall which looks for mapped windows
	update idletasks
	# this will need changed as well
	applyGraphOptions 0
	raise $gtop
	wm deiconify $gtop
	return
}



proc applyGraphOptions {update} {
    global GNU dsmode
    # called from showPlot and recoverPlotShow
    # update just calls showPlot if GNU
    applyAxScl 0
    plotLogScale 0
    plotSymbolSize 0
    plotErrbars 0
    plotConfig
    if { ! $dsmode } {
	viewMsrSet 0
	viewMsrFS 0
	plotSymbol 0
	plotSymbolColor 0
    } else {
	plotSymbol 0
	plotSymbolColor 0
    }
    viewElem 0
    if { $GNU && $update } { if { $dsmode } { showDSplot } else { showPlot } }
}

proc oneC {on} {
    return [expr 1 - $on]
}

proc viewElem {update} {
    global g gwds eview BLT GNU spcfg dsmode dseditY fopgroup gnum
    if { $BLT } {
	if { $dsmode } {
	    if { ! [info exists gwds] } { return }
	    for {set i 1} {$i <= $spcfg(ny)} {incr i} {
		if { ![string match $dseditY all] && \
			 ![string match $dseditY $spcfg(y${i}-lbl)] } {
		    $gwds element configure y$i -hide 1
		    continue
		}
		if { [info exists spcfg(y${i}-show)] } {
		    set hidestate [oneC $spcfg(y${i}-show)]
		    $gwds element configure y$i -hide $hidestate
		} else {
		    $gwds element configure y$i -hide 0
		}
	    }
	} else {
	    configHideStates $g
	    if { ! $fopgroup($gnum) } {
		set hidestateR [oneC $eview(R)]
		set hidestateM [oneC $eview(msr)]
		if { $hidestateR == 1 && $hidestateM == 1 } {
		    $g axis configure y2 -hide 1
		    $g axis configure rf -hide 1
		} elseif { $hidestateR == 1 } {
		    $g y2axis use y2
		    $g axis configure y2 -hide 0
		    $g axis configure rf -hide 1
		} else {
		    # if both msr and R are viewed use R axis
		    $g y2axis use rf
		    $g axis configure y2 -hide 1
		    $g axis configure rf -hide 0
		}
	    }
	}
    }
    if { $GNU && $update } {
	if { $dsmode } { showDSplot } else { showPlot }
    }
}

proc viewMsrFS {update} {
    global eview ngrpsets
    global g
    global plotbg
    global pencolor fsSymb
    global BLT GNU
    global fopgroup gnum

    if { $fopgroup($gnum) } { return }
    if { $BLT } {
    set plotbg [$g cget -plotbackground]
	for {set i 1} {$i <= $ngrpsets} {incr i} {
	    if { $eview(sfm) } {
		$g pen configure p3$i \
		    -errorbarcolor #$pencolor(3$i) -symbol $fsSymb(3)
		$g pen configure p4$i \
		    -errorbarcolor #$pencolor(4$i) -symbol $fsSymb(4)
	    } else {
		$g pen configure p3$i -errorbarcolor $plotbg -symbol ""
		$g pen configure p4$i -errorbarcolor $plotbg -symbol ""
	    }
	    if { $eview(nsfm) } {
		$g pen configure p1$i \
		    -errorbarcolor #$pencolor(1$i) -symbol $fsSymb(1)
		$g pen configure p2$i \
		    -errorbarcolor #$pencolor(2$i) -symbol $fsSymb(2)
	    } else {
		$g pen configure p1$i -errorbarcolor $plotbg -symbol ""
		$g pen configure p2$i -errorbarcolor $plotbg -symbol ""
	    }
	}
    }
    if { $GNU && $update } { showPlot }
}

proc viewMsrSet {update} {
    # command for selecting which src sets are viewed in plot
    # isrcplot isrcon button
    # April 2010 isrcplot is controlled directly from menu sfm
    # so isrcon variable no longer used
    # this proc is now called when msrSet radio is selected.

    global isrcon ngrpsets
    global g
    global plotbg
    global pencolor fsSymb
    global isrcplot plotconfig
    global BLT GNU
    global fopgroup gnum

    if { $fopgroup($gnum) } { return }
    if { $BLT } {
	# this is a hack to turn off data associated with a pen
	# set its color to plotbg
	# or better set it symbol to ""
	# the pens for snum are 4*(snum-1)+j  j=1,4
	#set snum $i
	#if { $snum < 1 || $snum > $ngrpsets } { return }
	set plotbg [$g cget -plotbackground]
	for {set j 1} {$j <= $ngrpsets} {incr j} {
	    for {set i 1} {$i <= 4} {incr i} {
		if { $isrcplot($j) } {
		    if { $plotconfig(errmsr) } {
			$g pen configure p$i$j -errorbarcolor #$pencolor($i$j)
		    } else {
			$g pen configure p$i$j -errorbarcolor $plotbg
		    }
		    $g pen configure p$i$j -symbol $fsSymb($i)
		} else {
		    $g pen configure p$i$j -errorbarcolor $plotbg
		    $g pen configure p$i$j -symbol ""
		}
	    }
	}
    }

    if { $GNU && $update } { showPlot }
}

# hexColorLinear implements linear color interpolation
proc hexColorLinear {icolor fcolor frac} {
    set r ""
    set nh [string length $icolor]
    for {set i 0} {$i < $nh} {incr i} {
	set di [scan [string index $icolor $i] %x]
	set df [scan [string index $fcolor $i] %x]
	set dx [format %1x [expr int((1-$frac)*$di + $frac*$df)]]
	append r $dx
    }
    return $r
}
# implement color vector rotation with hexColor
# hexout = cos(frac*Pi/2)icolor + sin(frac*Pi/2)fcolor
# colorVector(theta) = R(thet) (cos(thet)ich + sin(thet)(unit(ichxfch))xich)
# for thet=(0,thetaf) cos(thetaf)=ich.fch
# R(thet) = Ri + (Rf-Ri)(theta/thetaf)

proc unitvector {vecarray} {
    upvar $vecarray a
    set mag 0.
    foreach {i val} [array get a] { set mag [expr $mag + $val*$val] }
    if { $mag <= 0. } { return 0. }
    set mag [expr sqrt($mag)]
    foreach {i val} [array get a] { set a($i) [expr $a($i)/$mag] }
    return $mag
}
proc dotpro {avec bvec} {
    upvar $avec a $bvec b
    set dot 0.
    foreach {i av} [array get a] {j bv} [array get b] {
	set dot [expr $dot + $av*$bv]
    }
    return $dot
}
proc scalevector {scl avec ascl} {
    upvar $avec a $ascl s
    foreach {i val} [array get a] { set s($i) [expr $scl*$a($i)] }
}
proc addvectors {avec bvec cvec} {
    upvar $avec a $bvec b $cvec c
    foreach {i av} [array get a] {j bv} [array get b] {
	set c($i) [expr $av + $bv]
    }
}

proc crosspro {avec bvec cvec} {
    # 3-vecs
    upvar $avec a $bvec b $cvec c
    set c(1) [expr $a(2)*$b(3) - $a(3)*$b(2)]
    set c(2) [expr $a(3)*$b(1) - $a(1)*$b(3)]
    set c(3) [expr $a(1)*$b(2) - $a(2)*$b(1)]
}

proc hexColorList {fracLst colorLst frac} {
    # fracLst must be ordered low to high
    set nc [llength $colorLst]
    if { $nc < 1 } { return 000000 }
    if { $nc == 1 } { return [lindex $colorLst 0] }
    set ic 0
    set nfrac [expr [llength $fracLst] - 1]
    if { $nfrac < 1 } { return [lindex $colorLst 0] }

    while { $ic < $nfrac && $frac > [lindex $fracLst [expr $ic + 1]] } {
	incr ic
    }
    set nc1 [expr $nc - 1]
    if { $ic >= $nc1 || $ic >= $nfrac } {
	return [lindex $colorLst $nc1]
    }
    set ic1 [expr $ic + 1]
    set f2 [lindex $fracLst $ic1]
    set f1 [lindex $fracLst $ic]
    set df [expr $f2 - $f1]
    if { $df <= 0. } { return [lindex $colorLst 0] }
    set f [expr ($frac - $f1)/$df]
    return [hexColorLinear [lindex $colorLst $ic] [lindex $colorLst $ic1] $f]
}

proc hexColor3 {icolor mcolor fcolor frac} {
    if { $frac <= 0.5 } {
	return [hexColorLinear $icolor $mcolor [expr 2.*$frac]]
    } else {
	return [hexColorLinear $mcolor $fcolor [expr 2.*($frac-0.5)]]
    }
}

proc hexColor {icolor fcolor frac} {
    array set hexfmts {
	1 {%01x%01x%01x}
	2 {%02x%02x%02x}
	3 {%03x%03x%03x}
	4 {%04x%04x%04x}
    }
    # break the colors into rgb components
    set nh [string length $icolor]
    set nc [expr $nh/3]
    scan $icolor $hexfmts($nc) irgb(1) irgb(2) irgb(3)
    scan $fcolor $hexfmts($nc) frgb(1) frgb(2) frgb(3)
    # get the magnitude and unit vector for icolor and fcolor
    set ri [unitvector irgb]
    set rf [unitvector frgb]
    if { $ri <= 0. || $rf <= 0. } { return [format {%06x} 0] }
    array set p {1 0. 2 0. 3 0.}
    crosspro irgb frgb p
    set pmag [unitvector p]
    if { $pmag <= 0. } { return [format {%06x} 0] }
    array set ip {1 0. 2 0. 3 0.}
    crosspro p irgb ip
    set tf [expr acos([dotpro irgb frgb])]
    set t [expr $frac*$tf]
    set ct [expr cos($t)]
    set st [expr sin($t)]
    set rt [expr $ri + ($rf - $ri)*$frac]
    array set si {1 0. 2 0. 3 0.}
    scalevector [expr $rt*$ct] irgb si
    array set sf {1 0. 2 0. 3 0.}
    scalevector [expr $rt*$st] ip sf
    array set rcolor {1 0. 2 0. 3 0.}
    addvectors si sf rcolor
    return [format $hexfmts($nc) \
		[expr round($rcolor(1))] \
		[expr round($rcolor(2))] \
		[expr round($rcolor(3))]]
}

proc bltPS {} {
    global bltPSopt g gwds gnum iwin spfile dsmode
    set gw $g
    if { $dsmode } { 
	if { ! [info exists gwds] } { return }
	set gw $gwds
    }
    foreach opt $bltPSopt {
	$gw postscript configure $opt $bltPSopt($opt)
    }
    if { $dsmode } {
	set outfil $spfile
    } else {
	set outfil pbcor_group$gnum
    }
    $gw postscript output ${outfil}.ps
}
proc bltGIF {} {
    global g gwds gnum iwin spfile dsmode
    set image [image create photo]
    set gw $g
    if { $dsmode } {
	if { ! [info exists gwds] } { return }
	set gw $gwds
    }
    $gw snap $image
    if { $iwin < 1 } {
	set outfil $spfile
    } else {
	set outfil pbcor_group$gnum
    }
    $image write ${outfil}.gif -format GIF
    image delete $image
}

proc killPlotWindow {} {
    global iwin GNU BLT gpipe gpipes g gws gvs
    if { $iwin == 1 } {
	Dialog_Warn "Don't destroy plot window 1"
	return
    }
    if { $GNU } {
	if { [info exists gpipes($iwin)] && \
		 [string length $gpipes($iwin)] > 0 } {
	    close $gpipes($iwin)
	    unset gpipes($iwin)
	    set gpipe $gpipes(1)
	}
    } elseif { $BLT } {
	# user can just close window to destroy
	if { [info exists gvs($iwin)] && \
		 [winfo exists $gvs($iwin)] } {
	    destroy $gvs($iwin)
	    if { [winfo exists $gvs($iwin)] } { tkwait window $gvs($iwin) }
	    set g $gws(1)
	}
    }
    set iwin 1
    #updatePlotWindowsMenu 0
}

proc selectPlotWindow {args} {
    # trace on writing iwin
    global iwin BLT GNU gpipes gpipe gws g gvs gvtitle gnum iwin
    if { $iwin < 1 } { return }
    if { $GNU } {
	if {[info exists gpipes($iwin)] && \
		[string length $gpipes($iwin)] > 0} {
	    set gpipe $gpipes($iwin)
	}
	if {[info exists gvs($iwin)] && [winfo exists $gvs($iwin)] } {
	    wm title $gvs($iwin) "$gvtitle  group $gnum"
	}
    } elseif { $BLT } {
	if { [info exists gws($iwin)] && [winfo exists $gws($iwin)] } {
	    set g $gws($iwin)
	}
    }
}

proc updatePlotWindowsMenu {new} {
    # deprecated
    global iwin miwin
    global gpipes gws gvs
    global GNU BLT

    if { $GNU } {
	set pws [array names gpipes]
    } elseif { $BLT } {
	set pws {}
	foreach pw [array names gws] {
	    if { [winfo exists $gws($pw)] } {
		lappend pws $pw
	    } else {
		unset gws($pw)
		unset gvs($pw)
	    }
	}
    }
    if { [string match new $new] } {
	set iwin 1
	while { [lsearch $pws $iwin] >= 0 } { incr iwin }
	lappend pws $iwin
    } elseif { $new > 0 } {
	set iwin $new
	if { [lsearch $pws $new] < 0 } { lappend pws $new }
    }
    set pws [lsort -integer -increasing $pws]
    $miwin delete 0 end
    #puts "updatePlotWindowsMenu pws=$pws"
    foreach pw $pws {
	$miwin add radio -label $pw -variable iwin -value $pw
    }
}

proc updatePlotSelector {} {
    # call this when switching between PB and DS modes
    global pbWidgets dsWidgets
    global psEdit psPBmenu psDSmenu
    global dsmode
    if { $dsmode } {
	# DS mode
	foreach w $pbWidgets { grid remove $w }
	foreach w $dsWidgets { grid $w }
	$psEdit configure -menu $psDSmenu
    } else {
	foreach w $dsWidgets { grid remove $w }
	foreach w $pbWidgets { grid $w }
	$psEdit configure -menu $psPBmenu
    }
}

proc redrawFudge {gra} {
	set ht [$gra cget -height]
	set ht1 [expr $ht + 1]
	$gra configure -height $ht1
	$gra configure -height $ht
}

proc plotConfigDetectors {} {
	# this is last step in all pbcor elem configs
	# for current graph g or all graphs if plotconfigall is set
	# configure the per detector per spin state pens on or off according to detectorsOn array
	# by setting pixels to plotsymbolsize(sxx) or zero
	# This assumes the pens exist
	global g
	global nDetectors detectorsOn
	global plotconfigall plotsymbolsize plotsymbolcolor pbcorelems plotsymbolfill
	global plotbg
	
	if { $plotconfigall } {
		set gws [bltgraphwidgets 1]
    } else {
		set gws $g
    }
    foreach gw $gws {
    	set plotbg [$gw cget -plotbackground]	
       	for {set j 1} {$j <= $nDetectors} {incr j} {
       		for {set i 1} {$i <= 4} {incr i} {
       			set i1 [expr $i - 1]
       			set el [lindex $pbcorelems $i1]
       			set clr $plotbg
       			set ss 0
       			set ebw 0
       			set ebc $plotbg
       			set fil ""
       			if { $detectorsOn($j) } {
       				set ss $plotsymbolsize($el)
       				set ebw 1
       				set ebc $plotsymbolcolor($el)
       				set clr $plotsymbolcolor($el)
       				if { $plotsymbolfill($el) } {
       					set fil $plotsymbolcolor($el)
       				} else {
       					set fil ""
       				}
       			}
    			$gw pen configure sp$i$j \
    				-pixels $ss -errorbarwidth $ebw -errorbarcolor $ebc -color $clr -fill $fil
    		}
    	}
    	redrawFudge $gw
    }
    # RWE often does not update the graphs. I can fudge this with a routine that changes the graph height
    update idletasks
}

proc setDetectors {n} {
	global nDetectors detectorsOn detectorsMin detectorsMax
	# none
	if {$n == 0} {
		for {set i 1} {$i <= $nDetectors} {incr i} {
			set detectorsOn($i) 0
		}
	#all
	} elseif {$n == 2} {
		for {set i 1} {$i <= $nDetectors} {incr i} {
			set detectorsOn($i) 1
		}
	# range
	} elseif {$n == 3} {
		set r1 $detectorsMin
		set r2 $detectorsMax
		if { $r1 < 1 || $r1 > $nDetectors || $r2 < 1 || $r2 > $nDetectors } { return }
		if { $r1 > $r2 } {
			set rt $r1
			set r1 $r2
			set r2 $rt
		}
		for {set i 1} {$i <= $nDetectors} {incr i} {
			set detectorsOn($i) 0
		}
		for {set i $r1} {$i <= $r2} {incr i} {
			set detectorsOn($i) 1
		}
	}
	plotConfigDetectors
}

proc updateDetectorMenu {} {
	global detectorMenu detectorMenuMin detectorMenuMax nDetectors detectorsOn detectorsMin detectorsMax
	if { ![winfo exists $detectorMenu] } { return }
	$detectorMenu delete 0 end
	if { $nDetectors <= 1 } {
		return
	}
	set m $detectorMenu
	$m add command -label "all" -command "setDetectors 2"
	$m add command -label "none" -command "setDetectors 0"
	$m add command -label "range" -command "setDetectors 3"
	$m add cascade -label "range-min" -menu $m.min
	set detectorMenuMin [menu $m.min -tearoff 0]
	$m add cascade -label "range-max" -menu $m.max
	set detectorMenuMax [menu $m.max -tearoff 0]
	set mm $detectorMenuMin
	set mx $detectorMenuMax
	for {set i 1} {$i <= $nDetectors} {incr i} {
		$m add check -label $i -variable detectorsOn($i) -command "setDetectors 1"
		$mm add radio -label $i -variable detectorsMin -value $i
		$mx add radio -label $i -variable detectorsMax -value $i
	}
}

proc buildPlotSelector {} {
    # call this to setup plot-selector only if PLT

    # I want to be able to reconfigure the plot selector
    # for plotDS showDSplot
    # for which there are no msr
    # so build a separate edit menu that can be attached to the same
    # edit menu button
    # need to save the widgets so we can use grid remove and re grid later

    # this build preps for PB as initial state of the plot selector

    global g gv gvs gvtitle gws iwin miwin onePlotPerGrp
    global gnum snum iwin
    global eview
    global srcon srcfile isrcon sfm detm
    global fsSymb gfsSymb
    global pVecNames mVecNames xpts xmsr
    global titlefont tickfont iX
    global plotsymbolsize plotsymbolcolor plotsymbolfill symbolsizes pbcorelems
    global plotconfig plotimagetype imagefmts
    global BLT GNU PLT gnuftyp gnuPSopt bltPSopt toscreen
    global saveGnuCmnds
    global yctslbl y2ctslbl y2Rlbl 
    global psEdit psPBmenu psDSmenu pbWidgets dsWidgets
    global dseditY dseditm dsmode
    global spfile spcfg
    global bltSymbolsLst rgbColors
    global plotconfigall
    global mbbg mbab
    global detectorMenu

    # gv is the frame in group-selector which will hold the plot-selector
    #set gv [toplevel .g]
    #wm title $gv $gvtitle
    #wm iconname $gv "PB plt"
    #wm iconify $gv
    #set gvs(1) $gv

    #-font *-helvetica-bold-r-normal--*-240-*
    set title [label $gv.title -text "plot editor" \
	       -font {helvetica 24 bold} -foreground #f00]

    set mb $gv

    # throw away group number adjustment in favor of Edit menu
    #set inc $mb.inc
    #set dec $mb.dec
    #set cur $mb.cur
    #set curlbl $mb.curlbl

    set edit $mb.edit
    menubutton $edit -text Edit -menu $edit.pbMenu -bg $mbbg -relief raised \
	    -activebackground $mbab

    # make 2 versions of the edit menu. One for PB and one for DS
    set psEdit $edit
    set psPBmenu $edit.pbMenu
    set psDSmenu $edit.dsMenu
    foreach m [list $psPBmenu $psDSmenu] pbm {1 0} {
	menu $m -tearoff 1
	set plbl ""
	if { $pbm } {
	    set plbl "pbcor "
	    $m add check -label "apply plot-configs to all groups" \
		-variable plotconfigall
	    $m add separator
	}

	if { ! $pbm } {
	    $m add cascade -label "symbol" -menu $m.sym
	    set msym [menu $m.sym -tearoff 0]
	    foreach sym $bltSymbolsLst {
		$msym add radio -label $sym -variable spcfg(sym) \
			-value $sym -command "plotSymbol 1"
	    }
	    $m add cascade -label "symbol-color" -menu $m.color
	    set mcolor [menu $m.color -tearoff 0]
	    foreach colr $rgbColors {
		$mcolor add radio -label $colr -variable spcfg(color) \
			-value $colr -command "plotSymbolColor 1"
	    }
	} else {
		$m add check -label "pbcor symbols same color" \
		-variable plotsymbolcolor(same)
		
		$m add cascade -label "pbcor symbol colors" -menu $m.symcolor
	    set msymcolor [menu $m.symcolor -tearoff 0]
	    foreach el $pbcorelems {
	    	$msymcolor add cascade -label $el -menu $msymcolor.$el
	    	set mm [menu $msymcolor.$el -tearoff 0]
	    	foreach colr $rgbColors {
	    		$mm add radio -label $colr -variable plotsymbolcolor($el) \
	    			-value $colr -command "plotSymbolColor 1"
	    	}
	    }
	    
	    #$m add command -label "pbcor detector colors" \
		#-command "openColorSelector 0"
	    #$m add command -label "msr dataset colors" \
		#-command "openColorSelector 1"
	    $m add separator
	}
	
	if { $pbm } {
		$m add check -label "pbcor symbols same size" \
		-variable plotsymbolsize(same)
		
		$m add cascade -label "pbcor symbol sizes" -menu $m.symsize
	    set msymsize [menu $m.symsize -tearoff 0]
	    foreach el $pbcorelems {
	    	$msymsize add cascade -label $el -menu $msymsize.$el
	    	set mm [menu $msymsize.$el -tearoff 0]
	    	foreach size $symbolsizes {
	    		$mm add radio -label $size -variable plotsymbolsize($el) \
	    			-value $size -command "plotSymbolSize 1"
	    	}
	    }
	    $m add separator
	} else {
		$m add cascade -label "${plbl}symbol-size" -menu $m.pbsub
		set m2 [menu $m.pbsub -tearoff 0]
		foreach size $symbolsizes {
			$m2 add radio -label $size -variable spcfg(pixels) \
		    -value $size -command "plotSymbolSize 1"
		}
	}
	
	if { $pbm } {
		$m add check -label "pbcor symbols same fill" \
		-variable plotsymbolfill(same)
		
		$m add cascade -label "pbcor symbol fills" -menu $m.symfill
	    set msymfill [menu $m.symfill -tearoff 0]
	    foreach el $pbcorelems {
	    	$msymfill add check -label $el -variable plotsymbolfill($el) \
	    	-command "plotSymbol 1"
	    }
	    $m add separator
	}
	
	if { $pbm } {
	    $m add cascade -label "msr symbol-size" -menu $m.msrsub
	    set m3 [menu $m.msrsub -tearoff 0]
	    foreach size $symbolsizes {
		$m3 add radio -label $size -variable plotsymbolsize(msr) \
		    -value $size -command "plotSymbolSize 1"
	    }
	    $m add cascade -label "R symbol-size" -menu $m.rsub
	    set m4 [menu $m.rsub -tearoff 0]
	    foreach size $symbolsizes {
		$m4 add radio -label $size -variable plotsymbolsize(R) \
		    -value $size -command "plotSymbolSize 1"
	    }
	}
	$m add separator
	if { $pbm } {
	    $m add check -label "${plbl}errbars" -variable plotconfig(errpb) \
		-command "plotErrbars 1"
	    $m add check -label "msr errbars" -variable plotconfig(errmsr) \
		-command "plotErrbars 1"
	} else {
	    $m add check -label "${plbl}errbars" -variable spcfg(errs) \
		-command "plotErrbars 1"
	}
	if { $pbm } {
		$m add separator
		$m add cascade -label "detector select" -menu $m.dets
		set detectorMenu [menu $m.dets -tearoff 0]
		updateDetectorMenu
	}	
	
	$m add separator

	if { $pbm } {
	    if { $BLT } {
		$m add check -label zoom -variable plotconfig(zoom) \
		    -command plotConfig
		$m add check -label crosshairs \
		    -variable plotconfig(crosshairs) -command plotConfig
	    }
	    $m add check -label logscale -variable plotconfig(logscale) \
		-command "plotLogScale 1"
	} else {
	    if { $BLT } {
		$m add check -label zoom -variable spcfg(zoom) \
		    -command plotConfig
		$m add check -label crosshairs -variable spcfg(crosshairs) \
		    -command plotConfig
	    }
	    $m add check -label logscale -variable spcfg(logscale) \
		-command "plotLogScale 1"
	}

	$m add separator

	set mpb [menu $m.pbmin -tearoff 0]
	if { $pbm } {
	    $m add cascade -label "${plbl}axis min" -menu $m.pbmin
	    set mmsr [menu $m.msrmin -tearoff 0]
	    $m add cascade -label "msr axis min" -menu $m.msrmin
	} else {
	    $m add cascade -label "y axis min" -menu $m.pbmin
	}
	foreach mt {0 1 auto} {
	    if { $pbm } {
		$mpb add radio -label $mt \
		    -variable plotconfig(ymin) -value $mt -command "axismin y"
		$mmsr add radio -label $mt \
		    -variable plotconfig(y2min) -value $mt -command "axismin y2"
	    } else {
		$mpb add radio -label $mt -variable spcfg(ymin) -value $mt \
		    -command "axismin y"
	    }
	}
	$m add command -label "other axis scaling" -command axisScale

	$m add separator
	$m add command -label "start coord tracker" -command startWhereAmI
	
	#$m add separator
	# section for detaching and saving plots
	# RWE April 2019 now all window saving is from the File Menu -> Find and Save Windows

 
    }

    # add buttons to select which fs to view
    # select pbcorrect and/or raw data (from .m)
    # use symbols uptri=du downtri=ud filcirc=uu opencirc=dd
    # color coding: graded=rawsets coded by set   green=correct
    # as in text window select which sets to view

    #set pltwinL [label $mb.pltwinL -text pltwin]
    #set pltwin [label $mb.pltwin -textvariable iwin -width 2
    #		    -relief sunken -bd 1 -bg white]

    # switching modes reconfigures the plotselector
    set dsplt [checkbutton $mb.dsmode -variable dsmode -text "pltSelFile" \
		   -anchor e -command updatePlotSelector]

    # for DS use a menubutton to allow user to select
    # which y-element to edit
    # the dseditm menu gets radio populated with the DS y columnlabels
    set elemYL [label $mb.elemYL -textvariable dseditY -bg white -width 20]
    set elemY $mb.elemY
    menubutton $elemY -text EditY -menu $elemY.menu -bg $mbbg \
	-relief raised -activebackground $mbab
    set dseditm [menu $elemY.menu -tearoff 0]


    set uu $mb.uu
    set du $mb.du
    set ud $mb.ud
    set dd $mb.dd

    set srcview $mb.srcview
    set sfsrcview $mb.sfsrcview
    set nsfsrcview $mb.nsfsrcview
    set rfview $mb.rfview

    # change sn stuff to a single menubutton with
    # a menu of check foreach set file
    set sf $mb.sf

    # save the names of widgets that get toggled when plotDS
    set pbWidgets \
	[list $uu $du $ud $dd $srcview $sfsrcview $nsfsrcview $rfview $sf]
    set dsWidgets [list $elemY $elemYL]

    #set sninc $sf.sninc
    #set sndec $sf.sndec
    #set sncur $sf.sncur
    #set sncurlbl $sf.sncurlbl
    #set snview $sf.snview
    #set snfile $sf.file
    #set snfileL $sf.fileL


    #button $inc -command incrGroup -image uparrow \
    #	-background green -activebackground yellow
    #button $dec -command decrGroup -image downarrow \
    #	-background green -activebackground yellow
    #label  $cur -textvariable gnum -width 3 -relief sunken -bd 1 -bg white
    #label  $curlbl -text group

    checkbutton $uu -variable eview(suu) -text uu -anchor e \
	-command {viewElem 1}
    checkbutton $dd -variable eview(sdd) -text dd -anchor e \
	-command {viewElem 1}
    checkbutton $du -variable eview(sdu) -text du -anchor e \
	-command {viewElem 1}
    checkbutton $ud -variable eview(sud) -text ud -anchor e \
	-command {viewElem 1}

    checkbutton $srcview -variable eview(msr) -text msr -anchor e \
	-command {viewElem 1}

    checkbutton $nsfsrcview -variable eview(nsfm) -text nsfmsr -anchor e \
	-command {viewMsrFS 1}
    checkbutton $sfsrcview -variable eview(sfm) -text sfmsr -anchor e \
	-command {viewMsrFS 1}


    checkbutton $rfview -variable eview(R) -text R -anchor e \
	-command {viewElem 1}

    menubutton $sf -text msrSet -menu $sf.menu -bg $mbbg -relief raised \
	-activebackground $mbab
    set sfm [menu $sf.menu -tearoff 0]


    #frame $sf
    #button $sninc -command incrSn -image uparrow \
    #	-background green -activebackground yellow
    #button $sndec -command decrSn -image downarrow \
    #	-background green -activebackground yellow
    #label  $sncur -textvariable snum -width 3 -relief sunken -bd 1 -bg white
    #label  $sncurlbl -text msrset
    #checkbutton $snview -variable isrcon -command viewMsrSet
    #entry $snfile -textvariable srcfile -width 20
    #label $snfileL -text file
    # could put the srcfile name in wm border
    # replace the group selector widgets with an Edit menu
    # for adjusting symbol size, crosshairs, logY

    #grid $curlbl $cur $inc $dec
    grid $title -row 0 -column 0 -sticky w -ipadx 5
    grid $edit -row 0 -column 1 -ipadx 5 -padx 10
    #grid $pltwinL -row 0 -column 2
    #grid $pltwin -row 0 -column 3
    grid $dsplt -row 0 -column 4 -padx 10

    grid columnconfigure $mb 5 -minsize 20
    grid $elemY -row 0 -column 6
    grid $elemYL -row 0 -column 7
    # remove elemY and elemYL for pb configuration
    grid remove $elemY
    grid remove $elemYL

    grid $uu -row 0 -column 8
    grid $du -row 0 -column 9
    grid $ud -row 0 -column 10
    grid $dd -row 0 -column 11
    grid $srcview -row 0 -column 12
    grid $nsfsrcview -row 0 -column 13
    grid $sfsrcview -row 0 -column 14

    grid $sf -row 0 -column 15
    grid $rfview -row 0 -column 16 

    #grid $sf -row 0 -column 10 -sticky e -padx 15
    #grid $sninc $sndec $sncur $sncurlbl $snview $snfileL $snfile

    #pack $title $mb -side left -fill x


    #if { $BLT }
    #set g [graph $gv.g]
    #set gvs(1) $gv
    #set gws(1) $g
    #pack $g -side top -fill both -expand true
    #bltGraphSetup

    #setPlotGeo
}

# keep track of the groupnumber in each plotwindow
# and inverse plotwindownumber for each group
# iwin gets set when changing group
# if another group is plotted already in iwin via plotgrpiwin(iwin)
# need to make new plot and update plotgrpiwin and plotwinigrp
# if plot window is killed update plotgrpiwin array
# ie user will PLOTsoln
# unless the one-plot-window-per-grp flag is set onePlotPerGrp=1
# GlobalOptions
# then find the next open window for the group and set iwin to that.
# plotconfiguration stuff is in
# plotconfig  plotsymbolsize eview isrcplot which are all saved per group
# while plotconfiguration stuff can be changed runtime
# when using showPlot it should also be applied.

proc axismin {ax} {
    # just use applyAxScl which handles BLT and GNU
    global plotconfig spcfg dsmode
    if { $dsmode } {
	if { [string match auto $spcfg(${ax}min)] } {
	    set spcfg(${ax}automin) 1
	    set spcfg(${ax}min) 0
	} else {
	    set spcfg(${ax}automin) 0
	}
    } else {
	if { [string match auto $plotconfig(${ax}min)] } {
	    set plotconfig(${ax}automin) 1
	    set plotconfig(${ax}min) 0
	} else {
	    set plotconfig(${ax}automin) 0
	}
    }
    applyAxScl 1
}

proc axisScale {} {
    global ax iwin plotconfig spcfg dsmode spfile
    # present user with top level
    set ax .ax
    if { ! [winfo exists $ax] } {
	toplevel $ax

	foreach {c lbl} {1 min 2 max 3 automin 4 automax} {
	    grid [label $ax.$lbl -text $lbl] -row 0 -column $c
	}

	foreach {r lbl} {1 x 2 y 3 y2} {
	    grid [label $ax.$lbl -text $lbl] -row $r -column 0
	}
	foreach {r rlbl} {1 x 2 y 3 y2} {
	    foreach {c clbl} {1 min 2 max} {
		grid [entry $ax.$rlbl$clbl \
			  -textvariable plotconfig($rlbl$clbl) \
			  -width 12 -relief sunken] \
		    -row $r -column $c
	    }
	    foreach {c clbl} {3 automin 4 automax} {
		grid [checkbutton $ax.$rlbl$clbl \
			  -variable plotconfig($rlbl$clbl)] \
		    -row $r -column $c
	    }
	}

	grid [button $ax.apply -text apply -command "applyAxScl 1" \
		 -background green -activebackground orange] -row 0 -column 0
    }
    raise $ax

    if { $dsmode } {
	wm title $ax "dataset file $spfile axis limits"
	foreach {r rlbl} {1 x 2 y 3 y2} {
	    foreach {c clbl} {1 min 2 max} {
		$ax.$rlbl$clbl configure -textvariable spcfg($rlbl$clbl)
	    }
	    foreach {c clbl} {3 automin 4 automax} {
		$ax.$rlbl$clbl configure -variable spcfg($rlbl$clbl)
	    }
	}
    } else {
	wm title $ax "axis limits"
	foreach {r rlbl} {1 x 2 y 3 y2} {
	    foreach {c clbl} {1 min 2 max} {
		$ax.$rlbl$clbl configure -textvariable plotconfig($rlbl$clbl)
	    }
	    foreach {c clbl} {3 automin 4 automax} {
		$ax.$rlbl$clbl configure -variable plotconfig($rlbl$clbl)
	    }
	}
    }
}

proc applyAxScl {update} {
    # use this to configure plot ranges
    global g gwds plotconfig spcfg dsmode
    global PLT BLT GNU iwin plotconfigall
    global fopgroup gnum
    if { ! $PLT } { return }
    set axnlst [list x y]
    if { ! $fopgroup($gnum) } { lappend axnlst y2 }
    if { $dsmode } {
	if { ! [info exists gwds] } { return }
	set gws $gwds
    } elseif { $plotconfigall } {
	set gws [bltgraphwidgets 1]
    } else {
	set gws $g
    }
    foreach ax $axnlst axn $axnlst {
	set mn ""
	set mx ""
	if { $GNU } {
	    set mn *
	    set mx *
	}
	if { $dsmode } {
	    if { ! $spcfg(${axn}automin) && \
		     [scan $spcfg(${axn}min) {%g} mnval] > 0 } {
		set mn $mnval
	    } else {
		set spcfg(${axn}automin) 1
	    }
	    if { ! $spcfg(${axn}automax) && \
		     [scan $spcfg(${axn}max) {%g} mxval] > 0 } {
		set mx $mxval
	    } else {
		set spcfg(${axn}automax) 1
	    }
	} else {
	    if { ! $plotconfig(${axn}automin) && \
		     [scan $plotconfig(${axn}min) {%g} mnval] > 0 } {
		set mn $mnval
	    } else {
		set plotconfig(${axn}automin) 1
	    }
	    if { ! $plotconfig(${axn}automax) && \
		     [scan $plotconfig(${axn}max) {%g} mxval] > 0 } {
		set mx $mxval
	    } else {
		set plotconfig(${axn}automax) 1
	    }
	}

	if { $BLT } {
	    foreach gw $gws {
		catch {$gw axis configure $ax -min $mn -max $mx}
	    }
	    if { $plotconfigall } {
		plotconfigGrps \
		    [list ${axn}automin ${axn}automax ${axn}min ${axn}max]
	    }
	} else {
	    if { $dsmode } {
		gdspipeWrite "set ${ax}range \[$mn:$mx\]"
		if { $update } { gdspipeWrite "replot" }
	    } else {
		gpipeWrite "set ${ax}range \[$mn:$mx\]"
		if { $update } { gpipeWrite "replot" }
	    }
	}
    }
}

proc plotconfigGrps {elst} {
    global plotconfig plotconfigsave gnum group
    # propagate plotconfig($elem) to all groups
    array set plotconfigsave [array get plotconfig]
    saveGroup
    set nowgrp $gnum
    foreach grp [array names group] {
	set gnum $grp
	setGroup
	foreach elem $elst { set plotconfig($elem) $plotconfigsave($elem) }
	saveGroup
    }
    set gnum $nowgrp
    setGroup
}

proc plotsymbolsizeGrps {elst} {
    global plotsymbolsize plotsymbolsizesave gnum group
    # propagate plotsymbolsize($elem) to all groups
    array set plotsymbolsizesave [array get plotsymbolsize]
    saveGroup
    set nowgrp $gnum
    foreach grp [array names group] {
	set gnum $grp
	setGroup
	foreach elem $elst {
	    set plotsymbolsize($elem) $plotsymbolsizesave($elem)
	}
	saveGroup
    }
    set gnum $nowgrp
    setGroup
}


proc checkStyles {} {
    # BLT PB only
    # if stylst is using a pen that doesnt exist remove that style
    global g stylst rstylst ngrpsets
    set penlst [$g pen names]
    for {set i [expr [llength $stylst] - 1]} {$i >= 0} {incr i -1} {
		set sty [lindex $stylst $i]
		set p [lindex $sty 0]
		if { [lsearch -exact $penlst $p] < 0 } {
	    	set stylst [lreplace $stylst $i $i]
		}
    }
    for {set i [expr [llength $rstylst] - 1]} {$i >= 0} {incr i -1} {
		set sty [lindex $rstylst $i]
		set p [lindex $sty 0]
		if { [lsearch -exact $penlst $p] < 0 } {
	    	set rstylst [lreplace $rstylst $i $i]
		}
    }
}

proc configHideStates {g} {
    global eview leglbl ny fopgroup gnum
    set nsh 0
    foreach el {suu sdu sud sdd} {
	set h [oneC $eview($el)]
	$g element configure $el -hide $h
	set lege $el\L
	if { $h > 0 } {
	    incr nsh
	    $g element configure $lege -label ""
	} else {
	    $g element configure $lege -label $leglbl($el)
	}
    }
    if { $nsh == 4 } {
	$g element configure corL -label ""
    } else {
	if { $ny == 1 } {
	    $g element configure corL -label "cor-blue"
	} else {
	    $g element configure corL -label "cor-rgb"
	}
    }

    if { ! $fopgroup($gnum) } {
	set h [oneC $eview(msr)]
	$g element configure msr -hide $h
	if { $h > 0 } {
	    $g element configure msrL -label ""
	} else {
	    $g element configure msrL -label $leglbl(msr)
	}
	set h [oneC $eview(R)]
	$g element configure R -hide $h
	if { $h > 0 } {
	    $g element configure R -label ""
	} else {
	    $g element configure R -label R
	}
    }
}

proc ydetCompName {} {
    # for current group solved use
    # pltDet($gnum) and ynames to construct the plot Y-axis name
    # single det is easy ie pltDet > 0 or llength $ynames == 1
    # else try for indexed sequence of ynames ie SPEC0-3 PSDC2-5
    # for 3 or less non sequential ROOTi,j,k else ROOT...

    global gnum pltDet ynames

    if { ! [info exists pltDet($gnum)] || ! [info exists ynames] } {
	return counts
    }
    set idet $pltDet($gnum)
    set nyn [llength $ynames]
    set ny1 [expr $nyn - 1]
    if { $nyn < 1 || $idet > $nyn } { return counts }
    if { $nyn == 1 || $idet > 0 } {
	if { $idet > 0 } {
	    return [lindex $ynames [expr $idet - 1]]
	} else {
	    return [lindex $ynames 0]
	}
    }
    # nyn > 1 and idet == 0 solving all ynames
    # parse for trailing detector integer indices
    set inds {}
    set saveroot ""
    set chk 0
    foreach yn $ynames {
	set ireg [regexp {^([^0-9]*)([0-9]+)$} $yn wm root indx]
	if { $ireg < 1 } {
	    return "${yn}..."
	}
	if { $chk && ![string match $root $saveroot] } {
	    return "${yn}..."
	}
	lappend inds $indx
	set saveroot $root
	set chk 1
    }
    set inds [lsort -increasing $inds]
    scan [lindex $inds 0] %d ini
    set con 1
    for {set i 1} {$i < $nyn} {incr i} {
	scan [lindex $inds $i] %d inn
	if { $inn != [expr $ini + 1] } { set con 0; break }
	set ini $inn
    }
    if { $con } {
	# consecutive indices return the range appended
	return "${root}[lindex $inds 0]-[lindex $inds $ny1]"
    }
    if { $nyn <= 3 } {
	set comp $root
	for {set i 0} {$i < $nyn} {incr i} {
	    append comp [lindex $inds $i]
	    if { $i < $ny1 } { append comp "," }
	}
	return $comp
    }
    return "${root}..."
}

proc createWhereAmIBindings {} {
    # set up these bindings for any graph widget to track
    # the cursor position data values in the whereami toplevel
    bind whereAmI <ButtonPress-1> {initGraphCoords %W %x %y}
    bind whereAmI <B1-Motion> {curGraphCoords %x %y}
}

proc initGraphCoords {W X Y} {
    global wheregraph coordX coordY coordY2 coordR
    set wheregraph $W
    set coordLst [$wheregraph invtransform $X $Y]
    set coordX [lindex $coordLst 0]
    set coordY [lindex $coordLst 1]
    set coordY2 [$wheregraph axis invtransform y2 $Y]
    set coordR [$wheregraph axis invtransform rf $Y]
}
proc curGraphCoords {X Y} {
    # activated by whereAmI binding on current Graph
    # just display the invtransform for current graph
    # in the graphCoords entries via their textvariables
    global wheregraph coordX coordY coordY2 coordR
    set coordLst [$wheregraph invtransform $X $Y]
    set coordX [lindex $coordLst 0]
    set coordY [lindex $coordLst 1]
    set coordR [$wheregraph axis invtransform rf $Y]
}

proc startWhereAmI {} {
    # when starting whereami graph coords
    # add the whereAmI bindtag to all mapped graph widgets that dont have it
    # if user destroys a graph are the bindings destroyed??
    # check that the bindings dont already exist for that graph
    # first get the list if graph widgets via winfo children .

    # first make sure the whereami toplevel is created and mapped

    if { ! [winfo exists .wami] } { buildWhereAmI }
    if { ! [winfo ismapped .wami] } { wm deiconify .wami }
    raise .wami

    set gwsmapped [bltgraphwidgets 1]
    if { [llength $gwsmapped] < 1 } {
	Dialog_Warn "Need some graphs to start whereAmI"
	return
    }

    # make sure the bindtag whereAmI exits
    if { [llength [bind whereAmI]] < 1 } { createWhereAmIBindings }

    foreach gw $gwsmapped {
	set bnds [bindtags $gw]
	# if whereAmI is not in this graphs bindtags, then add it
	# removing zoom
	if { [lsearch $bnds whereAmI] < 1 } {
	    lappend bnds whereAmI
	}
	if { [set iz [lsearch $bnds zoom*]] >= 0 } {
	    set bnds [lreplace $bnds $iz $iz]
	}
	bindtags $gw $bnds
    }
}

proc stopWhereAmI {} {
    # if the whereami toplevel is closed turn off the whereami bindings
    # the graph widgets that were bound are in the list
    # gwwhere
    # check all bltgraphwidgets even if they arent mapped
    # and remove the whereAmI bindtag
    set gwslst [bltgraphwidgets 0]
    foreach gw $gwslst {
	set bnds [bindtags $gw]
	if { [set iw [lsearch $bnds whereAmI]] >= 0 } {
	    set bnds [lreplace $bnds $iw $iw]
	}
	if { [lsearch $bnds zoom*] < 0 } {
	    lappend bnds zoom-$gw
	}
	bindtags $gw $bnds
    }
}

proc buildWhereAmI {} {
    global coordX coordY coordY2 coordR
    if [winfo exists .wami] { return }
    set w .wami
    toplevel $w
    wm title $w "graph coordinates"
    set lx [label $w.lx -text "X "]
    set ex [entry $w.ex -textvariable coordX -width 16]
    set ly [label $w.ly -text "Y "]
    set ey [entry $w.ey -textvariable coordY -width 16]
    set ly2 [label $w.ly2 -text Y2]
    set ey2 [entry $w.ey2 -textvariable coordY2 -width 16]
    set lr [label $w.lr -text "R "]
    set er [entry $w.er -textvariable coordR -width 16]
    grid $lx $ex
    grid $ly $ey
    grid $ly2 $ey2
    grid $lr $er
    raise $w
    bind $w <Destroy> stopWhereAmI
}

proc bltGraphSetup {} {
    # this is only called from showPlot for BLT
    global g gnum fsSymb grpTitles
    global yctslbl y2ctslbl y2Rlbl xaxlbl titlefont tickfont
    global plotbg gtitlefont
    global pVecNames mVecNames pDatNames mDatNames
    global stylst rstylst sstylst ngrpsets ny nDetectors isrcplot eview pencolor spencolor
    global iWts suuWts sddWts sduWts sudWts
    global iX Xlabel
    global leglbl
    global fopgroup
    
    # make sure the graph pens and elements exist
    set isfop 0
    if { [info exists fopgroup($gnum)] } {set isfop $fopgroup($gnum)}
    checkPens
    if { ! [$g element exists suu] } {
	#$g element create suu -symbol $fsSymb(1) -color #0000ff -linewidth 0
	#$g element create sdd -symbol $fsSymb(2) -color #0000ff -linewidth 0
	#$g element create sdu -symbol $fsSymb(3) -color #0000ff -linewidth 0
	#$g element create sud -symbol $fsSymb(4) -color #0000ff -linewidth 0
	# Sxy now pen coded like msr to handle multi-det ?? is ny saved
	# RWE Jan 2011 make label blank since pens mess up the legend
		$g element create suu -linewidth 0 -label ""
		$g element create sdd -linewidth 0 -label ""
		$g element create sdu -linewidth 0 -label ""
		$g element create sud -linewidth 0 -label ""
		if { ! $isfop } {
	    	$g element create msr -linewidth 0 -label ""
	    	$g element create R -symbol cross -color red -linewidth 0
		}
    }

    if { ! [$g element exists suuL] } {
		# create dummy elements with no data for the legend
		$g element create suuL -linewidth 0 -symbol circle -fill "" \
		    -label $leglbl(suu)
		$g element create sduL -linewidth 0 -symbol arrow -fill "" \
		    -label $leglbl(sdu)
		$g element create sudL -linewidth 0 -symbol triangle -fill "" \
		    -label $leglbl(sud)
		$g element create sddL -linewidth 0 -symbol square -fill "" \
		    -label $leglbl(sdd)
		$g element create corL -linewidth 0 -symbol "" \
		    -label $leglbl(cor)
		if { ! $isfop } {
		    $g element create msrL -linewidth 0 -symbol "" \
			-label $leglbl(msr)
		}
    }
    
    # config the axes except for X

    # RWE May 2011 want to use the ynames=?Yselect to construct the
    # Y1 axis label as pbcor ynamesComposite

    set yctslbl "pbcor [ydetCompName]"
    $g axis configure y -title $yctslbl -color blue \
	-titlecolor blue -titlefont $titlefont -tickfont $tickfont
    if { ! $isfop } {
		$g axis configure y2 -title $y2ctslbl -hide 0 -color red \
	    -titlecolor red -titlefont $titlefont -tickfont $tickfont
		if { [llength [$g axis names rf]] < 1 } {
	    	$g axis create rf -title $y2Rlbl -color red \
			-titlecolor red -titlefont $titlefont \
			-tickfont $tickfont -hide 1
		}
    }

    if [info exists grpTitles($gnum)] {
		$g configure -title $grpTitles($gnum) -font $gtitlefont -fg black
    } else {
		$g configure -title ""
    }
    # make sure the vectors for this group exist
    if { ! [vectorExists suu$gnum] } {
        foreach vnam $pVecNames { vector create $vnam$gnum }
		vector create xpts$gnum
		if { ! $isfop } {
	    	foreach vnam $mVecNames { vector create $vnam$gnum }
	    	vector create xmsr$gnum rf$gnum
		}
    }
    # assign the group data lists to the vectors
    foreach vnam $pVecNames pdat $pDatNames {
		global $pdat
		eval $vnam$gnum set $$pdat
    }
    global Xpts
    xpts$gnum set $Xpts

    if { ! $isfop } {
		foreach vnam $mVecNames mdat $mDatNames {
	    	global $mdat
	    	eval $vnam$gnum set $$mdat
		}
		global Xmsr Rf

		xmsr$gnum set $Xmsr
		rf$gnum set $Rf
    }

    # assign the vectors to their graph elements
    $g element configure suu \
	-xdata ::xpts$gnum -ydata ::suu$gnum -yerror ::euu$gnum
    $g element configure sdd \
	-xdata ::xpts$gnum -ydata ::sdd$gnum -yerror ::edd$gnum
    $g element configure sdu \
	-xdata ::xpts$gnum -ydata ::sdu$gnum -yerror ::edu$gnum
    $g element configure sud \
	-xdata ::xpts$gnum -ydata ::sud$gnum -yerror ::eud$gnum

    if { ! $isfop } {
		$g element configure msr \
	    	-xdata ::xmsr$gnum -ydata ::cc$gnum  -yerror ::ccerr$gnum \
	    	-mapy y2 -color red
		$g element configure R -xdata ::xmsr$gnum -ydata ::rf$gnum -mapy rf
		# initially rf axis hidden
		$g y2axis use y2
    } else {
		# for fop make sure msr and R elements dont exist
		#puts "bltGraphSetup deleting elements for fop group=$gnum"
		if [$g element exists msr] { $g element delete msr }
		if [$g element exists R] { $g element delete R }
		$g axis configure y2 -hide 1
    }

    set plotbg [$g cget -plotbackground]

    checkPens
    if { ! $isfop } {
		checkStyles
		$g element configure msr -styles $stylst -weights $iWts
		$g element configure R -styles $rstylst -weights $iWts
    }

    # RWE Dec 2010 changed to color code the 4 Sxy elements by det indx yi
    # RWE Apr 2019 with MACS using 3-axis fmt no need to color code detectors so skip the styles config
    # RWE Apr 2019 but we want to select which detectors are plotted so we keep the sstylst and Wts
    # NB with styles defined the symbol must be in the pen configure
    $g element configure suu -symbol circle -styles $sstylst -weights $suuWts
    $g element configure sdd -symbol square -styles $sstylst -weights $sddWts
    $g element configure sdu -symbol arrow -styles $sstylst -weights $sduWts
    $g element configure sud -symbol triangle -styles $sstylst -weights $sudWts
    # config the pbcor elem pen symbols which are fixed
    for {set i 1} {$i <= 4} {incr i} {
    	for {set j 1} {$j <= $nDetectors} {incr j} {
    		$g pen configure sp$i$j -symbol $fsSymb($i)
    	}
    }

    configHideStates $g


    if { ! $isfop } {
	for {set i 1} {$i <= $ngrpsets} {incr i} {
	    for {set j 1} {$j <= 4} {incr j} {
		if { $isrcplot($i) } {
		    set pc $pencolor($j$i)
		} else {
		    set pc $plotbg
		}
		set pc [string trim $pc \#]
		$g pen configure p$j$i -color #$pc -symbol $fsSymb($j)
		$g pen configure r$j$i -color #$pc -symbol cross
	    }
	}
    }

    for {set i 1} {$i <= $ny} {incr i} {
	for {set j 1} {$j <= 4} {incr j} {
	    set pc $spencolor($j$i)
	    set pc [string trim $pc \#]
	    $g pen configure sp$j$i -color #$pc -symbol $fsSymb($j)
	}
    }
    # Looks like a bug. I missed setDetectors 2 to enable plot config
    setDetectors 2
	# here is where xaxis label gets setusing iX (>0) in xaxlbl
    $g axis configure x -title $Xlabel -color black \
	-titlecolor black -titlefont $titlefont -tickfont $tickfont    
    $g legend configure -font {helvetica 10}
    
    # blt enhancements
    Blt_ZoomStack $g
    Blt_Crosshairs $g
    Blt_ActiveLegend $g
    #Blt_ClosestPoint $g
    Blt_PrintKey $g    
}

proc bltRecoverGraphSetup {gr gnum srcgrp} {
    # this is showPlot for BLT Recover
    # g is the graph widget set in Recover gnum is group.vers assuming we can have dots in vector names
    global fsSymb grpTitles
    global yctslbl y2ctslbl y2Rlbl xaxlbl titlefont tickfont
    global plotbg gtitlefont
    global pVecNames mVecNames pDatNames mDatNames
    global stylst rstylst sstylst ngrpsets ny nDetectors isrcplot eview pencolor spencolor
    global iWts suuWts sddWts sduWts sudWts
    global iX Xlabel
    global leglbl
    global fopgroup
    global g
    
    # make sure the graph pens and elements exist
    set isfop 0
    # need global g to be current graph widget
    set g $gr
    # have to fix pens also
    checkPens
    if { ! [$g element exists suu] } {
	#$g element create suu -symbol $fsSymb(1) -color #0000ff -linewidth 0
	#$g element create sdd -symbol $fsSymb(2) -color #0000ff -linewidth 0
	#$g element create sdu -symbol $fsSymb(3) -color #0000ff -linewidth 0
	#$g element create sud -symbol $fsSymb(4) -color #0000ff -linewidth 0
	# Sxy now pen coded like msr to handle multi-det ?? is ny saved
	# RWE Jan 2011 make label blank since pens mess up the legend
		$g element create suu -linewidth 0 -label ""
		$g element create sdd -linewidth 0 -label ""
		$g element create sdu -linewidth 0 -label ""
		$g element create sud -linewidth 0 -label ""

	    $g element create msr -linewidth 0 -label ""
	    $g element create R -symbol cross -color red -linewidth 0
    }

    if { ! [$g element exists suuL] } {
		# create dummy elements with no data for the legend
		$g element create suuL -linewidth 0 -symbol circle -fill "" \
		    -label $leglbl(suu)
		$g element create sduL -linewidth 0 -symbol arrow -fill "" \
		    -label $leglbl(sdu)
		$g element create sudL -linewidth 0 -symbol triangle -fill "" \
		    -label $leglbl(sud)
		$g element create sddL -linewidth 0 -symbol square -fill "" \
		    -label $leglbl(sdd)
		$g element create corL -linewidth 0 -symbol "" \
		    -label $leglbl(cor)
		if { ! $isfop } {
		    $g element create msrL -linewidth 0 -symbol "" \
			-label $leglbl(msr)
		}
    }
    
    # config the axes except for X

    # RWE May 2011 want to use the ynames=?Yselect to construct the
    # Y1 axis label as pbcor ynamesComposite

    set yctslbl "pbcor [ydetCompName]"
    $g axis configure y -title $yctslbl -color blue \
	-titlecolor blue -titlefont $titlefont -tickfont $tickfont

	$g axis configure y2 -title $y2ctslbl -hide 0 -color red \
	 -titlecolor red -titlefont $titlefont -tickfont $tickfont
		
	if { [llength [$g axis names rf]] < 1 } {
	    $g axis create rf -title $y2Rlbl -color red \
			-titlecolor red -titlefont $titlefont \
			-tickfont $tickfont -hide 1
	}


    if [info exists grpTitles($srcgrp)] {
		$g configure -title $grpTitles($srcgrp) -font $gtitlefont -fg black
    } else {
		$g configure -title ""
    }
    # make sure the vectors for this group exist
    if { ! [vectorExists suu$gnum] } {
        foreach vnam $pVecNames { vector create $vnam$gnum }
		vector create xpts$gnum

	    foreach vnam $mVecNames { vector create $vnam$gnum }
	    vector create xmsr$gnum rf$gnum
		
    }
    # assign the group data lists to the vectors
    foreach vnam $pVecNames pdat $pDatNames {
		global $pdat
		eval $vnam$gnum set $$pdat
    }
    global Xpts
    xpts$gnum set $Xpts


	foreach vnam $mVecNames mdat $mDatNames {
	    global $mdat
	    eval $vnam$gnum set $$mdat
	}
	global Xmsr Rf

	xmsr$gnum set $Xmsr
	rf$gnum set $Rf


    # assign the vectors to their graph elements
    $g element configure suu \
	-xdata ::xpts$gnum -ydata ::suu$gnum -yerror ::euu$gnum
    $g element configure sdd \
	-xdata ::xpts$gnum -ydata ::sdd$gnum -yerror ::edd$gnum
    $g element configure sdu \
	-xdata ::xpts$gnum -ydata ::sdu$gnum -yerror ::edu$gnum
    $g element configure sud \
	-xdata ::xpts$gnum -ydata ::sud$gnum -yerror ::eud$gnum

    if { ! $isfop } {
		$g element configure msr \
	    	-xdata ::xmsr$gnum -ydata ::cc$gnum  -yerror ::ccerr$gnum \
	    	-mapy y2 -color red
		$g element configure R -xdata ::xmsr$gnum -ydata ::rf$gnum -mapy rf
		# initially rf axis hidden
		$g y2axis use y2
    } else {
		# for fop make sure msr and R elements dont exist
		#puts "bltGraphSetup deleting elements for fop group=$gnum"
		if [$g element exists msr] { $g element delete msr }
		if [$g element exists R] { $g element delete R }
		$g axis configure y2 -hide 1
    }

    set plotbg [$g cget -plotbackground]

    checkPens
    if { ! $isfop } {
		checkStyles
		$g element configure msr -styles $stylst -weights $iWts
		$g element configure R -styles $rstylst -weights $iWts
    }

    # RWE Dec 2010 changed to color code the 4 Sxy elements by det indx yi
    # RWE Apr 2019 styles used to select which detectors get plotted 
    $g element configure suu -styles $sstylst -weights $suuWts
    $g element configure sdd -styles $sstylst -weights $sddWts
    $g element configure sdu -styles $sstylst -weights $sduWts
    $g element configure sud -styles $sstylst -weights $sudWts
    
    # config the pbcor elem pen symbols which are fixed
    for {set i 1} {$i <= 4} {incr i} {
    	for {set j 1} {$j <= $nDetectors} {incr j} {
    		$g pen configure sp$i$j -symbol $fsSymb($i)
    	}
    }

    configHideStates $g


    if { ! $isfop } {
		for {set i 1} {$i <= $ngrpsets} {incr i} {
	    	for {set j 1} {$j <= 4} {incr j} {
				if { $isrcplot($i) } {
		    		set pc $pencolor($j$i)
				} else {
		    		set pc $plotbg
				}
				set pc [string trim $pc \#]
				$g pen configure p$j$i -color #$pc -symbol $fsSymb($j)
				$g pen configure r$j$i -color #$pc -symbol cross
	    	}
		}
    }

	# RWE Apr 2019 made pbcor elems configurable except for symbol = fsSymb
	# Since we are using styles again we need to configure pens from 
	# plotsymbolsize plotsymbolcolor plotsymbolfill plotConfigDetectors
	
    for {set i 1} {$i <= $ny} {incr i} {
		for {set j 1} {$j <= 4} {incr j} {
			set j1 [expr $j - 1]
	    	set pc $spencolor($j$i)
	    	set pc [string trim $pc \#]
	    	set el [lindex $pbcorelems $j1]
	    	set pf ""
	    	if {$plotsymbolfill($el)} {set pf $plotsymbolcolor($el)}
	    	$g pen configure sp$j$i -color $plotsymbolcolor($el) -fill $pf
		}
    }
	# here is where xaxis label gets setusing iX (>0) in xaxlbl
    $g axis configure x -title $Xlabel -color black \
	-titlecolor black -titlefont $titlefont -tickfont $tickfont    
    $g legend configure -font {helvetica 10}
    
    # blt enhancements
    Blt_ZoomStack $g
    Blt_Crosshairs $g
    Blt_ActiveLegend $g
    #Blt_ClosestPoint $g
    Blt_PrintKey $g    
}
proc bltDSGraphSetup {} {
    # this is showDSplot for BLT DSmode
    global gwds
    global spcfg
    global titlefont tickfont
    global plotbg gtitlefont
    global readlines readfinished
    
    # make sure the graph elements and vectors exist and assign the DS
    # data lists to the vectors and then the vectors to the element data

    if { ! [info exists gwds] } { return }
    set elems [$gwds element names]
    for {set i [expr $spcfg(ny) + 1]} {$i <= [llength $elems]} {incr i} {
	$gwds element delete y$i
    }

    if { ! [vectorExists xdspts] } { vector create xdspts }
    pipeWrite "c dl $spcfg(xcol)"
    
    # get rid of the leading label
    set Xds [lreplace [lindex $readlines 0] 0 0]
    xdspts set $Xds

    for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	if { ! [$gwds element exists y$i] } { $gwds element create y$i }
	if [regexp {(.*)-fill} $spcfg(y${i}-sym) mtch sym] {
	    set spcfg(y${i}-sym) $sym
	    set spcfg(y${i}-fill) defcolor
	}
	#puts "bltDS y$i symbol= $spcfg(y${i}-sym)"
	$gwds element configure y$i -symbol $spcfg(y${i}-sym) \
	    -color $spcfg(y${i}-color) -linewidth 0 -hide 0 \
	    -label $spcfg(y${i}-lbl) -fill $spcfg(y${i}-fill)
	if { ! [vectorExists y$i] } { vector create y$i }
	if { ! [vectorExists e$i] } { vector create e$i }
	pipeWrite "c dl $spcfg(y${i}-col)"
	
	# get rid of the leading label
	set ydatalst [lreplace [lindex $readlines 0] 0 0]
	y$i set $ydatalst
	pipeWrite "c dl $spcfg(e${i}-col)"
	
	# get rid of the leading label
	set edatalst [lreplace [lindex $readlines 0] 0 0]
	e$i set $edatalst
	$gwds element configure y$i -xdata ::xdspts -ydata ::y$i -yerror ::e$i
    }

    # config the axes
    
    $gwds axis configure y -title $spcfg(ylbl) -color blue \
	-titlecolor blue -titlefont $titlefont -tickfont $tickfont
    $gwds axis configure y2 -hide 1 -color blue -tickfont $tickfont
    $gwds axis configure x -title $spcfg(xlbl) -color black \
	-titlecolor black -titlefont $titlefont -tickfont $tickfont    
    $gwds legend configure -font {helvetica 10}

    if { [info exists spcfg(title)] && [string length $spcfg(title)] > 0 } {
	$gwds configure -title $spcfg(title) -font $gtitlefont \
		-fg black
    }

    # blt enhancements
    Blt_ZoomStack $gwds
    Blt_Crosshairs $gwds
    Blt_ActiveLegend $gwds
    #Blt_ClosestPoint $gwds
    Blt_PrintKey $gwds
}


proc saveImage {w lbl gn} {
    # only works for BLT widget and with ImageMagick utilities available
    global plotimagetype
    global g gvs gnum iwin spfile
    if [string match $w 0] {
	if { ! [info exists gvs($iwin)] || ! [winfo exists $gvs($iwin)] } {
	    Dialog_Warn \
		"NO plot for current window $iwin. Group probably unsolved\n"
	    return
	}
	set w $gvs($iwin)
    }
    if { ![winfo exists $w] || ![winfo ismapped $w] } { return }
    if { $iwin < 1 && [string match $lbl ds] } { set lbl $spfile }
    if { $iwin > 0 && $gn } { append lbl $gnum }
    set fname "$lbl.$plotimagetype"
    set wid [winfo id $w]
    if { [catch {exec import -window $wid $fname}] } {
	Dialog_Warn "Failed to save plotimage as $fname"
    }
}
proc detachImage {w lbl gn} {
    # this only works of the ImageMagick utilities import and display
    # are available and w is a tk widget
    # for the gnuplot window need another way
    # I'm not sure if there are xwindow ids under IMagick on Windows
    global g gvs gnum iwin spfile
    if [string match $w 0] {
	if { ! [info exists gvs($iwin)] || ! [winfo exists $gvs($iwin)] } {
	    Dialog_Warn \
		"NO plot for current window $iwin. Group probably unsolved\n"
	    return
	}
	set w $gvs($iwin)
    }
    if { ![winfo exists $w] } { return }
    if { $iwin < 1 && [string match $lbl ds] } { set lbl $spfile }
    if { $iwin > 0 && $gn } { append lbl $gnum }
    set fname "$lbl.gif"
    set wid [winfo id $w]
    if { [catch {exec import -window $wid $fname}] } {
	Dialog_Warn "Failed to save image as $fname"
    }
    if { [catch {exec display $fname &}] } {
	Dialog_Warn "Failed to detach image"
    }
}

proc bltgraphwidgets {mapped} {
    # return list of all (mapped) blt graph widgets
    set bltws {}
    foreach w [winfo children .] {
	if { [regexp {\.g([0-9]+)} $w mtch i] } {
	    set bltw .g$i.g$i
	    if { [winfo exists $bltw] } {
		if { $mapped && [winfo ismapped $bltw] } {
		    lappend bltws $bltw
		} else {
		    lappend bltws $bltw
		}
	    }
	}
    }
    return $bltws
}

proc plotConfig {} {
    # use this for BLT comp= zoom crosshairs
    global g gwds plotconfig spcfg dsmode
    global BLT plotconfigall
    if { ! $BLT } { return }
    set gws $g
    if { $dsmode } {
	if { ! [info exists gwds] } { return }
	set gws $gwds
    } elseif { $plotconfigall } {
	set gws [bltgraphwidgets 1]
    }
    foreach comp {zoom crosshairs} {
	if { $dsmode } {
	    set on $spcfg($comp)
	} else {
	    set on $plotconfig($comp)
	    if { $plotconfigall } {
		plotconfigGrps [list $comp]
	    }
	}
	foreach gw $gws {
	    set bndtgs [bindtags $gw]
	    set zi [lsearch $bndtgs $comp*]
	    if { $on } {
		if { $zi >= 0 } { continue }
		set bndtgs [linsert $bndtgs 0 $comp-$gw]
	    } else {
		if { $zi < 0 } { continue }
		set bndtgs [lreplace $bndtgs $zi $zi]
	    }
	    bindtags $gw $bndtgs
	}
    }
}

proc plotSymbolSize {update} {
    # use to change symbol size for comp=pbcor, msr, R
    # NB pen onfig doesnt get a graph update, thus the element config
    global g gwds plotsymbolsize ngrpsets pbcorelems spcfg dsmode dseditY
    global BLT plotconfigall fopgroup gnum
    global nDetectors detectorsOn

    # for dsmode start from plot selector configuration
    if { $dsmode } {
		if { ! [info exists gwds] } { return }
		for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	    	if { ![string match $dseditY all] && \
		     ![string match $dseditY $spcfg(y${i}-lbl)] } { continue }
	    	set spcfg(y${i}-pixels) $spcfg(pixels)
		}
    } elseif { $plotconfigall } {
		set gws [bltgraphwidgets 1]
    } else {
		set gws $g
    }
    if { $BLT } {
		if { $dsmode } {
	    	for {set i 1} {$i <= $spcfg(ny)} {incr i} {
				if [info exists spcfg(y${i}-pixels)] {
		    		$gwds element configure y$i -pixels $spcfg(y${i}-pixels)
				} elseif [info exists spcfg(pixels)] {
		    		set spcfg(y${i}-pixels) $spcfg(pixels)
		    		$gwds element configure y$i -pixels $spcfg(y${i}-pixels)
				}
	    	}
		} else {
			if {! [info exists plotsymbolsize(same)]} {
				# if same is missing arraySet in pbsession probably reinited the array
				array set plotsymbolsize {same 1 suu 6 sdd 6 sdu 6 sud 6}
			}
		
	    	foreach gw $gws {
				foreach elem $pbcorelems i {1 2 3 4} {
		    		if [$gw element exists $elem] {
		    			if {$plotsymbolsize(same)} {
							$gw element configure $elem \
			    			-pixels $plotsymbolsize(suu)
			    			set px $plotsymbolsize(suu)
			    			for {set j 1} {$j <= $nDetectors} {incr j} {
			    				if {$detectorsOn($j)} {
			    					$gw pen configure sp$i$j -pixels $px -errorbarwidth 1
			    				} else {
			    					$gw pen configure sp$i$j -pixels 0 -errorbarwidth 0
			    				}
			    			}
			    		} else {
			    			$gw element configure $elem \
			    			-pixels $plotsymbolsize($elem)
			    			set px $plotsymbolsize($elem)
			    			for {set j 1} {$j <= $nDetectors} {incr j} {
			    				if {$detectorsOn($j)} {
			    					$gw pen configure sp$i$j -pixels $px
			    				} else {
			    					$gw pen configure sp$i$j -pixels 0
			    				}
			    			}
			    		}
		    		}
				}
				plotConfigDetectors
				#foreach penname [$gw pen names sp*] {
		    	#	$gw pen configure $penname -pixels $plotsymbolsize(pbcor)
				#}
				if { ![info exists fopgroup($gnum] || ! $fopgroup($gnum) } {
		    		foreach penname [$gw pen names p*] {
						$gw pen configure $penname -pixels $plotsymbolsize(msr)
		    		}
		    		if [$gw element exists msr] {
						$gw element configure msr -pixels $plotsymbolsize(msr)
		    		}
		    		foreach penname [$gw pen names r*] {
						$gw pen configure $penname -pixels $plotsymbolsize(R)
		    		}
		    		if [$gw element exists R] {
						$gw element configure R -pixels $plotsymbolsize(R)
		    		}
				}
	    	}
	    	set ssgrps pbcor
	    	if { ![info exists fopgroup($gnum)] || ! $fopgroup($gnum) } { lappend ssgrps msr R }
	    	# update the value for all groups if requested
	   		if { $plotconfigall } {
				plotsymbolsizeGrps $ssgrps
	    	}
		}
    } elseif { $update } {
		# for GNU just replot if update requested
		if { $dsmode } { showDSplot } else { showPlot }
    }
}

proc plotSymbol {update} {
	# RWE Apr 2019 since MACS now looks like 3-axis with one detector pre datapt 
	# (and PTAI di det indx col)
	# the use of styles and weights for the pbcor elements suu sdd sdu sud no longer required.
	# We can then allow different symbol-fill and colors for the pbcor elements
    # DSmode only
    # symbol types are fixed for group plot
    # but might as well allow user to configure symbol types
    # for DS plot
    global g gwds spcfg dsmode dseditY
    global BLT bltSymbolIndex
    global plotconfigall plotsymbolfill plotsymbolcolor pbcorelems
    global nDetectors
    if { $dsmode } {
		if { ! [info exists gwds] } { return }
		for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	    	if { ![string match $dseditY all] && \
		     ![string match $dseditY $spcfg(y${i}-lbl)] } { continue }
	    	set spcfg(y${i}-gsym) $gnuSymbols($bltSymbolIndex($spcfg(sym)))
	    	set spcfg(y${i}-sym) $spcfg(sym)
	    	set spcfg(y${i}-fill) ""
	    	if [regexp {(.*)-fill} $spcfg(y${i}-sym) mtch sym] {
				set spcfg(y${i}-sym) $sym
				set spcfg(y${i}-fill) defcolor
	    	}	    
		}
    } elseif { $plotconfigall } {
		set gws [bltgraphwidgets 1]
    } else {
		set gws $g
    }
    if { $BLT } {
		if { $dsmode } {
	    	for {set i 1} {$i <= $spcfg(ny)} {incr i} {
				if { ![string match $dseditY all] && \
			 	![string match $dseditY $spcfg(y${i}-lbl)] } {
		    		continue
				}
				$gwds element configure y$i \
		    	-symbol $spcfg(y${i}-sym) -fill $spcfg(y${i}-fill)
	   		}
		} else {
			# because plotsymbolfile and plotsymbolcolor were added 2019 they are not in some pbsession files
			if {! [info exists plotsymbolfill(same)] } { set plotsymbolfill(same) 1 }
			if {! [info exists plotsymbolcolor(same)] } { set plotsymbolcolor(same) 1 }
			foreach elem $pbcorelems {
				if {! [info exists plotsymbolfill($elem)] } { set plotsymbolfill($elem) 1 }
				if {! [info exists plotsymbolcolor($elem)] } { set plotsymbolcolor($elem) blue }
			}
			foreach gw $gws {
				foreach elem $pbcorelems i {1 2 3 4} {
		    		if [$gw element exists $elem] {
		    			puts "config  $elem fill"
		    			if {$plotsymbolfill(same)} {
		    				if {$plotsymbolfill(suu)} {
		    					if {$plotsymbolcolor(same)} {
									$gw element configure $elem \
			    					-fill $plotsymbolcolor(suu)
			    					puts "config fillsame colorsame from suu  $elem fill $plotsymbolcolor(suu)\n"
									for {set j 1} {$j <= $nDetectors} {incr j} {
			    						$gw pen configure sp$i$j -fill $plotsymbolcolor(suu)
			    					}
			    				} else {
			    					$gw element configure $elem \
			    					-fill $plotsymbolcolor($elem)
			    					puts "config fillsame $elem fill $plotsymbolcolor($elem)\n"
			    					for {set j 1} {$j <= $nDetectors} {incr j} {
			    						$gw pen configure sp$i$j -fill $plotsymbolcolor($elem)
			    					}
			    				}
			    			} else {
			    				$gw element configure $elem -fill ""
			    				puts "config fillsame from suu $elem fill empty string\n"
			    				for {set j 1} {$j <= $nDetectors} {incr j} {
			    					$gw pen configure sp$i$j -fill ""
			    				}
			    			}
			    		} else {
			    			if {$plotsymbolcolor(same)} {
			    				set clr $plotsymbolcolor(suu)
			    			} else {
			    				set clr $plotsymbolcolor($elem)
			    			}
			    			
			    			if {$plotsymbolfill($elem)} {
				    			$gw element configure $elem -fill $clr
			    				puts "config not fillsame $elem fill color= $clr\n"
			    				for {set j 1} {$j <= $nDetectors} {incr j} {
			    					$gw pen configure sp$i$j -fill $clr
			    				}
			    			} else {
			    				$gw element configure $elem -fill ""
			    				puts "config not fillsame $elem fill color= empty string\n"
			    				for {set j 1} {$j <= $nDetectors} {incr j} {
			    					$gw pen configure sp$i$j -fill ""
			    				}
			    			}
			    		}
		    		}
				}
			}
			plotConfigDetectors
		}
    } elseif { $update } {
		if { $dsmode } { showDSplot } else { showPlot }
    }
}
proc plotSymbolColor {update} {
	# RWE Apr 2019 added color to pbcorelems
    # DSmode only
    # symbol color fixed for group plot
    # but might as well allow user to configure symbol color
    # for DS plot
    global gwds spcfg dsmode dseditY
    global BLT
    global plotsymbolcolor pbcorelems plotconfigall
    global nDetectors
    if { $dsmode } {
		if { ! [info exists gwds] } { return }
		for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	    	if { ![string match $dseditY all] && \
		     ![string match $dseditY $spcfg(y${i}-lbl)] } { continue }
	    	set spcfg(y${i}-color) $spcfg(color)
		}
    } elseif { $plotconfigall } {
		set gws [bltgraphwidgets 1]
    } else {
		set gws $g
    }
    if { $BLT } {
		if { $dsmode } {
	    	for {set i 1} {$i <= $spcfg(ny)} {incr i} {
				if { ![string match $dseditY all] && \
			 	![string match $dseditY $spcfg(y${i}-lbl)] } {
		    	continue
				}
				if { ! [info exists gwds] } { return }
				$gwds element configure y$i -color $spcfg(y${i}-color)
	    	}
	    } else {
	    	# because plotsymbolfile and plotsymbolcolor were added 2019 they are not in some pbsession files
			if {! [info exists plotsymbolcolor(same)] } { set plotsymbolcolor(same) 1 }
			foreach elem $pbcorelems {
				if {! [info exists plotsymbolcolor($elem)] } { set plotsymbolcolor($elem) blue }
			}
	    	foreach gw $gws {
				foreach elem $pbcorelems i {1 2 3 4} {
		    		if [$gw element exists $elem] {
		    			if {$plotsymbolcolor(same)} {
		    				set clr $plotsymbolcolor(suu)
		    			} else {
		    				set clr $plotsymbolcolor($elem)
		    			}
		    			$gw element configure $elem -color $clr
		    			for {set j 1} {$j <= $nDetectors} {incr j} {
		    				$gw pen configure sp$i$j -color $clr
			    		}
		    		}
		    	}
		    }
		    plotConfigDetectors
	    }
	    	
    } elseif { $update } {
		if { $dsmode } { showDSplot } else { showPlot }
    }
}

proc plotErrbars {update} {
    global g gwds plotconfig pbcorelems spcfg dsmode dseditY
    global BLT iwin plotconfigall spencolor plotbg
    global detectorsOn plotsymbolcolor
    
    if { $dsmode } {
	if { ! [info exists gwds] } { return }
		for {set i 1} {$i <= $spcfg(ny)} {incr i} {
	    	if { ![string match $dseditY all] && \
		     ![string match $dseditY $spcfg(y${i}-lbl)] } { continue }
	    	set spcfg(y${i}-errs) $spcfg(errs)
		}
    } elseif { $plotconfigall } {
		set gws [bltgraphwidgets 1]
    } else {
		set gws $g
    }
    if { $BLT } {
		if { $dsmode } {
	    	for {set i 1} {$i <= $spcfg(ny)} {incr i} {
				if { ![string match $dseditY all] && \
			 	![string match $dseditY $spcfg(y${i}-lbl)] } {
		    	continue
			}
			set eb none
			if { ([info exists spcfg(y${i}-errs)] && $spcfg(y${i}-errs)) \
			 || ([info exists spcfg(errs)] && $spcfg(errs)) } {
		    set eb both
			}
			if { ! [info exists gwds] } { return }
				$gwds element configure y$i -showerrorbars $eb
	    	}
		} else {
	    	set eb none
	    	if { $plotconfig(errpb) } { set eb both }
	    	foreach gw $gws {
	    		set plotbg [$gw cget -plotbackground]
				foreach elem $pbcorelems {
		    		if { ! [$gw element exists $elem] } { continue }
		    		foreach penname [$gw pen names sp*] {
						if [regexp {sp([1-4])([0-9]+)} $penname mtch i j] {
							set ebw 0
							if { $detectorsOn($j) } { set ebw 1 }
			    			if { $plotconfig(errpb) } {
								if [info exists spencolor($i$j)] {
				    				$gw pen configure $penname \
									-errorbarcolor $plotsymbolcolor($elem) -errorbarwidth $ebw
									# spencolor($i$j)
								}
			    			} else {
								$gw pen configure $penname \
				    			-errorbarcolor $plotbg -errorbarwidth 0
			    			}
						}
		    		}
		    		$gw element configure $elem -showerrorbars $eb
				}
	    	}
	    	plotConfigDetectors
	    	viewMsrSet 0
	    	if { $plotconfigall } {
				plotconfigGrps [list errpb]
	    	}
		}
    } elseif { $update } {
		if { $dsmode } { showDSplot } else { showPlot }
    }
}

proc plotLogScale {update} {
    # use to set logscale for pbcor and msr
    global g gwds plotconfig spcfg dsmode
    global BLT plotconfigall fopgroup gnum
    if { $dsmode } {
		if { ! [info exists gwds] } { return }
		set log $spcfg(logscale)
		set gws $gwds
    } else {
		set log $plotconfig(logscale)
		set gws $g
		if { $plotconfigall } {
	    	set gws [bltgraphwidgets 1]
	    	plotconfigGrps [list logscale]
		}
    }
    if { $log } {
		if { $BLT } {
	    	foreach gw $gws {
				$gw axis configure y -min 1 -logscale true -loose 1
				if { ![info exists fopgroup($gnum)] || ! $fopgroup($gnum) } {
			    	$gw axis configure y2 -min 1 -logscale true -loose 1
				}
	    	}
		}
    } else {
		if { $BLT } {
	    	foreach gw $gws {
				$gw axis configure y -logscale false -loose 0
				if {![info exists fopgroup($gnum)] || ! $fopgroup($gnum) } {
		    		$gw axis configure y2 -logscale false -loose 0
				}
	    	}
		}
    }
}

##################### auto group procs ########################

proc fileListToSetList {fls} {
    # put datasets corresponding to files in fls into setlist 1
    # fls list should have short filenames from lbxs(0)
    # so dataset numbers can be recovered from dsFile
    global readfinished readlines
    global currentDir dsFile
    global globalsetlist
    global recordScript

    # delete any previous setlists  NOT sure s da works
    #pipeWrite "s da"
    # clear the first setlist. This also makes it current setlist
    pipeWrite "s1 lc"
    
    # put the files in this setlist
    set nfls [llength $fls]
    #puts "fileListToSetlist nfls=$nfls"
    set globalsetlist {}
    for {set i 0} {$i < $nfls} {incr i} {

	# reconstruct the full src filename
	    set fl [lindex $fls $i]
	#set fl [file join $currentDir [lindex $fls $i]]
    	#puts "fileListToSetlist loop i=$i file=$fl"
	# get dataset index from input filename srcin
	# because libDF adds .x ext to read files even though not
	# written to disk because cols have been modified
	# yet they are in the sellectlist with orig srcname = srcin from d ne
	# why not store the dsnumber for every file that gets put into
	# lbxs(0) so we never have to look it up or worry about src name
	# in fact this was already done and the dsnumbers are
	# in dsFile(shortFileName) array
	
	#pipeWrite "d ni $fl"
	#
	# readline 0 has dataset inum filename= file
	#set dsn 0
	#if { [llength $readlines] > 0 } {
	#    set dsn [lindex [lindex $readlines 0] 1]
	#}
	# RWE Dec 2015  changed to add files to setlist by filename as debugCMDS can get the dataset indices
	# to be different than in the session. Note that sessionRecover or main session dont have this problem
	# so here lookup the src files dataset. NB the src file is not necessarily srcin
	# as libDF can modify srcin (eg to get .x files with added or different columns)
	# do the lookup using src via
	# d no src-filename. These should be the filenames that appear in the group-selector GUI
	# We need to make sure we can lookup the setnumber by src(out) filename
	
		# Dont care about dataset numbers if we are recording background script
		if {! $recordScript } {
        	pipeWrite "d no $fl"
        	if { [llength $readlines] < 1 } {
        		Dialog_Warn "failed to get dataset number for file: $fl"
	        	return
        	}
        	set dsn [lindex [lindex $readlines 0] 1]
        	if { $dsn < 1 } {
         		Dialog_Warn "failed to get dataset number for file: $fl"
	        	return       
        	}
	    	#if { ! [info exists dsFile($fl)] || $dsFile($fl) < 1 } {
	    	#    Dialog_Warn "failed to get dataset number for file: $fl"
	    	#    return
	    	#}
	    	#lappend globalsetlist $dsFile($fl)
	    	lappend globalsetlist $dsn
	    }
	    # change this for debugCMDS to add files to the setlist by filename
	    #puts "about to s l+ $fl"
	    pipeWrite "s l+ $fl"
	    #puts "added to setlist file= $fl"
    }
}

proc autoGroup {all} {
    # exec auto grouping button binding
    global currentgroup samepts exclmatch gnum
    global stndTolArr stndTolNames autoStndTol
    global readfinished readlines
    global lbxs

    # get curselection from lbxs(0) to check for seed
    set seedlst [$lbxs(0) curselection]
    set nseed [llength $seedlst]
    set seed 0

    # if currentgroup put just gnum files into setlist
    # else put all files into setlist
    if { $all } { set ig 0 } else { set ig $gnum }
    set fls [getFilesInGroup $ig]
    set nfls [llength $fls]
    if { $nfls < 2 } {
	Dialog_Warn "Cant auto-group with less than 2 files in source group."
	return
    }
    fileListToSetList $fls

    for {set i 0} {$i < $nfls} {incr i} {
	set fl [lindex $fls $i]
	# if not yet seeded see if we can use this fl
	if { $nseed > 0 && $seed < 1 } {
	    if { [set is [lsearch -exact $seedlst $fl]] >= 0 } {
		set seed [expr $i + 1]
	    }
	}
    }
    # update the tolerances and save in autoStndTol

    setStndTolerances
    array set autoStndTol [array get stndTolArr]

    # ready to match
    set p ""
    if { $samepts < 1 } { set p p }
    set x ""
    if { $exclmatch < 1 } { set x x }
    # exec the setlist match command
    pipeWrite "s M$p$x $seed"
    

    #puts "match complete. print lines"
    #for {set i 0} {$i < [llength $readlines]} {incr i} {
    #	puts [lindex $readlines $i]
    # }

    # get the resulting linked setlist numbers
    pipeWrite "s1 ll"
    
    if { [llength $readlines] < 2 } {
	Dialog_Warn "Match failed to produce new groupings."
	return
    }
    set ll [lindex $readlines 1]
    # first element is the source setlist number (1)
    # the link setlistnumbers follow and will contain the source as well
    # so remove any entries equal to 1
    while { [set is [lsearch -exact $ll 1]] >= 0 } {
	set ll [lreplace $ll $is $is]
    }

    # now where to put these setlists
    # user may have already setup groups with these setlist numbers
    # use nextEmptyGroup
    set eg 0
    for {set i 0} {$i < [llength $ll]} {incr i} {
	set sl [lindex $ll $i]
	set eg [nextEmptyGroup $eg]
	set gnum $eg
	setGroup
	# get the filenames in this setlist as they appear in the listbox
	# s li gives the srcin. Here we want the diskfile names
	pipeWrite "s$sl l"
	
	# start line 4 ilst match iset fname
	# put the data for these files into lbxs with gnum=$eg
	# first make the list of files from listbox into lbxfiles
	getLbxFiles
	set failed ""
	for {set j 3} {$j < [llength $readlines]} {incr j} {
	    set rl [lindex $readlines $j]
	    set slf [file tail [lindex $rl 3]]
	    # get the row of data from the lbxs for the file slf
	    #
	    set data [getLbxData $slf]
	    if { [llength $data] < 1 } {
		append failed "$slf\n"
		continue
	    }
	    set data [lreplace $data 1 1 $eg]
	    appendLbxData [list $data]
	}
	if { [string length $failed] > 0 } {
	    Dialog_Warn "The following files from group $gnum were not found in the listbox. This is a bug.\n$failed"
	}
	setGroup
    }
}

proc setdefltTol {} {
    # used on reading files, or user button to reset deflt tolerances
    # if user edits tolerances, then setting by file is disabled (instrType = -1 after edits)
    global instrType readlines stndTolArr stndTolNames
    global toltype tolerance
    set fac 1
    if [expr $instrType == 1] { set fac 33 }
    #puts "setdefltTol fac= $fac\n"
    #set ln [list $stndTolNames]
    # print all the deflt tolerances
    pipeWrite "c Stdp"
    for {set j 1} {$j < [llength $readlines]} {incr j} {
	    set rl [lindex $readlines $j]
	    #puts "setdefltTol readline= $rl\n"
	    set styp [lindex $rl 1]
	    set il [lsearch -exact $stndTolNames $styp]
	    #puts "setdefltTol $styp indx= $il\n"
	    if {$il < 0} { continue }
	    set sval [lindex $rl 2]
	    set stndTolArr($styp) [expr $fac * $sval]
	    #puts "setdefltTol stndTolArrValue= $stndTolArr($styp) about to updateTolMenuLabel"
	    updateTolMenuLabel $styp
	}
    if { [info exists stndTolArr($toltype)] } {
        set tolerance $stndTolArr($toltype)
    }
}

proc scaleTolBy {} {
    global tolmenu stndTolArr tolerance colds colLbls
    global toltype tolscale scaleall
    if { [scan $tolscale %g scl] < 1 } { return }
    if { ! $scaleall } {
        if { ! [info exists stndTolArr($toltype)] } { return }
        set stndTolArr($toltype) [expr $scl * $stndTolArr($toltype)]
        updateTolMenuLabel $toltype
    } else {
        foreach {key value} [array get stndTolArr] {
            set stndTolArr($key) [expr $scl * $value]
            updateTolMenuLabel $key
        }
    }
    if { [info exists stndTolArr($toltype)] } {
        set tolerance $stndTolArr($toltype)
    }
}

proc updateTolMenuLabel {name} {
    global tolmenu stndTolArr colds colLbls
    if { ![info exists stndTolArr($name)] } { return }
    set val $stndTolArr($name)
    set lbl ""
    if { [info exists colds($name)] && [info exists colLbls($colds($name))] } { set lbl $colLbls($colds($name)) }
    if { [catch {$tolmenu entryconfigure $name* -label "$name $val $lbl"} tmsg] } {
        #puts "failed tolerance menu update for $name\n$tmsg"
    }
}

proc updateTolMenu {} {
	global stndTolNames stndTolArr
	# when changing group update tolerances to new valuesw
	foreach name $stndTolNames {
    	if {![info exists stndTolArr($name)]} {continue}
    	updateTolMenuLabel $name
    }
}

proc buildTolMenu {} {
    # whenever a file gets read lLbls may change so call this to rebuild the menu
    global tolmenu stndTolNames stndTolArr colds colLbls
    global toltype
    $tolmenu delete 0 end
    if { [info exists colds(En)] } {
        #puts "colds(En)= $colds(En)"
        set col $colds(En)
        if { [info exists colLbls($col)] } {
            #puts "colLbls($col) = $colLbls($col)"
        }
    }
    foreach name $stndTolNames {
    	# in case the programatic list of stndTolNames changes over time make sure each name is in the current array
    	# for example if user recovers an old session that was missing some of the updated tolerance names
    	# we dont want to add those to the menu
    	if {![info exists stndTolArr($name)]} {continue}
        set tol $stndTolArr($name)
	    set lbl ""
	    if { [info exists colds($name)] && [info exists colLbls($colds($name))] } { set lbl $colLbls($colds($name)) }
	    $tolmenu add radio -variable toltype -label "$name $tol $lbl" -value $name -command "setStndTol $name"
	    #$tolmenu entryconfigure $name -command "setStndTol $name"
	}
}


proc buildAutoGroup {} {
    # This is built after  groupPrep so the help pop and showHelp are available
    # how to handle tolerances
    # punt and just use stnd col names
    # do an initial load of stndTol(stndName) = stndTol

    # this builds in $bot frame

    global bframes
    global stndTolNames stndTolArr tolerance toltype samepts exclmatch
    global tolscale scaleall
    global tolmenu relaxToler
    global currentgroup
    global mbbg mbab
    global colds


    set f $bframes(3)
    clearGrid $f

    #set title $f.title
    set exe $f.exe
    #set optcb $f.opt
    #set onecb $f.one
    set cb $f.cb
    set relax $cb.rlx
    set scalecb $cb.scl

    #set execur $exe.cur
    #set exeall $exe.all
    set tolMB $exe.mb
    set tolmenu $exe.mb.menu
    set tolType $exe.t
    set tolValu $exe.v
    set tolTypeL $exe.tl
    set tolValuL $exe.vl
    set scalebut $exe.sclbut
    set defltbut $exe.defltbut
    set scaleValu $exe.sclval




    if { ! [winfo exists $tolValu] } {
    # RWE May 2015 need a more user friendly tolerance
    # interface since it is important.
    # This means creating a normal menu (not tk_option)
    # The menu should have radio entries with commands for each column type.
    # the radio variable will be toltype and set to stndTolName=ctype
    # each radio label will have stndTolName tol dataColName(if available)
    #   showing the current tolerance value and the datafile ColumnLabel(if available)
    # The radio selection command, setStndTol $item, will load stndTolName in the toltype variable for entry widget tolType
    # and load tolerance variable for entry widget tolValu from stndTolArr(tolType)
    # if user edits tol: bind $tolValu <Leave> to trigger reading the tolerance variable into stndTolArr(toltype)
    # then also update the menu entry to relect the new tolerance value
    # Find menu entires by glob-pattern-match on label using the standard name
    # and send this to pbscript? NO I think the tolerances get sent via the SOLVE button and setStndTolerances
    # but also update the menu entry for ctype with the new tolerance value

	#-font *-helvetica-bold-r-normal--*-240-*
	#label $title -text "auto group"
	# -font {helvetica 24 bold} -foreground #f00
		frame $exe
		frame $cb
		#button $execur -text "AUTO-GROUP current group files"
		# -command "autoGroup 0"
		# -background green -activebackground yellow -width 28
		#button $exeall -text "AUTO-GROUPall"
		#-command "autoGroup 0"
		#-background green -activebackground yellow -width 14
	    menubutton $tolMB -text "data tolerances" -menu $tolmenu -bg $mbbg -activebackground $mbab -relief raised
		menu $tolmenu -tearoff 1

		label $tolTypeL -text "data column"
		label $tolValuL -text "toler/scl"
	
		#set tolmenu [eval tk_optionMenu $tolType toltype $stndTolNames]
		# configure each menu entry to write the tol value to the
		# entry widget when it is selected
	    entry $tolType -textvariable toltype -width 10
		entry $tolValu -textvariable tolerance -width 10
		set toltype [lindex $stndTolNames 0]
		set tolerance $stndTolArr($toltype)
		# initially there wont be any data file so column labels will be missing in the menu
		buildTolMenu

		bind $tolValu <Leave> getStndTol


		#checkbutton $optcb -variable samepts
		# -text "files must have all same data points"
		#checkbutton $onecb -variable exclmatch
		# -text "each file can only be in one group"
		checkbutton $relax -variable relaxToler \
		    -text "relaxed tolerances" -command setRelaxToler
		checkbutton $scalecb -variable scaleall -text "scale all"
		button $scalebut -text "Scale By" -bg green -activebackground yellow -command scaleTolBy
		entry $scaleValu -textvariable tolscale -width 10
		button $defltbut -text "Set Deflt" -bg green -activebackground yellow -command setdefltTol
		bind $relax <Enter> "showHelp $relax ."
		bind $relax <Leave> withdrawHelp
		bind $defltbut <Enter> "showHelp $defltbut ."
		bind $defltbut <Leave> withdrawHelp
    }
    #grid $title -sticky ew
    grid $exe -sticky ew
    grid $cb -sticky ew
    #grid $execur -sticky ew -columnspan 2
    grid $tolMB -row 0 -column 0 -columnspan 2 -sticky ew
    grid $scalebut -row 0 -column 2 -sticky ew
    grid $tolTypeL -row 1 -column 0 -sticky ew
    grid $tolType  -row 1 -column 1 -sticky ew
    grid $defltbut -row 1 -column 2 -sticky ew
    grid $tolValuL -row 2 -column 0 -sticky ew
    grid $tolValu  -row 2 -column 1 -sticky ew
    grid $scaleValu -row 2 -column 2 -sticky ew
    #grid $optcb -sticky ew
    #grid $onecb -sticky ew

    grid $relax -row 0 -column 0 -columnspan 2 -sticky ew
    grid $scalecb -row 0 -column 2 -sticky ew

}

################### file selector procs ###################

proc showSelfile {} {
    global mp
    if { $mp == 4 } { showText }
}


proc buildFileSelector {} {
    global sellectlist
    global bframes
    global mswindows

    # some widget widths are platform dependent
    set selwid 32
    set filtwid 10
    set butwid 4
    if { $mswindows } {
	set butwid 7
    }

    # this builds in top left frame
    set f $bframes(1)
    clearGrid $f
    if { ![winfo exists $f.top] } {
	# create the fileselector widgets if they dont exist
	set top [frame $f.top]
	#-font *-helvetica-bold-r-normal--*-240-*
	set title [label $top.title -text "data selector" \
		       -font {helvetica 24 bold} -foreground #f00]
	set do [frame $top.do]

	set bot [frame $f.bot]
	set sellectlist [listbox $bot.list -width $selwid -relief raised -bd 2 \
			     -yscrollcommand "$bot.listscroll set" \
			     -selectmode extended]
	bind $sellectlist <ButtonRelease-1>  {+ showSelfile; plotDS}
	bind $sellectlist <ButtonRelease-3> {tk::ListboxSelectAll %W}
	set sellectscroll [scrollbar $bot.listscroll \
			       -command "$bot.list yview"]

	set filterentry [entry $do.filterentry -relief sunken -bd 2 \
			     -width $filtwid -textvariable fileFilter]
	bind $filterentry <Return> "doFilter $sellectlist"
	#bind $filterentry <Enter> "doFilter $sellectlist"
	bind $filterentry <Tab> "doFilter $sellectlist"
	set bFILTER [button $do.filter -text FILTER -width $butwid \
			 -command "doFilter $sellectlist" \
			 -background green -activebackground yellow]
	set bALL [button $do.all -text "sel ALL" -width $butwid \
		      -command "tk::ListboxSelectAll $sellectlist" \
			 -background green -activebackground yellow]
	set bREAD [button $do.read -text "READ sel" -width $butwid \
		       -command "readFiles" \
		       -background green -activebackground yellow]
    } else {
	set top $f.top
	set title $top.title
	set do $top.do
	set bot $f.bot
	set sellectlist $bot.list
	set sellectscroll $bot.listscroll
	set filterentry $do.filterentry
	set bFILTER $do.filter
	set bALL $do.all
	set bREAD $do.read
    }
    grid $top -sticky ew
    grid $bot -sticky ew

    grid $title -sticky ew
    grid $do -sticky ew
    grid $bREAD $bALL $bFILTER $filterentry -sticky ew

    grid $sellectlist $sellectscroll -sticky news
}

# He3 cells
proc getDirectoryCellFiles {} {
    global readfinished
    global readlines
    global cellFiles cellFile
    global cellmen
    global currentDir
    global nsl1men nsl1Files nsl1File nsl1FilesEnable

    set cfm 0
    if [winfo exists $cellmen.menu] {
	    $cellmen.menu delete 0 end
	    set cfm 1
    }
    set cellFiles {}
    pipeWrite "p H *"

    # starting at 4th line: ifil match filename
    set nf [expr [llength $readlines] - 3]
    if { $nf <= 0 } {
	    Dialog_Warn "NO cell files in current directory: $currentDir"
	    lappend cellFiles none
	    return
    }

    for {set i 3} {$i < [llength $readlines]} {incr i} {
    	set ln [lindex $readlines $i]
	    if { [llength $ln] < 3 } { continue }
	    set cellFile [lindex $ln 2]
	    lappend cellFiles $cellFile
	    if { $cfm } {
	        $cellmen.menu add radio -variable cellFile \
		        -label $cellFile -value $cellFile

	    }
    }
    # also look for nsl files which should have nsl in filename (see p Ho)
    set cfm 0
    #puts "winfo exists nsl1men= [winfo exists $nsl1men]"
    if [winfo exists $nsl1men.menu] {
	    $nsl1men.menu delete 0 end
	    set cfm 1
	    #puts "set cfm= $cfm"
    }
    set nsl1Files {}
    pipeWrite "p Ho"
    global nread
    #puts "p Ho produced nread= $nread readlines= $readlines"
    #puts "p Ho looking for nsl files returned nread=$nread"
    for {set i 3} {$i < [llength $readlines]} {incr i} {
	    set ln [lindex $readlines $i]
	    #puts "p Ho readline= $ln"
	    if { [llength $ln] < 3 } { continue }
	    set nsl1File [lindex $ln 2]
	    lappend nsl1Files $nsl1File
	    #puts "cfm=$cfm about to menu add command label= $nsl1File"
	    if { $cfm } {
	        $nsl1men.menu add command -command nsl1FileDummy \
		        -label $nsl1File
	    }
    }
    #lappend nsl1Files none
    if { $cfm } {
	    $nsl1men.menu add checkbutton -variable nsl1FilesEnable \
	    	-label nslFilesEnable
		#$nsl1men.menu entryconfigure end -deselect
    }
}

# column assignment
proc readCurrentColAssigns {} {
    global colds colLbls colRanges colNdif
    global readfinished readlines

    # get the columns assigned for current dataset
    # and record labels and column data ranges

    pipeWrite "c al"
    

    # each line after the first has
    # ctyp index dscol datasetColLabel
    # need the dscol for each of our DS columns to get stats
    
    set nc [llength $readlines]
    arrayClear colLbls
    arrayClear colds
    arrayClear colRanges
    for {set i 1} {$i < $nc} {incr i} {
	set cspec [lindex $readlines $i]
	if { [llength $cspec] < 4 } { continue }
	set ctyp [lindex $cspec 0]
	set cindx [lindex $cspec 1]
	set dscol [lindex $cspec 2]
	set lbl [lindex $cspec 3]
	if { $cindx > 0 } { append ctyp $cindx }
	set colds($ctyp) $dscol
	set colLbls($dscol) $lbl
    }
    foreach ctyp [array names colds] {
	pipeWrite "c s $colds($ctyp)"
	
	# line 2 has col lbl min max vary ave stdev ndif
	set stat [lindex $readlines 1]
	set min [lindex $stat 2]
	set max [lindex $stat 3]
	set ndif [lindex $stat 7]
	if { $max > $min } {
	    set colRanges($ctyp) "$min $max"
	} else {
	    set colRanges($ctyp) $max
	}
	set colNdif($ctyp) $ndif
    }
    return [expr $nc - 1]
}

proc normalLbxValues {} {
    global rotH grplbxs elbx
    if { $rotH } { convertHlbx }
    set elbl [$grplbxs.name$elbx cget -text]
    if { [string match $elbl Ei] || [string match $elbl Ei] } {
		setElbx 0
    }
    setTSlbx 0
    setQEangs 0
}

proc convertQEangles {} {
    # swap Qxyz E with tts om ttm tta
    # colRanges uses stnd col types, ie x1-3 En ttm tta tts om
    global QEangles
    global elbx qxlbx qylbx qzlbx
    setQEangs [expr 1 - $QEangles]
}

proc initLbxSC {} {
	global lbxnames lbxlabels
	global TTlbls QElbxs lbxs grplbxs
	global colsrch colsrchS
	set lbxnames [lreplace $lbxnames 13 13 Bfld]
	set lbxlabels [lreplace $lbxlabels 13 13 Bfld]
	$grplbxs.name13 configure -text Bfld
	set colsrch [lreplace $colsrch 6 6 Bfld]
	set colsrchS [lreplace $colsrchS 5 5 Bfld]
}

proc initLbxAngles {} {
	global lbxnames lbxlabels
	global TTlbls QElbxs lbxs grplbxs
	global colsrch colsrchS
	# this is meanst to config group lbx columns when first file is read and Q=0 then use angles
	foreach n $TTlbls ibx $QElbxs is {0 1 2 3} {
		set lbxnames [lreplace $lbxnames $ibx $ibx $n]
		set lbxlabels [lreplace $lbxlabels $ibx $ibx $n]
		$grplbxs.name$ibx configure -text $n
		set colsrch [lreplace $colsrch $is $is $n]
		set colsrchS [lreplace $colsrchS $is $is $n]
	}
}

proc setQEangs {wantAngles} {
    global lbxs QEstnd TTlbls QElbls QEangles QElbxs

    if { $wantAngles == $QEangles } { return }
    set nl [llength [$lbxs(1) get 0 end]]
    if { $nl < 1 } { return }
    #puts "setQEangs wantAngles=$wantAngles nl=$nl" 

    set st $QEstnd
    set lbls $QElbls
    if { $wantAngles } {
		set st $TTlbls
		set lbls $TTlbls
    }
    set sl [getfilesetlist]
    #puts "setlist length = [llength $sl]"
    foreach s $st l $lbls ilbx $QElbxs {
		set newlst [getcolrangelist $s $sl]
		#puts "getcolrangelist $s return llength=[llength $newlst]"
		setlbxValues $newlst $ilbx $l
    }
    set QEangles $wantAngles
}

proc convertHlbx {} {
    # swap the H lbx to rotH
    global lbxs hlbx lbxlabels grplbxs Hlbls rotH
    set nl [llength [$lbxs(1) get 0 end]]
    if { $nl < 1 } { return }
    set rotH [expr 1 - $rotH]
    set newlst [HrotHswap $rotH]
    $lbxs($hlbx) delete 0 end
    for {set i 0} {$i < $nl} {incr i} {
	$lbxs($hlbx) insert end [lindex $newlst $i]
    }
    $grplbxs.name$hlbx configure -text $Hlbls($rotH)
    setLbxWidth $lbxs($hlbx) $Hlbls($rotH)
}
proc HrotHswap {wantRotH} {
    global lbxs dsFile colRanges Hlbls readfinished
    # get the list of files
    set el $Hlbls($wantRotH)
    set sl [getfilesetlist]
    return [getcolrangelist $el $sl]
}

proc convertElbx {Ei} {
    # display E Ei or Ef in the E listbox
    # can get the current state of the E lbx from its label
    # ewant=0 for E, =1 for Ei =2 for Ef
    global lbxs elbx lbxlabels grplbxs elbls
    set nl [llength [$lbxs(1) get 0 end]]
    if { $nl < 1 } { return }
    set elbl [$grplbxs.name$elbx cget -text]
    if { [string match $elbl E] || [string match $elbl En] } {
	if { $Ei } {
	    set ewant 1
	} else {
	    set ewant 2
	}
    } elseif [string match $elbl Ei] {
	if { $Ei } {
	    set ewant 0
	} else {
	    set ewant 2
	}
    } elseif [string match $elbl Ef] {
	if { $Ei } {
	    set ewant 1
	} else {
	    set ewant 0
	}
    }
    setElbx $ewant
}

proc setlbxValues {vlst ilbx lbl} {
    global lbxs grplbxs lbxfmts
    # set ilbx values from vlst, which can be list
    # and if lbl non-blank set the ilbx label
    set nl [llength $vlst]
    if { $nl < 1 } { return }
    #puts "setlbxValues nl=$nl"
    $lbxs($ilbx) delete 0 end

    if { [string length $lbxfmts($ilbx)] > 0 } {
		for {set i 0} {$i < $nl} {incr i} {
	    	set data [lindex $vlst $i]
	    	set nd [llength $data]
	    	for {set k 0} {$k < $nd} {incr k} {
				set kdata [lindex $data $k]
				if [catch "format $lbxfmts($ilbx) $kdata"] { break }
				set kdata [format $lbxfmts($ilbx) $kdata]
				set data [lreplace $data $k $k $kdata]
	    	}
	    	$lbxs($ilbx) insert end [join $data]
		}
    } else {
		for {set i 0} {$i < $nl} {incr i} {
	    	$lbxs($ilbx) insert end [join [lindex $vlst $i]]
		}
    }
    if { [string length $lbl] < 1 } {
		set lbl [$grplbxs.name$ilbx cget -text]
    }
    $grplbxs.name$ilbx configure -text $lbl
    setLbxWidth $lbxs($ilbx) $lbl
}

proc setElbx {ewant} {
    global lbxs elbx elbls
    set nl [llength [$lbxs(1) get 0 end]]
    if { $nl < 1 } { return }
    set newlst [eifswap $ewant]
    setlbxValues $newlst $elbx $elbls($ewant)
}

proc getfilesetlist {} {
    global lbxs dsFile
    # return the list of setnumbers for the files in lbxs(0)
    set fls [$lbxs(0) get 0 end]
    set retlst {}
    foreach fl $fls {
	if { ! [info exists dsFile($fl)] } {
	    lappend retlst ""
	    continue
	}
	lappend retlst $dsFile($fl)
    }
    return $retlst
}

proc getcolrangelist {stnd sl} {
    global colRanges
    global readfinished
    # return list of colRanges(stnd) one foreach set in sl
    set retlst {}
    foreach s $sl {
		if { [string length $s] < 1 } {
	    	lappend retlst ""
		} else {
	    	pipeWrite "d$s"
	    
	    	readCurrentColAssigns
	    	if { ! [info exists colRanges($stnd)] } {
				lappend retlst ""
	    	} else {
				lappend retlst $colRanges($stnd)
	    	}
		}
    }	    
    return $retlst
}

proc eifswap {ewant} {
    # return range list of E, Ei, Ef
    # for current files in groupselector
    # these files should have setnumbers in dsFile
    global elbls
    # get the list of files
    set el $elbls($ewant)
    set sl [getfilesetlist]
    return [getcolrangelist $el $sl]
}

proc convertTSlbx {} {
    # swap ts date in ts lbx
    global lbxs tslbx lbxlabels tsdate grplbxs tslbls
    set wantdate [expr 1 - $tsdate]
    setTSlbx $wantdate
}
proc setTSlbx {wantdate} {
    global lbxs tslbx tslbls tsdate grplbxs
    if { $wantdate == $tsdate } { return }
    set lst [$lbxs($tslbx) get 0 end]
    if { [set nl [llength $lst]] < 1 } { return }
    set newlst [tsdateswap $wantdate $lst]
    $lbxs($tslbx) delete 0 end
    for {set i 0} {$i < $nl} {incr i} {
	$lbxs($tslbx) insert end [lindex $newlst $i]
    }
    set tsdate $wantdate
    $grplbxs.name$tslbx configure -text $tslbls($wantdate)
    setLbxWidth $lbxs($tslbx) $tslbls($wantdate)
}

proc tsdateswap {wantdate lst} {
    global timedatefmt
    # convert lst items, ts to date OR date to ts using tcl clock cmnd
    if { [set nl [llength $lst]] < 1 } { return {} }
    set lok 0
    for {set i 0} {$i < $nl} {incr i} {
	if { [string length [string trim [lindex $lst $i]]] > 0 } {
	    set lok 1
	    set lchk $i
	}
    }
    if { ! $lok } { return {} }
    set ists 0
    if { [regexp {^[0-9\.]+$} [lindex $lst $lchk]] } { set ists 1 }
    if { ($wantdate && ! $ists) || (! $wantdate && $ists) } { return $lst }
    set newlst {}
    for {set i 0} {$i < $nl} {incr i} {
	set l [lindex $lst $i]
	if { [string length [string trim $l]] < 1 } {
	    lappend newlst ""
	    continue
	}
	if { $wantdate } {
	    set secs [expr int([lindex $lst $i])]
	    lappend newlst [clock format $secs -format $timedatefmt]
	} else {
	    lappend newlst [clock scan [lindex $lst $i]]
	}
    }
    return $newlst
}

# readFile
proc readFile {fil dw} {
    global dsFile srcfiles currentDir readfinished errlines nerr warnlines nwarn
    global readlines readfinished nread errs
    global Yselect ynames sessionrecover
    global a4enable xcolumns xvary newfiles

    # read fil into dataset and return the list
    # iset npts ncol ny nas df src fshort fname
    # user may reset the global errs string to "" before call
    # dw==1 to write the parsed file to disk if necessary
    # dw==0 NO write to disk

    set fshort [file tail $fil]
    # no need to use full filenames as pbscript will know the cd
    # this means src will get stored as short and can
    # be easily compared to filenames in listboxes
    set fname [file join $currentDir $fshort]
    #puts $fname
    # arrange for pbscript to read this file into the next dataset
    pipeWrite "f+ $fshort"
    
    # check for read errors
    if { $nwarn > 0 } {
			append errs "$warnlines"
    }
    if { $nerr > 0 } {
	# if fails on fixed-energy check for ICE A4 angle scan
	# RWE Jul 2010 try to get energy from A2 A6 = ttm tta
	# fixed libDF to try angles if cant get Efix
	# so no longer convertTTStoQ
	# but if QE are zeros must plot vs angle

	#if { [string match "*fixed energy*" $errlines] && $a4enable }
	# set newfile [convertTTStoQ [list $fshort]]
	#    if { [llength $newfile] < 1 } { return {} }
	#    set fil [lindex $newfile 0]
	#    set fshort [file tail $fil]
	#    set fname [file join $currentDir $fshort]
	#    pipeWrite "f+ $fshort"
	#    
	#    if { $nwarn > 0 }
	#	append errs "$warnlines"
	#
	#    if { $nerr > 0 }
	#      append errs "$errlines"
	#	return {}
	#
	# else 
	#
	# return {}
	
			append errs "$errlines"
			return {}
    }

    # scan 2nd readline for npts. 
    # always have the leading  * indicator of current set
    # save the set number for this file
    set rl [lindex $readlines 1]
    set nscan [scan $rl {%[*]%d %d %d %d %d %d %s} \
		   star iset npts ncol ny nas df src]
    if { $nscan < 8 } {
			append errs "FILEREAD ERROR: failed get npts in $fshort from $rl\n"
			return {}
    }
    # check for ng0 files to set instrType
		checkInstr $fshort

    # readFile must also handle reading raw with no autoCols
    # which means no Y assignment
    # count the number of yColNames to check for raw read
    set ynames [yColNames $iset]
    set nynam [llength $ynames]

    # apparently fixed energy OK but suppose no variation in En or x1-3
    # ICE may be writing incorrect Efix so use checkEiEfangles()
    # if no x1-3 En vary, convertTTStoQ
    # getXcolumns sets the global xcolumns array

    # if already in raw read mode, already trying to convert
    # so dont check for x varies else get inf loop on readFile
    #puts "readFile: nynam=$nynam"

 
    if { $nynam > 0 } {
		#puts "starting angle check"
		# not raw mode
		getXcolumns $iset
		set isangle 0
		pipeWrite "d h \#ScanType"
	
		set rl [lindex $readlines 0]
		if { [string length $rl] > 0 } {
	    	if {[string match "*ANGLE*" $rl] || [string match "*MOTOR*" $rl]} {
				set isangle 1
	    	} elseif { [string match "*Find Peak*" $rl] } {
				pipeWrite "d h \#Scan "
		
				set rl [lindex $readlines 0]
				#puts "d h Scan returned $rl"
				if { [string length $rl] > 0 } {
				   if {[string match "*A4*" $rl] || [string match "*A3*" $rl]} {
			       set isangle 1
			   }
			}
	    }
	}
	# check isangle or see if QE varies
	set QEvary [expr $xvary(En) || $xvary(x1) || $xvary(x2) || $xvary(x3)]
	if { $isangle || ($npts > 1 && ! $QEvary) } {
	    #puts "isangle=$isangle npts=$npts"
	    # or match d h ScanType = ANGLE
	    # ICE not reliably getting Efix in angle scans so get it
	    # from angles
	    if { ! $QEvary && $npts > 1} {
		# why do I have to direct assign x which is supposed to
		# be for Q, to angles???
		# why not just set nx=0 for the set if Q dont vary
		# reset x to angle columns that vary with direct assignment
			set nx 0
			foreach ang {tts om ttm tta} {
			    set xc [lindex $xcolumns($ang) 0]
			    if { $xvary($ang) && $xc > 0 } {
					incr nx
					#pipeWrite "c ad x$nx $xc"
					#
			    }
			}
			if { $nx < 1 } {
			    append errs \
				"FILEREAD ERROR: failed to find ind var in $fshort\n"
			    return {}
			}
			#pipeWrite "d nx $nx"
			#
	    }

	    pipeWrite "c ea"
	    
	    if { $nerr > 0 } {
		append errs $errlines
		return {}
	    }
	}

	#if { $varys < 0 }
	#set newfile [convertTTStoQ [list $fshort]]
	#puts "readFile: convertTTStoQ returned $newfile"
	# if { [llength $newfile] < 1 } { return {} }
	#    set fil [lindex $newfile 0]
	#    set fshort [file tail $fil]
	#    set fname [file join $currentDir $fshort]
	#    puts "readFile: about to r+ $fshort"
	#    pipeWrite "f+ $fshort"
	#    
	#    if { $nwarn > 0 }
	#      append errs "$warnlines"
	#
	#    if { $nerr > 0 }
	#       append errs "$errlines"
	#	return {}
	#
	#    set rl [lindex $readlines 1]
	#    set nscan [scan $rl {%[*]%d %d %d %d %d %d %s} \
	#		   star iset npts ncol ny nas df src]
	#    if { $nscan < 8 }
	#	append errs \
	#	    "FILEREAD ERROR: failed get npts in $fshort from $rl\n"
	#       return {}
	#
	#
    }


    # ICE looks at Signal header-derivative to find y1
    # I dont know what this is for PSD mode but maybe sum of PSD dets

    # if Yselect is non-blank try to set the ycolumns from that label list
    # if recovering a session however, Yselect should remain blank until after
    # recovery since files should have been previously read.

    set nY [llength $Yselect]
    # first check if Yselect is already the yColNames
    # check that Yselect columns are all in this dataset
    # check Y column settings

    if { ! $sessionrecover && $nynam > 0 && ! [sameLists $Yselect $ynames] && \
	     [columnLabelsExist $iset $Yselect] } {
	for {set i 0;set j 1} {$i < $nY} {incr i; incr j} {
	    set yselect [lindex $Yselect $i]
	    pipeWrite "c$iset ad y$j $yselect"
	    
	    # read it back to confirm
	    pipeWrite "c al y$j"
	    
	    if { [scan [lindex $readlines 1] {%s %d %d %s} \
		      typ indx col lbl] < 4 ||
		 ! [string match $lbl $yselect] } {
		append errs \
		"FILEREAD ERROR: failed to assign Y$j to $yselect in $fshort\n"
		return {}
	    }
	    # make sure df == 0 to force new file write if we are
	    # directly assigning y columns here
	    set df 0
	}
	if { $nY > 0 } {
	    pipeWrite "d$iset ny $nY"
	    
	    # also need to recompute errors
	    pipeWrite "c err"
	    
	}
	set ynames [yColNames $iset]
	if { [llength $ynames] < 1 } {
	    append errs "FILEREAD ERROR: failed to get Y column labels\n"
	    return {}
	}
	set ny [llength $ynames]
    }

    # check for DetectorEfficiencies in ICE files
    pipeWrite "d h #DetectorEfficiencies"
    
    if { $nread > 0 } {
	set effline [lindex $readlines 0]
	set yeff {}
	foreach yn $ynames {
	    if [regexp "$yn=(\[0-9\.\]+)" $effline wm eff] {
		lappend yeff $eff
	    } else {
		lappend yeff 1
	    }
	}
	pipeWrite "c y [join $yeff]"
	
    }

    # use the filename of diskfile after writing
    # version number may change if root already exists
    # can use df=0 to indicate that this is a new dataset
    # not yet on disk
    # filenames can be duplicate in different groups.
    set src [file tail $src]

    if { $dw && ($df == 0 || ! [string match $src $fshort]) } {
	# this means columns hd etc were added so write to disk
	# write the updated file to disk
	pipeWrite "d$iset w"
	
	# get back the actual disk file name
	
	pipeWrite "d$iset"
	
	set rl [lindex $readlines 1]
	set nscan [scan [string trimleft $rl "* "] {%d %d %d %d %d %d %s} \
		       iset npts ncol ny nas df src]
	if { $nscan < 7 } {
	    append errs \
		"FILEREAD ERROR: failed diskfilename for $fshort from $rl\n"
	    return {}
	}
	
	set fshort [file tail $src]
	set fname [file join $currentDir $fshort]
    }

    set dsFile($fshort) $iset
    set srcfiles($iset) $fshort
    # rebuild toleranc menu
    if { $newfiles } { buildTolMenu }
    # return list
    return [list $iset $npts $ncol $ny $nas $df $src $fshort $fname]
}

proc overlapLists {list1 list2} {
    set n1 [llength $list1]
    set n2 [llength $list2]
    if { $n1 == 0 || $n2 == 0 } { return {} }
    set res {}
    foreach l1 $list1 {
	foreach l2 $list2 {
	    if { [string match $l1 $l2] } { lappend res $l1; break }
	}
    }
    return $res
}

proc sameLists {list1 list2} {
    set n1 [llength $list1]
    set n2 [llength $list2]
    if { $n1 != $n2 } { return 0 }
    foreach l1 $list1 l2 $list2 {
	if { ! [string match $l1 $l2] } { return 0 }
    }
    return 1
}

proc columnLabelsExist {ds lblst} {
    # check that column labels from lblst exist for dataset ds
    # return 0 if any lbls dont exist
    global readlines readfinished
    pipeWrite "d$ds"
    
    foreach lbl $lblst {
	pipeWrite "c ls $lbl"
	
	scan [lindex $readlines 0] {%s %s %d} dum lbl icol
	if { $icol <= 0 } { return 0 }
    }
    return 1
}

proc applyYselectGroup {grp} {
    # apply Yselect to group by cloning to new datasets if necessary
    # then writing new disk files
    global gnum Yselect dsFile lbxs lbxi dsFile
    global readfinished readlines

    if { $grp < 1 } { set grp $gnum }
    set nY [llength $Yselect]
    if { $nY < 1 } {
	Dialog_Warn "No y columns specified in Yselect.\n"
	return
    }

    # get the files for this group listed in lbxs(0)
    set fls [getFilesInGroup $grp]
    # recall getFilesInGroup also prepares the listbox indices in lbxi
    if { [llength $fls] < 1 } {
	Dialog_Warn "group $grp has no files to apply Yselect to.\n"
	return
    }
    # if all the files already are Yselect do nothing
    set dif 0
    foreach fl $fls {
	set dsn $dsFile($fl)
	if { $dsn < 1 } {
	    # this shouldnt happen, but check anyway
	    Dialog_Warn "failed to get dataset number for file: $fl"
	    return
	}
	set ynames [yColNames $dsn]
	if { ! [sameLists $ynames $Yselect] } {
	    set dif 1
	    break
	}
    }
    if { ! $dif } { return }

    # repeat and clone the datasets that are different to new datasets
    # with new filenames and replace the filenames, ny in lbxs
    set errs ""
    foreach fl $fls i $lbxi {
	set dsn $dsFile($fl)
	if { $dsn < 1 } {
	    # this shouldnt happen, but check anyway
	    Dialog_Warn "failed to get dataset number for file: $fl"
	    return
	}
	# check that Yselect columns all exist for dsn
	if { ! [columnLabelsExist $dsn $Yselect] } {
	    append errs \
		"Cant set $Yselect columns for set $dsn. Labels missing.\n"
	    continue
	}
	set ynames [yColNames $dsn]
	#puts "file $fl set $dsn ynames=$ynames Yselect=$Yselect"
	if { [sameLists $ynames $Yselect] } { continue }
	# clone dsn with new filename $fl\.y
	# why clone new sets?? Why not just direct assign y

	# try skip cloning
	# set newfl $fl\.y
	# if { ! [set newds [cloneSet $dsn $newfl]] } { return }

	# set nY for the new set
	# this becomes set y for current set dsn
	pipeWrite "d$dsn ny $nY"
	

	# NOTANYMOREnewds becomes the current dataset
	# directly assign the Yselect columns
	# this is similar to handling Yselect in readFile

	#puts "applyY: dsn=$dsn for file= $fl new clone set= $newds"
	for {set ii 0;set j 1} {$ii < $nY} {incr ii; incr j} {
	    #puts "assign direct y$j to [lindex $Yselect $ii]"
	    pipeWrite "c ad y$j [lindex $Yselect $ii]"
	    
	}
	# must update data y errors
	pipeWrite "c errs"
	

	# avoid writing to disk as this will change filename
	# and make solnfiles invalid
	# NOrewrite the dataset to disk NOTnew dataset to disk
	#pipeWrite "d$dsn w"
	#pipeWrite "d$newds w"
	#

	# get back the actual disk file name
	
	pipeWrite "d$dsn"
	
	set rl [lindex $readlines 1]
	set nscan [scan [string trimleft $rl "* "] {%d %d %d %d %d %d %s} \
		       iset npts ncol ny nas df src]
	if { $nscan < 7 } {
	    Dialog_Warn \
		"FILEREAD ERROR: failed diskfilename for $newfl from $rl\n"
	}
	# NB d$dsn returns the filename for file that would be written to disk
	# which is diff from orig because we have modified col assigns
	# we are no longer writing to disk which would change filename
	#set dsFile($src) $newds
	#set dsFile($src) $dsn
	# replace the listbox filename and ny
	#$lbxs(0) delete $i
	#$lbxs(0) insert $i $src
	$lbxs(4) delete $i
	$lbxs(4) insert $i $nY
    }
    if { [string length $errs] > 0 } {
	Dialog_Warn $errs
    }
    updateSrcFileMenus
}

proc isblankline {line} {
    if { [string length [string trim $line]] < 1 } { return 1 }
    return 0
}

proc yColNames {iset} {
    # return the column names of detectors
    global nread readfinished readlines
    # list all y assignments
    pipeWrite "c$iset al y"
    
    set ynames {}
    # be careful, pipeRead now passes blank lines
    # should make sure pbscript doesnt add any
    for {set i 1} {$i < $nread} {incr i} {
	set rl [lindex $readlines $i]
	if [isblankline $rl] { continue }
	if { [scan [lindex $readlines $i] {%s %d %d %s} typ indx col lbl] < 4} {
	    return {}
	}
	lappend ynames $lbl
    }
    return $ynames
}

proc checkInstr {fshort} {
    global instrType
    global monitor monofilter
    global nDetectors
    # when reading files allowed to set tolerances
    # instrType is -2=undefined at startup or -1=if user edits tolerances
    # set to 0=zero for defltStndTol and 1 for MACSdefltStndTol = defltStndTol*33
    # RWE Mar 2019 DO NOT setdefltTol on file read.
    # The user can always use the defltTol button
    # but setting deflt on every fileread obfuscates the users settings
    set newtype 0
    if [string match -nocase *\.ng0* $fshort] { set newtype 1 }
    #if { ![expr $instrType == -1] && ![expr $instrType == $newtype] }
    set instrType $newtype
        #setdefltTol
        # also set the monitor position and Be filter for MACS
    if {$instrType == 1} {
        	set monofilter "Be filter"
        	set monitor "before He3 pol"
        	if { $nDetectors != 20 } {
        		set nDetectors 20
        		updateDetectorMenu
        	}
    } else {
        	set monitor "after He3 pol"
        	if { $nDetectors != 1 } {
        		set nDetectors 1
        		updateDetectorMenu
        	}

    }   
    #updateDetectorMenu
}

proc readFiles {args} {
    # read to current group files in [lindex $args 0]
    # else with no filelist in args get the selection from
    # the data selector listbox sellectlist
    # args is used for session recovery
    # everything in the lbxs can be recovered by reading the files
    # except for fbg, which is saved in group in array fbgFile(shortfilename)
    global sellectlist
    global currentDir
    global readfinished
    global readlines nread
    global errlines nerr
    global warnlines nwarn
    global colsrch
    global lbxnames lbxlabels
    global gnum
    global lbxs
    global lbxpad
    global fspb
    global dsFile fbgFile
    global colds colRanges isfop
    global errs
    global newfiles configQA configSC
    
    set configQA 0
    if {[$lbxs(0) size] < 1} {set configQA 1}

    set fillst {}
    if { [llength $args] < 1 } {
			set flstind [$sellectlist curselection]
			if { [llength $flstind] < 1 } {
	    	Dialog_Warn "NO files selected.\n"
	    	return
			}
			foreach i [lsort -integer $flstind] {
	    	set fshort [$sellectlist get $i]
	    	lappend fillst $fshort
			}
    } else {
			set fillst [lindex $args 0]
			#puts "set fillst from [lindex $args 0]= $fillst"
    }
    if { [llength $fillst] < 1 } {
			Dialog_Warn "NO files in filelist for reading.\n"
			return
    }

    set values(fnam) {}
    set values(gnum) {}
    set values(ts) {}
    set values(npts) {}
    set values(ny) {}
    set values(Fix) {}
    set values(ref) {}
    set values(fbg) {}
    set values(y) {}

    foreach i $colsrch { set values($i) {} }

    set nf 0
    set errs ""
    # newfiles controls when tolerance menu rebuilds
    set newfiles 1
    foreach fil $fillst {
	    set ret [readFile $fil 1]
	    # continue on fatal read error
	    #puts "fil=$fil readFile returned $ret"
	    if { [llength $ret] < 1 } { continue }
	    set newfiles 0
	    #returns [list $iset $npts $ncol $ny $nas $df $src $fshort $fname]
	    set iset [lindex $ret 0]
	    set npts [lindex $ret 1]
	    set ncol [lindex $ret 2]
	    set ny [lindex $ret 3]
	    set df [lindex $ret 5]
	    set fshort [lindex $ret 7]
	    set fname [lindex $ret 8]


	    lappend values(ny) $ny

	    # get Fix from Fixn hd
	    pipeWrite "d d Fixn"
	
    	# 3rd line: symbol value found
    	if { [llength $readlines] < 3 } {
	        append errs "FILEREAD ERROR: HD Fixn not found in $fshort\n"
	    }
	    set hd [lindex $readlines 2]
	    set fnd [lindex $hd 1]
	    if { $fnd == 1 } {
	        lappend values(Fix) "Ef"
	    } elseif { $fnd == 0 } {
	        lappend values(Fix) "Ei"
	    } else {
	        lappend values(Fix) "NA"
	    }

	    pipeWrite "d d refn"
	
	    # 3rd line: symbol value found
	    if { [llength $readlines] < 3 } {
	        append errs "FILEREAD WARNING: HD refn not found in $fshort\n"
	    }
	    set hd [lindex $readlines 2]
	    set fnd [lindex $hd 1]
	    if { $fnd == 2 } {
	        lappend values(ref) "mon"
	    } elseif { $fnd == 1 } {
	        lappend values(ref) "time"
	    } else {
	        lappend values(ref) "NA"
	    }

	    # get the assigned data columns and data ranges
	    set nc [readCurrentColAssigns]
    
	    if { $nc < 10 } {
	        append errs \
	    	"FILEREAD WARNING: assigned less than 10 columns in $fname\n"
	        continue
	    }

	    incr nf

	    lappend values(fnam) $fshort
	    lappend values(gnum) $gnum
	    lappend values(npts) $npts

	    if { ! [info exists fbgFile($fshort)] } {
	        set fbgFile($fshort) 0
	    }

	    lappend values(fbg) $fbgFile($fshort)
    
	    # now collect the stats for each column.
	    # dont have to report missing as they will show blank in the table
	    # set colsrch 
	    #  x1 x2 x3 En fs H V Temp scl
	    # only record stats for first detector = y1
	    # colds(colsrch) records the column numbers foreach search column
	    
	    # if this is first file read, check if Qx Qy Qx = x1 x2 x3 are all nonzero
	    # It this test fails reset the gui to use angles  tts om ttm tta  instead of Qx Qy Qz E  in column indices 6 7 8 9

		if {[$lbxs(0) size] < 1} {
			set configQA 0
			#set x1exists [info exists colds(x1)]
			#puts "colds(x1) exists= $x1exists"
			#if { $x1exists } {puts "colds(x1)= $colds(x1)  rangeValues= $colRanges(x1)"}
			# deflt is to use Qxyz, but if any are not found or are NA or if all are zeros go to angles
			if {![info exists colds(x1)] || ![info exists colds(x2)] || ![info exists colds(x3)]} {
				set configQA 1
			}
			set x1v $colRanges(x1)
			set x2v $colRanges(x2)
			set x3v $colRanges(x3)
			if { [lindex $x1v end] == "N/A" || [lindex $x2v end] == "N/A" || [lindex $x3v end] == "N/A"} {
				set configQA 1
			} elseif { [lindex $x1v end] == "0" && [lindex $x2v end] == "0" && [lindex $x3v end] == "0"} {
				set configQA 1
			}
			if {$configQA} {
				initLbxAngles
			}
			# if both H and V are missing then assign Bfld to V
			set configSC 0
			if {![info exists colds(V)] && ![info exists colds(H)]} {
				set configSC 1
				initLbxSC
			}
		}

    
	    foreach i $colsrch {
	        if { [info exists colds($i)] && $colds($i) > 0 } {
	        	if { ! [string compare $i "fs"] } {
	        	    # handle flipper states, find all used
	        	    pipeWrite "c sd $colds($i)"
		    
	        	    # line 2 has all distinct values
	        	    # use fspb to translate and order offoff onoff offon onon
	        	    set stat [lindex $readlines 1]
	        	    set fstates {}
	        	    foreach fsti {1 3 4 2} {
	            		if { [lsearch -exact $stat $fsti] < 0 } { continue }
	            		lappend fstates $fspb($fsti)
	        	    }
		            if { [llength $fstates] == 4 } { set fstates ALL }
		            lappend values($i) $fstates
		        } else {
		            lappend values($i) $colRanges($i)
		        }
    	    } else {
		        lappend values($i) ""
	        }
	    }

	    # handle detectors y columns
	    for {set i 1} {$i <= $ny} {incr i} {
	        set yn y$i
	        if { [info exists colds($yn)] && $colds($yn) > 0 } {
		        set yimn [lindex $colRanges($yn) 0]
		        set yimx [lindex $colRanges($yn) 1]
	        } else {
		        set yimn 0
		        set yimx 0
	        }
	        if { $i == 1 } {
		        set ymn $yimn
    		    set ymx $yimx
    	    } else {
        		if { $yimn < $ymn } { set ymn $yimn }
        		if { $yimx > $ymx } { set ymx $yimx }
    	    }
    	}
    	if { $ny > 0 } {
    	    lappend values(y) [list $ymn $ymx]
    	} else {
    	    lappend values(y) ""
    	}
    	if { [info exists colds(ts)] && $colds(ts) > 0 } {
    	    # use the min ts which is roughly start time for file
    	    lappend values(ts) [lindex $colRanges(ts) 0]
    	} else {
    	    lappend values(ts) 0
    	}
    }

    # now append the values to the lbxs
    # can use appendLbxDataArray here

    appendLbxDataArray values
    # sortGroupsIncreasing
    #sortGroupsIncreasing
    hiliteActiveGroup
    if { [string length $errs] > 0 } {
    	Dialog_Warn $errs
    }
    # finally updateBGjobs
    #puts "from readFiles caling updatePbcorJobs"
   	updatePbcorJobs
}

proc convertTTStoQ {args} {
	# NB This is currently NOT used
    # convert A4 scan to QX vector scan and write to file
    # returns the list of files converted
    # This is ICE specific
    # read an A4 ICE ANGLE scan file and convert to VECTOR scan
    # since ICE doesnt fill in Q E and FixedE????
    # putting in FixedE Qx = Q Qy=Qz=0
    # get E FixedE from Ei from A2 MonoDsp  Ef from A6 AnalDsp
    # this means change columns QX QY QZ E to numeric ones
    # QX = (4Pi/lambda)sin(thetaS)
    # also fixup header: FixedE  Ei|Ef value
    # ScanType VECTOR
    # Scan  column Label

    global sellectlist
    global readlines readfinished nread errs

    set fillst {}
    if { [llength $args] < 1 } {
	set flstind [$sellectlist curselection]
	if { [llength $flstind] < 1 } {
	    Dialog_Warn "NO files selected.\n"
	    return 0
	}
	foreach i [lsort -integer $flstind] {
	    set fshort [$sellectlist get $i]
	    lappend fillst $fshort
	}
    } else {
	set fillst [lindex $args 0]
    }
    if { [llength $fillst] < 1 } {
	Dialog_Warn "NO files in filelist for reading.\n"
	return 0
    }


    # to read angle scan A4 files with no FixedE must turn off PBcheckCols
    pipeWrite "f@ 0"
    

    #set errs ""
    set fconv {}
    foreach fil $fillst {
	# do NOT write the file back to disk which would make it DFlike
	#puts "convertTTStoQ about to read raw $fil"
	set ret [readFile $fil 0]
	#puts "convertTTStoQ read raw $fil returned"
	# continue on fatal read error
	if { [llength $ret] < 1 } { continue }
	#returns [list $iset $npts $ncol $ny $nas $df $src $fshort $fname]
	set iset [lindex $ret 0]
	set npts [lindex $ret 1]
	set ncol [lindex $ret 2]
	set df [lindex $ret 5]
	set fshort [lindex $ret 7]
	set fname [lindex $ret 8]
	# make sure this is current dataset
	pipeWrite "d$iset"
	
	pipeWrite "d h \"#Scan \""
	
	if { $nread < 1 || [string first A4 [lindex $readlines 0]] < 0 } {
	    append errs \
		"FILEREAD ERROR: ICE file A4 is not scanned in $fshort\n"
	    continue
	}
	
	# edit all the bogus N/A columns from an ICE ANGLE scan, rem app nums
	foreach na {QX1 QY1 QZ1 E1} {
	    pipeWrite "c s $na"
	    
	    if { $nread < 2 } { continue }
	    # delete the column
	    pipeWrite "c n- $na"
	    
	}
	# change the data type to numberic for QX QY QZ E
	set qok 1
	foreach na {QX QY QZ E} {
	    pipeWrite "c s $na"
	    
	    if { $nread < 2 } {
		append errs \
		    "FILEREAD ERROR: missing $na column in $fshort\n"
		set qok 0
		continue
	    }
	    # reset the column to numeric
	    pipeWrite "c dt $na 0"
	    
	}
	if { ! $qok } { continue }

	# need A2 A6 to get Ei Ef -> E FixedE
	# and  A4 to get new QX values
	set aok 1
	foreach ca {A2 A4 A6} {
	    pipeWrite "c s $ca"
	    
	    if { $nread < 2 || \
		     [scan [lindex $readlines 1] {%d %s %g %g %d %d %g} \
			  icol lbl mn mx nch var ave ] < 7 } {
		append errs \
		    "FILEREAD ERROR: missing $ca column in $fshort\n"
		set aok 0
		continue
	    }
	    set varies($ca) $var
	    set aver($ca) $ave
	}
	if { ! $aok } { continue }
	if { ! $varies(A4) } {
	    append errs \
		"FILEREAD ERROR: A4 doesnt vary in $fshort\n"
	    continue
	}
	set Pi [expr acos(-1.)]
	set angTOrad [expr $Pi/180.]
	set Dn 2.072141789

	pipeWrite "d d Dm"
	
	if { [scan [lindex $readlines 2] {%s %g %d} sym Dm fnd] < 3 || \
		 ! $fnd || $Dm <= 0. } {
	    append errs "FILEREAD ERROR: couldnt get Dsp-mono in $fshort\n"
	    continue
	}
	pipeWrite "d d Da"
	
	if { [scan [lindex $readlines 2] {%s %g %d} sym Da fnd] < 3 || \
		 ! $fnd || $Da <= 0. } {
	    append errs "FILEREAD ERROR: couldnt get Dsp-anal in $fshort\n"
	    continue
	}
	pipeWrite "c dl A2"
	
	set a2lst [lreplace [lindex $readlines 0] 0 0]
	pipeWrite "c dl A4"
	
	set a4lst [lreplace [lindex $readlines 0] 0 0]
	pipeWrite "c dl A6"
	
	set a6lst [lreplace [lindex $readlines 0] 0 0]

	# make new E column and set the data values
	#pipeWrite "c n QX QY QZ E"
	#

	for {set i 0; set j 1} {$i < $npts} {incr i; incr j} {
	    set ki [expr $Pi/$Dm/sin(0.5*$angTOrad*[lindex $a2lst $i])]
	    set Ei [expr $Dn * $ki * $ki]
	    set kf [expr $Pi/$Da/sin(0.5*$angTOrad*[lindex $a6lst $i])]
	    set Ef [expr $Dn * $kf * $kf]
	    set E [expr $Ei - $Ef]
	    pipeWrite "c de E $j $E"
	    
	}	    
	if { ! $varies(A2) } {
	    # Ei fixed. calc from MonoSpacing
	    # lamb = 2 Dm sin(A2/2) E = Dn (2Pi/lamb)^2
	    # E = Dn (Pi/(Dm sin(A2/2)))^2
	    set kfix $ki
	    set Efix $Ei
	    # replace FixedE in the header
	    pipeWrite "d hr #FixedE \"#FixedE    Ei $Efix\""
	    
	} elseif { ! $varies(A6) } {
	    # lamb = 2 Dm sin(A2/2) E = Dn (2Pi/lamb)^2
	    # E = Dn (Pi/(Dm sin(A2/2)))^2
	    set kfix $kf
	    set Efix $Ef
	    # replace FixedE in the header
	    pipeWrite "d hr #FixedE \"#FixedE    Ef $Efix\""
	    
	} else {
	    append errs \
		"FILEREAD ERROR: couldn't get fixed A2 or A6 in $fshort\n"
	    continue
	}
	# now calc QX = 2 kfix sin(A4/2)
	# get its col number
	pipeWrite "c l QX"
	
	if { [scan [lindex $readlines 1] {%d %s} cqx lbl] < 2 } {
	    append errs \
		"FILEREAD ERROR: couldn't get QX column in $fshort\n"
	    continue
	}

	for {set i 0; set j 1} {$i < $npts} {incr i; incr j} {
	    set Q [expr 2.*$kfix*sin(0.5*$angTOrad*[lindex $a4lst $i])]
	    pipeWrite "c de QX $j $Q"
	    
	    pipeWrite "c de QY $j 0."
	    
	    pipeWrite "c de QZ $j 0."
	    
	}	

	# fix ScanType and Scan
	pipeWrite "d hr #ScanType \"#ScanType VECTOR\""
	
	pipeWrite "d hr \"#Scan \" \"#Scan $cqx QX\""
	
	pipeWrite "d hr #ScanRanges \"#ScanRanges $cqx QX\""
	

	# finally write this as raw nonDF .Q file and delete the set
	pipeWrite "d nr $fshort.Q"
	
	pipeWrite "d wr"
	
	#puts "convertTTStoQ wrote raw $fshort.Q"
	pipeWrite "d-$iset"
	
	lappend fconv $fshort.Q
    }
    # this is always called from readFile so just accumulate errors
    #if { [string length $errs] > 0 }
    #Dialog_Warn $errs

    # turn checkCols back on
    pipeWrite "f@ 1"
    
    #puts "cponverTTStoQ finishing with raw mode off"
    return $fconv
}

proc setLbxWidth {lbw lbl} {
    global lbxpad lbxfixedwidth
    set lblw [string length $lbl]
    if { [info exists lbxfixedwidth($lbw)] && $lbxfixedwidth($lbw) > 0 } {
	set wid $lbxfixedwidth($lbw)
	if { $wid < $lblw } { set wid $lblw }
	$lbw configure -width $wid
	return
    }
    set lbxw [maxListItemLength [$lbw get 0 end]]
    if { $lblw > $lbxw } { set lbxw $lblw }
    set wid [expr $lbxw + $lbxpad]
    $lbw configure -width $wid
}

proc lbxIndicesForGroup {ig} {
    global lbxs
    set gvals [$lbxs(1) get 0 end]
    set indices {}
    for {set i 0} {$i < [llength $gvals]} {incr i} {
	if { [lindex $gvals $i] == $ig } { lappend indices $i }
    }
    return $indices
}

# use the files listbox hilite green to indicate active group
# use the group number listbox to hilite green for calc
# and yellow for fop
proc hiliteActiveGroup {} {
    global gnum calc calcHilite
    global lbxs
    set gvals [$lbxs(1) get 0 end]
    for {set i 0} {$i < [llength $gvals]} {incr i} {
	if { [lindex $gvals $i] == $gnum } {
	    $lbxs(0) itemconfigure $i -bg green
	    #$lbxs(1) itemconfigure $i -bg $calcHilite($calc)
	} else {
	    $lbxs(0) itemconfigure $i -bg white
	}
    }
}

proc hiliteCalcGroup {} {
    # apply hilite to current gnum
    global gnum calc
    global lbxs
    global calcHilite
    set gvals [$lbxs(1) get 0 end]
    for {set i 0} {$i < [llength $gvals]} {incr i} {
	if { [lindex $gvals $i] == $gnum } {
	    $lbxs(1) itemconfigure $i -bg $calcHilite($calc)
	}
    }
}

proc hiliteGroups {} {
    global calcgroup fopgroup lbxs
    set gns [$lbxs(1) get 0 end]
    for {set i 0} {$i < [llength $gns]} {incr i} {
	$lbxs(1) itemconfigure $i -bg white
	set gn [lindex $gns $i]
	if { [info exists calcgroup($gn)] && $calcgroup($gn) } {
	    $lbxs(1) itemconfigure $i -bg green
	} elseif { [info exists fopgroup($gn)] && $fopgroup($gn) } {
	    $lbxs(1) itemconfigure $i -bg red
	}
    }
}

proc hiliteFopGroup {} {
    # apply hilite to current gnum
    global gnum calc isfop
    global lbxs
    global fopHilite
    set gvals [$lbxs(1) get 0 end]
    for {set i 0} {$i < [llength $gvals]} {incr i} {if { $mswindows } {
    set windowCaptureProgram "C:\\Windows\\system32\\SnippingTool.exe"
} else {
    # unices look for /usr/bin/import
    set windowCaptureProgram "/usr/bin/import"
}

	if { [lindex $gvals $i] == $gnum } {
	    $lbxs(1) itemconfigure $i -bg $fopHilite($isfop)
	}
    }
}


proc maxListItemLength {lst} {
    set maxl 0
    foreach i $lst {
	set ilen [string length $i]
	if { $ilen > $maxl } { set maxl $ilen }
    }
    return $maxl
}

proc doFilter {w} {
    global fileFilter
    $w delete 0 end
    set flst [glob -nocomplain $fileFilter]
    foreach i [lsort $flst] {
	if { [file isfile $i] == 0 || [file executable $i] } { continue }
	$w insert end $i
    }
}

##################### main group selector procs ##########

proc captureWindow { win } {

   	regexp {([0-9]*)x([0-9]*)\+([0-9]*)\+([0-9]*)} [winfo geometry $win] - w h x y

   	# Make the base image based on the window
   	set image [image create photo -format window -data $win]
  
   	foreach child [winfo children $win] {
    	 captureWindowSub $child $image 0 0
   	}

   	return $image
}

proc captureWindowSub { win image px py } {
	#puts "captureWindowSub win=$win image=$image px=$px py=$py"
 	if {![winfo ismapped $win]} {
 		#puts "captureWindowSub $win NOT MAPPED"
 	    return
   	}

	regexp {([0-9]*)x([0-9]*)\+([0-9]*)\+([0-9]*)} [winfo geometry $win] - w h x y
	#puts "captureWindowSub win=$win offsets x=$x y=$y"
   	incr px $x
   	incr py $y

   	# Make an image from this widget
   	set tempImage [image create photo -format window -data $win]
  
   	# Copy this image into place on the main image
   	$image copy $tempImage -to $px $py
   	image delete $tempImage

   	foreach child [winfo children $win] {
     	captureWindowSub $child $image $px $py
   	}
}
#David Easton 17 Mar 2003 Here is a demo for above the above that creates a window and saves the screenshot to a file, when the user presses the 'x' key in the window.
# I can only get gif to work so don't offer any other choices

# RWE 2019 still have trouble getting tk programs like Img package to work
# so move to external programs.
# Windows snipper

if { $mswindows } {
    set windowCaptureProgram "C:\\Windows\\system32\\SnippingTool.exe"
} else {
    # unices look for /usr/bin/import
    set windowCaptureProgram "/usr/bin/import"
}

proc runWindowCapture {} {
    global mswindows windowCaptureProgram windowCaptureFile
    if { [string length $windowCaptureProgram] < 1 || ![file executable $windowCaptureProgram] } {
        Dialog_Warn "ERROR: cant execute windowCaptureProgram= $windowCaptureProgram"
        return
    }
    if { $mswindows } {
        if { [catch { exec $windowCaptureProgram & } msg] } {
            Dialog_Warn $msg
        }
        return
    }
    # on UNIX we need the output filename to supply the format using import
    if { [string length $windowCaptureFile] < 1 } {
        set windowCaptureFile windowCapture
    }
    if { [catch { exec $windowCaptureProgram $windowCaptureFile & } msg] } {
        Dialog_Warn $msg
    }
    return
}

proc chooseSaveDir {} {
	global saveWindowDir
    set selDir [tk_chooseDirectory \
			-title "choose save hardcopy directory"]
    if { [string length [string trim $selDir]] < 1 } { return }
    set saveWindowDir $selDir
}
proc saveCurrentPlot {} {
	global gnum
	windowToFile .g$gnum
}

proc windowToFile { win args } {
	# pass the window tk path as win
	global saveWindowDir graphicsFmt
  	set image [captureWindow $win]

   	if [catch {set title [wm title $win]}] {
   		set title $win
   	}
   	# replace all spaces in title with underscore
   	regsub -all { } $title _ title
   	if {[llength $args] > 0} {
   		set dr [lindex $args 0]
   		if [file isdirectory $dr] {
   			set saveWindowDir $dr
   		}
   	}
   	set filename [file join $saveWindowDir $title.$graphicsFmt]
   	#set filename [tk_getSaveFile -filetypes $types \
   	#                              -initialfile capture.gif \
   	#                             -defaultextension .gif]

   	if {[llength $filename]} {
       	$image write -format $graphicsFmt $filename
       	#puts "Written to file: $filename"
   	} else {
   		#puts "Write cancelled"
   	}
   	image delete $image
}

# start working on scrolledlistboxes for grid geo manager
# NB to prevent resizing a grid element set  -weight 0
# parent frame should be grid managed
# as scrolledlistboxes may add scrollbar using grid
# NB grid cant resize columns arbitrarily so may have to use -weight
# write separate proc to resize lbxs according to width of contents


# RWE Jan 2011 get rid of reconfiguring the main app window
# by putting auto-config into main edit menu
# and fixing height lbx height at 37 lines


proc setLbxHeight {nl} {
    global lbxs nlbxs
    for {set i 0} {$i < $nlbxs} {incr i} {
	$lbxs($i) configure -height $nl
    }
}

# RWE Jan 2011 build and map file selector and group options
proc buildLeft {} {
    global bframes
    for {set i 1} {$i <= 2} {incr i} {
		if { ! [winfo ismapped $bframes($i)] } { grid $bframes($i) }
	}
}

proc buildButtonConfig {ib} {
    # call with ib==1 when bld(1) is pressed
    # call with ib==2 when bld(2) is pressed
    # call with ib==0 when heightlines changes
    # store the button actions in bexe(1) and bexe(2)
    # the action values can be 1 2 or 3 indicating
    # display frame f/g/a
    # when ib==1,2 which frame f/g/a gets unviewed
    # isview(1-3)=0/1 are view states of f/g/a
    # if a bld button is removed its bexe(ib)=0
    # so we can tell if two, one or no buttons are displayed

    # there are two build buttons
    # if heightlines == 1 both buttons are displayed
    # with the names of the two non-viewed frame types
    # if heightlines == 2 only one button is displayed
    # if heightlines >= 3 no buttons are needed

    # keep track of which frames, top/mid/bot are viewed
    # with isview(f) isview(g) isview(a)
    # to remove one of the frames from view, grid remove thatframe
    # and to restore it, grid thatframe

    # with two buttons, only one of isview is active
    # swap the viewed one with the one requested by the button

    # with one button, two of isview are active.
    # unview the lowest priority window f/g/a is priority order


    global heightlines
    global bld bldtext bframes
    global bexe isview
    global lbxs nlbxs

    if { $ib > 0 } {
	# find the lowest priority viewed window
	for {set i 3} {$i >= 1} {incr i -1} {
	    if { $isview($i) } {
		set irem $i
		break
	    }
	}
	# clear irem frame
	if { [winfo ismapped $bframes($irem)] } { grid remove $bframes($irem) }
	# view the one requested by the button
	set iview $bexe($ib)
	if { ! [winfo ismapped $bframes($iview)] } { grid $bframes($iview) }
	# reconfigure the ib button to view the one removed
	$bld($ib) configure -text $bldtext($irem)

	set isview($irem) 0
	set isview($iview) 1
	set bexe($ib) $irem
	return
    }

    # now handle heightlines
    # by starting from a standard configuration
    for {set i 1} {$i <= 2} {incr i} {
	if { [winfo ismapped $bld($i)] } { grid remove $bld($i) }
	set bexe($i) 0
    }

    set nview $heightlines
    if { $nview > 3 } { set nview 3 }
    for {set i 1} {$i <= $nview} {incr i} {
	if {! [winfo ismapped $bframes($i)] } { grid $bframes($i) }
	set isview($i) 1
    }
    for {set i [expr $nview + 1] ; set nb 1} {$i <= 3} {incr i ; incr nb} {
	if { [winfo ismapped $bframes($i)] } { grid remove $bframes($i) }
	set isview($i) 0
	grid $bld($nb)
	$bld($nb) configure -text $bldtext($i)
	set bexe($nb) $i
    } 

    set nl 10
    if { $heightlines == 2 } { set nl 28 }
    if { $heightlines >= 3 } { set nl 37 }
    for {set i 0} {$i < $nlbxs} {incr i} {
	$lbxs($i) configure -height $nl
    }
}

proc nptsFormat {args} {
    global curnpts npts
    set nc [string length $npts]
    if { $nc > 0 } { $curnpts configure -width $nc }
}
proc nmsrFormat {args} {
    global curnmsr nmsr
    set nc [string length $nmsr]
    if { $nc > 0 } { $curnmsr configure -width $nc }
}

proc setTempTolFromRanges {all} {
    global lbxsr stndTolArr templbx toltype tolerance
    if { ! $all } {
		set sel [selectionOwned 0 4 0]
		if { [set n [llength $sel]] < 1 } {
	    	Dialog_Warn "NO selection in group-ranges listboxes.\n"
	    	return
		}
		set trs {}
		for {set i 0} {$i < $n} {incr i} {
	    	lappend trs [$lbxsr($templbx) get [lindex $sel $i]]
		}
    } else {
		set trs [$lbxsr($templbx) get 0 end]
		set n [llength $trs]
    }
    if { $n < 1 } { return }
    set tr [lindex $trs 0]
    for {set i 1} {$i < $n} {incr i} {
		set tr [updateNumRange [lindex $trs $i] $tr]
    }
    if { [llength $tr] < 1 } {
		set stndTolArr(Temp) 1.
    } else {
		set tmn [lindex $tr 0]
		set tmx $tmn
		if { [llength $tr] > 1 } { set tmx [lindex $tr 1] }
		set stndTolArr(Temp) [expr 1.1*abs($tmx - $tmn)]
    }
    if { [string match $toltype Temp] } { set tolerance $stndTolArr(Temp) }
}
proc setGrpScale {all} {
    global lbxsr grpScales grpScale
    if { ! $all } {
	set sel [selectionOwned 0 4 0]
	if { [set n [llength $sel]] < 1 } {
	    Dialog_Warn "NO selection in group-info listboxes.\n"
	    return
	}
	set gs {}
	for {set i 0} {$i < $n} {incr i} {
	    lappend gs [$lbxsr(0) get [lindex $sel $i]]
	}
    } else {
	set gs [$lbxsr(0) get 0 end]
	set n [llength $gs]
    }
    if { $n < 1 } { return }

    for {set i 0} {$i < $n} {incr i} {
	set g [lindex $gs $i]
	set grpScales($g) $grpScale
    }
    showGroupRanges
}
proc setGrpTitle {} {
    global lbxsr grpTitles grpTitle group
    set sel [selectionOwned 0 4 0]
    if { [set n [llength $sel]] < 1 } {
	Dialog_Warn "NO selection in group-info listboxes.\n"
	return
    }
    set g [$lbxsr(0) get [lindex $sel 0]]
    if { ! [info exists group($g)] } { return }
    set grpTitles($g) $grpTitle
    showGroupRanges
}

proc buildGroupRanges {} {
    # group selector-like listbox with ranges for all columns per group
    global grpr
    global gselSB lbxsr nlbxsr lbxrlabels grpScale grpTitle
    global lbxLabels lbxWidgets

    set grpr [toplevel .grpr]
    set top [frame $grpr.top]
    set bot [frame $grpr.bot]
    grid $top -sticky news
    grid $bot -sticky news
    set  title $top.title
    #-font *-helvetica-bold-r-normal--*-240-*
    label $title -text "group info" \
	-font {helvetica 24 bold} \
	-foreground #f00
    set ttol [button $top.tol -text "set Temp tol from T-range of selGrps" \
		  -bg green -activebackground yellow \
		  -command "setTempTolFromRanges 0" -width 28]
    set ttola [button $top.tola -text "from allGrps" \
		  -bg green -activebackground yellow \
		  -command "setTempTolFromRanges 1" -width 8]

    set gscl [button $top.scl -text "set solnScl of selGrps" \
		  -bg green -activebackground yellow \
		  -command "setGrpScale 0" -width 20]
    set gscla [button $top.scla -text "of allGrps" \
		  -bg green -activebackground yellow \
		  -command "setGrpScale 1" -width 8]

    set gscle [entry $top.scle -textvariable grpScale -width 12]

    set gtitl [button $top.ptitl -text "set title of selGrp" \
		  -bg green -activebackground yellow \
		  -command setGrpTitle -width 18]
    set gtitle [entry $top.ptitle -textvariable grpTitle -width 24]

    grid $title  -row 0 -column 0 -sticky w
    grid columnconfigure $top  1 -minsize 30
    grid $ttol   -row 0 -column 2 -sticky e
    grid $ttola  -row 0 -column 3 -sticky e
    grid columnconfigure $top  4 -minsize 30
    grid $gscl   -row 0 -column 5 -sticky e
    grid $gscla  -row 0 -column 6 -sticky e
    grid $gscle  -row 0 -column 7 -sticky e
    grid columnconfigure $top  8 -minsize 20
    grid $gtitl  -row 0 -column 9 -sticky e
    grid $gtitle -row 0 -column 10 -sticky e
    

    set lbxrlabels {
  group nfile ts npts ny cts Qx Qy Qz E  Fix  fs H V T scl ref fbg solnScl title
    }
    set widths     {
  3     3     10 3    2  12  12 12 12 12 3    12 6 6 6 12  4   3   12      24
    }
    set sorts  {
	-integer
	-integer -real -ascii -ascii
	-ascii -ascii -ascii -ascii -ascii -ascii -ascii
	-real -real -real -real
	-ascii -real -real
	-ascii
    }
      
    array set lbxsr \
	[scrolledlistboxes $bot $lbxrlabels $widths $sorts $gselSB {}]
    wm title $grpr "PB group info"

    # enable changing lbx column widths
    set lbxLabels($bot) $lbxrlabels
    set lbxWidgets($bot) [array get lbxsr]
    for {set i 0} {$i < [llength $lbxrlabels]} {incr i} {
	set wnam $bot.name$i
	bind $wnam <ButtonRelease-2> "+ adjustLbxWid $bot $i"
	bind $wnam <ButtonRelease-4> "+ adjustLbxWid $bot $i"
    }
}

proc showGroupRanges {} {
    global lbxs lbxsr lbxrlabels grpr
    global grpScales grpTitles
    global readfinished readlines
    # go thru lbxs and find ranges for each group
    # gran(grpnum) { }
    if { ! [winfo exists $grpr] } { buildGroupRanges }
    wm deiconify $grpr
    raise $grpr
    foreach b [array names lbxsr] { $lbxsr($b) delete 0 end }
    set gns {}
    set gnx [lsort -integer -increasing [$lbxs(1) get 0 end]]
    set n [llength $gnx]
    if { $n < 1 } { return }
    set gn [lindex $gnx 0]
    lappend gns $gn
    for {set i 1} {$i < $n} {incr i} {
	set newgn [lindex $gnx $i]
	if { $newgn == $gn } { continue }
	lappend gns $newgn
	set gn $newgn
    }
    array set gran {}
    set numcols {2 3 5 6 7 8 9  12 13 14 15}
    # fbg is 17
    set txtcols {10 11 16 17}
    for {set i 0} {$i < [llength $gns]} {incr i} {
	set gn [lindex $gns $i]
	# collect ranges for this group lbxs 2-17
	# 2-9 are numeric ranges
	# 10=Fix Ei Ef 11=fs 12-15 numeric 16=reftype (mon or time) 17 text fbg
	unset gran
	set nf 0
	for {set j 0} {$j < $n} {incr j} {
	    if { [$lbxs(1) get $j] != $gn } { continue }
	    # first do the numeric column range updates
	    incr nf
	    foreach nc $numcols {
		set nr [$lbxs($nc) get $j]
		if { ! [info exists gran($nc)] } {
		    set gran($nc) $nr
		} else {
		    set gran($nc) [updateNumRange $nr $gran($nc)]
		}
	    }
	    # now handle 10=Fix 11=fs 16=reftype 17 fbg
	    foreach nc $txtcols {
		set nr [$lbxs($nc) get $j]
		if { ! [info exists gran($nc)] } {
		    set gran($nc) $nr
		} else {
		    set gran($nc) [updateTxtRange $nr $gran($nc)]
		}
	    }
	}

	# grp number of detectors solved
	pipeWrite "g$gn ly"
	
	if { [scan [lindex $readlines 0] {%s %d %s %d} dum idum nyl ny] < 4 } {
	    set ny 1
	}
	set gran(4) $ny

	# grp soln-scalefactor
	set gran(18) 0
	if { [info exists grpScales($gn)] && $grpScales($gn) > 0 } {
	    set gran(18) $grpScales($gn)
	}
	# grp title
	set gran(19) ""
	if { [info exists grpTitles($gn)] } {
	    set gran(19) $grpTitles($gn)
	}

	# insert the gran data into lbxsr
	$lbxsr(0) insert end $gn
	$lbxsr(1) insert end $nf
	for {set j 2} {$j < 20} {incr j} {
	    $lbxsr($j) insert end $gran($j)
	}
	for {set j 0} {$j < 20} {incr j} {
	    setLbxWidth $lbxsr($j) [lindex $lbxrlabels $j]
	}
    }
}

proc updateTxtRange {newr oldr} {
    set rlst $oldr
    foreach n $newr {
	if { [lsearch -exact $rlst $n] < 0 } { lappend rlst $n }
    }
    return $rlst
}

proc updateNumRange {newr oldr} {
    # cumulative update of min max numeric ranges
    if { [llength $oldr] < 1 } {
	return $newr
    } elseif { [llength $oldr] > 1 } {
	set oldmin [lindex $oldr 0]
	set oldmax [lindex $oldr 1]
    } else {
	set oldmin [lindex $oldr 0]
	set oldmax $oldmin
    }
    if { $oldmin > $oldmax } {
	set temp $oldmin
	set oldmin $oldmax
	set oldmax $temp
    }
    if { [llength $newr] > 1 } {
	set newmin [lindex $newr 0]
	set newmax [lindex $newr 1]
    } else {
	set newmin [lindex $newr 0]
	set newmax $newmin
    }
    if { $newmin > $newmax } {
	set temp $newmin
	set newmin $newmax
	set newmax $temp
    }

    set mn $oldmin
    if { $newmin < $oldmin } { set mn $newmin }
    set mx $oldmax
    if { $newmax > $oldmax } { set mx $newmax }
    if { $mx > $mn } {
	return [list $mn $mx]
    } else {
	return $mx
    }
}

proc setCanvasScrollRegion {c f} {
    # c is the canvas path, f is the frame in the canvas that we want to scroll
    # this proc is bound to Configure $f
    # so that when the frame reconfigs i.e.
    # listboxes change width etc the canvas scrollregion
    # will change along with them.
    # unfortunately this gets activated when scrolling the canvas
    # as there is no Resize binding,
    # so must trap for no-resize
    global lastCwidth lastCheight

    if { ! [info exists lastCwidth($c)] } {
	set lastCwidth($c) 0
	set lastCheight($c) 0
    }
    set bbox [grid bbox $f 0 0]
    set inc [lindex $bbox 2]
    set width [winfo reqwidth $f]
    set height [winfo reqheight $f]
    # trap for no resize
    if { $height == $lastCheight($c) && $width == $lastCwidth($c) } { return }
    $c config -height $height
    $c config -scrollregion "0 0 $width $height"
    $c config -xscrollincrement $inc
    set lastCheight($c) $height
    set lastCwidth($c) $width
    mainWindowSize
}
proc mainWindowSize {} {
    # handle user resizing group selector width
    # configures the group selector canvas clip view and the toplevel geometry
    global botc
    global mainwidth
    # the main windowSize submenu has 
    # for mainwidth
    # min width 1
    # med width 2
    # max width 3
    # small height  -> set heightlines 1
    # medium height -> set heightlines 2
    # max   height  -> set heightlines 3
    # for these options call setGrpLines 0 to update the main window height
    
    # setGrpLines 0 triggers setCanvasScrollRegion so dont need it here

    # now we want the actual canvas widget width to get resized 
    # to some fraction of the full scrollregion width

    # use the grid bbox on .a.right.top to get the minwidth for canvas size
    # and     canvas scrollregion for maxwidth
    # then just $botc configure -width newwidth
    set bbt [grid bbox .a.right.top]
    set bbc [$botc cget -scrollregion]
    set minwid [lindex $bbt 2]
    set maxwid [lindex $bbc 2]
    if { $maxwid < $minwid } { set maxwid $minwid }
    set dw [expr ($maxwid - $minwid)/2]
    if { $dw < 0 } { set dw 0 }
    set wid [expr $minwid + ($mainwidth - 1)*$dw]

    # this will set the canvas clip view region that gets scrolled
    $botc configure -width $wid
    update idletasks
    # also have wm request new size for the toplevel keeping the position
    set tlgeo [winfo geometry .]
    regsub -all {\+} $tlgeo " " tlgeo
    scan $tlgeo {%dx%d%d%d} W H wx wy
    # use .a geometry to get the main window height
    set ageo [winfo geometry .a]
    regsub -all {\+} $ageo " " ageo
    scan $ageo {%dx%d%d%d} aW aH awx awy
    # use the canvas width + the width of .a.left
    set lgeo [winfo geometry .a.left]
    regsub -all {\+} $lgeo " " lgeo
    scan $lgeo {%dx%d%d%d} lW lH lwx lwy
    set totW [expr $lW + $wid + 10]

    wm geometry . $totW\x$aH+$wx+$wy
}

proc openHelp {idoc} {
    # open the pbcorHelp.pdf stored in the starkit doc directory
    # On Windows this can be done with start command which will find the
    # pdf reader
    # except that Windows cant find the vfs mounted directories
    # so use the Windows User/username/Documents
    # if the pdf is not there copy it from the starkit doc to Documents
    global mswindows
    global pbcorDir
    global helpdocs
    global isStarKit



   	set docdir [file join $pbcorDir Doc]
   	if {![file isdirectory $docdir]} {
   		Dialog_Warn "NO directory $docdir for Documents"
   		return
   	}
   	set helpdoc [file join $docdir $helpdocs($idoc)]
   	if {![file exists $helpdoc]} {
   		Dialog_Warn "Cant find $helpdocs($idoc)"
   		return
   	}
   	if {$mswindows} {
        exec {*}[auto_execok start] "" $helpdoc
    } else {
        # try evince for linux
        exec evince $helpdoc
    }
}


proc buildGroupSelector {} {
    # this only gets built once when heightlines=1
    global tv imagefmts
    global gselSB
    global lbxlabels
    global bld
    global npts nmsr calc gnum
    global curnpts curnmsr
    global right gv botc botf
    global lbxs
    global detm
    global grplbxs lbxLabels lbxWidgets lbxwidgetlst
    global heightlines mainwidth
    global useXscan multisoln
    global exclmatch samepts
    global mbbg mbab
    global graphicsFmt imgfmts

    set gr $right
    # right frame is already built and managed by grid
    # it has top for title and bot for listboxes which we want to scroll
    # using canvas containing frame window for groupsel scrolled frame
    # canvas path

    set top $gr.top
    set botf $gr.bot
    # add plot frame holder below group-selector
    set gv $gr.gv

    # put a canvas in the bot frame
    set botc $botf.c
    # canvas frame path
    set bot $botc.f

    #clearGrid $bot


    # save the group-selector lbx parent
    set grplbxs $bot

    set title $top.title
    set bld(1) $top.bld1
    set bld(2) $top.bld2
    set fvw $top.fvw
    set fil $top.fil
    set edit $top.edit
    set solv $top.solv
    set solvb $top.solvb
    set det $top.det
    set version $top.ver
    set help $top.help

    set curgrp $top.cur
    set inc $curgrp.inc
    set dec $curgrp.dec
    set cur $curgrp.cur
    set curlbl $curgrp.lbl
    set curnpts $curgrp.npts
    set curnptsL $curgrp.nptsL
    set curnmsr $curgrp.nmsr
    set curnmsrL $curgrp.nmsrL

    if { ![winfo exists $top] } {
	# create the groupselector widgets if they dont exist
	frame $top

	# botf is the frame containing the scrolled canvas
	frame $botf
	# botc = botf.c
	canvas $botc -width 1000 -height 600 \
	    -xscrollcommand [list $botf.xs set]
	scrollbar $botf.xs -orient horizontal \
	    -command [list $botc xview]
	pack $botc -side top -fill both -expand true
	pack $botf.xs -side bottom -fill x
	# bot is the frame inside the canvas bot = botc.f
	frame $bot -bd 0
	# canvas geo manages the frame via create window
	$botc create window 0 0 -anchor nw -window $bot
	# the trick is going to be setting the scrollregion
	# of the canvas to match the frame size of its contained frame $bot
	# try bind $g <Configure> setCanvasScrollRegion
	# botf is the frame that contains the canvas
	# canvas scrollregion should determine the size of its containing frame
	# canvas controls the geometry of its window $bot
	bind $bot <Configure> "setCanvasScrollRegion $botc $bot"


	frame $gv
	frame $curgrp
	button $inc -command incrGroup -image uparrow \
	    -background green -activebackground yellow
	button $dec -command decrGroup -image downarrow \
	    -background green -activebackground yellow
	label  $cur -textvariable gnum -width 3 -relief sunken -bd 1 -bg white
	label  $curlbl -text group
	label  $curnpts -textvariable npts \
	    -width 3 -relief sunken -bd 1 -bg white
	label  $curnptsL -text npts
	label  $curnmsr -textvariable nmsr \
	    -width 3 -relief sunken -bd 1 -bg white
	label  $curnmsrL -text nmsr
	trace variable npts w nptsFormat
	trace variable nmsr w nmsrFormat
	#trace variable gnum w groupSrcFiles

	#-font *-helvetica-bold-r-normal--*-240-*
	label $title -text "group selector" \
	    -font {helvetica 24 bold} \
	    -foreground #f00
	#button $bld(1) -bg green -activebackground yellow 
	#   -command "buildButtonConfig 1" -width 8
	#button $bld(2) -bg green -activebackground yellow
	#   -command "buildButtonConfig 2" -width 8

	button $fvw -bg green -activebackground yellow \
	    -command fileViewer -text "fileViewer" -width 8
	button $solv -text "SOLVE" -width 8 \
	    -command "solveGroup 0" \
	    -background green -activebackground yellow

    menubutton $help -bg $mbbg \
        -menu $help.menu -text HELP -relief raised \
        -activebackground $mbab
    set hlpm [menu $help.menu -tearoff 0]
    $hlpm add command -label "quickstart guide" -command "openHelp 0"
    $hlpm add command -label "gui tour" -command "openHelp 1"
    $hlpm add command -label "calculation details" -command "openHelp 2"
    $hlpm add command -label "polarized beam details" -command "openHelp 3"

	menubutton $version -text Version -menu $version.menu \
	    -bg $mbbg -relief raised \
	    -activebackground $mbab
	set verm [menu $version.menu -tearoff 0]
	global versioninfo platforminfo
	foreach vitem [array names versioninfo] {
	    $verm add command -label "$vitem  $versioninfo($vitem)"
	}
	foreach vitem [array names platforminfo] {
	    $verm add command -label "$vitem  $platforminfo($vitem)"
	}

	# RWE Mar 2019. I can drop the solvDet interface which was for MACS now that se are using 3-axis conversion
	# every detector is another datapt and we always solve all detectors even in MACS 3-axis mode fixed PTAI
	#menubutton $det -text solvDet -menu $det.menu \
	#    -bg $mbbg -relief raised \
	#    -activebackground $mbab
	#set detm [menu $det.menu -tearoff 0]
	
	# replace this with a SOLVEbg 
	button $solvb -text "SOLVEbg" -width 8 \
	    -command "solveGroup 0 bg" \
	    -background green -activebackground yellow

	menubutton $fil -text File -menu $fil.menu \
	    -bg $mbbg -relief raised \
	    -activebackground $mbab
	set m [menu $fil.menu -tearoff 1]
	$m add command -label "Select Data Directory" \
	    -command "selectDataDirectory 1"

	$m add separator
	$m add cascade -label "He3-cells" -menu $m.cell
	set mcel [menu $m.cell -tearoff 0]
	$mcel add command -label "Rescan for He-3-cellFiles" \
	    -command getDirectoryCellFiles
	$mcel add command -label "View cells for current group" \
	    -command "viewCells 1"
	$mcel add command -label "View cells selected files" \
	    -command "viewCells 0"
	$mcel add command -label "Open cell editor" \
	    -command "viewCells 2"

	$m add separator
	$m add command -label "find and save windows" -command openWfind

	# change save window images to use tk Img package. This is slow and only saves gif but is platform independent
    #if [file executable /usr/bin/import]
	# imageMagick under Linux
	#$m add separator
	#$m add cascade -label "save windows" -menu $m.mim
	#set mim [menu $m.mim -tearoff 0]
	#$mim add command -label "save window" -command "runWindowCapture"
	#$mim add command -label "save fileViewer window" -command "windowToFile $tv"
	#$mim add command -label "save current group plot" -command saveCurrentPlot
	#$mim add command -label "choose save Directory" -command chooseSaveDir
	#$mim add cascade -label "graphics format" -menu $mim.mgf
	#set mgf [menu $mim.mgf -tearoff 0]
	#foreach gfmt $imgfmts {
	#	$mgf add command -label $gfmt -command "set graphicsFmt $gfmt"
	#}
	
	$m add separator
	$m add command -label "Open correction simulator" \
	    -command openSim
	$m add command -label "Open bg job" \
	    -command "wm deiconify .pbcorBG"
	$m add command -label "Open prev solutions" \
	    -command "wm deiconify .plotprev"


	$m add separator
	$m add command -label "Save session" -command "saveSession 1"
	$m add command -label "Save and quit" -command "saveSession 2"
	$m add command -label "Quit" -command "saveSession 0"

	
	menubutton $edit -text Edit -menu $edit.menu \
	    -bg $mbbg -relief raised \
	    -activebackground $mbab
	set m [menu $edit.menu -tearoff 1]

	$m add command -label "Select Y data columns" \
	    -command "columnsFromFileView 1"
	$m add command -label "Select X data column" \
	    -command "columnsFromFileView 2"
	$m add check -label "Use X scan variable" -variable useXscan
	$m add check -label "Allow multiple solutions" -variable multisoln
	#$m add command -label "Apply select Y columns to current group"
	#    -command "applyYselectGroup 0"

	$m add separator
	$m add command -label "Subtract background" -command subtractBG
	$m add command -label "Scale datasets" -command scaleSETS

	$m add separator
	$m add command -label "Show group info" -command showGroupRanges

	$m add separator
	$m add command -label "Solved group file operations" \
	    -command groupResultFops

	$m add separator
	$m add cascade -label "remove groups" -menu $m.mrgr
	set mrgr [menu $m.mrgr -tearoff 0]
	$mrgr add command -label "Remove current group" -command "delGrps 0"
	$mrgr add command -label "Remove selected groups" -command "delGrps 1"
	$mrgr add command -label "Remove unselected groups" -command "delGrps 2"

	$m add separator
	$m add cascade -label "copy/move/del grp files" -menu $m.mgf
	set mgf [menu $m.mgf -tearoff 0]
	$mgf add command -label "Remove selected files from their group" \
	    -command delFiles

	# moveFiles just moves/copies selected rows from listbox
	# moveFiles choices
	# 0 move sel files to current grp
	# 1 copy sel files to current grp

	$mgf add command -label "Move selected files to current group" \
	    -command "moveFiles 0"
	$mgf add command -label "Copy selected files to current group" \
	    -command "moveFiles 1"
	$mgf add command -label "Reread current group source files" \
	    -command rereadGroup

	$m add separator
	$m add cascade -label "auto-group" -menu $m.mag
	set mag [menu $m.mag -tearoff 0]
	$mag add command -label "AUTO-GROUP current group files" \
	    -command "autoGroup 0"
	$mag add command -label "AUTO-GROUP will use group option tolerances"
	$mag add check -label "files must have all same data points" \
	    -variable samepts
	$mag add check -label "each file can only be in one group" \
	    -variable exclmatch

	$m add separator

	# $m add cascade -label "configure mainWindow size" -menu $m.mszm
	# set mszm [menu $m.mszm -tearoff 0]
	# $mszm add radio -label "min height" -variable heightlines -value 1
	# -command "setGrpLines 0"
	# $mszm add radio -label "med height" -variable heightlines -value 2
	# -command "setGrpLines 0"
	# $mszm add radio -label "max height" -variable heightlines -value 3
	# -command "setGrpLines 0"
	# $mszm add radio -label "min width" -variable mainwidth -value 1
	# -command mainWindowSize
	# $mszm add radio -label "med width" -variable mainwidth -value 2
	# -command mainWindowSize
	# $mszm add radio -label "max width" -variable mainwidth -value 3
	# -command mainWindowSize

	#$m add command -label "More lines in group selector"
	#   -command "setGrpLines 1" 
	#$m add command -label "Less lines in group selector"
	#   -command "setGrpLines -1" 

	$m add command -label "right-click listbox label to set col width"
	$m add cascade -label "swap column data" -menu $m.swap
	set mswp [menu $m.swap -tearoff 0]
	$mswp add command -label "swap ts/date" \
	    -command convertTSlbx
	$mswp add command -label "swap E/Ei" \
	    -command "convertElbx 1"
	$mswp add command -label "swap E/Ef" \
	    -command "convertElbx 0"
	$mswp add command -label "swap QxQyQzE/angles" \
	    -command convertQEangles
	$mswp add command -label "swap H/rotH" \
	    -command convertHlbx



    }

    grid $top -sticky news
    grid $botf
    grid $gv -sticky news

    grid $title -row 0 -column 0 -ipadx 5 -sticky w
    grid columnconfigure $top 1 -minsize 20
    grid $fil  -row 0 -column 2 -ipadx 5 -padx 10 -sticky w
    grid $edit -row 0 -column 3 -ipadx 5 -padx 10 -sticky w
    grid $solv -row 0 -column 4 -ipadx 5 -padx 10
    grid $solvb  -row 0 -column 5 -ipadx 5 -sticky w
    grid $curgrp -row 0 -column 6 -ipadx 5
    #grid $bld(1) -row 0 -column 6
    #grid $bld(2) -row 0 -column 7
    grid $fvw -row 0 -column 9
    grid columnconfigure $top 10 -minsize 50
    grid $version -row 0 -column 11 -sticky e
    grid columnconfigure $top 12 -minsize 50
    grid $help -row 0 -column 13 -sticky e

    grid $curlbl $cur $inc $dec $curnptsL $curnpts $curnmsrL $curnmsr

    set lbxlabels \
	{filename g ts npts ny cts Qx Qy Qz E  Fix fs H V T scl ref fbg}
    set widths \
	{16       3 10 3    2  12  12 12 12 12 3   12 6 6 6 12  4   3}
    set sorts  {
	-ascii
	-integer -real -integer -integer
	-ascii -ascii -ascii -ascii -ascii -ascii -ascii
	-real -real -real -real
	-ascii -real
    }
    array set lbxs \
	[scrolledlistboxes $bot $lbxlabels $widths $sorts $gselSB {}]
    bind $lbxs(0) <ButtonRelease-1>  {+ showSelfile; plotDS}

    # store the lbxlabels list and the widget names array as list
    # in lbxLabels($bot) lbxWidgets($bot)
    set lbxLabels($bot) $lbxlabels
    set lbxWidgets($bot) [array get lbxs]
    # also bind to column heading ButtonRelease-1 hiliteGroups
    set lbxwidgetlst {}
    for {set i 0} {$i < [llength $lbxlabels]} {incr i} {
	set wnam $bot.name$i
	bind $wnam <ButtonRelease-1> {+ hiliteGroups}
	bind $wnam <ButtonRelease-2> "+ adjustLbxWid $bot $i"
	bind $wnam <ButtonRelease-4> "+ adjustLbxWid $bot $i"
	lappend lbxwidgetlst $lbxs($i)
    }
    # do the config with startup heightlines=37
    #buildButtonConfig 0
    setLbxHeight 37
}

proc adjustLbxWid {f i} {
    # setLbxWidth for i column listbox in frame f using toplevel gui
    global adjlbxw adjlabel adjindex lbxlabels adjframe
    global lbxLabels
    if { ![info exists lbxLabels($f)] } { return }
    if { ![winfo exists $adjlbxw] } {
	buildAdjLbxWid
    }
    set adjlabel [lindex $lbxLabels($f) $i]
    set adjindex $i
    set adjframe $f
    wm deiconify $adjlbxw
    raise $adjlbxw
}
proc buildAdjLbxWid {} {
    global adjlbxw adjlabel adjwidth adjfixed
    set a [toplevel .adjcw] 
    set adjlbxw $a
    wm title $a "adjust listbox width"
    set nlbl [label $a.nlbl -text "listbox label"]
    set name [label $a.name -textvariable adjlabel -bg white -width 32]
    set wchk [checkbutton $a.wchk -variable adjfixed -text "fixed width" \
		 -command setFixedWidth]
    set wval [entry $a.wid -textvariable adjwidth -width 32]
    grid $nlbl $name
    grid $wchk $wval
    bind $wval <Leave> setFixedWidth
}
proc setFixedWidth {} {
    global lbxs lbxlabels adjindex adjfixed adjwidth adjframe
    global lbxfixedwidth lbxWidgets lbxLabels
    global lbxwidgets
    # adjfixed>0 and adjwidth must be numeric to set fixed
    if { ! [info exists lbxLabels($adjframe)] } { return }
    if { ! [info exists lbxWidgets($adjframe)] } { return }
    if { [scan $adjindex {%d} adjindex] < 1 } { return }
    # get the array of lbx widgets
    arraySet lbxwidgets $lbxWidgets($adjframe)
    if { ! [winfo exists $lbxwidgets($adjindex)] } { return }
    set w $lbxwidgets($adjindex)
    if { [scan $adjfixed {%d} adjfixed] < 1 || \
	     [scan $adjwidth {%d} adjwidth] < 1 || \
	     $adjfixed < 1 || $adjwidth < 1 } {
	set lbxfixedwidth($w) 0
    }
    if { $adjfixed && $adjwidth > 0 } {
	set lbxfixedwidth($w) $adjwidth
    }
    setLbxWidth $w [lindex $lbxLabels($adjframe) $adjindex]
}

proc deleteLbxEntries {egs} {
    global lbxs nlbxs
    set egs [lsort -integer -decreasing $egs]
    #puts "deleteLbxEntries starting with sorted egs= $egs ll= [llength $egs]"
    for {set i 0} {$i < [llength $egs]} {incr i} {
		set ig [lindex $egs $i]
		for {set j 0} {$j < $nlbxs} {incr j} {
	    	$lbxs($j) delete $ig
		}
    }
}
proc deleteGroup {ig} {
    global group groupplot groupfop fopgroup
    # make sure to del decreasing order so indices remain valid
    set egs [lsort -decreasing [getEntriesForGroup $ig]]
    deleteLbxEntries $egs
    set group($ig) {}
    set groupplot($ig) {}
    set groupfop($ig) {}
    set fopgroup($ig) 0
    changeGroupNoSave
}

proc filesSelected {is} {
    global lbxs lbxi nlbxs widgetsel
    # for selection options check the widget that owns PRIMARY sel
    # some selection is required

    set sel [selectionOwned 0 2 0]
    if { [llength $sel] < 1 } {
	Dialog_Warn "Selection is not in one of the group selector listboxes.\n"
	return {}
    }

    set fsels {}
    set lbxi {}
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set isel [lindex $sel $i]
	set fsel [$lbxs(0) get $isel]
	if {[lsearch -exact $fsels $fsel] < 0} {
	    lappend fsels $fsel
	    lappend lbxi $isel
	}
    }
    if { $is } { return $fsels }

    # get unselected files
    set allfs [$lbxs(0) get 0 end]
    set fnosels {}
    set lbxi {}
    for {set i 0} {$i < [llength $allfs]} {incr i} {
	set fsel [lindex $allfs $i]
	if {[lsearch -exact $fnosels $fsel] < 0 && 
	    [lsearch -exact $fsels $fsel] < 0 } {
	    lappend fnosels $fsel
	    lappend lbxi $i
	}
    }
    return $fnosels
}

proc filesInGroupsSelected {is} {
    global widgetsel lbxi
    set gsel [groupsSelected $is]
    set ngsel [llength $gsel]
    if { $ngsel < 1 } { return {} }
    set filst {}
    set lbxiloc {}
    foreach grp $gsel {
	set fils [getFilesInGroup $grp]
	for {set i 0} {$i < [llength $fils]} {incr i} {
	    set fil [lindex $fils $i]
	    if {[lsearch -exact $filst $fil] < 0} {
		lappend filst $fil
		lappend lbxiloc [lindex $lbxi $i]
	    }
	}
    }
    set lbxi $lbxiloc
    return $filst
}

proc selectionOwned {sellst nlbx getitems} {
    # return selection if
    # sellst==1 if owned by sellectlist
    # nlbx ==1 if owned by lbxs(0)
    # nlbx ==2 if owned by any lbxs
    # nlbx ==3 if owned by lbxr(0)
    # nlbx ==4 if owned by any lbxr
    # if getitems returns list of lbx selected items
    # else returns selection indices

    global sellectlist lbxs nlbxs lbxsr nlbxsr
    if { $sellst == 0 && $nlbxs == 0 } { return {} }
    if [catch {selection own} w] { return {} }
    if { $sellst &&  [string match $w $sellectlist] } {
	return [getLbxItems $w [$w curselection] $getitems]
    }
    if { $nlbx == 1 &&  [string match $w $lbxs(0)] } {
	return [getLbxItems $w [$w curselection] $getitems]
    }
    if { $nlbx == 3 &&  [string match $w $lbxsr(0)] } {
	return [getLbxItems $w [$w curselection] $getitems]
    }

    set selok 0
    if { $nlbx == 2 } {
	for {set i 0} {$i < $nlbxs} {incr i} {
	    if { [string match $w $lbxs($i)] } {
		set selok 1
		break
	    }
	}
    } elseif { $nlbx == 4 } {
	for {set i 0} {$i < $nlbxsr} {incr i} {
	    if { [string match $w $lbxsr($i)] } {
		set selok 1
		break
	    }
	}
    }
    if { ! $selok } {
	return {}
    } else {
	return [getLbxItems $w [$w curselection] $getitems]
    }
}
proc getLbxItems {w indices getitems} {
    if { ! $getitems } { return $indices }
    set items {}
    foreach i $indices { lappend items [$w get $i] }
    return $items
}

proc groupsSelected {is} {
    global lbxs nlbxs widgetsel
    # for selection options check the widget that owns PRIMARY sel
    # some selection is required

    set sel [selectionOwned 0 2 0]
    if { [llength $sel] < 1 } {
	Dialog_Warn "Selection is not in one of the group selector listboxes.\n"
	return {}
    }

    set gsels {}
    for {set i 0} {$i < [llength $sel]} {incr i} {
	set isel [lindex $sel $i]
	set gsel [$lbxs(1) get $isel]
	if {[lsearch -exact $gsels $gsel] < 0} {lappend gsels $gsel}
    }
    if { $is } { return $gsels }

    # get unselected
    set allgs [$lbxs(1) get 0 end]
    set gnosels {}
    for {set i 0} {$i < [llength $allgs]} {incr i} {
	set gsel [lindex $allgs $i]
	if {[lsearch -exact $gnosels $gsel] < 0 && 
	    [lsearch -exact $gsels $gsel] < 0 } {lappend gnosels $gsel}
    }
    return $gnosels
}
proc delGrps {id} {
    global lbxs nlbxs
    global gnum group
    # id=0 del current grp
    # id=1 del selected grps
    # id=2 del unselected grps
    # id=3 del all groups

    if { $id < 0 } {
	deleteGroup [expr -1*$id]
	return
    }

    if { $id == 0 } {
	# delete current group
	deleteGroup $gnum
	return
    }

    if { $id == 1 } {
	set gsels [groupsSelected 1]
	for {set i 0} {$i < [llength $gsels]} {incr i} {
	    deleteGroup [lindex $gsels $i]
	}
	return
    }
    # must be delete non selected
    if { $id == 2 } {
	set gnosels [groupsSelected 0]
	for {set i 0} {$i < [llength $gnosels]} {incr i} {
	    deleteGroup [lindex $gnosels $i]
	}
	return
    }
    if { $id == 3 } {
	if { ! [info exists group] } { return }
	foreach i [array names group] {
	    deleteGroup $i
	}
	return
    }
}

proc delFiles {} {
    global lbxs
    set sel [$lbxs(0) curselection]
    if { [llength $sel] < 1 } {
		Dialog_Warn "Selection must be in File listbox to delete files.\n"
		return
    }
    deleteLbxEntries $sel
}
proc moveFiles {cp} {
    # cp = 0 move sel files to current grp
    # cp = 1 copy sel files to current grp

    global lbxs nlbxs
    global gnum group groupplot
    global sortgroups

    set sel [$lbxs(0) curselection]
    if { [llength $sel] < 1 } {
	Dialog_Warn \
	    "Selection must be in group filename listbox to move/copy files.\n"
	return
    }

    if { $cp == 1 } {
	# for copy just reread the original files to the current group
	# this puts them into new datasets.
	# Hopefully this will allow them to be treated separately
	set newlst {}
	foreach ent $sel { lappend newlst [$lbxs(0) get $ent] }
	readFiles [getFilesOrigin $newlst]
	return
    } else {

	# move
	# make sure files are not already in target group
	set flst [getFilesInGroup $gnum]

	set already {}
	foreach ent $sel {
	    set fil [$lbxs(0) get $ent]
	    if { [lsearch -exact $flst $fil] >= 0 } {
		lappend already $fil
		continue
	    }

	    # if not copying just change the group number
	    # if copying change the group number in the appended copy

	    $lbxs(1) delete $ent
	    $lbxs(1) insert $ent $gnum
	}

	if { [llength $already] > 0 } {
	    Dialog_Warn "Files already in current group:\n$already"
	}
    }

    # do a sort on groups increasing
    if { $sortgroups } { sortGroupsIncreasing }
    hiliteGroups
    hiliteActiveGroup
}

proc getFilesOrigin {flst} {
    # reconstruct a files original name
    # by distilling down to one extension
    # except if there is also a .A .B .C or .D
    set newlst {}
    foreach fl $flst {
	set fl [file tail $fl]
	set i 0
	set root(0) $fl
	while {1} {
	    set ext($i) [file extension $root($i)]
	    set i1 [expr $i + 1]
	    set root($i1) [file root $root($i)]
	    if { [string length $ext($i)] < 1 } { break }
	    set i $i1
	}
	# root($i)   has no  extension = ext($i)
	# root($i-1) has one extension = ext($i-1) if $i > 0
	if { $i > 1 } {
	# two or more extensions 
	    # only keep two extensions if second ext is .A .B etc
	    set i1 [expr $i - 1]
	    set i2 [expr $i - 2]
	    if [regexp {^\.[A-D]$} $ext($i2)] {
		set fl $root($i2)
	    } else {
		set fl $root($i1)
	    }
	}
	lappend newlst $fl
    }
    return $newlst
}

proc rereadGroup {} {
    # clear current group and reread the source files
    # reconstruct the src file names from single extension
    # and possible added .A .B .C .D
    # updated May 2011
    global gnum group
    set fls [getFilesInGroup $gnum]
    delGrps 0
    set srcfls [getFilesOrigin $fls]
    #puts "llength $srcfls=[llength $srcfls]"
    readFiles $srcfls
}

proc setGrpLines {inc} {
    global heightlines
    # RWE Jan 2011 static main window
    return
    if { $inc > 0 } {
	if { $heightlines < 3 } { incr heightlines }
    } elseif { $inc < 0 } {
	if { $heightlines > 1 } { incr heightlines -1 }
    }
    buildButtonConfig 0
}

############### group coordinate-peaks  toplevel similar to file ops

proc loadGroupPeaks {} {
    # load the pk data for current group gnumpk
    # pksdata is array by group number with value rowlist of listboxdatlist
    global gnumpk
    global pksdata ipks lbxps
    if { ! [info exists lbxps(0)] || ! [winfo exists $lbxps(0)] } { return }
    for {set i 0} {$i < 7} {incr i} { $lbxps($i) delete 0 end }
    set pkgrps [array names pksdata]
    if { [llength $pkgrps] < 1 } { return }
    if { [lsearch $pkgrps $gnumpk] < 0 } { return }
	set data $pksdata($gnumpk)
	set nrows [llength $data]
	for {set i 0} {$i < $nrows} {incr i} {
	    set rowdata [lindex $data $i]
	    for {set j 0} {$j < 7} {incr j} {
            $lbxps($j) insert end [lindex $rowdata $j]
        }
	}
    set ipks -1
    if { $nrows > 0 } {
        set ipks [expr $nrows - 1]
	    $lbxps(0) selection set $ipks
    }
}

proc savePkGroup {} {
    global gnumpk
    global pksdata ipks lbxps
    if { $gnumpk < 1 } { return }
    if { ! [info exists lbxps(0)] } { return }
    if { ! [winfo exists $lbxps(0)] } { return }
    set pksdata($gnumpk) [list]
    set n [$lbxps(0) index end]
    if { $n < 1 } { return }
    for {set i 0} {$i < $n} {incr i} {
        set ln [expr $i + 1]
        set rowdata [list]
        for {set j 0} {$j < 7} {incr j} {
            lappend rowdata [$lbxps($j) get $i]
        }
        # do sanity checking on min max pos fwhm here
        for {set j 3} {$j < 7} {incr j} {
            set bn [expr $j + 1]
            set x [lindex $rowdata $j]
            if { [scan $x "%g" xvar] != 1 } {
                Dialog_Warn "Peak data at line $ln box $bn is invalid"
                return
            }
        }
        set mn [lindex $rowdata 3]
        set mx [lindex $rowdata 4]
        set fwhm [lindex $rowdata 6]
        if { $fwhm <= 0 } {
            Dialog_Warn "Peak fwhm at line $ln is not positive"
            return
        }
        if { $mx < $mn } {
            set rowdata [lreplace $rowdata 3 4 $mx $mn]
        }
        lappend pksdata($gnumpk) $rowdata
    }
}

proc groupPeaks {} {
    # start the groupPeaks toplevel
    global pks
    if { ![winfo exists $pks] } { buildPeaks }
    if { ![winfo ismapped $pks] } { wm deiconify $pks }
    loadGroupPeaks
    raise $pks
}

proc selectPk {} {
    # click in the uid listbox to select a peak, loading data into edit entries
    # NB number of entries in each lbx must be same
    global lbxps ipks
    global pkeToi
    set npks [$lbxps(0) index end]
    if { $npks < 1 } { return }
    set ipkstest [$lbxps(0) curselection]
    if { $ipkstest < 0 } { return }
    set ipks $ipkstest
    # I think insert the text into entries will set the value of the entry's variable
    foreach w [array names pkeToi] {
        set i $pkeToi($w)
        catch {$w insert 0 [$lbxps($i) get $ipks]}
    }
}

proc setCurFS {fs} {
    # fill in lbxps(1) index ipks with the radio label passed as fs
    global lbxps ipks
    if { $ipks < 0 } { return }
    $lbxps(1) delete $ipks
    $lbxps(1) insert $ipks $fs
}

proc setPeaksCoord {lbl} {
    global lbxps ipks
    if { $ipks < 0 } { return }
    $lbxps(2) delete $ipks
    $lbxps(2) insert $ipks $lbl
}

proc incrPkGroup {} {
    global gnumpk
    savePkGroup
    incr gnumpk
    updatePkGroup
}
proc decrPkGroup {} {
    global gnumpk
    savePkGroup
    incr gnumpk -1
    updatePkGroup
}
proc updatePkGroup {} {
    global gnumpk
    configPkCoords
    loadGroupPeaks
}

proc addNewPkRow {vlist} {
    global lbxps ipks
    set nv [llength $vlist]
    for {set i 0} {$i < 7} {incr i} {
        set value ""
        if { $i < $nv } { set value [lindex $vlist $i] }
        $lbxps($i) insert end $value
    }
    set ipks [expr [$lbxps(0) index end] - 1]
    $lbxps(0) selection set end
}
proc newPeakUID {} {
    global gnumpk pksuid
    addNewPkRow {}
    focus $pksuid
}
proc newPeakCoord {} {
    global gnumpk
    global ipks lbxps pkscoord pksuid
    # get current uid and fs and copy to new
    if {$ipks < 0} { return }
    set nowuid [$lbxps(0) get $ipks]
    set nowfs [$lbxps(1) get $ipks]
    addNewPkRow [list $nowuid "" $nowfs]
    $pksuid delete 0 end
    $pksuid insert 0 $nowuid
    focus $pkscoord
}
proc removePeakUID {} {
    global lbxps ipks
    if {$ipks < 0} { return }
    set nowuid [$lbxps(0) get $ipks]
    set npks [$lbxps(0) index end]
    # remove all rows with this nowuid
    set urows {}
    for {set i 0} {$i < $npks} {incr i} {
        uid = [$lbxps(0) get $i]
        if { [string match $uid $nowuid] } {
            lappend urows $i
        }
    }
    set nr [llength $urows]
    if { $nr < 1 } { return }
    set drows [lsort -decreasing -integer $urows]
    for {set i 0} {$i < $nr} {incr i} {
        set ir [lindex $drows $i]
        removePkRow $ir
    }
    set newnpks [expr $npks - $nr]
    if { $newnpks > 0 } {
        set ipks [expr $newnpks -1]
        $lbxps(0) selection set $ipks
    }
}

proc removePeakCoord {} {
    global lbxps ipks
    if {$ipks < 0} { return }
    set nowcoord [$lbxps(2) get $ipks]
    set npks [$lbxps(0) index end]
    # remove ipks row
    removePkRow $ipks
    set newnpks [expr $npks - 1]
    if { $newnpks > 0 } {
        set ipks [expr $newnpks -1]
        $lbxps(0) selection set $ipks
    }
}

proc removePkRow {irow} {
    # irow is a listbox index >= 0
    global lbxps
    set npks [$lbxps(0) index end]
    if {$irow >= $npks} { return }
    for {set i 0} {$i < 7} {incr i} {
        $lbxps($i) delete $irow
    }
}


proc buildPeaks {} {
    global pks
    # list of flipper state combinations in the pksfs menu
    global fssels
    # list of coords in the pkscoord menu
    global pkscoords
    global gselSB
    global lbxps lbxplabels
    # below are entry and menu selected values
    global pkuid pkfs pkcoord pkmin pkmax pkpos pkfwhm
    # below are entry and menu widgets
    global pksuid pkscoord pksfs pksmin pksmax pkspos pksfwhm
    global gnumpk
    global mbbg mbab
    global coordnames
    global pkscoordmenu pksfsmenu
    global pkeToi

    set pks .pks
    set top $pks.top
    set midbot $pks.midbot
    set mid $midbot.mid
    set bot $midbot.bot


    set title $top.title
    set edit $top.edit
    set pksuid $mid.uid
    set pkscoord $mid.coord
    set pkscoordmenu $pkscoord.menu
    set pksmin $mid.min
    set pksmax $mid.max
    set pkspos $mid.pos
    set pksfwhm $mid.fwhm
    set pksfs $mid.fs
    set pksfsmenu $pksfs.menu

    set curgrp $top.curgrp
    set inc $curgrp.inc
    set dec $curgrp.dec
    set cur $curgrp.cur
    set curlbl $curgrp.lbl

    if { ![winfo exists $pks] } {
	    toplevel $pks
	    wm title $pks "PB peaks"
	    # add a close window protocol to save data
	    wm protocol $pks WM_DELETE_WINDOW {
	        savePkGroup
	        destroy $pks
	    }

    	frame $top
    	frame $midbot
	    frame $mid
	    frame $bot

    	label $title -text "gaussian peak weighting" \
	        -font {helvetica 24 bold} \
	        -foreground #f00


	    frame $curgrp
	    button $inc -command incrPkGroup -image uparrow \
	        -background green -activebackground yellow
	    button $dec -command decrPkGroup -image downarrow \
	        -background green -activebackground yellow
	    label  $cur -textvariable gnumpk -width 3 -relief sunken -bd 1 -bg white
	    label  $curlbl -text group
	    bind $cur <Return> updatePkGroup
	    bind $cur <Tab> updatePkGroup


	    menubutton $edit -text Edit -menu $edit.menu -bg $mbbg \
	        -relief raised -activebackground $mbab -width 10
	    set m [menu $edit.menu -tearoff 1]
	    $m add command -label "New peak uid for current grp" -command newPeakUID
	    $m add command -label "New coord for current peak uid" -command newPeakCoord

	    $m add separator
	    $m add command -label "Remove current uid" -command removePeakUID
	    $m add command -label "Remove current coord" -command removePeakCoord

        # second row(mid) for column menus and entries

	    menubutton $pksfs -text "fs weighted" -menu $pksfsmenu -bg $mbbg \
	        -relief raised -activebackground $mbab -width 13
	    menu $pksfsmenu -tearoff 0
	    set fs3 [list "uu dd du" "uu dd ud" "uu du ud" "dd du ud"]
	    set fi3 [list 7          11         13         14]
	    set fs2 [list "uu dd" "uu du" "uu ud" "dd du" "dd ud" "du ud"]
	    set fi2 [list 3       5       9       6       10      12]
	    set fs1 [list uu dd du ud]
	    set fi1 [list 1  2  4  8]
	    set fssels [concat all $fs3 $fs2 $fs1 none]
	    set ifsels [concat 15  $fi3 $fi2 $fi1 0]
	    foreach fssel $fssels ifsel $ifsels {
	        $pksfs.menu add radio -label $fssel -variable pkfs -value $ifsel \
	            -command "setCurFS $fssel"
	    }
	    menubutton $pkscoord -text "coord" -menu $pkscoordmenu -bg $mbbg \
	        -relief raised -activebackground $mbab -width 10
	    menu $pkscoordmenu -tearoff 0
	    entry $pksuid -textvariable pkuid -width 12
	    entry $pksmin -textvariable pkmin -width 12
	    entry $pksmax -textvariable pkmax -width 12
	    entry $pkspos -textvariable pkpos -width 12
	    entry $pksfwhm -textvariable pkfwhm -width 12
    }

    clearGrid $pks

    grid $top -sticky news
    grid $title -row 0 -column 0 -columnspan 4 -ipadx 5 -sticky w
    grid $curlbl $cur $inc $dec
    grid $curgrp -row 0 -column 4 -sticky w
    grid $edit -row 0 -column 5 -sticky w

    grid $midbot
    grid $mid
    # need to match up the mid column widths with the scrolledlistboxes
    # using grid columnconfigure $mid $widget -minsize pixels
    grid $pksuid $pksfs $pkscoord $pksmin $pksmax $pkspos $pksfwhm

    grid $bot
    # scrolledlistboxes clears its parent frame when created so give it a separate frame
    set lbxplabels {uid   fsweighted   coord   min     max     pos      fwhmm}
    set widths     {10    14           12      12      12      12       12}
    set sorts  {
	    -ascii -ascii -ascii -float -float -float -float
    }
    array set lbxps \
	[scrolledlistboxes $bot $lbxplabels $widths $sorts $gselSB {}]

    bind $lbxps(0) <ButtonRelease-1> {+ selectPk}
    array set pkeToi [list $pksuid 0 $pksmin 3 $pksmax 4 $pkspos 5 $pksfwhm 6]
    foreach w [array names pkeToi] {
        bind $w <Return> "+ entryPkToLbx $w"
        bind $w <Tab> "+ entryPkToLbx $w"
    }
    configPkCoords
}


proc entryPkToLbx {w} {
    global lbxps pkeToi ipks
    # move an entry value to its listbox at ipks index by replacement
    if { $ipks < 0 } { return }
    set ib $pkeToi($w)
    set lb $lbxps($ib)
    set nb [$lb index end]
    if { $ipks >= $nb } { return }
    # to replace, first insert before ipks, then delete the element pushed down
    set ipks1 [expr $ipks + 1]
    $lb insert $ipks [$w get]
    $lb delete $ipks1
}

proc configPkCoords {} {
    # use the stndScanNames in the coord menu
    # if there is data in current group use data column names assigned to stnd
    global pkscoordmenu
    global gnumpk
    global lbxs
    global dsFile colds colLbls
    global stndScanNames
    $pkscoordmenu delete 0 end
    set gi [lbxIndicesForGroup $gnumpk]
    #puts "configPkCoords: gnumpk=$gnumpk gi=$gi"
    if { [llength $gi] > 0 } {
        set indx [lindex $gi 0]
        set fil [$lbxs(0) get $indx]
        if { [info exists dsFile($fil)] } {
            set ds $dsFile($fil)
            pipeWrite "d$ds"
            readCurrentColAssigns
            set menuitems {}
            foreach sn $stndScanNames {
                if { [info exists colds($sn)] } {
                    set dscol $colds($sn)
                    lappend menuitems $colLbls($dscol)
                }
            }
            set nm [llength $menuitems]
            if { $nm > 0 } {
                for {set im 0} {$im < $nm} {incr im} {
                    set lbl [lindex $menuitems $im]
                    $pkscoordmenu add command -label $lbl -command "setPeaksCoord $lbl"
                }
                return
            }
        }
    }
    # else just set menuitems from the stndScanNames
    set nm [llength $stndScanNames]
    for {set im 0} {$im < $nm} {incr im} {
        set lbl [lindex $stndScanNames $im]
        $pkscoordmenu add command -label $lbl -command "setPeaksCoord $lbl"
    }    
}

############### group file ops - subtract or average group solutions ######

proc buildFileOps {} {
    global fop
    global gselSB
    global lbxos lbxolabels
    global grpsop grpsub
    global npts nmsr calc gnum
    global fopcontrol useFF fopTolerSrc
    global mbbg mbab

    set fop .fop
    set top $fop.top
    set bot $fop.bot

    set title $top.title

    set edit $top.edit
    set grpsop $top.gent
    set grpsub $top.gsub
    set grpres $top.gres
    set execg $top.exec

    if { ![winfo exists $fop] } {
	toplevel $fop
	wm title $fop "PB fop"

	frame $top
	frame $bot

	label $title -text "subtract or average groups" \
	    -font {helvetica 24 bold} \
	    -foreground #f00


	menubutton $edit -text Edit -menu $edit.menu -bg $mbbg \
	    -relief raised -activebackground $mbab -width 10
	set m [menu $edit.menu -tearoff 1]
	$m add command -label "New groups subtraction" -command newSubtraction
	$m add command -label "New groups average" -command newAverage

	$m add separator
	$m add command -label "Remove current Op" -command removeOp
	$m add command -label "Edit current Op" -command editOp

	$m add separator
	$m add command -font {helvetica 12 bold} -foreground #f00 \
	    -label "Choose independent variables"
	$m add radio -label "common scan column" \
	    -variable fopcontrol -value 4
	$m add radio -label "all stnd ind vars" \
	    -variable fopcontrol -value 2
	$m add radio -label "x-stnd ind vars only" \
	    -variable fopcontrol -value 0
	$m add radio -label "non-x stnd ind vars only" \
	    -variable fopcontrol -value 1
	$m add radio -label "choose ind vars" -variable fopcontrol -value 3
	$m add command -label "config ind var choice" -command configIV

	$m add separator
	$m add check -label "use first file ind var values" -variable useFF

	$m add separator
	$m add radio -label "tolerances from current grp" \
	    -variable fopTolerSrc -value 0
	$m add radio -label "tolerances from first op grp" \
	    -variable fopTolerSrc -value 1
	$m add radio -label "tolerances from max all grps" \
	    -variable fopTolerSrc -value 2

	entry $grpsop -textvariable gop  -state disabled -width 16
	entry $grpsub -textvariable gsub -state disabled -width 12

	button $execg -bg green -activebackground yellow \
	    -command execCurFop -text "Exec Current Op" -width 12
    }

    clearGrid $fop

    grid $top -sticky news
    grid $bot

    grid $title -row 0 -column 0 -columnspan 4 -ipadx 5 -sticky w
    grid $edit $grpsop $grpsub $execg

    set lbxolabels {op   groups   minus-group result-group}
    set widths     {10   16       12          12}
    set sorts  {
	-ascii -ascii -integer -integer
    }
    array set lbxos \
	[scrolledlistboxes $bot $lbxolabels $widths $sorts $gselSB {}]

    bind $lbxos(0) <ButtonRelease-1> {+ selectFop}
}

proc updateFOPtable {} {
    global fopdata ifop lbxos
    if { ! [info exists lbxos(0)] || ! [winfo exists $lbxos(0)] } { return }
    for {set i 0} {$i < 4} {incr i} { $lbxos($i) delete 0 end }
    set fops [array names fopdata]
    if { [llength $fops] < 1 } { return }
    foreach ip [lsort -integer -increasing $fops] {
	set data $fopdata($ip)
	for {set i 0} {$i < 4} {incr i} {
	    $lbxos($i) insert end [lindex $data $i]
	}
    }
    if { [lsearch $fops $ifop] >= 0 } {
	$lbxos(0) selection set $ifop
	selectFop
    }
}

proc configIV {} {
    # display toplevel to configIV
    global ivc
    if { ![winfo exists $ivc] } { buildConfigIV }
    if { ![winfo ismapped $ivc] } { wm deiconify $ivc }
    raise $ivc
}

proc clearIVC {} {
    global ivce
    for {set i 0} {$i < 3} {incr i} {
	for {set j 1} {$j <= 4} {incr j} {
	    set ivce($i$j) ""
	}
    }
}

proc setActiveIVCcolumn {} {
    global ivcEnt ivcCol
    if { ! [winfo exists $ivcEnt] } { return }
    $ivcEnt delete 0 end
    $ivcEnt insert 0 $ivcCol
}

proc buildConfigIV {} {
    global ivc ivcChi ivcOrder ivcCol ivcEnt ivce
    global stndColTypes stndColNames
    global mbbg mbab

    array set rownames {0 column 1 bincenter 2 binwidth}
    
    set ivc .ivc
    set top $ivc.top
    set bot $ivc.bot
    set title $top.title
    set selcol $top.selcol
    set chiL $top.chiL
    set chi $top.chi
    set fitL $top.fitL
    set fit $top.fit

    if { ![winfo exists $ivc] } {
	toplevel $ivc
	wm title $ivc "Config FOP"

	frame $top
	frame $bot

	label $title -text "ind vars for fop" \
	    -font {helvetica 24 bold} \
	    -foreground #f00

	menubutton $selcol -text "Sel active Column" -menu $selcol.menu \
	    -bg $mbbg \
	    -relief raised -activebackground $mbab -width 17
	set m [menu $selcol.menu -tearoff 0]
	foreach cl $stndColNames {
	    $m add radio -label $cl -variable ivcCol -value $cl \
		-command setActiveIVCcolumn
	}

	label $chiL -text chi
	entry $chi -textvariable ivcChi
	label $fitL -text polyFitorder
	entry $fit -textvariable ivcOrder

	for {set i 0} {$i < 3} {incr i} {
	    label $bot.$rownames($i) -text $rownames($i)
	    for {set j 1} {$j <= 4} {incr j} {
		entry $bot.e$i$j -textvariable ivce($i$j)
		bind $bot.e$i$j <ButtonRelease-1> "+ set ivcEnt $bot.e$i$j"
	    }
	}
    }
    clearGrid $ivc

    grid $top -sticky news
    grid $bot

    #grid $title -sticky news
    grid $selcol 
    grid columnconfigure $top 1 -minsize 20
    grid $chiL -row 0 -column 2
    grid $chi -row 0 -column 3
    grid $fitL -row 0 -column 4
    grid $fit -row 0 -column 5

    for {set i 0} {$i < 3} {incr i} {
	grid $bot.$rownames($i) -row $i -column 0
	for {set j 1} {$j <= 4} {incr j} {
	    grid $bot.e$i$j -row $i -column $j
	}
    }
}

proc removeOp {} {
    global ifop lbxos
    if { $ifop < 0 || $ifop >= [$lbxos(0) index end] } {
	Dialog_Warn "Invalid file operation.\nMake sure a row is selected in the PB fop listboxes.\n"
	return
    }
    set fopstr ""
    for {set i 0} {$i <= 3} {incr i} {
	append fopstr [$lbxos($i) get $ifop] ","
    }
    
    set rgrps [$lbxos(3) get $ifop]
    set qmsg "Are you sure you want to delete fop $ifop = $fopstr?\n"
    if { [llength $rgrps] > 0 } {
	append qmsg "This will delete the result groups $rgrps.\n"
    }
    if { ! [Dialog_Query $qmsg] } { return }

    for {set i 0} {$i <= 3} {incr i} {
	$lbxos($i) delete $ifop
    }
    foreach grp $rgrps { deleteGroup $grp }
}

proc editOp {} {
    global ifop lbxos editfop
    set editfop 1
    set editok [selectFop]
    set editfop 0
    if { $editok == 0 } {
	Dialog_Warn "Edit fop failed. Make sure selection is in a fop listbox."
    }
}

proc setMaxTolerance {aout ain} {
    upvar #0 $aout ao $ain ai
    foreach name [array names ao] {
	if { $ai($name) > $ao($name) } { set ao($name) $ai($name) }
    }
} 

proc execCurFop {} {
    # ifop is the index in the fop table for the fop
    # to execute
    global PLT
    global lbxos lbxi ifop curOp gnum calc isfop fopgroup group groupfop
    global pset ny pltDet ngrpsets maxgrpsets
    global calcgroup
    global calc iX xaxlbl textf
    global readfinished readlines nread nerr errlines nwarn warnlines
    global colds colLbls colRanges colsrch colsrchS lbxnames nlbxs
    global dsFile
    global solnresultsets
    global stndForiX natfs
    global fopTolerSrc stndTolArr saveStndTol
    # to debug fop result set 
    global slds eg

    global fopcontrol useFF ivcOrder ivcChi ivce stndColNames stndColTypes

    if { $ifop < 0 || $ifop >= [$lbxos(0) index end] } {
		Dialog_Warn "Invalid file operation.\nMake sure a row is selected in the PB fop listboxes.\n"
		return
    }
    set sub -1
    if { [string match [set op [$lbxos(0) get $ifop]] subtraction] } {
		set sub 1
    } elseif { [string match $op average] } {
		set sub 0
    }
    if { $sub < 0 } {
		Dialog_Warn "Invalid file operation.\n"
		return
    }
    # if user requests tolerances from current group just set them
    if { $fopTolerSrc == 0 } {
		setStndTolerances
        # Aug 2012 the following line was missing
		array set saveStndTol [array get stndTolArr]
    }

    # check that groups exist and are calc and all have same ny

    if { [info exists iXa] } { unset iXa }
    set grps [$lbxos(1) get $ifop]
    set ngrp [llength $grps]
    if { $ngrp < 1 } {
		Dialog_Warn "No groups to operate on.\n"
		return
    }
    set ninsets 0
    set iny 0
    set idet 0
    foreach grp $grps {
		if { ! [info exists group($grp)] } {
	    	Dialog_Warn "Group $grp doesn't exist.\n"
	    	return
		}
		setGnum $grp
		# setGnum will update stndTolArr
		if { $calc == 0 } {
	    	Dialog_Warn "Group $grp has not been solved.\n"
	    	return
		}
		# if user requests first group as toler source send them
		if { $ninsets == 0 && $fopTolerSrc > 0 } {
	    	setStndTolerances
	    	array set saveStndTol [array get stndTolArr]
		} elseif { $ninsets > 0 && $fopTolerSrc == 2 } {
	    	# update saveStndTol to max values
	    	setMaxTolerance saveStndTol stndTolArr
		}

		# set the tolerances from the first group used
		# this is kludgy for doing multiple subtractions
		# as one would like to use the tolerances
		# for each group, but this would be very difficult to program.
		# on the other hand it is reasonable when averaging multiple grps
		if { $ninsets == 0 } {
	    	#setGnum $grp
	    	#setStndTolerances
	    	if { [info exists ny] } { set iny $ny }
	    	if { [info exists pltDet($grp)] } { set idet $pltDet($grp) }
		} else {
	    	if { [info exists ny] && $iny != $ny } {
				Dialog_Warn "group $grp number of detectors=$ny inconsistent"
				return
	    	}
	    	if { [info exists pltDet($grp)] && $idet != $pltDet($grp) } {
	      		Dialog_Warn "group $grp detector-index=$pltDet($grp) inconsistent"
				return
	    	}
		}

		incr ninsets

		set iXa($grp) $iX
		if { ! [info exists solnresultsets($grp)] } {
	    	groupSrcFiles [list $grp]
		}
    }

    # make sure iny has been set
    if { $iny < 1 } {
		Dialog_Warn "Failed to set ny for FOP"
		return
    }

    # more sanity checks, and make sure we have solnresultsets for each grp
    set allgrps $grps
    if { $sub } {
		# supposed to be only one grp listed for subtraction from all the srcs
		set grps [$lbxos(2) get $ifop]
		set ngrp [llength $grps]
		if { $ngrp < 1 } {
	    	Dialog_Warn "No subtraction group.\n"
	    	return
		}
		foreach grp $grps {
	    	if { ! [info exists group($grp)] } {
				Dialog_Warn "Group $grp doesn't exist.\n"
				return
	    	}
	    	setGnum $grp
	    	if { $calc == 0 } {
				Dialog_Warn "Group $grp has not been solved.\n"
				return
	    	}
	    	if { $fopTolerSrc == 2 } {
				# update saveStndTol to max values
				setMaxTolerance saveStndTol stndTolArr
	    	}
	    	lappend allgrps $grp
	    	set iXa($grp) $iX
	    	if { ! [info exists solnresultsets($grp)] } {
				groupSrcFiles [list $grp]
	    	}
		}
    }
    # at this point we can send the saveStndTol
    if { $fopTolerSrc > 0 } {
		array set stndTolArr [array get saveStndTol]
		setStndTolerances
    }

    foreach ig [array names iXa] {
		if { $iXa($ig) != $iX } {
	    	Dialog_Warn \
			"Group $ig ind. var. \n$xaxlbl($iXa($ig)) != $xaxlbl($iX).\n"
	    	return
		}
    }

    # passed sanity checks
    # and result setnumbers should be in the list solnresultsets(gnum)

    # set up for fop with .p .A-D files
    # by putting them in a setlist first from grpsop
    # use groupSrcFiles [list gnum]
    # for getting the .p and native setnumbers
    # for a group which are then in global solnresultsets 
    # RWE Jan 2011 added stnd yi=27 to fop control since
    # we must always distinguish datapoints by detector index for multidet data
    # This is not valid for native sets where all detectors are on one line
    # but subtract should then ignore yi=27 stnd as it isnt assigned

    # put the result setlist into the next available group
    # or if reslt-grps already set and sufficient (are-fop-froups)
    # use them

    set nmsr 0

    # configure the operation FOP or FOPindVarsDeflt
    # using setlist 1
    pipeWrite "s1"
    

    # use first file flag
    pipeWrite "s o f $useFF"
    

    if { $fopcontrol < 3 } {
	# using stnd tolerances
	# fopcontrol 2 = x+stnd is deflt
	# 0=x 1=stnd 2=x+stnd
	# set the tolerances to max values used
	# NB if stnd is omitted operation wont distinguish .p detector indx yi
		pipeWrite "s o-"
	
		pipeWrite "s o i $fopcontrol"
	
    } else {
	# fopcontrol==3 user defined ind var columns
	# fopcontrol==4 try to find col with all distinct values (this is deflt)
	#
	# fit order
		pipeWrite "s o o $ivcOrder"
	
	# fitChi
		pipeWrite "s o x $ivcChi"
	

	# config the fop variables
		if { $fopcontrol == 4 } {
	    # try to find common stndcol in allgrps with all distinct values
	    # c s  col  now  returns ndif
	    # c ss col  returns  col isscan ismono
            # add the minimum spacing as fourth return arg from c ss colnum
	    # so just use readCurrentColAssigns then colNdif array has ndif

	    #puts "fopcontrol=4 starting allgrps= $allgrps"
	    	foreach gr $allgrps {
				setGnum $gr
				set iset [lindex $solnresultsets($gr) 0]
				set isscan($gr) 0
				set scancol($gr) {}
				set scanms($gr) {}
				pipeWrite "d$iset"
		
				foreach cnam $stndColNames {
		    		pipeWrite "c al $cnam"
		    
		    		if { $nread < 2 } { continue }
		    		if {[scan [lindex $readlines 1] {%s %d %d %s} \
			     		ct id co lb] < 4} { continue }
		    		pipeWrite "c ss $co"
		    
		    		if { $nread < 2 } { continue }
		    		# Aug 2012 c ss colnum now returns minStep as well
		    		if {[scan [lindex $readlines 1] {%d %d %d %g} co sc mo ms] < 3} {
						continue
		    		}
		    		#puts "gr=$gr cnam=$cnam co=$co sc=$sc mo=$mo"
		    		if { $sc } { set isscan($gr) 1 }
		    		if { $mo && $sc } {
						set scancol($gr) [linsert $scancol($gr) 0 $cnam]
						set scanms($gr) [linsert $scanms($gr) 0 $ms]
		    		} elseif { $sc } {
						lappend scancol($gr) $cnam
						lappend scanms($gr) $ms
		    		}
				}		
	    	}
	    	foreach gr $allgrps {
				if { ! $isscan($gr) } {
		    		Dialog_Warn \
					"Can't find scan column for all groups in this file Op."
		    		return
				}
	    	}
	    	# now try to find first common scanned column
	    	set grp0 [lindex $allgrps 0]
	    	set chklst $scancol($grp0)
	    	foreach gr $allgrps {
				if { $gr == $grp0 } { continue }
				set chklst [overlapLists $chklst $scancol($gr)]
	    	}
	    	if { [llength $chklst] < 1 } {
				Dialog_Warn \
		    	"Can't find common scan col for all groups in this file Op."
				return
	    	}
	    	# use the first col in chklst, put it into ivce
	    	clearIVC
	    	set ivar [lindex $chklst 0]
	    	set ivce(01) $ivar
	    	set ivce(11) 0
	    	set ivce(21) 0
	    	# set the tolerance for the scan variable from minstep in scanms
	    	set ivdex [lsearch $scancol($grp0) $ivar]
	    	set ms [lindex $scanms($grp0) $ivdex]
	    	set ms [expr $ms/4]
	    	pipeWrite "c St $ivar $ms"
	    	# set the ind var. make sure to add the detector index yi_stnd=27
	    	# RWE Jan 2011 added 27 for fop on multidetector data
	    	pipeWrite "s o c $stndColTypes($ivar) 27"
	    
	    	# and binwid and bincenter
	    	pipeWrite "s o b 0 0 0 0"
	    
		} else {
	    	# fopcontrol == 3

	    	# get consecutive non-blank column labels from ivce to determine
	    	# number of ind vars selected
	    	set nv 0
	    	set clst {}
	    	for {set j 1} {$j <= 4} {incr j} {
				if { [string length $ivce(0$j)] > 0 && \
				 [lsearch $stndColNames $ivce(0$j)] >= 0 } {
		    		lappend clst $stndColTypes($ivce(0$j))
		    		incr nv
				} else {
		    		break
				}
	    	}
	    
	    	if { $nv < 1 } {
				Dialog_Warn \
		    	"ERROR: Custom file operation, but no ind-var columns selected\n"
				return
	    	}
	    	# set the ind-vars
	    	# RWE Jan 2011 added 27 detector-index=di
	    	pipeWrite "s o c [join $clst] 27"
	    
	    	# set the binwids and centers, if empty strings set 0
	    	set blst {}
	    	for {set j 1} {$j <= $nv} {incr j} {
				set bw $ivce(2$j)
				if { [string length $bw] < 1 || [scan $bw {%g} bw] == 0 } {
		    		set bw 0
				}
				set bc $ivce(1$j)
				if { [string length $bc] < 1 || [scan $bc {%g} bc] == 0 } {
		    		set bc 0
				}
				lappend blst $bw $bc
	    	}
	    	# add bin spec for 27 yi
	    	lappend blst 0 0
	    	pipeWrite "s o b [join $blst]"
	    
		}
    }


    # ready to do the FOP for p-set and 4 native sets
    # create new group (with 5 file members .p+native) for each result set
    # still using setlist 1 for fop control and filelist
    # unless reslt-groups are already OK

    set failed ""

    foreach st {0 2 3 4 5} {
	# indices of setnumbers in solnresultsets(gnum)
	# 0=p 2-5= A D B C

	# clear files from setlist 1
		pipeWrite "s1 lc"
	

	# process fop source group list
		set grps [$lbxos(1) get $ifop]
		set allgrpsop $grps
		if { ! $sub && [llength $grps] < 2 } {
	    	Dialog_Warn "need at least two groups for averaging.\n"
	    	return
		}

		set ninsets 0
		foreach grp $grps {
	    	set iset [lindex $solnresultsets($grp) $st]
	    	incr ninsets
	    	pipeWrite "s l+ $iset"
	    
		}

	# if this is subtraction add the subtrahend dataset to the setlist
	# and define it
	# group soln filenames always come from the first dataset filename
	# with _g$gnum appended = grouprootfilename
	# then .p .m .A .B .C .D
	# these are overwritten whenever a group is solved
	# the group rootfilename is available from cmd: g f
	# returns: group gnum output-file-rootname: filename
		if { $sub } {
	    	set gsub [lindex [$lbxos(2) get $ifop] 0]
	    	#lappend allgrpsop $gsub
	    	set iset [lindex $solnresultsets($gsub) $st]
	    	pipeWrite "s l+ $iset"
	    
	    	# define the subtractor
	    	pipeWrite "s ss $iset"
	    
		}


		# do the fop
		if { $sub } {
	    	pipeWrite "s s"
		} else {
	    	pipeWrite "s a"
		}
	
		# check for errors
		if { $nerr > 0 } {
	    	Dialog_Warn "FAILED file operation:\n$errlines\n"
	    	return
		}
		if { $nwarn > 0 } {
	    	append failed "file operation:\n$warnlines\n"
		}


		# list the setlist to find out which setlist it wrote the result(s) to
		pipeWrite "s1 l"
	
		# the second line of output should have: op lnr opc lns, lnr is link
		# to result
		if { [scan [lindex $readlines 1] {%s %s %s %s %d %d %d %d} \
		  lop llnr lopc llns op lnr opc lns] < 8 } {
	    	Dialog_Warn "failed to get file-op result setlist.\n"
	    	return
		}
	# for subtract the result setlist will have nInputsets = nread - 3
	# for average  the result setlist will have only one set


	# list the result setlist to find the result dataset numbers
		pipeWrite "s$lnr l"
	
		# readline(3) i match set srcfile
		set nrsets [expr $nread - 3]
		if { $sub && $nrsets != $ninsets } {
	    	Dialog_Warn "subtraction produced $nrsets != number input sets.\n"
	    	return
		} elseif { ! $sub && $nrsets != 1 } {
	    	Dialog_Warn "average produced $nrsets result-sets != 1.\n"
	    	return
		}
		# read result set list to get the result set-numbers in slds
		set slds {}
    	set rdls $readlines
		for {set j 3} {$j < [llength $rdls]} {incr j} {
	    	set rl [lindex $rdls $j]
	    	if { [llength $rl] < 4 } {
				Dialog_Warn "FOP result setlist has an invalid line.\n"
				continue
	    	}
	    	set iset [lindex $rl 2]
	    	lappend slds $iset
	    	# write this result to disk
	    	pipeWrite "d$iset w"
		}

		# create new group for each result setlist (or use current)
		# only do this at st==0 since all the st=0 2 3 4 5 subtractions
		# should produce the same number of result sets


		if { $st == 0 } {
	    	# set up the result groups on first iteration
	    	# check if there are already valid result grps from the fop table
	    	set crgrps [$lbxos(3) get $ifop]
	    	set crgrpsok 0
	    	if { [llength $crgrps] == $nrsets } {
				# check that each reslt grp is fop
				set crgrpsok 1
				foreach rgrp $crgrps {
		    		if {! [info exists fopgroup($rgrp)] || ! $fopgroup($rgrp)} {
						set crgrpsok 0
						break
		    		}
				}
	    	}
	    	for {set k 0} {$k < $nrsets} {incr k} {
				if { $crgrpsok == 1 } {
		    		set eg($k) [lindex $crgrps $k]
		    		set egs [lsort -decreasing [getEntriesForGroup $eg($k)]]
		    		if { [llength $egs] > 0 } {
						deleteLbxEntries $egs
		    		}
				} else {
		    		set eg($k) [nextEmptyGroup 0]
				}
	    	}

	    	set savenrsets $nrsets
	    	set resultgrps {}
	    	for {set k 0} {$k < $nrsets} {incr k} {
				lappend resultgrps $eg($k)
				set gnum $eg($k)
				setGroup

				set isfop 1
				set fopgroup($gnum) 1
				set calcgroup($gnum) 0
				set calc 0
		
				# inherit the number of detectors solved
				# and the soln detector index
				set ny $iny
				set ngrpsets 1
				set pltDet($gnum) $idet
				# might as well record these in pbscript
				pipeWrite "g$gnum ly $ny"
		
				pipeWrite "g y $idet"
		
				set groupfop($gnum) {}
				saveGroup
	    	}
		} else {
	    	if { $nrsets != $savenrsets } {
	    		Dialog_Warn "number of results sets for fop inconsisitent at st=$st"
				return
	    	}
		}
	

	# These will be new files, so collect the stats and put them in the
	# listboxes
	# get the filenames in this setlist as they appear in the listbox
	# s li gives the srcin. Here we want the diskfile names
	# start line 4 ilst match iset fname
	# put the data for these files into lbxs with gnum=$eg
	# first make the list of files from listbox

		set nslds 0

	# rslt sets should be in fop order, so put them into consecutive groups
	# rslt sets st=0=.p ndy=4  st>0=native ndy=1 (one file each for Sxy)
	# ndy is dataset number of Y columns
	#  ny is group total number of detectors
	# pltDet(gnum) = 0 solve all detectors else solve 1

		for {set k 0} {$k < $nrsets} {incr k} {
	    	set gnum $eg($k)
	    	set iset [lindex $slds $k]
	    	pipeWrite "d$iset"
	    
	    	# read back npts this group, set nmsr=0
	    	set nscan [scan [string trimleft [lindex $readlines 1] "* "] \
				   {%d %d %d %d %d %d %s} \
				   is npts ncol ndy nass df srcf]
	    	if { $nscan < 7 } {
				append failed "Failed to get set $iset npts.\n"
				continue
	    	}
	    	if {$npts < 1 || ($st == 0 && $ndy < 4) || ($st > 0 && $ndy < 1)} {
				append failed "No data or not enough Y for fop result $srcf.\n"
				continue
	    	}
	    	set srcf [file tail $srcf]
	    	set dsFile($srcf) $iset
	    	if { $st == 0 } { set pset($gnum) $iset }

	    	set values(fnam) {}
	    	set values(gnum) {}
	    	set values(ts) {}
	    	set values(npts) {}
	    	# NB put ny=number of detectors into lbxs. NOT ndy=4
	    	set values(ny) {}
	    	set values(Fix) {}
	    	set values(ref) {}
	    	set values(fbg) {}
	    	set values(y) {}
	    	
	    	foreach i $colsrch { set values($i) {} }

	    	lappend values(fnam) $srcf
	    	lappend values(gnum) $gnum
	    	lappend values(npts) $npts
	    	# the ny in the group selector is supposed to be number of
	    	# Y columns defined for each dataset
	    	# while ny in group info is where the group ny goes
	    	lappend values(ny) 4
	    	lappend values(fs) $natfs($st)
	    	lappend values(ts) [datasetTSrange $iset]
	    
	    	set nas [readCurrentColAssigns]
	    
	    	if { [llength $colRanges(Ei)] < 2 } {
				lappend values(Fix) Ei
	    	} elseif { [llength $colRanges(Ef)] < 2 } {
				lappend values(Fix) Ef
	    	} else {
				lappend values(Fix) None
	    	}
	    
	    	if { $st == 0 } {
				lappend values(y) [getRange \
				       [concat $colRanges(y1) $colRanges(y2) \
					    $colRanges(y3) $colRanges(y4)]]
	    	} else {
				lappend values(y) [getRange $colRanges(y1)]
	    	}

	    	# to determine beam refer type, 
	    	# check for PBSCRIPT: corrected in header

	    	pipeWrite "d h \"PBSCRIPT: corrected\""
	    
	    	set rl [lindex $readlines 0]
	    	if { [string length $rl] > 0 && \
			     [string match "*corrected for monitor*" $rl] } {
				lappend values(ref) mon
	    	} else {
				lappend values(ref) time
	    	}
	    	lappend values(fbg) 0
	    
	    	foreach col $colsrchS {
				if [info exists colRanges($col)] {
			    	lappend values($col) $colRanges($col)
				}
	    	}

	    	# put the new values into lbxs
	    	# set lbxnames 
	    	# {fnam gnum ts npts ny y x1 x2 x3 En Fix fs H V Temp scl ref fbg}
	
	    	lappend groupfop($gnum) [appendLbxDataArray values]

	    	incr nslds
		}
    }
    if { [string length $failed] > 0 } {
		Dialog_Warn "$failed"
    }

    # need a sort on group number increasing since group processing
    # was interleaved

    sortGroupsIncreasing
    hiliteGroups

    # on success put the result group into lbxos(3)
    # and save the fop info in FOP pop up
    $lbxos(3) delete $ifop
    $lbxos(3) insert $ifop $resultgrps
    saveFop

    # try plotting
    for {set k 0} {$k < $nrsets} {incr k} {
		set gnum $eg($k)
		setGroup
		if { $PLT } {
	    	if { ! [checkFOPpset] } {
			Dialog_Warn "Failed FOP group setup for plotting"
			return
	    	}
		}
    	# need to use gotoGroup which has enterGroupShow to get plot window
    	gotoGroup $eg($k)

		hiliteFopGroup
		#plotDS 1
		# try plotting this fop group

		#    savePlot
		#    showPlot

		saveGroup
    }

    # dont put the sl into the group as the group is not being solved
    # mark this group as FOP result (not PB group) so that it wont ever be calc
}

proc appendLbxDataArray {aname} {
    global lbxnames
    upvar $aname values
    # the array has elements for each of lbxs columns
    # and each element value is a list of that columns data
    # convert this to a list of rowdata for appendLbxData
    if { ! [info exists values(fnam)] } { return }
    set napp [llength $values(fnam)]
    set lbxdatalst {}
    for {set j 0} {$j < $napp} {incr j} {
		set lbxdata {}
		foreach col $lbxnames {
	    	lappend lbxdata [lindex $values($col) $j]
		}
		lappend lbxdatalst $lbxdata
    }
    appendLbxData $lbxdatalst
    return $lbxdatalst
}

proc getRange {vlst} {
    if { [set vlen [llength $vlst]] < 1 } { return [list 0 0] }
    set vinc [lsort -real -increasing $vlst]
    set mn [lindex $vinc 0]
    set mx [lindex $vinc [expr $vlen - 1]]
    if { $mx > $mn } {
	return [list $mn $mx]
    } else {
	return $mx
    }
}

proc newSubtraction {} {
    global lbxos ifop
    for {set i 1} {$i < 4} {incr i} {
	$lbxos($i) insert end ""
    }
    $lbxos(0) insert end subtraction
    set ifop [expr [$lbxos(0) index end] - 1]
    $lbxos(0) selection set $ifop
    selectFop
}
proc newAverage {} {
    global lbxos ifop
    for {set i 1} {$i < 4} {incr i} {
	$lbxos($i) insert end ""
    }
    $lbxos(0) insert end addition
    set ifop [expr [$lbxos(0) index end] - 1]
    $lbxos(0) selection set $ifop
    selectFop
}

proc selectFop {} {
    global lbxos ifop
    global curOp editfop
    set ops [$lbxos(0) get 0 end]
    if { [llength $ops] < 1 } { return 0 }
    set sel [$lbxos(0) curselection]
    if { [llength $sel] < 1 } { return 0 }
    set ifop [lindex $sel 0]
    for {set i 0} {$i < 4} {incr i} {
	if [catch {$lbxos($i) get $ifop} curOp($i)] { set curOp($i) "" }
    }
    opEntState 0 0
    opEntState 1 0
    if { $editfop == 1 || [string length $curOp(3)] < 1 } {
	if { [string length $curOp(0)] > 0 } {
	    opEntState 0 1
	    if { [string match $curOp(0) subtraction] } {
		opEntState 1 1
	    }
	}
    }
    return 1
}

proc opEntState {sub normal} {
    global grpsop grpsub
    # config the groupOp entries normal or disabled along with binding
    set ent $grpsop
    if { $sub } { set ent $grpsub }
    if { $normal } {
	$ent configure -state normal
	bind $ent <Leave> "updateOp $sub"
    } else {
	$ent configure -state disabled
	bind $ent <Leave> {}
    }
}

proc updateOp {sub} {
    global lbxos grpsop grpsub ifop
    if { $sub } {
	$lbxos(2) delete $ifop
	$lbxos(2) insert $ifop [$grpsub get]
    } else {
	$lbxos(1) delete $ifop
	$lbxos(1) insert $ifop [$grpsop get]
    }
}

proc groupResultFops {} {
    global fop
    if { ![winfo exists $fop] } { buildFileOps }
    if { ![winfo ismapped $fop] } { wm deiconify $fop }
    updateFOPtable
    raise $fop
}

#################### saving/setting group data and session ############

proc listGroupOptions {} {
    # these are the group setup variables/arrays
    global cellFile nsl1FilesEnable monofilter monitor
    global nsfv sfv
    global acszero
    global cm cen

    global npts nmsr calc isfop ngrpsets ny
    global textf
    # scrfiles dsFile mset pset are all dsnumber dependent
    global solnfiles
    global fbgFile
    global stndTolArr
    global grpScale
    global relaxToler useXscan ny

    # return a list of commands that will set the current group setup values
    set cmdlst {}
    lappend cmdlst "set cellFile $cellFile"
    lappend cmdlst "set nsl1FilesEnable $nsl1FilesEnable"

    lappend cmdlst "set monofilter \{$monofilter\}"
    lappend cmdlst "set monitor \{$monitor\}"
    lappend cmdlst "arraySet nsfv \{[array get nsfv]\}"
    lappend cmdlst "arraySet sfv \{[array get sfv]\}"
    lappend cmdlst "set acszero $acszero"
    lappend cmdlst "arraySet cm \{[array get cm]\}"
    lappend cmdlst "arraySet cen \{[array get cen]\}"

    lappend cmdlst "set npts $npts"
    lappend cmdlst "set nmsr $nmsr"
    lappend cmdlst "set calc $calc"
    lappend cmdlst "set ny $ny"
    lappend cmdlst "set grpScale $grpScale"
    lappend cmdlst "set isfop $isfop"
    #lappend cmdlst "set mset $mset"
    #lappend cmdlst "set pset $pset"
    lappend cmdlst "set ngrpsets $ngrpsets"
    lappend cmdlst "set relaxToler $relaxToler"
    lappend cmdlst "set useXscan $useXscan"
    lappend cmdlst "arraySet textf \{[array get textf]\}"
    lappend cmdlst "arraySet stndTolArr \{[array get stndTolArr]\}"
    lappend cmdlst "set solnfiles \{$solnfiles\}"
    # dsFile is set in readFile or subtractFBG
    #lappend cmdlst "arraySet dsFile \{[array get dsFile]\}"

    return $cmdlst
}
proc setPlotOptions {cmdlist} {
    # call with groupplot(i) as cmdlist
    global ngrpsets ny iwin dsns
    global pDatNames mDatNames
    global Xpts Xmsr iX Rf
    global stylst rstylst sstylst iWts suuWts sddWts sduWts sudWts
    global pencolor spencolor
    global isrcplot eview
    global gnufil gnustyl gnusstyl gnucol nsetfs 
    global plotsymbolsize plotsymbolcolor plotsymbolfill plotconfig
    global iwin gnuPSopt bltPSopt

    foreach pdat $pDatNames { global $pdat }
    foreach mdat $mDatNames { global $mdat }

    foreach cmd $cmdlist {eval $cmd}
}

proc listGlobalOptions {} {
    global gnum fbgFile fileFilter heightlines mainwidth onePlotPerGrp
    global autoStndTol PLT BLT GNU lastPlot
    global grpScales grpTitles
    global lbxfixedwidth ifop multisoln
    global mcolorFracs mcolors scolorFracs scolors
    global plotconfigall
    set cmdlst {}
    set lastPlot 0
    if { $BLT } { set lastPlot 1 }
    if { $GNU } { set lastPlot 2 }
    lappend cmdlst "set fileFilter $fileFilter"
    lappend cmdlst "set heightlines $heightlines"
    lappend cmdlst "set mainwidth $mainwidth"
    #lappend cmdlst "mainWindowSize"
    lappend cmdlst "set onePlotPerGrp $onePlotPerGrp"
    lappend cmdlst "set plotconfigall $plotconfigall"
    lappend cmdlst "set scolors [list $scolors]"
    lappend cmdlst "set scolorFracs [list $scolorFracs]"
    lappend cmdlst "set mcolors [list $mcolors]"
    lappend cmdlst "set mcolorFracs [list $mcolorFracs]"
    lappend cmdlst "set ifop $ifop"
    lappend cmdlst "set multisoln $multisoln"
    lappend cmdlst "array set autoStndTol \{[array get autoStndTol]\}"
    lappend cmdlst "arraySet fbgFile \{[array get fbgFile]\}"
    lappend cmdlst "arraySet grpScales \{[array get grpScales]\}"
    lappend cmdlst "arraySet grpTitles \{[array get grpTitles]\}"
    lappend cmdlst "arraySet lbxfixedwidth \{[array get lbxfixedwidth]\}"
    #lappend cmdlst "gotoGroup $gnum"
    return $cmdlst
}
proc listFopOptions {} {
    global ivce ivcOrder ivcChi fopcontrol useFF fopTolerSrc
    set cmdlst {}
    lappend cmdlst "set ivcOrder $ivcOrder"
    lappend cmdlst "set ivcChi $ivcChi"
    lappend cmdlst "set useFF $useFF"
    lappend cmdlst "set fopTolerSrc $fopTolerSrc"
    lappend cmdlst "set fopcontrol $fopcontrol"
    lappend cmdlst "arraySet ivce \{[array get ivce]\}"
    return $cmdlst
}


proc setGlobalOptions {cmdlist} {
    global gnum fbgFile fileFilter heightlines mainwidth onePlotPerGrp
    global autoStndTol grpScales grpTitles lbxfixedwidth
    global mcolorFracs mcolors scolorFracs scolors
    global plotconfigall multisoln
    foreach cmd $cmdlist {eval $cmd}
}
proc listGroupPlot {} {
    # separately save plot options since BLT is optional
    global ngrpsets ny iwin dsns
    global pDatNames mDatNames
    global Xpts Xmsr iX Rf
    global stylst rstylst sstylst iWts suuWts sddWts sduWts sudWts
    global pencolor spencolor
    global isrcplot eview
    global gnufil gnustyl gnusstyl gnucol nsetfs 
    global plotsymbolsize plotsymbolcolor plotsymbolfill plotconfig detectorsOn
    global iwin gnuPSopt bltPSopt

    foreach pdat $pDatNames { global $pdat }
    foreach mdat $mDatNames { global $mdat }

    set cmdlst {}
    lappend cmdlst "set iwin $iwin"
    lappend cmdlst "set ngrpsets $ngrpsets"
    #lappend cmdlst "set ny $ny"
    # plot menu options

    lappend cmdlst "set dsns [list $dsns]"
    lappend cmdlst "array set eview \{[array get eview]\}"
    lappend cmdlst "arraySet isrcplot \{[array get isrcplot]\}"
    lappend cmdlst "arraySet pencolor \{[array get pencolor]\}"
    lappend cmdlst "arraySet spencolor \{[array get spencolor]\}"
    lappend cmdlst "set stylst [list $stylst]"
    lappend cmdlst "set rstylst [list $rstylst]"
    lappend cmdlst "set sstylst [list $sstylst]"
    lappend cmdlst "set iWts [list $iWts]"
    lappend cmdlst "set suuWts [list $suuWts]"
    lappend cmdlst "set sddWts [list $sddWts]"
    lappend cmdlst "set sduWts [list $sduWts]"
    lappend cmdlst "set sudWts [list $sudWts]"
    lappend cmdlst "set iX $iX"
    lappend cmdlst "set Xpts [list $Xpts]"
    lappend cmdlst "set Xmsr [list $Xmsr]"
    lappend cmdlst "set Rf [list $Rf]"
    lappend cmdlst "arraySet gnufil \{[array get gnufil]\}"
    lappend cmdlst "arraySet gnustyl \{[array get gnustyl]\}"
    lappend cmdlst "arraySet gnusstyl \{[array get gnusstyl]\}"
    lappend cmdlst "arraySet gnucol \{[array get gnucol]\}"
    lappend cmdlst "arraySet nsetfs \{[array get nsetfs]\}"
    lappend cmdlst "arraySet plotsymbolsize \{[array get plotsymbolsize]\}"
    lappend cmdlst "arraySet plotsymbolcolor \{[array get plotsymbolcolor]\}"
    lappend cmdlst "arraySet plotsymbolfill \{[array get plotsymbolfill]\}"
    lappend cmdlst "arraySet detectorsOn \{[array get detectorsOn]\}"
    lappend cmdlst "array set plotconfig \{[array get plotconfig]\}"
    lappend cmdlst "arraySet gnuPSopt \{[array get gnuPSopt]\}"
    lappend cmdlst "arraySet bltPSopt \{[array get bltPSopt]\}"
    foreach dat $pDatNames {
	lappend cmdlst "set $dat [list [subst $$dat]]"
    }
    foreach dat $mDatNames {
	lappend cmdlst "set $dat [list [subst $$dat]]"
    }
    return $cmdlst
}

proc printStack {args} {
    # print info about levels above this one
    set level [info level]
    puts ""
    for {set i -1} {$i > -$level} {incr i -1} {
	puts "   RelLevel $i: [info level $i]"
    }
}

proc checkSessionRecovery {} {
    global sessionrecover DEBUG currentDir debugFPT
    if { $DEBUG } {

        puts "checkSessionRecovery: opened new empty debugCMDS with cwd= $currentDir"
    }

    if { ![istextfile pbsession] } {
	    # this may get called when changing data directories so if
	    # no pbsession so return
	    return
    }
    # about to recover a session so delete all groups
    delGrps 3
    set sessionrecover 1
    set ok 1
    if { $DEBUG } {
	    # execute script line by line so we can print line errors
	    set fs [open pbsession r]
	    set ln 1
	    while { [gets $fs line] >= 0 } {
	        if [catch {uplevel #0 $line} msg] {
    	    	#if { [string length $msg] > 32 } {
		        #    set msg [string range $msg 0 31]
		        #}
		        Dialog_Warn "Error reading pbsession file. Unable to recover session at line $ln: $msg\n"
		        close $fs
		        set ok 0
		        #printStack
	        	puts "pbsession failed line:\n$line"
		        break
	        }
	        incr ln
	    }
    } else {
	    if [catch {uplevel #0 source pbsession} msg] {
	        Dialog_Warn "Error reading pbsession file. Unable to recover session: $msg\n"
	        set ok 0
	    }	
    }

    if { $ok } {
	    Dialog_Warn "previous session recovered.\nIf you want a new session rename or delete the pbsession file.\n"
    }
    set sessionrecover 0
}

proc saveSession {act} {
    # for session saving
    # act = 0 == quit
    # act = 1 == savesession
    # act = 2 == saveANDquit
    global group groupplot gnum fopgroup groupfop fopinfo fopdata pksdata
    global currentDir BLT PLT debugFPT DEBUG
    global setplot pltDet
    global mswindows msAPPDATA

    if { $act == 0 } {
    # must be quitting to close debug and exit
		if { $DEBUG } {
	    	puts "saveSession: debug closing file"
	    	catch {close $debugFPT}
	    	# not using debugServer anymore so skip debugClose
		}
		exit
    }

    # first save the current group
    saveGroup
    set currentGroup $gnum
    set f [open pbsession w]
    # save globals
    # NB this includes fbg required for filling listboxes
    foreach go [listGlobalOptions] {
    	puts $f "setGlobalOptions [list [list $go]]"
    }
    #puts $f "setGlobalOptions [list [listGlobalOptions]]"
    puts $f "array set setplot \{[array get setplot]\}"
    puts $f "array set fopgroup \{[array get fopgroup]\}"
    puts $f "array set calcgroup \{[array get calcgroup]\}"
    puts $f "array set groupfop \{[array get groupfop]\}"
    puts $f "array set pltDet \{[array get pltDet]\}"
    puts $f "array set fopinfo \{[array get fopinfo]\}"
    puts $f "array set fopdata \{[array get fopdata]\}"
    puts $f "array set pksdata \{[array get pksdata]\}"
    
    set grps [lsort -integer [array names group]]
    foreach grp $grps {
		# these filenames will be short.
		# readFiles no need to prepend currentDir
		# dont save fop groups
		# they get added onto the lbxs at the end
		if { [info exists fopgroup($grp)] && $fopgroup($grp) } {
	    	puts $f "setGnum $grp"
	    	foreach gi $group($grp) {
	    		puts $f "lappend group($grp) [list $gi]"
	    	}
	    	#puts $f "set group($grp) [list $group($grp)]"
	    	continue
		}
		set fls [getFilesInGroup $grp]
		if { [llength $fls] < 1 } { continue }
		puts $f "setGnum $grp"
		foreach gi $group($grp) {
			puts $f "lappend group($grp) [list $gi]"
		}
		#puts $f "set group($grp) [list $group($grp)]"
		if { $PLT && [info exists groupplot($grp)] } {
	    	foreach pi $groupplot($grp) {
	    		puts $f "if { \$PLT } {lappend groupplot($grp) [list $pi]}"
	    	}
		}
		puts $f "setGnum $grp"
		puts $f "readFiles [list $fls]"
		# on recovery re-solve any groups that had been solved
		# this makes it possible to use either plot package
		# I think it also makes it unnecessary to save all the data lists???
		# RWE 2018 only using BLT and solveGroup can be time expensive to allow just plotting previous solution
		puts $f "if { \$calc } { recoverPlot }"
    }

    foreach grp $grps {
	if { ! [info exists fopgroup($grp)] || ! $fopgroup($grp) } { continue }
	if { [info exists groupfop($grp)] } {
	    puts $f "appendLbxData [list $groupfop($grp)]"
	}
    }
    puts $f "hiliteGroups"
    # restore currentGroup
    puts $f "setGnum $currentGroup"
    puts $f "setGroup"
    puts $f "updateFOPtable"

    # finally save the existing toplevel ismapped window geometries
    set tlevs [winfo children .]
    lappend tlevs .
    foreach tl $tlevs {
	set tlw [winfo toplevel $tl]
	if { ! [string match $tlw $tl] } { continue }
	if { ! [winfo ismapped $tl] } { continue }
	puts $f "if { \[winfo exists $tl\] && \[winfo ismapped $tl\] } { wm geometry $tl [wm geometry $tl] }"
    }
    #puts $f "setGrpLines 0"
    close $f

    if { $mswindows } {
	# try to save current data directory in msAPPDATA folder
	if { [string length $msAPPDATA] > 0 && [file isdirectory $msAPPDATA] } {
	    set testfile [file join $msAPPDATA pbcorLastData.tcl]
	    if [catch {open $testfile w} fileID] {
		# failed to save last folder location in APPDATA
	    } else {
		puts $fileID "cd $currentDir"
		close $fileID
	    }
	}
    }

    if { $act >= 2 } {
    # saveANDquit
	if { $DEBUG } {
	    close $debugFPT
	    debugClose
	}
	exit
    }
}

proc setPlotTitle {args} {
    # trace on writing grpTitle(grpnumber)
    # args 0 = grpTitle  args 1 = arrayelem = grpnumber
    global grpTitles groupiwin gtitlefont
    global BLT
    global gws

    if {[llength $args] < 2} { return }
    if { ! $BLT } { return }
    set gn [lindex $args 1]
    if { ! [info exists grpTitles($gn)] } { return }
    # lookup the plotwindow
    set gw -1

    foreach iw [array names groupiwin] {
	if { $groupiwin($iw) == $gn } {
	    set gw $iw
	    break
	}
    }

    if { $gw < 0 } { return }
    if { $BLT } {
		if { [info exists gws($gw)] && [winfo exists $gws($gw)] } {
	    	$gws($gw) configure -title $grpTitles($gn) -font $gtitlefont \
			-fg black
		}
    }
}

proc recordCalc {args} {
    # trace for writing calc
    # store calc value for current group
    global groupsolved calc gnum
    set groupsolved($gnum) $calc
}

proc setGnum {n} {
    global gnum
    set gnum $n
    setGroup
}
proc initGroup {} {
    global textf npts nmsr calc isfop mset pset
    arraySet textf {0 "" 1 "" 2 ""}
    set npts 0
    set nmsr 0
    set calc 0
    set isfop 0
    #set mset 0
    #set pset 0
}
proc setGroupOptions {cmdlist} {
    # during session only thing to track is group options
    # these are the group setup variables/arrays
    global cellFile fileFilter monofilter monitor
    global nsfv sfv
    global acszero
    global cm cen
    global nsl1FilesEnable

    global npts nmsr calc ngrpsets ny
    global textf
    # scrfiles dsFile mset pset are all dsnumber dependent
    # it is possible that on session recovery the file will
    # endup in different setnumbers than last session
    # so dont rely on these.
    # instead whenever gnum changes regenerate them
    # from the filenames
    # NB srcfiles and dsFile are just inverse arrays of each other
    # and mset and pset are dsnumbers for textf
    global srcfiles dsFile mset pset
    global fbgFile
    global grpScale
    
    global stndTolArr relaxToler
    
    global useXscan isfop

    # mset and pset are dataset numbers!!!
    # can regen them from textf file names

    foreach cmd $cmdlist {
		uplevel #0 $cmd
    }
}
proc showSrcFile {i} {
     global snum mp
     set snum $i
     # activate the trace on mp to call showText
     if {$mp == 5} {
     	set mp 5
     } else {
	 	set mp 3
	 }
}

# group utilities
proc updateSrcFileMenus {} {
    # called via setGroup and solveGroup via groupSrcFiles
    # srcfile menus are in text viewer(sfvm) and in plot-selector(sfm)
    # detm menu is in the group-selector menubar next to the SOLVE button
    global sfvm sfm detm PLT calc gnum
    global srcfiles ngrpsets isrcplot solnfiles solnsets
    global dsFile pltDet ynames lbxi lbxs Yselect
    global nread readlines readfinished
    global fopgroup
    # in text viewer, command select one of the srcfiles for showSrcFile
    # in plot-selector, check turns srcset view on/off in plot via viewMsrSet
    if { ! [winfo exists $sfvm] } {
	buildShowText -width 60 -height 30 -wrap none -font {courier 10}
    }
    if { [info exists sfvm] && [winfo exists $sfvm] } { $sfvm delete 0 end }
    if { $PLT && [info exists sfm] && [winfo exists $sfm] } { $sfm delete 0 end}
    if { $PLT && [info exists detm] && [winfo exists $detm] } {
	$detm delete 0 end
    }
    set maxpercol 32
    set j 1
    for {set i 1;set j 0} {$i <= $ngrpsets} {incr i;incr j} {
        set fil [file tail [lindex $solnfiles $j]]
	set cbrk 0
	if { $j > $maxpercol } {
	    set cbrk 1
	    set j 1
	}
	if { [info exists sfvm] && [winfo exists $sfvm] } {
	    $sfvm add command -label "$i $fil" -columnbreak $cbrk \
		-command "showSrcFile $i"
	}
	if { $PLT && [info exists sfm] && [winfo exists $sfm] } {
	  $sfm  add check -label "$i $fil" -variable isrcplot($i) \
	      -columnbreak $cbrk -command "viewMsrSet 1"
	}
    }
    if { $PLT && [info exists detm] && [winfo exists $detm] } {    
	# update the detector selection menu for current group
	# this must be possible even before solution

	# first get the current group ny
	# OOPS cant get group ny until group prep
	#pipeWrite "g$gnum ly"
	#
	#set nscan [scan [lindex $readlines 0] {%s %d %s %d} grpl grpn ylbl ny]
	#if { $nscan < 4 || $ny < 1 }
	#    Dialog_Warn "ERROR: failed to get ny>0 for group $gnum\n"
	#    return
     
	# check that current group all ny are same
	# then get the yColNames for one of the groups sets
	set flst [getFilesInGroup $gnum]
	if { [llength $flst] < 1 } { return }
	set fil [file tail [lindex $flst 0]]
	# the indices are in lbxi
	set ny [$lbxs(4) get [lindex $lbxi 0]]
	if { ! [info exists fopgroup($gnum)] || $fopgroup($gnum) == 0 } {
	    for {set i 1} {$i < [llength $lbxi]} {incr i} {
		if { $ny != [$lbxs(4) get [lindex $lbxi $i]] } {
		    # NO longer warn just return
		    #Dialog_Warn
		    #  "Not all files in group $gnum have the same ny.\n"
		    return
		}
	    }
	}

	# get the setnumber for the first file in this group

	if { [info exists solnsets($gnum)] && [llength $solnsets($gnum)] > 0 } {
	    set iset [lindex $solnsets($gnum) 0]
	} else {
	    if { ! [info exists dsFile($fil)] } {
		return
	    }
	    #pipeWrite "d no $fil"
	    #
	    #set nscan [scan [lindex $readlines 0] {%s %d} dlbl iset]
	    #if { $nscan < 2 || $iset < 1 }
	    #Dialog_Warn "ERROR: failed to get setnumber for $fil\n"
	    #return
	    set iset $dsFile($fil)
	}

	set ynames [yColNames $iset]
	set ny [llength $ynames]
	if { $ny < 1 } {
	    #Dialog_Warn "ERROR: failed to find Y column for group $gnum\n"
	    return
	}
	if { ! [info exists pltDet($gnum)] } { set pltDet($gnum) 0 }
	if { $ny > 1 } {
	    $detm add radio -label all -variable pltDet($gnum) -value 0
	}
	set Yselect {}
	for {set i 1;set j 0} {$i <= $ny} {incr i; incr j} {
	    set ynam [lindex $ynames $j]
	    $detm add radio -label $ynam -variable pltDet($gnum) \
		-value $i
	    lappend Yselect $ynam
	}
    }
}

proc setGroup {} {
    # this just saves/loads local variables from the group arrays
    global gnum group groupplot BLT PLT fopgroup
    global lockCells
    #printStack
    set lockCells 0
    if { $gnum < 1 } { set gnum 1 }
    if { ! [info exists group($gnum)] } {
		# new groups only inherit options
		initGroup
		if {![info exists fopgroup($gnum)] } { set fopgroup($gnum) 0 }
		saveGroup
    } else {
		setGroupOptions $group($gnum)
		if { $PLT } { setPlotOptions $groupplot($gnum) }
		updateSrcFileMenus
    }
    # RWE Mar 2019 move the stndTolArr to the gui
    updateTolMenu
    # clear any assigned nsl1 file indications
    #nslMenuMarkAssign {}
    hiliteActiveGroup
    setSn 1
    if { [info exists fopgroup($gnum)] && $fopgroup($gnum) > 0} {return}
    #updatePbcorJobs
}

proc checkFOPpset {} {
    global gnum pset Xpts dsFile ngrpsets npts
    global readlines readfinished
    # if pset($gnum) aleady exists and Xpts list > 0
    # should be OK
    set ngrpsets 1
    #if { [info exists pset($gnum)] && $pset($gnum) > 0 &&
    #[info exists Xpts] && [llength $Xpts] > 0 } { return 1 }

    set spfile [getFOPpset]
    if { [string length [string trim $spfile]] < 1 } { return 0 }
    set pset($gnum) [checkDSfile $spfile]
    if { $pset($gnum) < 1 } { return 0 }
    #if { [set npts [llength $Xpts]] > 0 } { return 1 }
    pipeWrite "d$pset($gnum)"
    
    set nscan [scan [string trimleft [lindex $readlines 1] "* "] \
		   {%d %d %d %d %d %d %s} \
		   is npts ncol ndy nass df srcf]
    if { $nscan < 7 } { return 0 }

    groupPlotSetup
    if { [set npts [llength $Xpts]] < 1 } { return 0 }
    return 1
}

proc enterGroupShow {} {
    global gnum tv PLT BLT gvs fopgroup ngrpsets npts
    if { [winfo exists $tv] && [winfo ismapped $tv] } { showText }
    if { ! $PLT } { return }
	# for fop check for Xpts llength > 0
	# if no Xpts need to groupPlotSetup
	# but first must have pset($gnum)
	# if pset($gnum) isnt valid need to read the fop-groups
	# .p set
	# pset($gnum) is set from groupSrcFiles after solveGroup
	# and in execCurFop
	# but after a sessionRecover fops filenames just get loaded

	# get the .p file name and read it to set pset
	# then call groupPlotSetup

	if { [info exists fopgroup($gnum)] && $fopgroup($gnum) } {
	    #   plotDS 1
	    #  return
	    # make sure the pset file is ready for gnum
	    if { ! [checkFOPpset] } {
			Dialog_Warn "Failed FOP group setup for plotting"
			return
	    }

	    global spencolor
	    saveGroup
	}

	if { [set iw [checkPlotWindow]] == 0 } {
	    # window for this group plot doesnt exist
	    # do we need a groupPlotSetup?
	    # showPlot checks for calc on non-fop groups
	    showPlot
	} else {
	    if { $BLT } {
			if [winfo exists $gvs($iw)] { raise $gvs($iw) }
	    }
	    showPlot
	}
}

proc changeGroupNoSave {} {
    # to find a new group after deleting current group
    global gnum lockCells
    set lockCells 0
    if { $gnum > 1 } { incr gnum -1 } else { incr gnum }
    setGroup
    enterGroupShow
}

proc incrGroup {} {
    global gnum tv PLT BLT gvs
    global lockCells
    #printStack
    set lockCells 0
    # save previous group
    # checkPlotWindow returns iwin if plot exists
    # else set iwin to next empty and returns 0
    # ? how do we get window ids for gnuplot windows
    saveGroup
    incr gnum
    setGroup
    enterGroupShow
}
proc gotoGroup {gn} {
    global gnum tv PLT BLT gvs
    global lockCells
    #printStack
    set lockCells 0
    if { $gn < 1 } { return }
    saveGroup
    set gnum $gn
    setGroup
    enterGroupShow
}
proc decrGroup {} {
    global gnum tv PLT BLT gvs
    global lockCells
    #printStack
    set lockCells 0
    if { $gnum == 1 } return
    saveGroup
    incr gnum -1
    setGroup
    enterGroupShow
}
proc saveGroup {} {
    global gnum group groupplot PLT isfop
    set group($gnum) [listGroupOptions]
    if { $PLT && ! $isfop } { set groupplot($gnum) [listGroupPlot] }
}
proc savePlot {} {
    global gnum groupplot
    set groupplot($gnum) [listGroupPlot]
}
proc saveFop {} {
    global ifop fopinfo fopdata
    set fopinfo($ifop) [listFopOptions]
    set fopdata($ifop) [listFopData]
}
proc listFopData {} {
    global lbxos ifop
    set op [$lbxos(0) get $ifop]
    if { [string length [string trim $op]] < 1 } { return {} }
    set datalst $op
    for {set i 1} {$i < 4} {incr i} {
	lappend datalst [$lbxos($i) get $ifop]
    }
    return $datalst
}

proc setFopOptions {cmdlist} {
    global ivce ivcOrder ivcChi fopcontrol useFF ifop
    foreach cmd $cmdlist {
	uplevel #0 $cmd
    }
}

proc checkPlotWindow {} {
    # keep track of plot windows for enterGroupShow
    # returns 1 if gnum is already plotted in the correct window
    global iwin gnum onePlotPerGrp
    global BLT
    global groupiwin iwingroup
    # have setGroup which sets iwin

    # first get the current plotwindow inventory
    set iwinlst {}
    if { $BLT } {
		set tls [winfo children .]
		foreach w $tls {
	    	if { [regexp {\.g([0-9]+)} $w match i] } {
				lappend iwinlst $i
	    	}
		}
    }
    set iwinlst [lsort -integer $iwinlst]
    foreach i $iwinlst {
	if { ![info exists groupiwin($i)] } { set groupiwin($i) 0 }
    }
    foreach i [array names groupiwin] {
	if { [lsearch -exact $iwinlst $i] < 0 } { unset groupiwin($i) }
    }
    foreach i [array names groupiwin] {
	set igrp $groupiwin($i)
	set iwingroup($igrp) $i
    }
    # first, if the requested plotwindow=iwin already has another
    # gnum plotted in it,

    # if the onePlotPerGrp flag is set
    # check to see if current group is plotted to some other existing iwin
    # and if so reset to that, or if current group is in not in any
    # plotwindow set iwin to next available

    # RWE June 2010 onePlotPerGrp is now standard
    # so there is no PlotWindowsMenu
    if { $onePlotPerGrp } {
	if { [info exists iwingroup($gnum)] && $iwingroup($gnum) > 0 } {
	    set iwin $iwingroup($gnum)
	    return $iwin
	} else {
	    set iwin 1
	    while { [info exists groupiwin($iwin)] && $groupiwin($iwin) > 0 } {
		incr iwin
	    }
	    #updatePlotWindowsMenu $iwin
	    return 0
	}
    } else {
	# must plot current group to iwin
	if { [info exists groupiwin($iwin)] && $groupiwin($iwin) == $gnum } {
	    return 1
	}
	return 0
    }
}

proc scrolledlistboxes {parent labels widths sorttypes sb hsb} {
    # returns [array get lbxs] from the indexed array of listbox widget names
    # also the column label widget names will be $parent.name$i
    # create multi listboxes to represent columns of parent table
    # and set up coherent vertical scanning/scrollbar bindings
    # optional horiz scrollbar on listboxs with indices in the hsb list
    global lbxLblW

    set n [llength $labels]
    set m [llength $widths]
    if { $m < $n } { set n $m }
    set m [llength $sorttypes]
    if { $m < $n } { set n $m }
    if { $n < 1 } { return 0 }
    set nTableColumns $n
    clearGrid $parent
    set lbxws {}

    set nhsb [llength $hsb]
    set lbxLblW($parent) {}

    for {set i 0; set j 0} {$i < $n} {incr i; incr j 2} {
	set lb [lindex $labels $i]
	set wd [lindex $widths $i]
	if { [set lblen [string length $lb]] > $wd } { set $wd $lblen }
	set lbx $parent.lb$i
	set lbxs($i) $lbx
	set head $parent.name$i
	set bar $parent.bar$i
	#set entry $parent.e$i
	if { ! [winfo exists $lbx] } {
	    listbox $lbx -bd 0 -selectmode extended -width $wd -height 10
	    label $head -background black -foreground white
	    frame $bar -width 2 -bg grey
	    # -cursor sb_h_double_arrow
	}
	#if { ! [winfo exists $entry] && $row0 } { entry $entry } 
	$lbx delete 0 end
	if { $sb } { $lbx configure -yscrollcommand "$parent.sb set" }
	if { $nhsb && [lsearch $hsb $i] >= 0 } {
	    $lbx configure -xscrollcommand "$parent.xsb$i set"
	    set xsb $parent.xsb$i
	    if { ! [winfo exists $xsb] } { scrollbar $xsb -orient horizontal }
	    $xsb configure -command [list $lbx xview]
	    grid $xsb -row 2 -column $j -sticky ew
	}
	$head configure -text $lb

	lappend lbxws $lbx

	set jc $j
	if { $sb < 0 } { set jc [expr $j + 1] } 
	grid $head -row 0 -column $jc -sticky ew
	grid $lbx -row 1 -column $jc -sticky news
	grid $bar -row 0 -column [expr $jc+1] -rowspan 2 -sticky ns
	grid columnconfigure $parent [expr $jc+1] -weight 0
	grid columnconfigure $parent $jc -minsize $wd
    }

    #set cmnd ""
    #for {set i 0} {$i < $n} {incr i} {
    #set cmnd "$cmnd $parent.lb$i yview\;"
    #}

    if { $sb != 0 } {
	set sb $parent.sb
	if { ! [winfo exists $sb] } { scrollbar $sb -orient vertical }
	$sb configure -command [list BindYview $lbxws]
	if { $sb > 0 } {
	    set sbcol [expr 2*$n]
	} else {
	    set sbcol 0
	}
	grid $sb -row 1 -column $sbcol -sticky ns
	grid columnconfigure $parent $sbcol -weight 0
    }

    grid rowconfigure $parent 0 -weight 0

    #foreach w $lbxws
    #	bind $w <B2-Motion>
    #	bind $w <B2-Motion> "BindDragto %x %y $lbxws"
    #	bind $w <Button-2>
    #	bind $w <Button-2> "BindMark %x %y $lbxws"


    for {set i 0} {$i < $n} {incr i} {
	set lblWidget $parent.name$i
	bind $lblWidget <Button-1> \
	    [list sortlistboxes $lbxws $i [lindex $sorttypes $i]]
	#setLbxWidth $lbxs($i) [lindex $labels $i]
    }
    return [array get lbxs]
}



proc clearGrid {container} {
    # clear grid widgets but remember configs
    foreach w [grid slaves $container] {
	if {[winfo exists $w]} {grid remove $w}
    }
}
proc globalarr {avar} {
    # in response to 2009 FC11 global cant work for arr(elem)
    # if avar is array element name just return the array name
    if { [regexp {([^\(]*)\(.+\)$} $avar wm anam] } {
	return $anam
    }
    return $avar	
}
proc BindYview {wlist args} {
    foreach w $wlist {
	eval {$w yview} $args
    }
}
proc BindDragto { x y args } {
    foreach w $args {
	$w scan dragto $x $y
    }
}
proc BindMark { x y args } {
    foreach w $args {
	$w scan mark $x $y
    }
}

# procedure to sort list of lists even when elements are missing = {}

proc checkType {type elem} {
    if { [string length [string trim $elem]] < 1 } { return 0 }
    switch -exact -- $type {
	-integer { if { ! [scan $elem {%d} val] } {return 0} }
	-real { if { ! [scan $elem {%g} val] } {return 0} }
    }
    return 1
}
proc mcompare {a b} {
    global cmptyp cmpind
    # a and b are lists for which we want to compare elements
    # at index using comparison type
    set ai [lindex $a $cmpind]
    set bi [lindex $b $cmpind]
    set ai [lindex $ai 0]
    set bi [lindex $bi 0]
    set aok [checkType $cmptyp $ai]
    set bok [checkType $cmptyp $bi]
    if { $aok == 0 } { return 1 }
    if { $bok == 0 } { return -1 }
    switch $cmptyp {
	-real -
	-integer {
	    if {$ai < $bi} {
		return -1
	    } elseif {$ai > $bi} {
		return 1
	    } else {
		return 0
	    }
	}
	-ascii {
	    return [string compare $ai $bi]
	}
    }
    return 0
}

proc sortlistboxes {listwidgets column type args} {
    # user responsibitly to make sure sort column can be sorted
    # for indexed sort to work the data gets collected as a list of rows
    # with each row containing a data list one datum per column.
    # sorting on a full column or on selected elements in a column
    # does congruent sort on all other columns preserving selection states

    #global wctrl
    global sortdirection sortdirectionindex
    global cmptyp cmpind
    #if { ! $wctrl(tablesortenable) } { return }
    set n [llength $listwidgets]
    if {$n <= $column} { return }
    set widget [lindex $listwidgets $column]
    set onlyselected 0
    if { [llength [$widget curselection]] > 1 } { set onlyselected 1 }
    set N [$widget index end]
    if { [string length $args] > 0 && [llength $args] > 0 } {
	set Nent [lindex $args 0]
	if { $Nent > 0 && $Nent < $N } { set N $Nent }
    }
    set workl {}
    set tsortlist {}
    set ilst {}

    for {set i 0} {$i < $N} {incr i} {
	set sl {}
	set thisline {}
	for {set j 0} {$j < $n} {incr j} {
	    set w [lindex $listwidgets $j]
	    # need the subst to get rid of braces around quoted strings
	    #lappend thisline "subst [$w get $i]"
	    lappend thisline [subst [$w get $i]]
	    lappend sl [$w selection includes $i]
	}
	lappend thisline $i $sl
	lappend workl $thisline
	if { $onlyselected } {
	    if { [lindex $sl $column] } {
		lappend tsortlist $thisline
		lappend ilst $i
	    }
	} else {
	    lappend tsortlist $thisline
	}
    }
    if { [llength $tsortlist] < 1 } { return }
    if { [llength $args] > 1 } {
	set incrsort [lindex $args 1]
	if { $incrsort } {
	    set sorttyp -increasing
	} else {
	    set sorttyp -decreasing
	}
    } else {
	set sorttyp [lindex $sortdirection $sortdirectionindex]
	set sortdirectionindex [expr 1 - $sortdirectionindex]
    }

    if [catch {set sortlist [eval lsort $sorttyp -index $column $type \
				 {$tsortlist}]} msg] {
	# if sort fails try mcompare
	set cmptyp $type
	set cmpind $column
	if [catch {set sortlist [eval lsort \
				     -command mcompare {$tsortlist}]} msg] {
	    Dialog_Warn $msg
	    return
	}
    }


    set nsort [llength $sortlist]
    if { $onlyselected } {
	for {set i 0} {$i < $nsort} {incr i} {
	    set thisline [lindex $sortlist $i]
	    set indx [lindex $ilst $i]
	    set workl [lreplace $workl $indx $indx $thisline]
	}
	writelistboxrows $listwidgets $workl
    } else {
	writelistboxrows $listwidgets $sortlist
    }
    return $nsort
}
proc writelistboxrows {listwidgets sortlist} {
    # sortlist is a list of table row data
    # the rows can have two extra columns appended by sort
    set n [llength $listwidgets]
    if {$n < 1} { return }
    for {set i 0} {$i < $n} {incr i} {
	set widget [lindex $listwidgets $i]
	$widget selection clear 0 end
	$widget delete 0 end
    }
    set N [llength $sortlist]
    for {set i 0} {$i < $N} {incr i} {
	set thisline [lindex $sortlist $i]

	set nc [llength $thisline]
	set slst {}
	set sok 0
	if { $nc > [expr $n + 1] } {
	    set slst [lindex $thisline [expr $nc + 1]]
	    set sok 1
	}
	if { $nc > $n } { set nc $n }
	if { $sok && [llength $slst] < $nc } { set sok 0 }
	for {set j 0} {$j < $nc} {incr j} {
	    set widget [lindex $listwidgets $j]
	    $widget insert end [lindex $thisline $j]
	    if { $sok && [lindex $slst $j] } {
		$widget selection own
		$widget selection set $i
	    }
	}
    }
    hiliteActiveGroup
}

proc sortGroupsIncreasing {} {
    # use this after copying files or reading files into a group
    # since the new files will be at the end of the listboxes otherwise
    global lbxwidgetlst sortdirectionindex
    set sortdirectionindex 0
    sortlistboxes $lbxwidgetlst 1 -integer
}

# He3 cell gui procs
##################### celledt gui ##########################

proc checkid {irow typ} {
    global celldata transdata exptdata exptmed
    global curcellrow frames
    if { $irow == 0 } {
	set irow $curcellrow($frames(1))
    }
    if { $typ == 2 } {
	set id $celldata(11,$irow)
	if { [string length [string trim $id]] < 1 } { return }
	set n [countEntries transdata]
	for {set i 1} {$i <= $n} {incr i} {
	    if { [string match $id $transdata(0,$i)] } { return }
	}
	# create a new trans with this label
	incr n
	if { $n >= $transdata(nrows) } {
	    addEntryTableRows .tf.d.e 1 transdata {}
	}
	set transdata(0,$n) $id
	set transdata(1,$n) 1.
	set transdata(2,$n) 0.
	set transdata(3,$n) 1.
	set transdata(4,$n) 0.
	set transdata(5,$n) ""
    } elseif { $typ == 3 } {
	set id $celldata(12,$irow)
	if { [string length [string trim $id]] < 1 } { return }
	set n [countEntries exptdata]
	for {set i 1} {$i <= $n} {incr i} {
	    if { [string match $id $exptdata(0,$i)] } { return }
	}
	# create new deflt expt
	incr n
	if { $n >= $exptdata(nrows) } {
	    addEntryTableRows .ef.d.e 1 exptdata {}
	}
	for {set i 0; set j 1} {$i < [llength $exptmed]} {incr i; incr j} {
	    set exptdata($j,$irow) [lindex $exptmed $i]
	}
    }    

}

proc entryTable {f title colLbls nRows aname colbindlst} {
    upvar #0 $aname a
    global he3titlefont titlecolor lblfont lblcolor
    # builds entry tables in He3 cell editor gui
    # f is the toplevel or frame in which to build the table


    set m [frame $f.m]
    set t [label $m.t -text $title -font $he3titlefont -fg $titlecolor]
    set d [frame $f.d]
    set ncol [llength $colLbls]
    set a(mbar) $m
    set a(nrows) $nRows
    set a(ncol) $ncol
    set a(lbls) $colLbls
    set a(frame) $d
    for {set j 1} {$j <= $nRows} {incr j} {
	for {set i 0} {$i < $ncol} {incr i} {
	    set lbl [lindex $colLbls $i]
	    if { $j == 1 } {
		label $d.l$i -text $lbl -font $lblfont -fg $lblcolor
		grid $d.l$i -row 0 -column $i
	    }
	    set aelem $aname\($i,$j\)
	    set a($i,$j) ""
	    entry $d.e$i-$j -textvariable $aelem
	    foreach b $colbindlst {
		set col [lindex $b 0]
		set key [lindex $b 1]
		set scr [lindex $b 2]
		if { $i == $col || "*" == $col } {
		    bind $d.e$i-$j $key $scr
		}
	    }
	    bind $d.e$i-$j <Button-1> \
		"+ set curcellrow($d.e) $j; set curcellcol($d.e) $i"
	    grid $d.e$i-$j -row $j -column $i
	}
    }
    for {set i 0} {$i < $ncol} {incr i} {
	entryTableColwid $aname $i,1 w
    }
    grid $m
    grid $t -row 0 -column 0 -sticky e
    grid $d
}

proc parseTimeDate {dts} {
    global timedatefmt
    # to make portable to non-UNIX environ
    # replace date command with tcl clock scan
    # old was return [exec date --date=$dts $datetimefmt]
    # puts "parseTimeDate: dts=$dts"
    if [catch [list clock scan $dts] isecs] {
	Dialog_Warn $isecs
	return
    }
    return [clock format $isecs -format $timedatefmt]
}

proc parseTimeDateRow {irow} {
    global celldata
    global curcellrow frames
    if { $irow == 0 } {
	set irow $curcellrow($frames(1))
    }
    set celldata(1,$irow) [parseTimeDate $celldata(1,$irow)]
}

proc directEditCells {fnum} {
    global celldata
    global ncell curcellrow curcellcol frames framearrays
    global lockCells
    global readfinished

    # fnum is the frame number in cellEditor with values to edit.
    # data that can be edited and column numbers start from zero
    # tEmpty(3) iPol(4) iPolErr(5) fPol(6) fPErr(7) Tf(8) Tc(9) TcErr(10)
    # tEmpty(3) iPol(4) iPolErr(5) fPol(6) fPErr(7) Tf(8) lpf9 Tc10 TcErr11
    # beamArea is in exptdata
    # cellname (0). user can edit name
    # PorA
    # recall p h will print current cell info in pbscript.
    # user can check this in simulation toplevel.
    # fixed data in frame 4 add Dia
    # nsL0(21=1 in fixedcelldata) nsL0Err(22=2 in fixedcelldata)
    # Leng(23=3 in fixedcelldata) curvRad(24=4 in fixedcelldata)

    # RWE May 2011 add:
    # Dia(25=5 in fixedcelldata)
    # S3 SD SH A4 dA4 He3off=tilt in experiment data
    # table in transport

    # RWE Oct 2011 add:
    # lpf after fPErr before Tf

    # must lock cells to do direct editing in pbscript
    if { ! $lockCells } { return }

    # this is to be bound to cell editor entries to allow
    # direct changes to the current cells in pbscript
    # if lockCells flag is set

    # when a cell is read into the cell editor one
    # can lock out anymore reading of cells by pbscript
    # if one choses this immediate mode, the cell file loaded to the editor
    # should be read by pbscript via p h cellFile and
    # then locked via p hl 1
    # this allows user to modify a group cellfile and resolve
    # using the direct cell edit commands p he cellindex
    # without having to export a new file and rescan and reset it for group.
    # Thus it is useful to have a load current group cellFile option in
    # the editor.
    # in pbscript when cellfiles are read the cells get ordered by
    # install time (equal time keep same order as in file).
    # When immediate-mode is set for the editor reload the last cellFile
    # loaded to the editor in case user has already made changes
    # and also load it to pbscript
    # then lock it out.
    # This is dangerous as user may forget that immediate mode is selected
    # so always kill this mode if user changes group

    #puts "directEditCells starting"
    set f $frames($fnum)
    set fcell $frames(1)

    # if frame is 2 or 3 must edit all cells that match the 
    # trans-name or expt-name

    if { ! [info exists curcellrow($f)] || ! [info exists curcellcol($f)] } {
	return
    }

    set r $curcellrow($f)

    #puts "current framenum=$fnum framerow= $r"
    # fnum = 1 is He3CellData =4 is fixedCellData
    # if frame is 2 or 3 must edit all cells that match the 
    # trans-name or expt-name
    set celli {}
    if { $fnum == 1 || $fnum == 4 } {
	if { ! [info exists curcellrow($fcell)] } { return }
	set rcell $curcellrow($fcell)
	lappend celli $rcell
    } elseif { $fnum == 2 } {
	# trans. get the transport id name from col 0, and all same entries
	set tname [${f}0-$r get]
	for {set i 1} {$i <= $ncell} {incr i} {
	    set ctname [${fcell}12-$i get]
	    if { [string match $ctname $tname] } { lappend celli $i }
	}
    } elseif { $fnum == 3 } {
	# expt
	set ename [${f}0-$r get]
	for {set i 1} {$i <= $ncell} {incr i} {
	    set cename [${fcell}13-$i get]
	    if { [string match $cename $ename] } { lappend celli $i }
	}
    }

    #puts "cells to edit= $celli"
    set fa $framearrays($fnum)
    global $fa
    # get the labels list
    eval set lbls $$fa\(lbls\)
    # get the current row col for this frame

    set c $curcellcol($f)

    set lbl [lindex $lbls $c]
    # get the entry string
    set value [$f$c-$r get]
    foreach r $celli {
	set ri [expr $r - 1]
	pipeWrite "p he $ri $lbl $value"
	
    }
}

proc setDirectEditCells { } {
    global readfinished
    global lockCells
    global curcellf
    global plotconfigall
    if { ! $lockCells } {
	# turn off cell locking
	pipeWrite "p hl 0"
	
	return
    }
    # reload the last cellfile in the editor and sort the cells by install
    # so they correspond to pbscript ordering
    sortCells
    if { [info exists curcellf] && [string length $curcellf] > 0 } {
	importCellFile 0 $curcellf
	sortCells
    }

    # resend the last cellFile for cell editor
    if { [string length $curcellf] > 0 } {
	pipeWrite "p h $curcellf"
	
    }
    # now lock the cells from being reread
    pipeWrite "p hl 1"
    
    # disable plotconfigall else will set all groups and lockCells=0
    set plotconfigall 0
}

proc addEntryTableRows {d nRows aname colbindlst} {
    global curcellrow
    upvar #0 $aname a
    set nrows $a(nrows)
    set Nrows [expr $nrows + $nRows]
    set a(nrows) $Nrows
    set ncol $a(ncol)
    for {set j [expr $nrows + 1]} {$j <= $Nrows} {incr j} {
	for {set i 0} {$i < $ncol} {incr i} {
	    set aelem $aname\($i,$j\)
	    set $aelem ""
	    if { ! [winfo exists $d$i-1] } {
		set wid 12
		if { [info exists $aname(lbls)] } {
		    eval set lbls $$aname(lbls)
		    set wid [string length [lindex $lbls $i]]
		}
	    } else {
		set wid [$d$i-1 cget -width]

	    }
	    if { ! [winfo exists $d$i-$j] } {
		entry $d$i-$j -textvariable $aelem -width $wid
	    } else {
		$d$i-$j configure -width $wid
	    }
	    foreach b $colbindlst {
		set col [lindex $b 0]
		set key [lindex $b 1]
		set scr [lindex $b 2]
		if { $i == $col || "*" == $col } {
		    bind $d$i-$j $key $scr
		}
	    }
	    bind $d$i-$j <Button-1> \
		"+ set curcellrow($d) $j; set curcellcol($d) $i"
	    grid $d$i-$j -row $j -column $i
	}
    }
}

proc entryTableColwid {aname elem op} {
    # the entry widgets are $ef.e$icol-$row 1-nRows
    # the data are in a($icol,$jrow)
    upvar #0 $aname a
    if { [scan $elem {%d,%d} icol irow] < 2 } { return }
    set clbls $a(lbls)
    set clbl [lindex $clbls $icol]
    set wid [string length $clbl]
    set nrows $a(nrows) ;
    set fr $a(frame)
    for {set j 1} {$j <= $nrows} {incr j} {
	if [info exists a($icol,$j)] {
	    set sw [string length $a($icol,$j)]
	    if { $sw > $wid } { set wid $sw } 
	}
    }
    for {set j 1} {$j <= $nrows} {incr j} {
	$fr.e$icol-$j configure -width $wid
    }
}

proc clearEntries {anam} {
    global $anam
    set els [array names $anam]
    foreach el $els {
	if { [scan $el {%d,%d} i j] == 2 } {
	    eval set $anam\($el\) {" "}
	}
    }
}
proc countEntries {anam} {
    upvar #0 $anam a
    set n 0
    for {set ii 1} {$ii <= $a(nrows)} {incr ii} {
	if { [string length [string trim $a(0,$ii)]] < 1 } {
	    break
	}
	incr n
    }
    return $n
}

proc importCellFile {apnd fil} {
    global lockCells readfinished
    if [catch "open $fil r" f] {
	Dialog_Warn "failed to import $fil cellfile"
	return
    }
    set lines {}
    while { [gets $f line] >= 0 } {
	if { [string match \#* $line] || [string length $line] < 1 } {
	    continue
	}
	lappend lines $line
    }
    close $f
    # RWE Aug 2010 changed to dbcheck = 0
    # cant append in cellLock mode
    if { $lockCells } { set apnd 0 }
    importCells $apnd 0 $lines
    if { $lockCells } {
	sortCells
	pipeWrite "p hl 0"
	
	pipeWrite "p h $fil"
	
	pipeWrite "p hl 1"
	
    }
}

proc validTransTable {name} {
    global transTables
    set nam [string trim $name]
    if { [string length $nam] < 1 } { return 0 }
    if { ! [info exists transTables($nam)] } { return -1 }
    set lst $transTables($nam)
    set n [llength $lst]
    set ivars [lindex $lst 1]
    set dlen 3
    if { [string length $ivars] > 1 } { set dlen 4 }
    if { $n < [expr 2 + $dlen] } { return -1 }
    for { set i 2 } { $i < $n } { incr i } {
	if { ! [scan [lindex $lst $i] "%g" var] } { return -1 }
    }
    return 1
}


    # lines is a list of He3-cellfile datalines
    # RWE Aug 2010: on import no longer consult database
    # RWE Nov 2015 get ready to import database as cellDB in cwd
    # RWE May 2011: any transport tables are checked for validity 
    # after all lines
    # are read as the actual definition can be on any line

# RWE May 2011 table(transport) is:
# name indvars(E|H|EH) data
# where data is:
# indvarValues(E|H|E H) trans transErr ...

	# June 2014 implement new cell file format which skips the extraneous information
	# that is in the original spread sheet format.
	# The first fields that are skipped are Energy and lambda so that field #6 becomes Pi
	# We can distinguish this new format because Pi is always less than 1 
	# whereas neither Energy or lambda should be less than 1 and certaintly not zero.
	# start by reading the first 9 fields to see which format


	#if { [set icell [lsearch $Cells $name]] < 0 }
	#    Dialog_Warn "cant import unknown cell $name"
	#    continue
	#
	#set cellname [lindex $Cells $icell]

	#if { $nscan < 19 } { set tEmpty $dbcell(tEmpty) }
	#if { $nscan < 20 } { set tEmptySlope $dbcell(tEmptySlope) }
	#if { $nscan < 21 } { set Pbar $dbcell(Pbar) }
	#if { $nscan < 22 } { set L $dbcell(L) }
	#if { $nscan < 23 } { set D $dbcell(D) }
	#if { $nscan < 24 } { set R $dbcell(R) }
	#if { $nscan < 25 } { set volcc $dbcell(volcc) }
	#if { $nscan < 26 } { set nsL0 $dbcell(nsL0) }
	#if { $nscan < 27 } { set nsL0err $dbcell(nsL0err) }

	# next compare the transport specs for this cell to any previous ones
	# and if a new spec is required, add it to the transport gui
	# giving it a name like trans#
	# transport data in the gui are stored in transdata($col,$row)
	# to make the gui easy to build


# if table strip list element 0 as the name which becomes the value of table
# while the whole string list is stored in transTables(name)

# if the table is only a name, do not update the transTables array


proc importCells {apnd dbcheck lines} {

    global cfvnames cell Cells ctLbls
    global dbchk overrideDB
    global ncell ntrans nexpt celldata transdata exptdata
    global transTables
    global cf tf ef
    global cellsColBindList transColBindList exptColBindList

    # if not apnd clear entries for any previous celldata transdata or exptdata
    if { ! $apnd } {
	clearEntries celldata
	clearEntries transdata
	clearEntries exptdata
	arrayClear transTables
	set ncell 0
    }
    set ncellstart $ncell

    for {set il 0} {$il < [llength $lines]} {incr il} {
	set line [lindex $lines $il]
	if { [string match \#* $line] || [string length $line] < 1 } {
	    continue
	}
	# set deflts
	set curvCor 1.
	set angcor 1.
	set t1 0.
	set t2 0.
	set hsigsq 0.
	set vsigsq 0.
	set xsigsq 0.
	set tEmptySlope 0.
	set Tf 0.
	set tf 0
	set L 10.
	set R 25.
	set beamArea 8.
	set Hmos 40.
	set Vmos 40.
	set Hcols 40.
	set Hcol2 40.
	set VcolsDeg 2.
	set Vcol2Deg 2.
	set S3 0.
	set SD 0.
	set SH 0.
	set rfloss 0.
	set A4 0.
	set dA4 0.
	set tilt 0.
	set table ""


	set nscan [scan $line \
		       {%s %s %s %s \
			    %g %g %g %g %g} \
name PorA date tim \
startSecs Energy lambda Pi Pierr]
	if { $nscan < 9 } {
	    Dialog_Warn "not enough celldata at line $il"
	    continue
	}
	set shortform 0
	if { $Energy > 0. && $Energy < 1. } { set shortform 1 }
	if { $shortform > 0 } {
	set nscan [scan $line \
		       {%s %s %s %s \
			    %g %g %g %g %g \
			    %g %g %g %g %g \
			    %g %g %g %g %g \
			    %g %g \
			    %g %g %g %g %g %g %g \
			    %g %g \
			    %g %g %g %g %g %g %g %[^~]} \
name PorA date tim \
startSecs Pi Pierr T Terr \
beamArea teff terr feff ferr \
tEmpty tEmptySlope L D R \
nsL0 nsL0err \
Hmos Vmos dsp Hcols Hcol2 VcolsDeg Vcol2Deg \
xsigsq Tf \
S3 SD SH A4 dA4 tilt rfloss table]
	} else {
	set nscan [scan $line \
		       {%s %s %s %s \
			    %g %g %g %g %g %g %g \
			    %g %g %g %g %g %g %g \
			    %g %g %g %g %g %g \
			    %g %g %g %g %g \
			    %s %g %g %g %g %g %g %g %g \
			    %g %g %g %g %g %g \
			    %g %g %g %g %g %g %g %[^~]} \
name PorA date tim \
startSecs Energy lambda Pi Pierr T Terr \
beamArea nsL0E nsL0Eerr teff terr feff ferr \
tEmpty tEmptySlope Pbar L D R \
volcc nsL0 nsL0err nsLE nsLEerr \
rnam Hmos Vmos dsp Hcols Hcol2 VcolsDeg Vcol2Deg omRad \
hsigsq vsigsq xsigsq curvCor angcor Tf \
S3 SD SH A4 dA4 tilt rfloss table]
	}
	


	if { [string length [string trim $table]] > 0 } {
	    set tabname [lindex $table 0]
	    if { ! [info exists transTables($tabname)] ||
		 [llength $transTables($tabname)] < 2 } {
		set transTables($tabname) $table
	    }
	    set table $tabname
	}
	set cellname $name
	global $cellname

	#array set dbcell [array get $cellname]
	if { $nscan < 27 } {
	    Dialog_Warn "not enough celldata at line $il"
	    continue
	}

	incr ncell
	if { $ncell > $celldata(nrows) } {
	    addEntryTableRows $cf.d.e 1 celldata $cellsColBindList
	}
	set cell($ncell) {}
	foreach vnam $cfvnames {
	    eval set val $$vnam
	    lappend cell($ncell)  $vnam $val
	}
    }
    # finished reading lines

    set dboverridemsg "db override is set so imported values will be used."
    set dbimportmsg "db values will be used."

    # now import the data into the gui
    for {set i [expr $ncellstart + 1]} {$i <= $ncell} {incr i} {
	array set cellinfo $cell($i)
	set cellname $cellinfo(name)
	set celldata(0,$i) $cellname

	# join together the start date and time and parse with tcl clock cmd
	set dt $cellinfo(tim)
	append dt " $cellinfo(date)"
	set celldata(1,$i) [parseTimeDate $dt]

	set celldata(2,$i) $cellinfo(PorA)

	set Pi $cellinfo(Pi)
	set Pierr $cellinfo(Pierr)
	set T $cellinfo(T)
	set Tf $cellinfo(Tf)
	set Terr $cellinfo(Terr)

	# insert tEmpty before Pi
	set celldata(3,$i) $tEmpty
	set celldata(4,$i) $Pi
	set celldata(5,$i) $Pierr

	# RWE Oct 2011 lpf=rfloss becomes col 9, so bump up column nos 9->10 etc
	set celldata(9,$i) $rfloss

	# now if Tf > 0 a two point Pi Pf was used
	# if the Terr < 0 it is actually Pferr
	# easiest case is Tf <= 0 then use Tc TcErr to calc fPol fPolErr
	set celldata(8,$i) $Tf
	set celldata(10,$i) $T
	if { $Tf <= 0. } {
	    set celldata(11,$i) $Terr
	} else {
	    # compute the final polarization and error at Tf
	    set Pf [expr $Pi*exp(-$Tf/$T)]
	    if { $Terr < 0. } {
		# then Terr is actually Pferr
		set Pferr [expr abs($Terr)]
		set a [expr $Pierr/$Pi]
		set b [expr $Pferr/$Pf]
		set c [expr log($Pi/$Pf)]
		set Terr [expr ($Tf/$c/$c)*sqrt($a*$a + $b*$b)]
	    } else {
		# calc Pferr from Terr Pierr
		set a [expr log($Pi/$Pf)*($Terr/$T)]
		set b [expr $Pierr/$Pi]
		set c [expr $a*$a - $b*$b]
		set Pferr 0.
		if { $c > 0. } { set Pferr [expr $Pf*sqrt($c)] }
	    }
	    set celldata(11,$i) $Terr
	    set celldata(6,$i) $Pf
	    set celldata(7,$i) $Pferr
	}

	# this finishes the basic He3-polarization stuff
	# now check imported cell specs nsL R L tEmpty slope against dbase
	set dbaname $cellname
	global $dbaname


	set warnmsg ""
	# NB dbcheck is turned off circa 2010
	if { $dbcheck } {
	    foreach dbc $dbchk {
		eval set dbval $$dbaname\($dbc\)
		set rval $cellinfo($dbc)
		if { [expr abs($rval-$dbval)] > 0.00001 } {
		    append warnmsg "imported cell $cellname value for $dbc = $rval differs from db value = $dbval.\n"
		    if { ! $overrideDB } {
			set cellinfo($dbc) $dbval
		    }
		}
	    }
	}

	if { [string length $warnmsg] > 0 } {
	    if { $overrideDB } {
		append warnmsg $dboverridemsg
	    } else {
		append warnmsg $dbimportmsg
		set cell($i) [array get cellinfo]
	    }
	    Dialog_Warn $warnmsg
	}


	# check validity of any transTables

	set tchk(1) $cellinfo(teff)
	set tchk(2) $cellinfo(terr)
	set tchk(3) $cellinfo(feff)
	set tchk(4) $cellinfo(ferr)

	set vt [validTransTable $cellinfo(table)]
	if { $vt < 0 } {
	    Dialog_Warn "invalid transport table for cell $cellinfo(name)"
	    set cellinfo(table) ""
	}

	set ntrans [countEntries transdata]
	set newt 1
	set match 0
	if { $ntrans > 0 } {
	    # check them all trying to find a match
	    for {set ii 1} {$ii <= $ntrans} {incr ii} {
		set newt 0
		for {set j 1} {$j <= 4} {incr j} {
		    if { ! [string match $tchk($j) $transdata($j,$ii)] } {
			set newt 1
			break
		    } 
		}
		if { [info exists cellinfo(table)] && \
			 [string length [string trim $cellinfo(table)]] > 0 && \
			 ![string match $cellinfo(table) $transdata(5,$ii)] } {
		    set newt 1
		}
		if { $newt == 0 } { set match $ii ; break }
	    }
	}

	if { $newt } {
	    if { $ntrans == $transdata(nrows) } {
		addEntryTableRows $tf.d.e 1 transdata transColBindList
	    }
	    incr ntrans
	    set tname trans$ntrans
	    set transdata(0,$ntrans) $tname
	    set celldata(12,$i) $tname
	    for {set j 1} {$j <= 4} {incr j} {
		set transdata($j,$ntrans) $tchk($j)
	    }
	    set transdata(5,$ntrans) $cellinfo(table)
	} else {
	    set celldata(12,$i) $transdata(0,$match)
	}

	# same for experiment. NB rfloss implemented in He3CellData
	set echk(1) [string trim $cellinfo(Hcols)]
	set echk(2) [string trim $cellinfo(Hcol2)]
	set echk(3) [string trim $cellinfo(VcolsDeg)]
	set echk(4) [string trim $cellinfo(Vcol2Deg)]
	set echk(5) [string trim $cellinfo(Hmos)]
	set echk(6) [string trim $cellinfo(dsp)]
	set echk(7) [string trim $cellinfo(xsigsq)]
	set echk(8) [string trim $cellinfo(beamArea)]
	set echk(9) [string trim $cellinfo(S3)]
	set echk(10) [string trim $cellinfo(SD)]
	set echk(11) [string trim $cellinfo(SH)]
	set echk(12) [string trim $cellinfo(A4)]
	set echk(13) [string trim $cellinfo(dA4)]
	set echk(14) [string trim $cellinfo(tilt)]

	set nexpt [countEntries exptdata]

	set match 0
	set newt 1
	if { $nexpt > 0 } {
	    # check them all trying to find a match
	    for {set ii 1} {$ii <= $nexpt} {incr ii} {
		set newt 0
		for {set j 1} {$j <= 14} {incr j} {
		    if { ! [string match $echk($j) \
				[string trim $exptdata($j,$ii)]] } {
			set newt 1
			break
		    } 
		}
		if { $newt == 0 } { set match $ii ; break }
	    }
	}

	if { $newt } {
	    if { $nexpt == $exptdata(nrows) } {
		addEntryTableRows $ef.d.e 1 exptdata exptColBindList
	    }
	    incr nexpt
	    set ename expt$nexpt
	    set exptdata(0,$nexpt) $ename
	    set celldata(13,$i) $ename
	    for {set j 1} {$j <= 14} {incr j} {
		set exptdata($j,$nexpt) $echk($j)
	    }
	} else {
	    set celldata(13,$i) $exptdata(0,$match)
	}
    }
}

proc setExptFocus {exptlst} {
    global cfvexpt exptdata ef curcellrow frames

    set w [focus]
    if { [scan $w {$ef.d.e%d-%d} icol irow] != 2 } {
	if { ! [info exists curcellrow($frames(3))] } {
	    Dialog_Warn "focus is not in one of the expt entries"
	    return
	} else {
	    set irow $curcellrow($frames(3))
	}
    }
    for {set i 0; set j 1} {$i < [llength $exptlst]} {incr i; incr j} {
	set exptdata($j,$irow) [lindex $exptlst $i]
    }
}

proc delFocus {typ} {
    global cell ncell celldata transdata exptdata
    global frames frametype
    if { $ncell < 1 } { return }
    set w [focus]
    set f $frames($typ)
    if { [scan $w "$f%d-%d" icol irow] != 2 } {
	Dialog_Warn "focus is not in one of the $frametype($typ) entries"
	return
    }
    set n -1
    if { $typ == 1 } {
	set n 13
    } elseif { $typ == 2 } {
	set n 5
    } elseif { $typ == 3 } {
	set n 14
    }
    for {set i 0} {$i <= $n} {incr i} {
	$f$i-$irow delete 0 end
    }
    # move the remaining cells up one
    if { $typ == 1 } {
	for {set i $irow; set j [expr $i + 1]} {$i < $ncell} {incr i; incr j} {
	    set cell($i) $cell($j)
	    for {set k 0} {$k <= $n} {incr k} {
		set celldata($k,$i) $celldata($k,$j)
	    }
	}
    } elseif { $typ == 2 } {
	for {set i $irow; set j [expr $i + 1]} {$i < $transdata(nrows)} \
	    {incr i; incr j} {
	    for {set k 0} {$k <= $n} {incr k} {
		set transdata($k,$i) $transdata($k,$j)
	    }
	}
    } elseif { $typ == 3 } {
	for {set i $irow; set j [expr $i + 1]} {$i < $exptdata(nrows)} \
	    {incr i; incr j} {
	    for {set k 0} {$k <= $n} {incr k} {
		set transdata($k,$i) $transdata($k,$j)
	    }
	}
    }
    # delete the last one
    for {set i 0} {$i <= $n} {incr i} {
	$f$i-$ncell delete 0 end
    }
    # decr ncell
    if { $typ == 1 } { incr ncell -1 }
}
proc addRow {typ} {
    # 1 = He3  2 = transport  3=expt
    global frames framearrays cellsColBindList
    global celldata transdata exptdata
    global ncell ntrans nexpt
    if { $typ == 1 } {
	if { ! [info exists celldata(nrows)] } { set celldata(nrows) 0 }
	incr ncell
	if { $ncell > $celldata(nrows) } {
	    addEntryTableRows \
		$frames($typ) 1 $framearrays($typ) $cellsColBindList
	}
    } elseif { $typ == 2 } {
	if { ! [info exists transdata(nrows)] } { set transdata(nrows) 0 }
	incr ntrans
	if { $ntrans > $transdata(nrows) } {
	    addEntryTableRows $frames($typ) 1 $framearrays($typ) {}
	}
    } elseif { $typ == 3 } {
	if { ! [info exists exptdata(nrows)] } { set exptdata(nrows) 0 }
	incr nexpt
	if { $nexpt > $exptdata(nrows) } {
	    addEntryTableRows $frames($typ) 1 $framearrays($typ) {}
	}
    }
}

proc sortCells {} {
    global cell ncell celldata utcfmt
    # make sure startSecs is defined for each cell
    for {set i 1} {$i <= $ncell} {incr i} {
	array set cellinfo $cell($i)
	if [catch [list clock scan $celldata(1,$i)] isecs] {
	    Dialog_Warn $isecs
	    return
	}
	set cellinfo(startSecs) [clock format $isecs -format $utcfmt]
	#[exec date --date=$celldata(1,$i) $utcfmt]
	set cell($i) [array get cellinfo]
	set secs($i) $cellinfo(startSecs)
    }
    for {set i 1} {$i < $ncell} {incr i} {
	set isecs $secs($i)
	set imin $i
	for {set j [expr $i + 1]} {$j <= $ncell} {incr j} {
	    if { $secs($j) < $isecs } {
		set isecs $secs($j)
		set imin $j
	    }
	}
	if { $imin == $i } { continue }
	# swap i with imin both celldata and cell(index)
	set temp $secs($i)
	set secs($i) $secs($imin)
	set secs($imin) $temp
	set temp $cell($i)
	set cell($i) $cell($imin)
	set cell($imin) $temp
	for {set k 0} {$k <= 13} {incr k} {
	    set temp $celldata($k,$i)
	    set celldata($k,$i) $celldata($k,$imin)
	    set celldata($k,$imin) $temp
	}
    }	
}

proc checkTransName {name} {
    global transdata celldata
    set ncell $celldata(nrows)
    for {set i 1} {$i <= $ncell} {incr i} {
	    if { [string match $name $transdata(0,$i)] } { return $i }
    }
    return 0
}
proc checkExptName {name} {
    global exptdata celldata
    set ncell $celldata(nrows)
    for {set i 1} {$i <= $ncell} {incr i} {
	    if { [string match $name $exptdata(0,$i)] } { return $i }
    }
    return 0
}


proc exportAS {} {
    global exportname
    if { ! [Dialog_Entry "save cellfile as:" exportname] } { return }
    if [file exists $exportname] {
	if { ! [Dialog_Query "File $exportname already exist. Overwrite?"] } {
	    return
	}
    }
    exportCellFile $exportname
}

proc isFixedDevice {cellname} {
    global fixedDevices
    foreach dev $fixedDevices {
        set nc [string length $dev]
        if [string equal -nocase -length $nc $dev $cellname] { return 1 }
    }
    return 0
}

proc exportCellFile {fil} {
    global cfvnames cfvexpt dbchk exptmed
    global reqcellC reqtransC reqcellFC
    global cell ncell
    global datefmt exportdatefmt timefmt utcfmt
    global celldata transdata exptdata cellinfo transTables

    # sanity check for blanks in required columns
    # check all rows celldata(nrows)
    # NB transTables are checked for validity on input

    set okcells {}
    set badcells {}
    #for {set i 1} {$i <= $ncell} {incr i}
    for {set i 1} {$i <= $celldata(nrows)} {incr i} {
        # make sure there is a cellname and check it against fixedDevices
        set cellname [string trim $celldata(0,$i)]
	    if { [string length [string trim $cellname]] < 1 } {
	        continue
	    }
	    lappend okcells $i
	    # deflt Tf to zero
	    if { [string length [string trim $celldata(9,$i)]] < 1 } {
	        set celldata(9,$i) 0.
	    }
    }
    for {set i 0} {$i < [llength $okcells]} {incr i} {
        set cel [lindex $okcells $i]
        set cellname [string trim $celldata(0,$cel)]
        set reqCols $reqcellC
        # for fixed device allow Tc Terr Tf to be blanks, just set outputs to zero
        if [isFixedDevice $cellname] { set reqCols $reqcellFC }
        foreach c $reqCols {
	        if {[string length [string trim $celldata($c,$cel)]] < 1} {
		        #set okcells [lreplace $okcells $i $i]
		        lappend badcells $cel
	        }
	    }
    }
    # after loop finishes update okcells
    foreach cel $badcells {
        set i [lsearch -exact -integer $okcells $cel]
        if { $i < 0 } { continue }
        Dialog_Warn "missing required cell information for row $cel"
        set okcells [lreplace $okcells $i $i]
    }
    for {set i 0} {$i < [llength $okcells]} {incr i} {
	    set cel [lindex $okcells $i]
	    set it [checkTransName $celldata(12,$cel)]
	    if { $it < 1 } {
	        #set okcells [lreplace $okcells $i $i]
	        lappend badcells $cel
            Dialog_Warn "transport name problem for row $cel"
	        continue
	    }
	    foreach c $reqtransC {
	        if {[string length [string trim $transdata($c,$it)]] < 1} {
		        #set okcells [lreplace $okcells $i $i]
		        lappend badcells $cel
		        break
	        }
	    }
	    set teff $transdata(1,$it)
	    set terr $transdata(2,$it)
	    set feff $transdata(3,$it)
	    set ferr $transdata(4,$it)
	    set ttable $transdata(5,$it)
	    set l1 [string length [string trim $teff]]
	    set l2 [string length [string trim $terr]]
	    set l3 [string length [string trim $feff]]
	    set l4 [string length [string trim $ferr]]
	    set l5 [string length [string trim $ttable]]
	    if { [expr $l1*$l2] < 1 && $l5 < 1 } {
	        #set okcells [lreplace $okcells $i $i]
	        lappend badcells $cel
	        Dialog_Warn "transport blank data problem for row $cel"
	    }
	    if { $l5 < 1 && ($teff < 0. || $teff > 1. || $terr < 0.) } {
	        lappend badcells $c
            Dialog_Warn "trans data teff terr invalid or missing table for cell row $cel"
	        continue
	    }
	    # if feff ferr are missing or invalid set to 1 0
	    if { $l3 < 1 || $feff < 0. || $feff > 1. } {
            set transdata(3,$it) 1.
        }	    
	    if { $l4 < 1 || $ferr < 0. } {
	        set transdata(4,$it) 0.
	    }
    }
    # after loop finishes update okcells
    foreach cel $badcells {
        set i [lsearch -exact -integer $okcells $cel]
        if { $i < 0 } { continue }
        set okcells [lreplace $okcells $i $i]
    }

    # data checks
    # trans is required, missing expt go to defaults
    #puts "n okcells= [llength $okcells]"
    for {set i 0} {$i < [llength $okcells]} {incr i} {
	    set c [lindex $okcells $i]
        set cellname [string trim $celldata(0,$c)]
        set reqCols $reqcellC
        if [isFixedDevice $cellname] { set reqCols $reqcellFC }
	    set PorA $celldata(2,$c)
	    set tEmpty $celldata(3,$c)
	    set Pi $celldata(4,$c)
	    set Pierr $celldata(5,$c)
	    set Pf $celldata(6,$c)
	    set Pferr $celldata(7,$c)
	    set Tf $celldata(8,$c)
	    set rfloss $celldata(9,$c)
	    set Tc $celldata(10,$c)
	    set Tcerr $celldata(11,$c)
	    set trans $celldata(12,$c)
	    set expt $celldata(13,$c)
	    if {[scan $celldata(2,$c) {%[pPaA]} res] < 1 || \
	        $Pi < -1. || $Pi > 1. || $Pierr < 0. || \
		    ([string length [string trim $Pf]] > 0 \
		    && ($Pf < -1. || $Pf > 1.)) || \
		    ([string length [string trim $Pferr]] > 0 && $Pferr < 0.)} {
	            #set okcells [lreplace $okcells $i $i]
	        lappend badcells $c
            Dialog_Warn "Pi or Pf data problem for cell row $c"
	        continue
	    }
	    if { ![isFixedDevice $cellname] } { 
		    if { $Tc < 0. } {
		    	lappend badcells $c
                Dialog_Warn "Tc invalid for cell row $c"
	            continue
		    }
		}
    }
    # after loop finishes update okcells
    foreach cel $badcells {
        set i [lsearch -exact -integer $okcells $cel]
        if { $i < 0 } { continue }
 
        set okcells [lreplace $okcells $i $i]
    }

    if { [llength $okcells] < 1 } {
	    Dialog_Warn \
	        "Failed cellfile export. All cells were missing required data."
	    return
    }
    if { [llength $badcells] > 0 } {
	    Dialog_Warn \
            "The following cells are missing required data and will be omitted:\n$badcells"
    }

    # first get the celldata exptdata transdata 
    # for each cell in turn into cellinfo(cfvnames)

    set outcells {}
    for {set ii 0} {$ii < [llength $okcells]} {incr ii} {
	    set i [lindex $okcells $ii]
	    # start with the values in cell(i)
	    if [info exists cell($i)] {
	        array set cellinfo $cell($i)
	    }
	    set name $celldata(0,$i)
	    #upvar #0 $name cellnam
	    #if { ! [info exists cellnam] }
	    #    Dialog_Warn "Cell $name is not a valid cellname."
	    #    continue
	    #

	    set cellinfo(name) $name
	    if [catch [list clock scan $celldata(1,$i)] isecs] {
	        # if clock scan fails set to now and warn
	        if [catch [list clock scan "now"] isecs] {
	            Dialog_Warn "ERROR: failed to get seconds for now"
	            return
	        }
	        Dialog_Warn "Cell=$name install time set to now."
	    }
	    set cellinfo(date) [clock format $isecs -format $exportdatefmt]
	    set cellinfo(tim) [clock format $isecs -format $timefmt]
	    set cellinfo(startSecs) [clock format $isecs -format $utcfmt]
	    #set cellinfo(date) [exec date --date=$celldata(1,$i) $datefmt]
	    #set cellinfo(tim) [exec date --date=$celldata(1,$i) $timefmt]
	    #set cellinfo(startSecs) [exec date --date=$celldata(1,$i) $utcfmt]
	    set cellinfo(PorA) $celldata(2,$i)
	    set cellinfo(tEmpty) $celldata(3,$i)
	    set cellinfo(Pi) $celldata(4,$i)
	    set cellinfo(Pierr) $celldata(5,$i)
	    # fixed device T Terr Tf may be blank, if so set them 0 here
	    foreach icl {8 10 11} {
	        if { [string length [string trim $celldata($icl, $i)]] < 1 } {
	            set $celldata($icl, $i) 0
	        }
	    }
	    set cellinfo(Tf) $celldata(8,$i)
	    set cellinfo(rfloss) $celldata(9,$i)
	    set cellinfo(T) $celldata(10,$i)
	    set cellinfo(Terr) $celldata(11,$i)
	    set it [checkTransName $celldata(12,$i)]
	    set cellinfo(teff) $transdata(1,$it)
	    set cellinfo(terr) $transdata(2,$it)
	    set cellinfo(feff) $transdata(3,$it)
	    set cellinfo(ferr) $transdata(4,$it)
	    set cellinfo(table) $transdata(5,$it)
	    if { [validTransTable $cellinfo(table)] > 0 } {
	        #puts "$cellinfo(table) is valid transTable"
	        set cellinfo(table) $transTables($cellinfo(table))
	    } else {
	        #puts "$cellinfo(table) is invalid transTable"
	        set cellinfo(table) ""
	    }

	    foreach vnam $cfvexpt val $exptmed { set cellinfo($vnam) $val }
	    set ie [checkExptName $celldata(13,$i)]
	    if { $ie > 0 } {
	        for {set k 0;set j 1} {$k < [llength $cfvexpt]} {incr k;incr j} {
		        set vnam [lindex $cfvexpt $k]
		        if { [string length [string trim $exptdata($j,$ie)]] > 0 } {
		            set cellinfo($vnam) $exptdata($j,$ie)
		        }
	        }
	    }
	    # now just put cellinfo back into cell(i)
	    #puts "export: cell $i:\n[array get cellinfo]"
	    set cell($i) [array get cellinfo]
	    lappend outcells $cell($i)
    }

    writeCellsToFile $fil $outcells
}
proc writeCellsToFile {fil outcells} {
    global cfvnames
    # outcells is a list of celldataarraylists from array get
    # first collect the data string widths from the outcells
    foreach vnam $cfvnames {
	set wid 0
	for {set ii 0} {$ii < [llength $outcells]} {incr ii} {
	    array set cellinfo [lindex $outcells $ii]
	    if { [set sl [string length $cellinfo($vnam)]] > $wid } {
		set wid $sl
	    }
	}
	if { [set sl [string length $vnam]] > $wid } { set wid $sl }
	set width($vnam) [expr $wid + 1]
    }
    # now write
    if [catch "open $fil w" fid] {
	Dialog_Warn "failed to open $fil for cellfile export"
	return
    }

    puts -nonewline $fid "#cellFile "
    set ncfv [llength $cfvnames]
    set ncfv1 [expr $ncfv - 1]
    for {set i 0} {$i < $ncfv1} {incr i} {
	set vnam [lindex $cfvnames $i]
	set wid $width($vnam)
	puts -nonewline $fid [format "%${wid}s" $vnam]
    }
    set vnam [lindex $cfvnames $ncfv1]
    puts $fid " $vnam"
    for {set ii 0} {$ii < [llength $outcells]} {incr ii} {
	array set cellinfo [lindex $outcells $ii]
	puts -nonewline $fid "          "
	for {set i 0} {$i < $ncfv1} {incr i} {
	    set vnam [lindex $cfvnames $i]
	    set wid $width($vnam)
	    puts -nonewline $fid [format "%${wid}s" $cellinfo($vnam)]
	}
	set vnam [lindex $cfvnames $ncfv1]
	puts $fid " $cellinfo($vnam)"	
    }
    close $fid
}

proc dirCellFiles {} {
    # return list of cellfiles in current directory
    # this is a tcl version of the check option to PBdefineCells
    global Cells
    set flst [glob -nocomplain *]
    set cellfiles {}
    foreach fil $flst {
	if { [file isdirectory $fil] || ![file readable $fil] } { continue }
	if [catch "open $fil r" f] { continue }
	while { [gets $f line] >= 0 } {
	    if { [string match #cellName* $line] || \
		 [string match #cellFile* $line] } {
		lappend cellfiles $fil
		break
	    }
	    if [string match #* $line] { continue }
	    foreach cel $Cells {
		if [string match $cel* $line] {
		    lappend cellfiles $fil
		    break
		}
	    }
	    break
	}
	close $f
    }
    return $cellfiles
}

proc setCellSelRow {c} {
    global celldata cell ncell cf
    global curcellrow frames
    set w [focus]
    if { [scan $w {$cf.d.e%d-%d} icol irow] < 2 } {
	set irow $curcellrow($frames(1))
	#Dialog_Warn "Focus is not in one of the He3 Cell Log rows"
	#return
    } else {
	set curcellrow($frames(1)) $irow
    }
    set celldata(0,$irow) $c
    global $c
    set aname $c
    # make sure the db basic cell values are in cell(irow)
    arrayClear cellall
    if { [info exists cell($irow)] } { array set cellall $cell($irow) }
    array set cellall [array get $c]
    array set cellall "name $c"
    set cell($irow) [array get cellall]
    #puts [array get cellall]
    if { $irow > $ncell } { set ncell $irow }
}

proc editCellFixedParams {} {
    # pop-up fixed cell data gui
    # cellName nsL(@1Ang) nsLErr Length Rcurv Dia
    # this is frame 4 for cell editing
    global ecfp cf
    global curcellrow frames
    global cell celldata fixedcelldata

    if { ! [info exists ecfp] || ! [winfo exists $ecfp] } {
	buildECFP
    }
    if { ![winfo ismapped $ecfp] } { wm deiconify $ecfp }
    raise $ecfp

    # look in the cell editor to see what cell is selected
    set w [focus]
    if { [scan $w {$cf.d.e%d-%d} icol irow] < 2 } {
	if { ! [info exists curcellrow($frames(1))] } {
	    Dialog_Warn "NO cell currently selected"
	    return
	}
	set irow $curcellrow($frames(1))
    } else {
	set curcellrow($frames(1)) $irow
    }
    if { ! [info exists cell($irow)] } {
	Dialog_Warn "NO cell defined for row $irow"
	return
    }
    array set locell $cell($irow)
    # load the cell($irow) fixed data into the fixedcelldata array
    set fixedcelldata(0,1) $celldata(0,$irow)
    set fixedcelldata(1,1) $locell(nsL0)
    set fixedcelldata(2,1) $locell(nsL0err)
    set fixedcelldata(3,1) $locell(L)
    set fixedcelldata(4,1) $locell(R)
    set fixedcelldata(5,1) $locell(D)
}

proc setCellFD {ic} {
    global cell frames curcellrow fixedcelldata
    set irow $curcellrow($frames(1))
    array set locell $cell($irow)
    if { $ic == 1 } {
	set locell(nsL0) $fixedcelldata(1,1)
    } elseif { $ic == 2 } {
	set locell(nsL0err) $fixedcelldata(2,1)
    } elseif { $ic == 3 } {
	set locell(L) $fixedcelldata(3,1)
    } elseif { $ic == 4 } {
	set locell(R) $fixedcelldata(4,1)
    } elseif { $ic == 5 } {
	set locell(D) $fixedcelldata(5,1)
    }
    set cell($irow) [array get locell]
}

proc buildECFP {} {
    global ecfp ecfpColBindList fixedcelldata

    if { [winfo exists $ecfp] } { return }
    toplevel $ecfp
    wm title $ecfp "He3-Cell Fixed Params"

    set f [frame $ecfp.f]
    grid $f

    set ecfpLbls { \
	 He3-CellName nsL(@1Ang) nsLerr  Length(cm) CurvRadius(cm) Dia(cm)}

    entryTable $f "He3-Cell Fixed Params" $ecfpLbls 1 \
	fixedcelldata $ecfpColBindList
}

proc importCurrentGroupCellfile {} {
    global cellFile
    importCellFile 0 $cellFile
}

############### cell gui build ##################3

# He3FlipLoss added Oct 2011
proc startFlipLossTableGui {} {
    if { ! [winfo exists .flg] } { buildFlipLossTableGui }
    if { ! [winfo ismapped .flg] } { wm deiconify .flg }
    raise .flg
    # put the current fliploss tables into the gui
    reloadFlipLossTables
}

proc showCellFL {} {
    global fliplossCell textFL
    global nerr nread readlines
    # rebuild any flip loss table for selected cell
    set i [expr $fliplossCell - 1]
    pipeWrite "p hfcd $i"
    # this shouldn't fail
    # if it does fail just return
    if { $nerr > 0 } { return }
    # print the results to textFL text widget
    $textFL delete 1.0 end
    for {set i 0} {$i < $nread} {incr i} {
	$textFL insert end "[lindex $readlines $i]\n"
    }
}

proc setfliplossCell {} {
    global fliplossCell ccfliploss showcellflmlbls showcellflmb 
    global celldata ncell bccFL
    global nerr errlines
    # call whenever fliplossCell changes via menu or fliplossFile changes
    # or manual init. This always builds the current cell flip loss table.
    # fliplossCell is index>0 of current cell row from cell editor
    # ccfliploss is entry variable for lpf value at celldata(9,$i).
    if { ! [info exists fliplossCell] || \
	     [string length [string trim $fliplossCell]] < 1 || \
	     [scan $fliplossCell %d fl] < 1 || $fl < 1 || $fl > $ncell } {
	return
    }
    set i [expr $fliplossCell - 1]
    # update the currentcell menubutton text
    $showcellflmb configure -text [lindex $showcellflmlbls $i]
    # update the flip loss deflt value using value for current cell
    set ccfliploss $celldata(9,$fliplossCell)
    # rebuild any flip loss table for selected cell req errs
    pipeWrite "p hfcd $i 1"
    # if this fails disable the ShowCurrentCellFLtable button
    # else enable the button and add the table to the text window
    if { $nerr > 0 } {
	$bccFL configure -state disabled
	Dialog_Warn "$errlines"
    } else {
	$bccFL configure -state normal
    }
}

proc updateShowFlipLossCellMenu {} {
    # use this when fliploss gui inits and
    # when fliploss gui is mapped and cell editor changes cells
    global showcellflm showcellflmlbls
    global celldata ncell
    if { ! [info exists showcellflm] } { return }
    $showcellflm delete 0 end
    set showcellflmlbls {}
    for {set i 1} {$i <= $ncell} {incr i} {
	set lbl "$celldata(0,$i) $celldata(1,$i) $celldata(2,$i)"
	lappend showcellflmlbls $lbl
	$showcellflm add radio \
	    -label $lbl \
	    -variable fliplossCell -value $i \
	    -command setfliplossCell
    }
}

proc reloadFlipLossTables {} {
    # called when startFlipLossTableGui for init filenames menu and
    # current cell
    # do a sortCells so order in gui is same as in libPB
    # get current He3 Ei and Ef FlipLoss Filenames
    global celldata ncell
    global showcellflm
    global fliplossFileEi fliplossFileEf ccfliploss fliplossCell
    global bEiFile bEfFile
    global readlines

    sortCells
    pipeWrite "p hf 1"
    set fliplossFileEi [lindex [lindex $readlines 0] 1]
    if { ! [file exists $fliplossFileEi] } {
	$bEiFile configure -state disabled
    } else {
	$bEiFile configure -state normal
	readFlipLossEi
    }
    pipeWrite "p hf 2"
    set fliplossFileEf [lindex [lindex $readlines 0] 1]
    if { ! [file exists $fliplossFileEf] } {
	$bEfFile configure -state disabled
    } else {
	$bEfFile configure -state normal
	readFlipLossEf
    }

    # load all the cells from the gui into the ShowCell FlipLossTable Menu
    # can use celldata array to do this
    updateShowFlipLossCellMenu

    # manual init
    if { ! [info exists fliplossCell] || \
	     [string length [string trim $fliplossCell]] < 1 || \
	     [scan $fliplossCell %d fl] < 1 || $fl < 1 || $fl > $ncell } {
	set fliplossCell 1
    }
    set ccfliploss ""
    if { $ncell > 0 } { setfliplossCell }
}

proc readFlipLossEi {} {
    global nerr bEiFile
    global fliplossFileEi
    pipeWrite "p hf 1 $fliplossFileEi"
    if { $nerr > 0 } {
	$bEiFile configure -state disabled
    } else {
	$bEiFile configure -state normal
    }
}
proc readFlipLossEf {} {
    global nerr bEfFile
    global fliplossFileEf
    pipeWrite "p hf 2 $fliplossFileEf"
    if { $nerr > 0 } {
	$bEfFile configure -state disabled
    } else {
	$bEfFile configure -state normal
    }
}

proc showEiFL {} {
    global fliplossFileEi textFL
    global nerr nread readlines errlines
    pipeWrite "p hfd 0 0"
    if { $nerr > 0 } {
	Dialog_Warn "$errlines"
	return
    }
    $textFL delete 1.0 end
    for {set i 0} {$i < $nread} {incr i} {
	$textFL insert end "[lindex $readlines $i]\n"
    }
}
proc showEfFL {} {
    global fliplossFileEf textFL
    global nerr nread readlines errlines
    pipeWrite "p hfd 1 0"
    if { $nerr > 0 } {
	Dialog_Warn "$errlines"
	return
    }
    $textFL delete 1.0 end
    for {set i 0} {$i < $nread} {incr i} {
	$textFL insert end "[lindex $readlines $i]\n"
    }
}

proc plotCellPol {} {
}

proc buildFlipLossTableGui {} {
    # this lists Ei Ef fliploss Files that have been located
    # list current cell editor He3cell flip loss table  and lpf-value
    # of list a fliploss File
    # for current cell it is UTC date-time accum-loss
    # for fliplossfile it is UTC date-time lpf (-1) deflts to cell->lpf
    #
    # ShowEi   FlipLossFile  ____
    # ShowEf   FlipLossFile  ____
    # ShowCell FlipLossTable cellsMenu
    # deflt cell LossPerFlip ____
    # PlotCellPolarization
    # Text
    global showcellflmb showcellflm
    global fliplossFileEi fliplossFileEf ccfliploss
    global bEiFile bEfFile bccFL textFL
    global mbbg mbab

    set w .flg
    toplevel $w
    wm title $w "Show He3 Flip Loss"
    set tf [frame $w.tf]

    grid $tf

    set bEiFile [button $tf.ei -text "Show Ei FlipLoss File" \
		  -bg green -activebackground yellow \
		  -command showEiFL -width 20]
    set bEfFile [button $tf.ef -text "Show Ef FlipLoss File" \
		  -bg green -activebackground yellow \
		  -command showEfFL -width 20]
    set bccFL [button $tf.cfl -text "Show Cell FlipLoss Table" \
		  -bg green -activebackground yellow \
		  -command showCellFL -width 20]
    set cclbl [label $tf.cclbl -text "Current Cell LossPerFlip" -width 20]
    set bcPP [button $tf.cpp -text "Plot Cell Polarization" \
		  -bg green -activebackground yellow \
		  -command plotCellPol -width 20]


    set eEi [entry $tf.eei -textvariable fliplossFileEi -width 30]
    set eEf [entry $tf.eef -textvariable fliplossFileEf -width 30]
    bind $eEi <Return> readFlipLossEi
    bind $eEi <Tab> readFlipLossEi
    bind $eEf <Return> readFlipLossEf
    bind $eEf <Tab> readFlipLossEf

    set showcellflm $tf.mbc.menu
    set showcellflmb [menubutton $tf.mbc -menu $showcellflm -relief raised \
			  -width 30 -bg $mbbg -activebackground $mbab]    
    menu $showcellflm -tearoff 0
    set ccfl [entry $tf.ccfl -textvariable ccfliploss -width 30]

    grid $bEiFile $eEi
    grid $bEfFile $eEf
    grid $bccFL   $showcellflmb
    grid $cclbl   $ccfl
    grid $bcPP

    set bf $w.bf
    set textFL [eval scrolledText $bf -wrap none -width 60 -height 16]
    grid $bf
    grid $textFL
}

proc buildTransTableGui {} {
    # save the container for the table entries, used in reload etc
    global ttents

    set w .ttg
    toplevel $w
    wm title $w "PB transport tables"
    # top frame for buttons, bottom frame for tables
    # top frame buttons: new-table delete-table save reload
    # mid frame grid are the labels
    # bot frame grid rows are entries for each transport table
    # index name vars data
    set tf [frame $w.tf]
    set mf [frame $w.mf]
    set ttents [frame $w.bf]
    # bf is the container for all the entries
    grid $tf
    grid $mf
    grid $ttents

    set bnew [button $tf.new -text "new-table" \
		  -bg green -activebackground yellow \
		  -command newTransTable -width 10]
    set bdel [button $tf.del -text "del-table" \
		  -bg green -activebackground yellow \
		  -command delTransTable -width 10]
    set brel [button $tf.rel -text "reload" \
		  -bg green -activebackground yellow \
		  -command reloadTransTables -width 10]
    set bsav [button $tf.sav -text "save" \
		  -bg green -activebackground yellow \
		  -command saveTransTables -width 10]
    grid $bnew $bdel $brel $bsav

    set ln [label $mf.ln -text name -width 10]
    set lv [label $mf.lv -text vars -width 10]
    set ld [label $mf.ld -text data -width 60]
    grid $ln $lv $ld -sticky w
    grid columnconfigure $mf 2 -minsize 60

    # make sure the help toplevel popup exists and
    # put help bindings for these labels into helptext array below
    # labels widgets are .ttg.mf.ln .lv .ld
    buildHelpPop
    bind $ln <Enter> "showHelp $ln .ttg"
    bind $ln <Leave> withdrawHelp
    bind $lv <Enter> "showHelp $lv .ttg"
    bind $lv <Leave> withdrawHelp
    bind $ld <Enter> "showHelp $ld .ttg"
    bind $ld <Leave> withdrawHelp
}


proc newTransTable {} {
    global ttents nttents
    # just add a new row of entries
    set ri $nttents
    incr nttents
    set j $nttents
    if { ! [winfo exists $ttents.en$j] } {
	entry $ttents.en$j -textvariable ttname($j) -width 10
	entry $ttents.ev$j -textvariable ttvars($j) -width 10
	entry $ttents.ed$j -textvariable ttdata($j) -width 60
    }
    grid $ttents.en$j -row $ri -column 0
    grid $ttents.ev$j -row $ri -column 1
    grid $ttents.ed$j -row $ri -column 2
}

proc delTransTable {} {
    global ttents nttents
    if { $nttents < 1 } { return }
    # selection must be in one of the ttents
    if { [catch {selection own} w] || ! [string match $ttents* $w] } {
	Dialog_Warn "Selection must be in a table entry to delete that table."
	return
    }
    # parse the widget to get the row number
    if { ! [regexp {^\.ttg\.bf\.e[nvd]+([0-9]+)$} $w wm ir] } {
	Dialog_Warn "Failed to parse selection widget name $w."
	return
    }
    # copy rows below ir up 1 row
    for {set j $ir; set i [expr $ir + 1]} {$i <= $nttents} {incr i; incr j} {
	$ttents.en$j delete 0 end
	$ttents.en$j insert 0 [$ttents.en$i get]
	$ttents.ev$j delete 0 end
	$ttents.ev$j insert 0 [$ttents.ev$i get]
	$ttents.ed$j delete 0 end
	$ttents.ed$j insert 0 [$ttents.ed$i get]
    }
    # finally remove the last row
    grid remove $ttents.en$nttents
    grid remove $ttents.ev$nttents
    grid remove $ttents.ed$nttents
    incr nttents -1
}

proc saveTransTables {} {
    # save transTables to both pbcor transTables array
    # and to pbscript as well
    global ttents nttents
    global nerr errlines
    global transTables
    for {set i 1} {$i <= $nttents} {incr i} {
	set name [string trim [$ttents.en$i get]]
	set vars [string trim [$ttents.ev$i get]]
	set data [$ttents.ed$i get]
	if { [string length $name] < 1 } {
	    Dialog_Warn "table name at $i must be non-blank."
	    return
	}

	if { ! [regexp {^[EeHh]+$} $vars] } {
	    Dialog_Warn "table vars at $i must E and|or H."
	    return
	}
	set nvars [string length $vars]
	if { [llength $data] < [expr 2 + $nvars] } {
	    Dialog_Warn "table data at $i must have at least one data pt."
	    return
	}
    }
    # everything looks OK so delete all current tables
    pipeWrite "p hx-"
    if { $nerr > 0 } {
	Dialog_Warn "$errlines\n"
    }
    for {set i 1} {$i <= $nttents} {incr i} {
	set name [string trim [$ttents.en$i get]]
	set vars [string trim [$ttents.ev$i get]]
	set data [$ttents.ed$i get]
	pipeWrite "p hxd $name $vars $data"
	if { $nerr > 0 } {
	    Dialog_Warn "$errlines\n"
	}
	set transTables($name) [list $name $vars]
	eval lappend transTables($name) $data
    }
}

proc reloadTransTables {} {
    global ttents nttents
    global readlines nread
    global ttname ttvars ttdata
    # read trans tables with  p hx
    # and load entries
    # the entries are  .ttg.bf.en# .ttg.ev# .ttg.ed#
    # reload first forgets all the entries
    foreach w [grid slaves $ttents] {
	if [winfo exists $w] { grid remove $w }
    }
    # get the tables list and write to entries
    pipeWrite "p hx"
    for {set i 2; set j 1} {$i < $nread} {incr i; incr j} {
	set tbl [lindex $readlines $i]
	if { ! [winfo exists $ttents.en$j] } {
	    entry $ttents.en$j -textvariable ttname($j) -width 10
	    entry $ttents.ev$j -textvariable ttvars($j) -width 10
	    entry $ttents.ed$j -textvariable ttdata($j) -width 60
	}
	grid $ttents.en$j $ttents.ev$j $ttents.ed$j
	$ttents.en$j delete 0 end
	$ttents.en$j insert 0 [lindex $tbl 0]
	$ttents.ev$j delete 0 end
	$ttents.ev$j insert 0 [lindex $tbl 1]
	$ttents.ed$j delete 0 end
	$ttents.ed$j insert 0 [lrange $tbl 2 end]
    }
    set nttents [expr $nread - 2]
}


proc startTransTableGui {} {
    if { ! [winfo exists .ttg] } { buildTransTableGui }
    if { ! [winfo ismapped .ttg] } { wm deiconify .ttg }
    raise .ttg
    # put the current transport tables into the gui
    reloadTransTables
}


proc useParamErrors {} {
    global useParamErrs
    global cell celldata transdata
    global savecellerrs
    
    # command for useParamErrs checkbutton
    # errors are:
    # Pierr(5,row) fPErr(derived not incelldata)
    # Terr(10,row) nsL0err is not in celldata
    # terr ferr nsL0err
    # rows start at 1, cols start at 0
    # celldata(nrows) is number of rows in editor

    # if useParamErrs, look for savecellerrs(row)
    # that matches cell(row) for  name PorA date tim startSecs
    # if matches and any of the savecellerrs errors are non-zero,
    # then load cell from savecellerrs
    # update celldata(col,row) and transdata foreach row

    # else not useParamErrs, if any cell(row) errors are non-zero
    # then save cell(row) to savecellerrs(row)
    # set all cell(row) errors to zero
    # and update celldata(col,row) and transdata

    if { ! [info exists celldata(nrows)] || $celldata(nrows) < 1 } {
	return
    }

    set evars { Pierr Terr nsL0err terr ferr }
    #set ecols { 5     10   0       2    4 }
    set match { name PorA date tim startSecs }

    if { $useParamErrs } {
	# if any current cell errors are non-zero just return
	for { set i 1 } { $i <= $celldata(nrows) } { incr i } {
	    if { ! [info exists cell($i)] } { continue }
	    arraySet ld $cell($i)
	    foreach ev $evars {
		if { [info exists ld($ev)] && $ld($ev) != 0. } { return }
	    }
	}
	# all errors are zero. Is there a savecellerrs(row) that matches
	# cell(row) in name PorA date tim startSecs? If so, load it
	# and update celldata transdata
	for { set i 1 } { $i <= $celldata(nrows) } { incr i } {
	    if { ![info exists cell($i)] || ![info exists savecellerrs($i)] } {
		continue
	    }
	    if [info exists ld] { unset ld }
	    if [info exists lde] { unset lde }
	    array set ld $cell($i)
	    array set lde $savecellerrs($i)
	    set matches 1
	    foreach m $match {
		if { ! [string match $ld($m) $lde($m)] } {
		    set matches 0
		    break
		}
	    }
	    if { ! $matches } { continue }
	    # this may pick up fPErr which is not normally in cell array
	    set cell($i) $savecellerrs($i)
	    set i1 [expr $i - 1]
	    pipeWrite "p he $i1 nsLerr $lde(nsL0err)"
	    set celldata(5,$i) $lde(Pierr)
	    pipeWrite "p he $i1 iPolErr $lde(Pierr)"
	    set celldata(11,$i) $lde(Terr)
	    pipeWrite "p he $i1 TcErr(hr) $lde(Terr)"
	    if [info exists lde(fPErr)] {
		set celldata(7,$i) $lde(fPErr)
		#pipeWrite "p he $i fPErr $lde(fPErr)"
		# updated by PBcalcCellParams
	    }
	    set tid $celldata(12,$i)
	    for {set j 1} {$j <= $transdata(nrows)} {incr j} {
		if { [string length $transdata(0,$i)] < 1 } { continue }
		if { ! [string match $tid $transdata(0,$i)] } { continue }
		set transdata(2,$i) $lde(terr)
		pipeWrite "p he $i1 tErr $lde(terr)"
		set transdata(4,$i) $lde(ferr)
		pipeWrite "p he $i1 fErr $lde(ferr)"
	    }
	}
	return
    }

    # here useParamErrs = 0
    # if all current param errors are zero just return
    set allzeros 1
    for { set i 1 } { $i <= $celldata(nrows) } { incr i } {
	if { ! [info exists cell($i)] } { continue }
	if [info exists ld] { unset ld }
	array set ld $cell($i)
	foreach ev $evars {
	    if { [info exists ld($ev)] && $ld($ev) != 0. } {
		set allzeros 0
		break
	    }
	}
	if { $allzeros } { return }
    }
    # save current to savecellerrs(row) and update errors in gui to zero
    for { set i 1 } { $i <= $celldata(nrows) } { incr i } {
	if { ! [info exists cell($i)] } { continue }
	set savecellerrs($i) $cell($i)
	if { [string length $celldata(7,$i)] > 0 } {
	    lappend savecellerrs($i) fPErr $celldata(7,$i)
	}
	#puts "before array set ld cell($i)= $cell($i)"
	if [info exists ld] { unset ld }
	array set ld $cell($i)
	#puts "ld from cell($i)= [array get ld]"
	foreach ev $evars {
	    set ld($ev) 0.
	}
	set i1 [expr $i - 1]
	pipeWrite "p he $i1 nsLerr 0."
	set cell($i) [array get ld]
	#puts "cell($i) reset from ld= $cell($i)"
	set celldata(5,$i) 0.
	pipeWrite "p he $i1 iPolErr 0."
	set celldata(11,$i) 0.
	pipeWrite "p he $i1 TcErr(hr) 0."
	if { [info exists ld(fPErr)] && [string length $ld(fPErr)] > 0 } {
	    set celldata(7,$i) 0.
	    #pipeWrite "p he $i1 fPErr 0."
	}
	set tid $celldata(12,$i)
	for {set j 1} {$j <= $transdata(nrows)} {incr j} {
	    if { [string length $transdata(0,$i)] < 1 } { continue }
	    if { ! [string match $tid $transdata(0,$i)] } { continue }
	    set transdata(2,$i) 0.0
	    pipeWrite "p he $i1 tErr 0.0"
	    set transdata(4,$i) 0.0
	    pipeWrite "p he $i1 fErr 0.0"
	}
    }
}

proc buildCellGui {} {
    global standalone
    global celledtTop celledtPre
    global cf tf ef
    global celldata transdata exptdata
    global Cells curcellf
    global exptopen exptmed exptfine
    global dircellfiles lockCells useParamErrs
    global cellsColBindList transColBindList exptColBindList
    global overrideDB
    global help helpmsg
    global mbbg mbab

    if { ! $standalone && [winfo exists $celledtTop] } { return }

    set mn [toplevel $celledtTop]
    set mnlbl "He3 Cell Log"
    wm title $mn $mnlbl
    wm iconname $mn "He3 Log"


    # need 3 frames
    # top cells
    # bot left transport
    # bot right expt conditions

    set cf [frame $celledtPre.cf -bd 2 -relief raised]
    set tf [frame $celledtPre.tf -bd 2 -relief raised]
    set ef [frame $celledtPre.ef -bd 2 -relief raised]
    grid $cf -columnspan 2 -sticky ew
    grid $tf $ef -sticky ew

    set ctLbls {
	cellName installTimeDate PorA
	tEmpty iPol iPolErr fPol fPErr Tf(hr) lpf Tc(hr) TcErr(hr) trans expt
    }


    entryTable $cf "He3 cells" $ctLbls 4 celldata $cellsColBindList
    set m $celldata(mbar)

    set mcells $m.cells
    menubutton $mcells -text "select cell current row" -menu $mcells.menu \
	-bg $mbbg -relief raised -activebackground $mbab
    set m1 [menu $mcells.menu -tearoff 1]
    foreach cel $Cells {
	$m1 add radio -label $cel -variable curcell -value $cel \
	    -command "setCellSelRow $cel"
    }

    # make a menubutton to import a cellfile from the dircellfiles
    #set mimport $m.imp
    #menubutton $mimport -text "import cellfile" -menu $mimport.menu \
	#    -bg #00aa00 -relief raised -activebackground #ffaa00
    #set m2 [menu $mimport.menu -tearoff 1]
    #foreach celf $dircellfiles {
    #    $m2 add radio -label $celf -variable curcellf -value $celf \
	#	    -command "importCellFile $celf"
    #}
    #grid $mimport -sticky e -row 0 -column 2 -padx 15
    
    #set bexport $m.exp
    #button $bexport -text "export cellfile AS" -command exportAS \
	#    -bg green -relief raised -activebackground #ffaa00
    #grid $bexport -sticky e -row 0 -column 3 -padx 15
    

    set film $m.file
    menubutton $film -text File -menu $film.menu \
	-bg $mbbg -relief raised -activebackground $mbab
    set mf [menu $film.menu -tearoff 1]

    $mf add cascade -label "import cellfile" -menu $mf.imp
    set mi [menu $mf.imp]
    foreach celf $dircellfiles {
	$mi add radio -label $celf -variable curcellf -value $celf \
	    -command "importCellFile 0 $celf"
    }
    $mf add cascade -label "import append cellfile" -menu $mf.impa
    set ma [menu $mf.impa]
    foreach celf $dircellfiles {
	$ma add radio -label $celf -variable curcellf -value $celf \
	    -command "importCellFile 1 $celf"
    }
    $mf add command -label "import current group cellfile" \
	-command importCurrentGroupCellfile
    $mf add separator
    
    $mf add command -label "export cellfile AS" -command exportAS
    
    
    set edit $m.edit
    menubutton $edit -text Edit -menu $edit.menu \
	-bg $mbbg -relief raised -activebackground $mbab
    set m3 [menu $edit.menu -tearoff 1]
    $m3 add command -label "Sort cells by installTimeDate" -command sortCells
    $m3 add separator
    $m3 add command -label "Delete selected cell" -command "delFocus 1"
    $m3 add command -label "Delete selected trans" -command "delFocus 2"
    $m3 add command -label "Delete selected expt" -command "delFocus 3"
    $m3 add separator
    $m3 add command -label "Add He3-cell" -command "addRow 1"
    $m3 add command -label "Add transport" -command "addRow 2"
    $m3 add command -label "Add experiment" -command "addRow 3"
    $m3 add separator
    #$m3 add check -label "Import file over-rides cellDB" -variable overrideDB
    $m3 add command -label "Edit selected He3-cell fixed params" \
	-command editCellFixedParams
    $m3 add command -label "Edit transport tables" \
	-command startTransTableGui
    $m3 add command -label "Show He3FlipLoss" \
	-command startFlipLossTableGui
    
    $m3 add separator
    $m3 add cascade -label "Set selected expt to stnd" -menu $m3.sub
    set m4 [menu $m3.sub -tearoff 0]
    $m4 add command -label "open resolution" \
	-command "setExptFocus [list $exptopen]" 
    $m4 add command -label "med resolution" \
	-command "setExptFocus [list $exptmed]" 
    $m4 add command -label "fine resolution" \
	-command "setExptFocus [list $exptfine]" 
    #grid $edit -sticky e -row 0 -column 4 -padx 15
    
    # add checkbutton for immediate mode
    set immode $m.im
    set immode [checkbutton $m.im -variable lockCells -text "lock" \
		 -command setDirectEditCells]
    set perrs [checkbutton $m.perrs -variable useParamErrs -text "paramErrs" \
		   -command useParamErrors]

    # expt has hcols hcol2 vcols vcol2 hmos dsp beamAreacm S3 SD SH A4 tilt
    # trans has transeff transErr flipeff flipErr table
    
    grid $film -row 0 -column 1 -padx 15
    grid $edit -sticky e -row 0 -column 2 -padx 15
    grid $mcells -sticky e -row 0 -column 3 -padx 15
    grid $immode -sticky e -row 0 -column 4 -padx 15
    grid $perrs -sticky e -row 0 -column 5 -padx 15
    
    # shorten labels for experiment as we need to add more columns
    # use the help facility to indicate field units
    set exLbls {
	name
	hcs hc2 vcs vc2 hmos dsp dlam2
	area s3 sd sh a4 da4 tilt
    }
    entryTable $ef "experiment" $exLbls 4 exptdata $exptColBindList
    
    set transLbls {
	name
	trans tErr flip fErr table
    }
    entryTable $tf "transport" $transLbls 4 transdata $transColBindList

    # create a toplevel help message window that
    # displays help text when mouse moves over label
    buildHelpPop

    # create bindings for Enter/Leave the labels .cf.d.l# .tf.d.l# ef.d.l#
    for {set i 0} {$i <= 13} {incr i} {
	set w $celledtPre.cf.d.l$i
	bind $w <Enter> "showHelp $w $mn"
	bind $w <Leave> withdrawHelp
    }
    set w $celledtPre.cf.m.im
    bind $w <Enter> "showHelp $w $mn"
    bind $w <Leave> withdrawHelp
    set w $celledtPre.cf.m.perrs
    bind $w <Enter> "showHelp $w $mn"
    bind $w <Leave> withdrawHelp

    for {set i 0} {$i <= 5} {incr i} {
	set w $celledtPre.tf.d.l$i
	bind $w <Enter> "showHelp $w $mn"
	bind $w <Leave> withdrawHelp
    }
    for {set i 0} {$i <= 14} {incr i} {
	set w $celledtPre.ef.d.l$i
	bind $w <Enter> "showHelp $w $mn"
	bind $w <Leave> withdrawHelp
    }
}

proc buildHelpPop {} {
    global help helpmsg
    if { [winfo exists .help] } { return }
    # create a toplevel help message window that
    # displays help text when mouse moves over widget
    set help [toplevel .help]
    set helpmsg [message $help.msg -aspect 500]
    pack $helpmsg
    wm title $help help
    wm withdraw $help
}

proc showHelp {w tl} {
    global help helpmsg helptext
    # w is the widget that wants help in toplevel tl
    # locate the w top left using winfo geometry
    # we need w height to place the help so keep it outside the loop

    if { ! [winfo exists .help] } { return }
    set wgeo [winfo geometry $w]
    regsub -all {\+} $wgeo " " wgeo
    scan $wgeo {%dx%d%d%d} L H wx wy

    set fx 0
    set fy 0
    set wd [winfo parent $w]
    # add up all the intermediate frame sizes
    while { ! [string match $wd $tl] } {
	set fgeo [winfo geometry $wd]
	regsub -all {\+} $fgeo " " fgeo
	scan $fgeo {%dx%d%d%d} dL dH dx dy
	set fx [expr $fx + $dx]
	set fy [expr $fy + $dy]
	set wd [winfo parent $wd]
    }
    # finally get geometry for top level
    set mgeo [winfo geometry $tl]
    regsub -all {\+} $mgeo " " mgeo
    scan $mgeo {%dx%d%d%d} mL mH mx my
    # mx,y + fx,y + dx,y + wx,y locates the top left = tlx,y
    # and w LxH
    # place the help msg at tlx,y + 3H just below the label
    # make sure it outside the label widget or the <Enter>
    # binding keeps firing
    set tlx [expr $mx + $fx + $wx]
    set tly [expr $my + $fy + $wy + 3*$H]
    wm deiconify $help
    wm geometry $help [format "%+d%+d" $tlx $tly]
    $helpmsg configure -text $helptext($w)
}

proc withdrawHelp {} {
    global help
    wm withdraw $help
}


proc openCellGui {} {
    global celledtTop
    if [winfo exists $celledtTop] {
	wm deiconify $celledtTop
	raise $celledtTop
	return
    }
    # remove Colwid traces and rebuild Cells toplevel
    # traces can survive toplevel destruction ?    
    # add the traces after the entry tables are created
    # otherwise will trigger entryTableColwid with upvar name a

    uplevel #0 trace remove variable celldata {write} entryTableColwid
    uplevel #0 trace remove variable transdata {write} entryTableColwid
    uplevel #0 trace remove variable exptdata {write} entryTableColwid

    buildCellGui
    wm deiconify $celledtTop
    raise $celledtTop

    uplevel #0 trace add variable celldata {write} entryTableColwid
    uplevel #0 trace add variable transdata {write} entryTableColwid
    uplevel #0 trace add variable exptdata {write} entryTableColwid
}


proc viewCells {grp} {
    global gnum
    global readlines readfinished nread
    global cellFile

    if { $grp == 2 } {
	openCellGui
	return
    }
    # if grp == 2 just open the cell editor toplevel
    # if grp, go thru files in group
    # else use selectedfiles
    # find the overall timestamp range for these files
    # then use p hta ti tf to get the cellfiles
    # which will return as full cellfile dataline that
    # can be read into He3Cell gui using importCells

    if { $grp } {
	set flst [getFilesInGroup $gnum]
    } else {
	set flst [selectionOwned 1 1 1]
    }
    if { [set nfls [llength $flst]] < 1 } {
	if { $grp } {
	    Dialog_Warn "No files in current group\n"
	} else {
	    Dialog_Warn "No files currently selected\n"
	}
	return
    }
    set trtot {}
    foreach fl $flst {
	set tr [fileTSrange $fl]
	if { [llength $tr] < 2 } { continue }
	set trtot [updateNumRange $tr $trtot]
    }

    # make sure to read current cellFile
    pipeWrite "p h $cellFile"
    

    #puts "about to p hta [lindex $trtot 0] [lindex $trtot 1]"
    pipeWrite "p hta [lindex $trtot 0] [lindex $trtot 1]"
    
    if { $nread < 3 } {
	Dialog_Warn "No cells from $cellFile apply to files:\n$flst\n"
	return
    }

    # open the cellGui and importCells
    openCellGui
    set cellLines {}
    for {set i 2} {$i < $nread} {incr i} {
	lappend cellLines [lindex $readlines $i]
    }
    importCells 0 0 $cellLines
}

proc datasetTSrange {iset} {
    global nread readlines readfinished
    pipeWrite "d$iset"
    
    # get the statistics for ts column
    pipeWrite "c al ts"
    
    if {$nread < 2 || [scan [lindex $readlines 1] {%s %d %d} typ ind col] < 3} {
	#Dialog_Warn "No timestamps found in file $fshort\n"
	return {}
    }
    pipeWrite "c s $col"
    
    if { [scan [lindex $readlines 1] {%d %s %g %g} col lbl mn mx] < 4 } {
	return {}
    }
    return [list $mn $mx]
}

proc fileTSrange {fl} {
    # get total timestamp range from file fl
    # if file is not already in a dataset read it
    global dsFile
    global readlines nread readfinished

    set fshort [file tail $fl]
    if { [info exists dsFile($fshort)] } {
	set iset $dsFile($fshort)
	set tempset 0
    } else {
	set ret [readFile $fshort 0]
	# continue on fatal read error
	if { [llength $ret] < 1 } {
	    Dialog_Warn "failed to open $fshort\n"
	    return
	}
	#returns [list $iset $npts $ncol $ny $nas $df $src $fshort $fname]
	set iset [lindex $ret 0]
	set tempset 1
    }
    set ret [datasetTSrange $iset]
    if { [llength $ret] < 1 } {
	#Dialog_Warn "No timestamps found in file $fshort\n"
	if { $tempset } {
	    pipeWrite "d-$iset"
	    
	}
	return {}
    }
    return $ret
}

#################  find Windows gui #######################

proc raisePlots {} {
    global wfgrphs
    foreach w $wfgrphs {
	raise $w
    }
}
proc tilePlots {v} {
    global wfgrphs
    set ix 0
    set iy 25
    set Hmax 0
    set Lmax 0
    for {set i 0} {$i < [llength $wfgrphs]} {incr i} {
	set w [lindex $wfgrphs $i]
	set wgeo [winfo geometry $w]
	regsub -all {\+} $wgeo " " wgeo
	scan $wgeo {%dx%d%d%d} L H wx wy
	if { $H > $Hmax } { set Hmax $H }
	if { $L > $Lmax } { set Lmax $L }
	wm geometry $w +$ix+$iy
	if { $v == 0 } {
	    set ix [expr $ix + $L]
	    if { $ix > 1000 } {
		set iy [expr $iy + $Hmax]
		set ix 0
		set Hmax $H
	    }
	} else {
	    set iy [expr $iy + $H]
	    if { $iy > 600 } {
		set ix [expr $ix + $Lmax]
		set iy 25
		set Lmax $L
	    }
	}
    }
}

proc raiseWFsel {} {
    global wtbox wgbox wfall
    set st [$wtbox curselection]
    set sg [$wgbox curselection]
    if { [llength $st] > 0 } {
	set iw [lindex $st 0]
    } elseif { [llength $sg] > 0 } {
	set iw [lindex $sg 0]
    } else {
	return
    }
    set w [lindex $wfall $iw]
    raise $w
}
proc moveWFsel {} {
    global wtbox wgbox wfall
    set st [$wtbox curselection]
    set sg [$wgbox curselection]
    if { [llength $st] > 0 } {
	set iw [lindex $st 0]
    } elseif { [llength $sg] > 0 } {
	set iw [lindex $sg 0]
    } else {
	return
    }
    set w [lindex $wfall $iw]
    wm geometry $w +0+20
    raise $w
}
proc saveWFsel {} {
    global wtbox wgbox wfall
    set st [$wtbox curselection]
    set sg [$wgbox curselection]
    if { [llength $st] > 0 } {
	set iw [lindex $st 0]
    } elseif { [llength $sg] > 0 } {
	set iw [lindex $sg 0]
    } else {
	return
    }
    set w [lindex $wfall $iw]
    raise $w
}

proc updateWlist {} {
    global wtbox wpbox wgbox wfw wfgrphs wfall
    $wtbox delete 0 end
    $wgbox delete 0 end
    $wpbox delete 0 end
    set aw [winfo children .]
    lappend aw .
    set atlm {}
    set grph {}
    set wfall {}
    foreach w $aw {
	if { [string match $w [winfo toplevel $w]] && [winfo ismapped $w] } {
	    if { [regexp {\.g([0-9]+)} $w match i] } {
		lappend grph $w
	    } else {
		lappend atlm $w
	    }
	}
    }
    set wfgrphs [lsort $grph]
    set atlm [lsort $atlm]
    set ngrph [llength $wfgrphs]
    set natlm [llength $atlm]
    set n [expr $ngrph + $natlm]
    $wtbox configure -height $n
    $wgbox configure -height $n
    $wpbox configure -height $n
    for {set i 0} {$i < $ngrph} {incr i} {
		set w [lindex $wfgrphs $i]
		$wtbox insert end [wm title $w]
		$wgbox insert end [winfo geometry $w]
		$wpbox insert end $w
		lappend wfall $w
    }
    for {set i 0} {$i < $natlm} {incr i} {
		set w [lindex $atlm $i]
		$wtbox insert end [wm title $w]
		$wgbox insert end [winfo geometry $w]
		$wpbox insert end $w
		lappend wfall $w
    }
}

proc openWfind {} {
    global wf
    if { ! [winfo exists $wf] } { buildWfind }
    updateWlist
    wm deiconify $wf
    raise $wf
}

proc saveSelectedWindows {} {
	global wtbox wpbox wgbox
	set ct [$wtbox curselection]
	set cp [$wpbox curselection]
	set cg [$wgbox curselection]
	set is -1
	if {[string length $ct] > 0} {
		set is $ct
	} elseif {[string length $cp] > 0} {
		set is $cp
	} elseif {[string length $cg] > 0} {
		set is $cg
	}
	foreach i $is {
		windowToFile [$wpbox get $is]
	}
}

# find windows
proc buildWfind {} {
    global wf wtbox wpbox wgbox wfw
    global mswindows
    global mbbg mbab
    global imgfmts graphicsFmt IMG

    toplevel $wf
    wm title $wf "find windows for PB"
    wm iconname $wf "PBfind"

    # top frame of cmnds
    # bottom frame two lbxs  window-title  geometry

    set top [frame $wf.t]
    set mid [frame $wf.i]
    set bot [frame $wf.b -bd 3 -relief sunken]
    grid $top
    grid $mid
    grid $bot

    set wtl [label $bot.tl -text window-title]
    set wpl [label $bot.pl -text window-path]
    set wgl [label $bot.gl -text window-geom]
    set wtbox [listbox $bot.tb -width 30]
    set wpbox [listbox $bot.pb -width 30]
    set wgbox [listbox $bot.gb -width 20]
    grid $wtl   $wpl   $wgl
    grid $wtbox $wpbox $wgbox

    set tcrp [button $top.rp -bg green -activebackground yellow \
		  -command raisePlots -text "raise all plots" -width 16]
    set tcth [button $top.tph -bg green -activebackground yellow \
		  -command "tilePlots 0" -text "tile plots horiz" -width 16]
    set tctv [button $top.tpv -bg green -activebackground yellow \
		  -command "tilePlots 1" -text "tile plots vert" -width 16]
		  
	set tcsv [button $top.tsv -bg green -activebackground yellow \
		  -command saveSelectedWindows -text "save selected windows" -width 16]
	if { ! $IMG } { $tcsv configure -state disabled }
	set tcsm [menubutton $top.tsm -width 17 -text "graphicsFmt" -menu $top.tsm.menu \
	    -bg $mbbg -relief raised \
	    -activebackground $mbab]
	set m [menu $top.tsm.menu -tearoff 1]
	foreach gfmt $imgfmts {
		$m add radio -label $gfmt -variable graphicsFmt -value $gfmt
	}
	if {$mswindows} {
		set tcss [button $top.tss -bg green -activebackground yellow \
		-command runWindowCapture -text "system windows saver" -width 16]
	}
	
	set tcrw [button $top.rw -bg green -activebackground yellow \
		  -command updateWlist -text "refresh window lists" -width 16]
	
    set tcir [label $mid.tinr -text "sel window title then left click to raise. right click (maybe middle button) to move upper-left"]
    set tcim [label $mid.tinm -text "saved windows must be visible on screen."]

    grid $tcrp $tcth $tctv
    if {$mswindows} {
	    grid $tcsv $tcsm $tcss
	} else {
		grid $tcsv $tcsm
	}
	grid $tcrw

    grid $tcir
    grid $tcim

    bind $wtbox <Button-1> {+ raiseWFsel}
    bind $wpbox <Button-1> {+ raiseWFsel}
    bind $wgbox <Button-1> {+ raiseWFsel}
    bind $wtbox <Button-2> {+ moveWFsel}
    bind $wpbox <Button-2> {+ moveWFsel}
    bind $wgbox <Button-2> {+ moveWFsel}
}


#################  simulation gui #########################

# user enters counts for cross-section and Ei Ef
# then series of fs msr-delta-time in hours
# wrt exp cell file start time
#
# simulation gets Poisson deviate for counts if deviate requested
# PBcross-section  Counts  deltaTime(Hrs) Ei Ef
#
# simulation collects all of these and generates measured counts using
# constraints + cellFile (no mon cor) from the transmission coefs
# this is done using the sim flags

# into a PBdataPt
# and corrects using gui constraints (mon cor would require countTime Ei etc)
# and cellFile
# prints the correction info from  p# d ptOpt msrOpt
#  Polarized-Beam Data Correction Simulation
# 4 sections

# 1 Solved Group Sim with CS functions
#   Xcol ________
#   flip-state  bg  slope  gaussH  gaussX gaussFWHM

# 2 CS entry simulations
#   elastic-cb  Ei ___________  Ef __________
#   flip-state     Counts          DeltaTimes(hrs)
#
#  goes with section 3 Transmission-Details
#   angle(deg)  ___________
#   detoff(deg) ___________
#   rotH(deg)   ___________

# 4 Solved Datapts: /Simulation   /Transmission-Details

# section 4 includes the dpt listing and buttons
# Print Cells   Clear Output  Save Output to pbsim


# text widget displays results from  p hs  OR p h

# following only called from the sim toplevel

proc clearSimTxt {} {
    global simtxt
    $simtxt delete 1.0 end
}
proc saveSimTxt {} {
    global simtxt
    if [catch "open pbsim w" f] {
	Dialog_Warn "Failed to open pbsim."
	return
    }
    puts $f [$simtxt get 1.0 end]
    close $f
}

proc doSimType {is} {
    global simtxt skipblanklines
    global nerr errlines nread readlines
    # do the sim and print the output to simtxt
    # remember to to disable skipblanklines, so output is formatted properly
    set skipblanklines 0
    pipeWrite "p s $is"
    #puts "doSimType nerr= $nerr  nread= $nread"
    if { $nerr > 0 } { Dialog_Warn "$errlines\n" }
    
    set skipblanklines 1
    $simtxt insert end "\n"
    for {set i 0} {$i < $nread} {incr i} {
		$simtxt insert end [lindex $readlines $i]
		$simtxt insert end "\n"
    }
}

proc doSimGroup {} {
    # handles group CS function simulation mode
    global simDev
    global simXcol
    global simFun simFunW
    global nerr errlines
    global PLT gnum

    # entry textvariables simFun(fs=1:4,pkparam=1:5)
    # entry widgets simFunW(fs,pkparam)

    # make sure we are in current group
    pipeWrite "g$gnum"

    # set sim deviate flag
    pipeWrite "p sv $simDev"

    # set the group sim Xcol for functions
    pipeWrite "p sgx $simXcol"

    # send the function for each flip-state
    for {set i 0;set k 1} {$i < 4} {incr i;incr k} {
	set vlst ""
	for {set j 1} {$j <= 5} {incr j} {
	    set v [$simFunW($i,$j) get]
	    if { [string length [string trim $v]] < 1 } {
		append vlst "," 0.
	    } else {
		append vlst "," $v
	    }
	}
	set vlst [string trim $vlst ","]
	pipeWrite "p sgc $k $vlst"
    }
    # update the group sim CS functions
    pipeWrite "p sgu"
    if { $nerr > 0 } {
	Dialog_Warn "$errlines\n"
	return
    }

    # do the group sim
    pipeWrite "p sg"
    if { $nerr > 0 } {
	Dialog_Warn "$errlines\n"
	return
    }

    # when succesful write .p.sim .m.sim and native.sim
    # files to disk
    # and hijacks the current group .p .m datasets
    # to hold the simulation results
    # so now can plot the group showing the simulation
    # Must resolve the group to get back to the non-sim solution
    # but be careful about cellfile lock when you do this.

    if { $PLT } {
	groupPlotSetup
	savePlot
	showPlot
    }
}

proc doSimDatapts {trans} {
    # trans= 0 | 1
    global simDev CFdpt gnum

    if { [llength $CFdpt] < 1 } {
	Dialog_Warn "NO datapts specified."
	return
    }
    # make sure we are in current group
    pipeWrite "g$gnum"
    # set sim deviate flag
    pipeWrite "p sv $simDev"
    # set the datapt indices
    pipeWrite "p stp $CFdpt"    

    doSimType $trans
}

proc doSim {trans} {
    # handles CS entry mode  trans= 0 | 1
    # trans=0 for CS simulation
    # trans=1 for transmission details
    global gnum
    global simtxt
    global simEi simEf simDev simct simtm simElastic
    global CFangle CFdetoff CFrotH CFdpt CFa4
    global readfinished readlines nread skipblanklines nerr errlines

    # update the constraints for current group should be done by a
    # re-solve
    # since writeGroupOpts will clear the group calc flag
    #if { ! [writeGroupOpts] } {
    #	Dialog_Warn "Failed to set grp $gnum options for simulation."
    #	return
    #}

    # RWE Aug 29 2011
    # enable multiple datapt CS values to see what happens to a peak
    # when the pb correction is applied for example.
    # Use the longest list of CS values to determine how many datapts to
    # sim and replicate the shorter lists by last value.
    # solve each datapt generating msr PBcounts for each time entered
    # This is a little hokey as we would actually like to have different
    # msr times for each CS input. But for the purpose this is
    # probably OK

    # Ei Ef

    # make sure we are in the current group
    pipeWrite "g$gnum"

    # set sim deviate flag
    pipeWrite "p sv $simDev"
    

    if { $simElastic } { set simEf $simEi }

    # find the longest list of the simct($i)
    # and make that the number of simulation datapts
    # shorter simct($i) get filled in by repetition of previous values
    # of zero if blank

    set nsimct 0
    for {set i 0} {$i < 4} {incr i} {
		if { [set isimct [llength $simct($i)]] > $nsimct } {
	    	set nsimct $isimct
		}
    }
    array set smct {}

    for {set id 0 } {$id < $nsimct} {incr id} {

		for {set i 0} {$i < 4} {incr i} {
	    	set isimct [llength $simct($i)]
	    	if { $isimct < 1 } {
				set smct($i) 0.
	    	} elseif { $id < $isimct } {
				set smct($i) [lindex $simct($i) $id]
	    	}
	    	# else uses last defined simct($i)
		}

		pipeWrite "p sc $smct(0) $smct(1) $smct(2) $smct(3) $simEi $simEf"
    
		# clear the sim data (times)
		pipeWrite "p sd-"
    
		# send the sim times data
		set isdata 0
		for {set i 0; set j 1} {$i < 4} {incr i; incr j} {
	    	if { [llength $simtm($i)] > 0 } {
				set isdata 1
				break
	    	}
		}
		if { ! $isdata && [llength $CFdpt] < 1 } {
	    	Dialog_Warn "missing delta times for simulation or no dpt spec."
	    	return
		}
	
		for {set i 0; set j 1} {$i < 4} {incr i; incr j} {
	    	if { [llength $simtm($i)] < 1 } { continue }
	    	foreach t $simtm($i) {
				pipeWrite "p sd $j $t"
		
	    	}
		}
	
		# in both sim modes
		# send CFangle CFdetoff CFrotH CFa4
		pipeWrite "p sta $CFangle"
		pipeWrite "p sto $CFdetoff"
		pipeWrite "p sth $CFrotH"
		pipeWrite "p st4 $CFa4"
	
		doSimType [expr $trans + 2]
		# type 2 is entry CS simulation
		# type 3 is entry CS transmission details

    }
}

proc printSimCells {} {
    global simtxt
    global readfinished readlines nread skipblanklines
    # get the pbscript output from p h and print it to the simtxt widget
    set skipblanklines 0    
    pipeWrite "p h"
    
    set skipblanklines 1
    $simtxt insert end "\n"
    for {set i 0} {$i < $nread} {incr i} {
	$simtxt insert end [lindex $readlines $i]
	$simtxt insert end "\n"
    }
}

proc openSim {} {
    global sim
    if { ! [winfo exists $sim] } { buildSim }
    wm deiconify $sim
    raise $sim
}

proc buildSim {} {
    global sim simtxt
    global he3titlefont titlecolor
    global simct simtm simEi simEf simDev simElastic
    global CFangle CFdetoff CFrotH CFa4 CFdpt
    global simFunW simFun simXcol
    global mswindows
    
    set fslbl [list offoff ONON ONoff offON]
    toplevel $sim
    wm title $sim "PB correction simulation and solvedGroup info"
    wm iconname $sim "PBsim"


    # RWE Sep 2011 redesigned sim toplevel to add group simulation
    # and make solved-datapts separate.
    # title-frame  modes-frame  output-text-frame
    set tf [frame $sim.tf]
    set mf [frame $sim.mf]
    set rf [frame $sim.of]

    # 5 mode frames for: group CS-sim CS-trans Solved-Datapts CmndButtons
    set gf [frame $mf.gf -relief raised -bd 2]
    set cs [frame $mf.cs -relief raised -bd 2]
    set csf [frame $cs.csf]
    set cst [frame $cs.cst]
    set dp [frame $mf.dp -relief raised -bd 2]
    set cmd [frame $mf.cmd]

    grid $tf
    grid $mf
    grid $rf

    grid $gf  -row 0 -column 0 -rowspan 2 -sticky news
    grid $cs  -row 0 -column 1 -rowspan 2 -sticky news
    grid $dp  -row 0 -column 2 -sticky news
    grid $cmd -row 1 -column 2 -sticky news

    grid $csf $cst -sticky news

    # title frame with deviaters checkbutton
    set title "Polarized-Beam Data Correction Simulation and SolvedGroup Info"
    set t [label $tf.t -text $title -font $he3titlefont -fg $titlecolor]
    set dev [checkbutton $tf.dev -variable simDev -text deviates]
    grid $t -row 0 -column 0
    grid $dev -row 0 -column 2
    grid columnconfigure $tf 1 -minsize 20

    # group sim frame
    set dosimG [button $gf.do -text "solved group simulation" \
		  -bg green -activebackground yellow \
		  -command doSimGroup -width 40]
    set glbl [label $gf.g -text "CS functions"]
    set gxcoll [label $gf.xcoll -text "Xcolumn"]
    set gxcole [entry $gf.xcole -textvariable simXcol -width 12]

    grid $dosimG -row 0 -column 0 -columnspan 6
    grid $glbl -row 1 -column 0 -columnspan 2
    grid $gxcoll -row 1 -column 3
    grid $gxcole -row 1 -column 4 -columnspan 2

    # add the 5 lines labels function params
    foreach w {fsl gbl sll ghl gxl gwl} i {0 1 2 3 4 5} \
	lbl {flip-state bg slope gaussH gaussX gFWHM} {
	    set ww [label $gf.$w -text $lbl]
	    grid $ww -row 2 -column $i
	}
    
    for {set i 0;set j 3} {$i < 4} {incr i;incr j} {
	set fsl [label $gf.fs$i -text [lindex $fslbl $i]]
	grid $fsl -row $j -column 0
	foreach w {gb sl gh gx gw} k {1 2 3 4 5} {
	    set simFunW($i,$k) \
		[entry $gf.$w$i-$k -textvariable simFun($i,$k) -width 6]
	    grid $simFunW($i,$k) -row $j -column $k
	}
    }


    # the CS simulation frame
    set dosim [button $csf.do -text "CS simulation" \
		  -bg green -activebackground yellow \
		  -command "doSim 0" -width 40]
    set elast [checkbutton $csf.elast -variable simElastic -text elastic]
    set eil [label $csf.eil -text Ei]
    set ei  [entry $csf.ei  -textvariable simEi -width 12]
    set efl [label $csf.efl -text Ef]
    set ef  [entry $csf.ef  -textvariable simEf -width 12]


    grid $dosim -row 0 -column 0 -columnspan 5
    grid $elast $eil $ei $efl $ef

    # add the 5 lines labels data
    set fs [label $csf.fs -text flip-state]
    set ct [label $csf.ct -text CS-counts]
    set tm [label $csf.tm -text msrDeltaTimes(Hrs)]
    grid $fs -row 2 -column 0
    grid $ct -row 2 -column 1 -columnspan 2
    grid $tm -row 2 -column 3 -columnspan 2

    for {set i 0;set j 3} {$i < 4} {incr i;incr j} {
	set fsl [label $csf.fs$i -text [lindex $fslbl $i]]
	set cts [entry $csf.ct$i -textvariable simct($i) -width 16]
	set tms [entry $csf.tm$i -textvariable simtm($i) -width 16]
	grid $fsl -row $j -column 0
	grid $cts -row $j -column 1 -columnspan 2
	grid $tms -row $j -column 3 -columnspan 2
    }

    # the CS transmission-details frame
    set dosimT [button $cst.do -text "CS transmission details" \
		    -bg green -activebackground yellow \
		    -command "doSim 1" -width 30]
    grid $dosimT -row 0 -column 0 -columnspan 2
    set transLbls {angle(deg) detoff(deg) rotH(deg) a4(deg)}
    set transVars {CFangle    CFdetoff    CFrotH   CFa4}
    for {set i 0;set j 1} {$i < 4} {incr i; incr j} {
	set cfl [label $cst.cfl$i \
		     -text [lindex $transLbls $i] -width 12]
	set cfe [entry $cst.cfe$i \
		     -textvariable [lindex $transVars $i] -width 12]
	grid $cfl -row $j -column 0
	grid $cfe -row $j -column 1
    }

    # the datapts frame
    set dlbl [label $dp.l -text "solved datapts"]
    set dosimD [button $dp.do -text "Examine solved datapts" \
		    -bg green -activebackground yellow \
		    -command "doSimDatapts 0" -width 28]
    set dosimDT [button $dp.dot -text "Examine transmission-details" \
		    -bg green -activebackground yellow \
		    -command "doSimDatapts 1" -width 28]
    set dptsl [label $dp.dptsl -text datapts(>0)]
    set dptse [entry $dp.dptse -textvariable CFdpt]

    grid $dlbl -row 0 -column 0 -columnspan 2
    grid $dosimD -row 1 -column 0 -columnspan 2 -sticky news
    grid $dosimDT -row 2 -column 0 -columnspan 2 -sticky news
    grid $dptsl -row 3 -column 0
    grid $dptse -row 3 -column 1

    # cmd frame
    set cw 28
    if { $mswindows } { set cw 31 }
    set prntCells [button $cmd.pc -text "print Cells" \
		  -bg green -activebackground yellow \
		  -command printSimCells -width $cw]
    set clearsim [button $cmd.cs -text "clear output" \
		  -bg green -activebackground yellow \
		  -command clearSimTxt -width $cw]
    set filesim  [button $cmd.sf -text "save output to pbsim" \
		  -bg green -activebackground yellow \
		  -command saveSimTxt -width $cw]

    grid $prntCells -sticky  news
    grid $clearsim -sticky news
    grid $filesim -sticky news

    set tw 160
    if { $mswindows } { set tw 150 }
    # now add the result text widget with both scrollbars
    set simtxt [text $rf.txt -wrap none -width $tw -height 40 \
		    -yscrollcommand "$rf.sy set" -xscrollcommand "$rf.sx set" \
		    -font {courier 10}]
    set sy [scrollbar $rf.sy -orient vert -command "$rf.txt yview"]
    set sx [scrollbar $rf.sx -orient horiz -command "$rf.txt xview"]
    grid $sy $simtxt -sticky news
    grid $sx -row 1 -column 1 -sticky ew
}

